/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/********************************
 ***	AgpAdaptivePalette.c   ***
 ********************************/

#include	"Uigp/igp.h"
#include	"ImageType/PaletteType.h"
#include	"GpType/GpType.h"


#define NBITS 4
#define CELL_NO (1 << (3 * NBITS))


#define		COLOR_TO_INDEX4( r, g, b )		( ((r&0xf0)<<4) | ((g&0xf0)<<0) | ((b&0xf0)>>4))

#ifdef WIN32
#define		COLOR_TO_INDEX4F( color )		( ((color&0xf00000)>>12) | ((color&0xf000)>>8) | ((color&0xf0)>>4))
#else
#define		COLOR_TO_INDEX4F( color )		( ((color&0xf00000)<<4) | ((color&0xf000)>>8) | ((color&0xf0)>>20))
#endif

#define		INDEX4_TO_RED( index )			( ( index >>4) & 0xf0 )
#define		INDEX4_TO_GREEN( index )			( ( index >>0) & 0xf0 )
#define		INDEX4_TO_BLUE( index )			( ( index <<4) & 0xf0 )





typedef struct	cell_type {
	int	r;
	int	g;
	int	b;

	int	n;

}	cell_type;


static int	agp_adaptive_palette_get_max( cell_type *h, int no,
										   palette_type *palette, int delta );


static void	agp_adaptive_palette_inverse( palette_type *pal );


static void	agp_adaptive_palette_histogram( gp_array *agp, cell_type H[] );

static void	agp_adaptive_palette_histogram_add( cell_type H[], qcolor_type val[] );



void 
agp_adaptive_palette( gp_array *agp[], int nAgp,
					   palette_type *palette, int nColor )
{
int	i,	k;
cell_type *H;



	H = (cell_type *)malloc(sizeof(cell_type)*  CELL_NO);

	for (i = 0; i < CELL_NO;  i++){
		H[i].n = 0;
		H[i].r = H[i].b = H[i].g = 0;
	}


	for( i = 0 ; i < nAgp ; i++ )
		agp_adaptive_palette_histogram( agp[i], H );


	for( i = palette->colorsNumber; i < 64 ; i++ ){
		k = agp_adaptive_palette_get_max( H, CELL_NO, palette, 32 );
		if( k < 0 )	break;


		palette->data[i].Red   = H[k].r / H[k].n;
		palette->data[i].Green = H[k].g / H[k].n;
		palette->data[i].Blue  = H[k].b / H[k].n;

		palette->colorsNumber++;

		H[k].n = 0;
	}


	for( i = palette->colorsNumber; i < nColor ; i++ ){
		k = agp_adaptive_palette_get_max( H, CELL_NO, palette, 0 );
		if( k < 0 )	break;


		palette->data[i].Red   = H[k].r / H[k].n;
		palette->data[i].Green = H[k].g / H[k].n;
		palette->data[i].Blue  = H[k].b / H[k].n;

		palette->colorsNumber++;

		H[k].n = 0;
	}

	free(H);


	palette->type = PALETTE_ADAPTIVE4;

	agp_adaptive_palette_inverse( palette );

}



void
agp_adaptive_palette_histogram( gp_array *agp, cell_type H[] )
					 
{
gp_type	*gp;
link_type	*link;
hunch_type	*hunch;
bgp_type	*bgp;
 
int	i,	j;

	if( agp->color_type != AGP_RGB )	return;

 
	for( i = 0 ; i < agp->Ni ; i++ )
	    for( j = 0 ; j < agp->Nj ; j++ ){
		gp = &agp->gp[i][j];
 

		for( bgp = gp->bgp ; bgp != NULL ; bgp = bgp->p )
			agp_adaptive_palette_histogram_add( H, bgp->val );

		

		for( hunch = gp->hunch ; hunch != NULL ; hunch = hunch->nh )
			agp_adaptive_palette_histogram_add( H, hunch->gval );

			


		for( link = gp->link ; link != NULL ; link = link->nl ){
			if( link->Fbr == 0 )	continue;

			switch( link->type ){
			case NF_N3:
				agp_adaptive_palette_histogram_add( H, link->pr[0].n3->mval );
				agp_adaptive_palette_histogram_add( H, link->pr[0].n3->side[0].bval );
				agp_adaptive_palette_histogram_add( H, link->pr[0].n3->side[1].bval );
			
				if( LINK_NEXT( link ) == NULL ){
					agp_adaptive_palette_histogram_add( H, link->pr[1].n3->mval );
					agp_adaptive_palette_histogram_add( H, link->pr[1].n3->side[0].bval );
					agp_adaptive_palette_histogram_add( H, link->pr[1].n3->side[1].bval );
				}
			    break;

			case NF_EDGE:
				agp_adaptive_palette_histogram_add( H, link->pr[0].edge->side[0].bval );
				agp_adaptive_palette_histogram_add( H, link->pr[0].edge->side[1].bval );
			
				if( LINK_NEXT( link ) == NULL ){
					agp_adaptive_palette_histogram_add( H, link->pr[1].edge->side[0].bval );
					agp_adaptive_palette_histogram_add( H, link->pr[1].edge->side[1].bval );
				}
			    break;
			case NF_N3R:
			case NF_RIDGE:
				agp_adaptive_palette_histogram_add( H, link->pr[0].ridge->mval );
			
				if( LINK_NEXT( link ) == NULL )
					agp_adaptive_palette_histogram_add( H, link->pr[1].ridge->mval );
			    break;
			}
		}
	    }
}



static void
agp_adaptive_palette_histogram_add( cell_type H[], qcolor_type val[] )
					 
{
int	r,	g,	b;
int	k;


	r = QCOLOR2F(val[SP_Y]);
	g = QCOLOR2F(val[SP_Q]);
	b = QCOLOR2F(val[SP_A]);

	k = COLOR_TO_INDEX4( r, g, b );


	H[k].r += r;
	H[k].g += g;
	H[k].b += b;
	H[k].n++;
}








static int
agp_adaptive_palette_get_max( cell_type *h, int no, palette_type *palette, int delta )
{
int	i,	j;
int	max,	k,	flag;
int	r,	g,	b;
int	rt,	gt,	bt;

	max = 0;
	k = -1;
	for (i = 0; i < no;  i++ ){
		if( h[i].n <= max )	continue;

		if( delta > 0 ){
			r = INDEX4_TO_RED( i )+8;
			g = INDEX4_TO_GREEN( i )+8;
			b = INDEX4_TO_BLUE( i)+8;


			flag = 0;
			for( j = 0 ; j < palette->colorsNumber ; j++ ){
				if( (rt = r - palette->data[j].Red ) < 0 )	rt = -rt;
				if( (gt = g - palette->data[j].Green ) < 0 )	gt = -gt;
				if( (bt = b - palette->data[j].Blue ) < 0 )	bt = -bt;

				if( rt < delta && gt < delta && bt < delta ){
					flag = 1;
					break;
				}
			}

			if( flag == 1 )	continue;
		}

		max = h[i].n;
		k = i;
	}

	if( delta > 0 && max < 25 )	return( -1 );

	return( k );
}



static void
agp_adaptive_palette_inverse( palette_type *pal )
{
int	r,	g,	b,	rt,	gt,	bt;
int	i,	j,	k,	max;

	pal->table = (u_char *)malloc( CELL_NO );

	for( i = 0 ; i < CELL_NO ; i++ ){
		r = INDEX4_TO_RED( i )+8;
		g = INDEX4_TO_GREEN( i )+8;
		b = INDEX4_TO_BLUE( i)+8;

		max = 1000;
		k = -1;
		for( j = 0 ; j < pal->colorsNumber ; j++ ){
			if( (rt = r - pal->data[j].Red ) < 0 )	rt = -rt;
			if( (gt = g - pal->data[j].Green ) < 0 )	gt = -gt;
			if( (bt = b - pal->data[j].Blue ) < 0 )	bt = -bt;

			if( rt < bt )	rt = bt;
			if( rt < gt ) rt = gt;

			if( rt > max )	continue;

			max = rt;
			k = j;
		}

		pal->table[i] = k;
	}
}



