/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/******************************
 ***	AgpChangeProfile.c	***
 ******************************/

#include    <stdio.h>   
#include	<math.h>

#include    "AgpColor.h"
#include    "AgpTool/AgpTool.h"

static void     agp_change_profile_edge( edge_type *e, qcolor_type *deltaC, float scale );
static void     agp_change_profile_ridge( ridge_type *e, qcolor_type *deltaC, float scale );
static void     agp_change_profile_n3( n3_type *e, qcolor_type *deltaC, float scale );

static void     agp_change_profile_hunch( hunch_type *e, qcolor_type *deltaC, float scale );
static void     agp_change_profile_bgp( bgp_type *e, qcolor_type *deltaC, float scale );

static void		color_rgb_change( qcolor_type *c, qcolor_type *deltaC, float scale );

void
agp_change_profile( gp_array *agp, qcolor_type *deltaC, float scale )
{
gp_type	*gp;
link_type	*l;
hunch_type	*hunch;
bgp_type	*bgp;
 
int	i,	j;

	if( agp->color_type != AGP_RGB )	return;


 
	for( i = 0 ; i < agp->Ni ; i++ )
	    for( j = 0 ; j < agp->Nj ; j++ ){
		gp = &agp->gp[i][j];
 

		for( bgp = gp->bgp ; bgp != NULL ; bgp = bgp->p )
			agp_change_profile_bgp( bgp, deltaC, scale );


		for( hunch = gp->hunch ; hunch != NULL ; hunch = hunch->nh )
			agp_change_profile_hunch( hunch, deltaC, scale );


		for( l = gp->link ; l != NULL ; l = l->nl ){

			switch( l->type ){
			case NF_N3:
				agp_change_profile_n3( l->pr[0].n3, deltaC, scale );
				if( LINK_NEXT( l ) == NULL )
				    agp_change_profile_n3( l->pr[1].n3, deltaC, scale );
			    break;

			case NF_EDGE:
				agp_change_profile_edge( l->pr[0].edge, deltaC, scale );
				if( LINK_NEXT( l ) == NULL )
				    agp_change_profile_edge( l->pr[1].edge, deltaC, scale );
			    break;
			case NF_N3R:
			case NF_RIDGE:
				agp_change_profile_ridge( l->pr[0].ridge, deltaC, scale );
				if( LINK_NEXT( l ) == NULL )
				    agp_change_profile_ridge( l->pr[1].ridge, deltaC, scale );
			    break;
			}
		}
	    }
}

int	agp_get_extrem_gistogram( gp_array *agp )
{
image_type *im, *im_y;
int	H[256];
int Hs15[256], H_max;
int i, j, i_H_max;

	im = agp_to_image( agp, 0x00FFFFFF, NULL );
	im_y = image4_to_y( im );

	image_histogram( im_y, H );

	image_destroy( im_y, TRUE );
	image_destroy( im, TRUE );

	for ( i = 0; i < 255; i++ )
		Hs15[i] = 0;
	for ( i = 7; i < 247; i++ )
		for ( j = i-7; j <= i+7; j++ )
			Hs15[i] += H[j];

	H_max = -1;
	for ( i = 0; i < 255; i++ )
		if ( Hs15[i] > H_max )
		{
			H_max = Hs15[i];
			i_H_max = i;
		}

	return i_H_max;
}

static void
agp_change_profile_edge( edge_type *e, qcolor_type *deltaC, float scale )
{
	color_rgb_change( e->side[F_RIGHT].val, deltaC, scale );

	color_rgb_change( e->side[F_LEFT].val, deltaC, scale );


	color_rgb_change( e->side[F_RIGHT].bval, deltaC, scale );

	color_rgb_change( e->side[F_LEFT].bval, deltaC, scale );
}


static void
agp_change_profile_n3( n3_type *e, qcolor_type *deltaC, float scale )
{
	color_rgb_change( e->side[F_RIGHT].val, deltaC, scale );

	color_rgb_change( e->side[F_LEFT].val, deltaC, scale );


	color_rgb_change( e->mval, deltaC, scale );

	color_rgb_change( e->side[F_RIGHT].bval, deltaC, scale );

	color_rgb_change( e->side[F_LEFT].bval, deltaC, scale );
}


static void
agp_change_profile_ridge( ridge_type *e, qcolor_type *deltaC, float scale )
{
	color_rgb_change( e->mval, deltaC, scale );
}



static void
agp_change_profile_hunch( hunch_type *e, qcolor_type *deltaC, float scale )
{
	color_rgb_change( e->gval, deltaC, scale );
}


static void
agp_change_profile_bgp( bgp_type *e, qcolor_type *deltaC, float scale )
{
	color_rgb_change( e->val, deltaC, scale );
}


static void
color_rgb_change( qcolor_type *c, qcolor_type *deltaC, float scale )
{
int A;

	for ( A = 0; A < SP_NO; A++ )
		c[A] = (qcolor_type)(c[A] * scale) + deltaC[A];
}
