/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/*************************
 ***    AgpColorF.c   ***
 *************************/ 
#include        <stdio.h>
#include        <math.h>

#include        "Uigp/igp.h"
 
#include        "GpType/GpType.h"
#include        "EpType/EpType.h"
 
#include        "Ep/Ep.h"
#include        "EpBgp/EpBgp.h"

#include        "N3Prime/N3Prime.h"


#include        "Quantize/Quantize.h"

 
 
#include        "AgpColor.h"

#include        "AgpColorF.h"


static	char	*Fblock;
#define         BST_SIZE         4000

typedef struct	st_bound_type {
                int	i,	j;
		link_type	*link;
		char	lside;
		char	side;

		struct st_bound_type	*p;
}	st_bound_type;



static struct           BLOCK_ST {
                int	i,	j;

		st_bound_type	*p;
}       Bst[BST_SIZE];




static int      Bst_b,   Bst_e;

#define BST_ADD( n )     (( n == BST_SIZE-1)? 0 : n++)


static	int	Brow,	Bcol;

static void	block_st_add( ep_array *aep,
			int i, int j, ep_type *ep, int bno );

static void	agp_color_F_from_point( gp_array *agp, ep_array *aep,
						box2d *box, vec2d *p );

static void	agp_color_F_begin_from_boundray( ep_array *aep );

static void	block_st_free( int st );




void
agp_color_F( gp_array *agp, box2d *box, vec2d *p, int step_no )
{
ep_array	*aep;
int	i0,	j0,	n,	k,	step;
int	Fn3_prime;



	quantize16_agp( agp );


        if( (Fn3_prime = agp->Fn3_prime) == 0 ) n3_prime_agp( agp );


	Brow = agp->row /( EP_N * agp->bgp ) + 1;
	Bcol = agp->column /( EP_N * agp->bgp ) + 1;

	Fblock = calloc( Brow*Bcol , 1 );

	aep = aep_alloc();

	Bst_b = Bst_e = 0;



	i0 = p->x /( EP_N * agp->bgp );
	j0 = p->y /( EP_N * agp->bgp );
	if( i0 < 0 )	i0 = 0;
	if( j0 < 0 )	j0 = 0;

	Fblock[ i0 * Bcol + j0 ] = 1;

	i0 = i0 * EP_N + 1;
	j0 = j0 * EP_N + 1;
        ep_create( agp, i0, j0, EP_N, EP_N, 0, aep );
	ep_bgp_get_average( agp, aep );


	agp_color_F_from_point( agp, aep, box, p );



	for( step = 1 ; Bst_b != Bst_e &&
			( step_no <= 0 || step < step_no ) ; step++ ){

		i0 = Bst[Bst_b].i;
		j0 = Bst[Bst_b].j;

		n = i0/EP_N;
		k = j0/EP_N;
	

		if( i0 <= 0 || i0 >= agp->Ni-1 ||
				j0 <= 0 || j0 >= agp->Nj-1 ){
			block_st_free( Bst_b );
			BST_ADD( Bst_b );
			continue;
		}

		if( Fblock[ n * Bcol + k ] == 1 ){
			block_st_free( Bst_b );
			BST_ADD( Bst_b );
			continue;
		}


		Fblock[ n * Bcol + k ] = 1;

        	ep_create( agp, i0, j0, EP_N, EP_N, 0, aep );
		ep_bgp_get_average( agp, aep );

		agp_color_F_begin_from_boundray( aep );

		agp_color_F1_ep( agp, aep, box );


		BST_ADD( Bst_b );
	}

	free( Fblock );
	Fblock = NULL;

	aep_destroy( aep );



	n3_prime_load( agp );

	if( Fn3_prime == 0 )    n3_prime_free( agp );
}



static void
agp_color_F_from_point( gp_array *agp, ep_array *aep, box2d *box, vec2d *p )
{
ep_type *ep;
vec2d   v;
int     i,      j,      bno;
float	dis;


        i = ( p->x - aep->V0.x + LSIZE*aep->bgp) * aep->bgp_inv;
        j = ( p->y - aep->V0.y + LSIZE*aep->bgp) * aep->bgp_inv;
        ep = &aep->ep[i][j];

        v.x = p->x - aep->V0.x - ep->ctr.x;
        v.y = p->y - aep->V0.y - ep->ctr.y;
        bno = ep_bound_point( ep, &v, &dis );
        if( bno == -1 ) return;
 

	agp_color_F1_st_clear();
	agp_color_F1_st_add( ep, bno );
 
 
        agp_color_F1_ep( agp, aep, box );
}



static void
agp_color_F_begin_from_boundray( ep_array *aep )
{
st_bound_type   *p;
ep_type	*ep;
al_type	*al;

	agp_color_F1_st_clear();

	while( ( p = Bst[Bst_b].p ) != NULL ){

		ep = &aep->ep[ p->i -aep->Ni0 + LSIZE][p->j - aep->Nj0 + LSIZE];

		if( p->link != NULL ){
			for( al = ep->al ; al != NULL ; al = al->nal ){
				if( p->link != al->link )
					continue;

				if( p->lside != al->link->side )
					continue;

				agp_color_F1_st_add( ep, al->bno[p->side] );
				break;
			}
		}
		else	agp_color_F1_st_add( ep, 0 );

		Bst[Bst_b].p = p->p;
		free( p );
	}
}




int
agp_color_F_test_boundary( ep_array *aep, ep_type *ep, int bno )
{
int	i0,	j0,	i,	j;


	i = ( ep->ctr.x + LSIZE*aep->bgp) * aep->bgp_inv;
	j = ( ep->ctr.y + LSIZE*aep->bgp) * aep->bgp_inv;

	if( i < LSIZE )		i0 = aep->Ni0 - EP_N;
	else	if( i >= aep->Ni - LSIZE )	i0 = aep->Ni0 +	EP_N;
		else	i0 = aep->Ni0;

	if( j < LSIZE )		j0 = aep->Nj0 - EP_N;
	else	if( j >= aep->Nj - LSIZE )	j0 = aep->Nj0 +	EP_N;
		else	j0 = aep->Nj0;

	if( i0 == aep->Ni0 && j0 == aep->Nj0 )	return( 0 );



	block_st_add( aep, i0, j0, ep, bno ); 

	return( 1 );
}


static void
block_st_add( ep_array *aep, int i, int j,  ep_type *ep, int bno )
{
st_bound_type	*b;
al_type	*al;
int	n,	k;
	if( Fblock == NULL )	return;


	n = i/EP_N;
	k = j/EP_N;
	
	if( Fblock[ n * Bcol + k ] == 1 )	return;




	b = (st_bound_type *)malloc( sizeof(st_bound_type) );

	b->i = ( ep->ctr.x + aep->V0.x + aep->bgp )* aep->bgp_inv;
	b->j = ( ep->ctr.y + aep->V0.y + aep->bgp )* aep->bgp_inv;
	b->p = NULL;

	if( ep->b_no > 1 ){
		for( al = ep->al ; al != NULL ; al = al->nal ){
			if( al->bno[F_RIGHT] == bno ){
				b->link = al->link;
				b->lside = al->link->side;
				b->side = F_RIGHT;
				break;
			}

			if( al->bno[F_LEFT] == bno ){
				b->link = al->link;
				b->lside = al->link->side;
				b->side = F_LEFT;
				break;
			}
		}
	}
	else	{
		b->link = NULL;
	}


	for( k = Bst_b ; k != Bst_e ; BST_ADD(k) ){

		if( Bst[ k ].i != i || Bst[ k ].j != j )	continue;

		b->p = Bst[ k ].p;
		Bst[ k].p = b;

		return;

	}


	Bst[ Bst_e ].i = i;
	Bst[ Bst_e ].j = j;

	Bst[ Bst_e ].p = b;

	BST_ADD( Bst_e );
}







static void
block_st_free( int st )
{
st_bound_type   *p;

	while( ( p = Bst[st].p ) != NULL ){

		Bst[st].p = p->p;
		free( p );
	}
}
