/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/*************************
 ***	AgpColorF1.c   ***
 *************************/

#include	<stdio.h>
#include	<math.h>

#include        "Uigp/igp.h"

#include        "GpType/GpType.h"
#include        "EpType/EpType.h"


#include        "RelValue/RelValue.h"

#include        "Ep/Ep.h"
#include        "EpBgp/EpBgp.h"


#include        "AgpColor.h"

#include        "AgpColorF.h"

#define		ST_SIZE		4000

static struct		ST {
		ep_type	*ep;
		int	bno;
}	St[ST_SIZE];

static int	St_b,	St_e;

#define	ST_ADD( n )	(( n == ST_SIZE-1)? 0 : n++)



static void	agp_color_F1_ep_one( ep_array *aep,
				ep_type *ep, int bno, box2d *box );


static float	Fcolor[SP_NO];

static int	Fill_op;

static int	Fprofile;


void
agp_color_F1_set( int fill_op, int fprofile, float color[] )
{
	Fill_op = fill_op;

	Fprofile = fprofile;

	Fcolor[SP_R] = color[SP_R];
	Fcolor[SP_G] = color[SP_G];
	Fcolor[SP_B] = color[SP_B];
}



void
agp_color_F1_ep( gp_array *agp, ep_array *aep, box2d *box )
{
ep_type	*ep;
int	i,	j,	k;



	/* clear br.no */
	for( i = JSIZE ; i < aep->Ni-JSIZE ; i++ )
		for( j = JSIZE ; j < aep->Nj-JSIZE ; j++ ){

			ep = &aep->ep[i][j];
			for( k = 0 ; k < ep->b_no ; k++ )
				ep->bound[k]->b2_no = 0;
		}


	while( St_b != St_e ){


	        i = ( aep->V0.x + St[St_b].ep->ctr.x + agp->bgp) * agp->inv_bgp;
        	j = ( aep->V0.y + St[St_b].ep->ctr.y + agp->bgp) * agp->inv_bgp;

 
		agp_color_F1_ep_one( aep, St[St_b].ep, St[St_b].bno, box );

		ST_ADD( St_b );
	}



	ep_bgp_put_1( agp, aep, Fprofile );

/*
	rel_value_agp( agp, aep, 0 );
*/
	abs_value_agp( agp, aep );
}




static void
agp_color_F1_ep_one( ep_array *aep, ep_type *ep, int bno, box2d *box )
{
ep_type	*nep;
bound_type	*b;
ebgp_type	*bgp;
vec2d	p;
int	i,	n;

	b = ep->bound[bno];

	if( b->b2_no != 0 )	return;
	b->b2_no = 1;

	if( agp_color_F_test_boundary( aep, ep, bno ) == 1 )	return;



	if( (bgp = b->bgp) != NULL ){

		switch( Fill_op ){
		case 1:
			bgp->val[SP_Y] = Fcolor[SP_Y];
			bgp->val[SP_Q] = Fcolor[SP_Q];
			bgp->val[SP_A] = Fcolor[SP_A];
		    break;
		case 2:
			bgp->val[SP_Y] = 0.5*(bgp->val[SP_Y] + Fcolor[SP_Y] );
			bgp->val[SP_Q] = 0.5*(bgp->val[SP_Q] + Fcolor[SP_Q] );
			bgp->val[SP_A] = 0.5*(bgp->val[SP_A] + Fcolor[SP_A] );
		    break;
		}
	}
		

	for( n = 0 ; n < 9 ; n++ ){
		if( n == 4 )	continue;
		if( b->Fcont[n] == 0 )	continue;
		nep = ep->ng[n];


		for( i = 0 ; i < nep->b_no ; i++ ){
			if( !( b->Fcont[n] & (1<<i) ))	continue;
			if( nep->bound[i]->b2_no != 0 )	continue;

			p.x = aep->V0.x + nep->ctr.x;
			p.y = aep->V0.y + nep->ctr.y;
			if( nep->bound[i]->bgp != NULL ){
				p.x += nep->bound[i]->bgp->ctr.x;
				p.y += nep->bound[i]->bgp->ctr.y;
			}
			if( !BOX2D_IS_IN( *box, p.x, p.y ) )	continue;


			agp_color_F1_st_add( ep->ng[n], i );
		}
	}
}


void
agp_color_F1_st_clear()
{
	St_b = St_e = 0;
}

void
agp_color_F1_st_add( ep_type *ep, int bno )
{
int	i;

	for( i = St_b ; i != St_e ; ST_ADD(i) )
		if( St[i].ep == ep && St[i].bno == bno )	return;

	St[St_e].ep = ep;
	St[St_e].bno = bno;

	ST_ADD( St_e );
}
