/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/************************
 ***	AgpGrbRgb.c   ***
 ************************/
#include        <stdio.h>   
#include	<math.h>

#include        "AgpColor.h"





#define		GRB_RGB( G, R, B ) \
			{ \
				float tmp; \
				tmp = G; \
				G = R; \
				R = tmp; \
			}

static void	agp_Grb_Rgb_edge( edge_type *e );
static void	agp_Grb_Rgb_ridge( ridge_type *e );
static void	agp_Grb_Rgb_n3( n3_type *e );

static void	agp_Grb_Rgb_hunch( hunch_type *e );
static void	agp_Grb_Rgb_bgp( bgp_type *e );



void
agp_Grb_Rgb( gp_array *agp )
{
gp_type	*gp;
link_type	*link;
hunch_type	*hunch;
bgp_type	*bgp;
 
int	i,	j;


/*
	if( agp->color_type == AGP_RGB )        return;
	agp->color_type = AGP_RGB;
*/

 
	for( i = 0 ; i < agp->Ni ; i++ )
	    for( j = 0 ; j < agp->Nj ; j++ ){
		gp = &agp->gp[i][j];
 

		for( bgp = gp->bgp ; bgp != NULL ; bgp = bgp->p )
			agp_Grb_Rgb_bgp( bgp );


		for( hunch = gp->hunch ; hunch != NULL ; hunch = hunch->nh )
			agp_Grb_Rgb_hunch( hunch );


		for( link = gp->link ; link != NULL ; link = link->nl ){

			switch( link->type ){
			case NF_N3:
				agp_Grb_Rgb_n3( link->pr[0].n3 );
				if( LINK_NEXT( link ) == NULL )
					agp_Grb_Rgb_n3( link->pr[1].n3 );
			    break;

			case NF_EDGE:
				agp_Grb_Rgb_edge( link->pr[0].edge );
				if( LINK_NEXT( link ) == NULL )
					agp_Grb_Rgb_edge( link->pr[1].edge );
			    break;
			case NF_N3R:
			case NF_RIDGE:
				agp_Grb_Rgb_ridge( link->pr[0].ridge );
				if( LINK_NEXT( link ) == NULL )
					agp_Grb_Rgb_ridge( link->pr[1].ridge);
			    break;
			}
		}
	    }
}



static void
agp_Grb_Rgb_edge( edge_type *e )
{
	GRB_RGB( e->side[F_RIGHT].val[SP_Y],
		e->side[F_RIGHT].val[SP_Q], e->side[F_RIGHT].val[SP_A] );

	GRB_RGB( e->side[F_LEFT].val[SP_Y],
		e->side[F_LEFT].val[SP_Q], e->side[F_LEFT].val[SP_A] );


	GRB_RGB( e->side[F_RIGHT].bval[SP_Y],
		e->side[F_RIGHT].bval[SP_Q], e->side[F_RIGHT].bval[SP_A] );

	GRB_RGB( e->side[F_LEFT].bval[SP_Y],
		e->side[F_LEFT].bval[SP_Q], e->side[F_LEFT].bval[SP_A] );
}


static void
agp_Grb_Rgb_n3( n3_type *e )
{
	GRB_RGB( e->side[F_RIGHT].val[SP_Y],
		e->side[F_RIGHT].val[SP_Q], e->side[F_RIGHT].val[SP_A] );

	GRB_RGB( e->mval[SP_Y], e->mval[SP_Q], e->mval[SP_A] );

	GRB_RGB( e->side[F_LEFT].val[SP_Y],
		e->side[F_LEFT].val[SP_Q], e->side[F_LEFT].val[SP_A] );


	GRB_RGB( e->side[F_RIGHT].bval[SP_Y],
		e->side[F_RIGHT].bval[SP_Q], e->side[F_RIGHT].bval[SP_A] );

	GRB_RGB( e->side[F_LEFT].bval[SP_Y],
		e->side[F_LEFT].bval[SP_Q], e->side[F_LEFT].bval[SP_A] );
}


static void
agp_Grb_Rgb_ridge( ridge_type *e )
{
	GRB_RGB( e->mval[SP_Y], e->mval[SP_Q], e->mval[SP_A] );
	GRB_RGB( e->rval[SP_Y], e->rval[SP_Q], e->rval[SP_A] );

	GRB_RGB( e->lm[SP_Y], e->lm[SP_Q], e->lm[SP_A] );
}


static void
agp_Grb_Rgb_bgp( bgp_type *e )
{
	GRB_RGB( e->val[SP_Y], e->val[SP_Q], e->val[SP_A] );
}


static void
agp_Grb_Rgb_hunch( hunch_type *e )
{
	GRB_RGB( e->gval[SP_Y], e->gval[SP_Q], e->gval[SP_A] );
	GRB_RGB( e->rval[SP_Y], e->rval[SP_Q], e->rval[SP_A] );

	GRB_RGB( e->lm1[SP_Y], e->lm1[SP_Q], e->lm1[SP_A] );
	if( e->type == NF_HUNCH )
		GRB_RGB( e->lm2[SP_Y], e->lm2[SP_Q], e->lm2[SP_A] );
}
