/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**************************
 ***	AgpRgbY.c   ***
 **************************/
#include        <stdio.h>   
#include	<math.h>

#include        "AgpColor.h"


/*
 *      Y  =  0.29900 * R + 0.58700 * G + 0.11400 * B
 */



static void	agp_Rgb_Y_edge( edge_type *e );
static void	agp_Rgb_Y_ridge( ridge_type *e );

static void	agp_Rgb_Y_edge( edge_type *e );
static void	agp_Rgb_Y_ridge( ridge_type *e );
static void	agp_Rgb_Y_n3( n3_type *e );

static void	agp_Rgb_Y_hunch( hunch_type *e );
static void	agp_Rgb_Y_bgp( bgp_type *e );


static void	color_Rgb_Y( qcolor_type c[], int Frel );




void
agp_Rgb_Y( gp_array *agp )
{
gp_type	*gp;
link_type	*link;
hunch_type	*hunch;
bgp_type	*bgp;
 
int	i,	j;

	if( agp->color_type != AGP_RGB )	return;

 
	for( i = 0 ; i < agp->Ni ; i++ )
	    for( j = 0 ; j < agp->Nj ; j++ ){
		gp = &agp->gp[i][j];
 

		for( bgp = gp->bgp ; bgp != NULL ; bgp = bgp->p )
			agp_Rgb_Y_bgp( bgp );


		for( hunch = gp->hunch ; hunch != NULL ; hunch = hunch->nh )
			agp_Rgb_Y_hunch( hunch );


		for( link = gp->link ; link != NULL ; link = link->nl ){

			switch( link->type ){
			case NF_N3:
				agp_Rgb_Y_n3( link->pr[0].n3 );
				if( LINK_NEXT( link ) == NULL )
					agp_Rgb_Y_n3( link->pr[1].n3 );
			    break;

			case NF_EDGE:
				agp_Rgb_Y_edge( link->pr[0].edge );
				if( LINK_NEXT( link ) == NULL )
					agp_Rgb_Y_edge( link->pr[1].edge );
			    break;
			case NF_N3R:
			case NF_RIDGE:
				agp_Rgb_Y_ridge( link->pr[0].ridge );
				if( LINK_NEXT( link ) == NULL )
					agp_Rgb_Y_ridge( link->pr[1].ridge);
			    break;
			}
		}
	    }
}



static void
agp_Rgb_Y_edge( edge_type *e )
{
	color_Rgb_Y( e->side[F_RIGHT].val, 0 );

	color_Rgb_Y( e->side[F_LEFT].val, 0 );


	color_Rgb_Y( e->side[F_RIGHT].bval, 0 );

	color_Rgb_Y( e->side[F_LEFT].bval, 0 );
}


static void
agp_Rgb_Y_n3( n3_type *e )
{
	color_Rgb_Y( e->side[F_RIGHT].val, 0 );

	color_Rgb_Y( e->mval, 0 );

	color_Rgb_Y( e->side[F_LEFT].val, 0 );


	color_Rgb_Y( e->side[F_RIGHT].bval, 0 );

	color_Rgb_Y( e->side[F_LEFT].bval, 0 );
}


static void
agp_Rgb_Y_ridge( ridge_type *e )
{
	color_Rgb_Y( e->mval, 0 );
	color_Rgb_Y( e->rval, 1 );

	color_Rgb_Y( e->lm, 1 );
}


static void
agp_Rgb_Y_bgp( bgp_type *e )
{
	color_Rgb_Y( e->val, 0 );
}


static void
agp_Rgb_Y_hunch( hunch_type *e )
{
	color_Rgb_Y( e->gval, 0 );
	color_Rgb_Y( e->rval, 1 );

	color_Rgb_Y( e->lm1, 1 );
	if( e->type == NF_HUNCH )
		color_Rgb_Y( e->lm2, 1 );
}



static void
color_Rgb_Y( qcolor_type c[], int Frel )
{
#ifdef  QFLOAT
float	R,	G,	B;
float	A;
	A = 0;//( Frel != 0 )? 0 : 128.5;

	R = c[SP_R] - A;
	G = c[SP_G] - A;
	B = c[SP_B] - A;

	c[SP_Y] = .29900*R + 0.58700*G + 0.11400*B;
	c[SP_Q] = c[SP_A] = c[SP_Y];
#endif

#ifdef  QSHORT
qcolor_type	R,	G,	B;
qcolor_type	A;
	A = 0;// ( Frel != 0 )? 0 : 128.5*COLOR_F2FP;

	R = c[SP_R] - A;
	G =  c[SP_G] - A;
	B =  c[SP_B] - A;

	c[SP_Y] = .29900*R + 0.58700*G + 0.11400*B;
	c[SP_Q] = c[SP_A] = c[SP_Y];
#endif

}
