/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**************************
 ***	AgpRgbYmax.c   ***
 **************************/
#include        <stdio.h>   
#include	<math.h>

#include        "AgpColor.h"


/*
 *      Y  =  0.29900 * R + 0.58700 * G + 0.11400 * B
 */

#ifdef  QFLOAT

#define COLOR_SHIFT_UP( c, shift ) \
	( (c) = ((c) >= (255.0F-((float)(shift)))) ? 255.0F : ((c)+((float)(shift))); )

#define COLOR_SHIFT_DOWN( c, shift ) \
	( (c) = ((c) <= ((float)(shift))) ? 0.0F : ((c)-((float)(shift))); )

#endif


#ifdef  QSHORT

#define COLOR_SHIFT_UP( c, shift ) \
	( (c) = ((c) >= (255*256-((shift)*256))) ? 255*256 : ((c)+((shift)*256)) )

#define COLOR_SHIFT_DOWN( c, shift ) \
	( (c) = ((c) <= ((shift)*256)) ? 0 : ((c)-((shift)*256)) )

#endif


static void	agp_Rgb_Ymax_edge( edge_type *e );
static void	agp_Rgb_Ymax_edge_1( edge_type *e );
static void	agp_Rgb_Ymax_edge_2( edge_type *e );

static void	agp_Rgb_Y_ridge( ridge_type *e );

static void	agp_Rgb_Ymax_n3( n3_type *e );
static void	agp_Rgb_Ymax_n3_1( n3_type *e );
static void	agp_Rgb_Ymax_n3_2( n3_type *e );

static void	agp_Rgb_Y_hunch( hunch_type *e );
static void	agp_Rgb_Y_bgp( bgp_type *e );


static void	color_Rgb_Y( qcolor_type c[], int Frel );




void
agp_Rgb_Ymax( gp_array *agp )
{
gp_type	*gp;
link_type	*link;
hunch_type	*hunch;
bgp_type	*bgp;
 
int	i,	j;

	if( agp->color_type != AGP_RGB )	return;

 
	for( i = 0 ; i < agp->Ni ; i++ )
	    for( j = 0 ; j < agp->Nj ; j++ ){
		gp = &agp->gp[i][j];
 

		for( bgp = gp->bgp ; bgp != NULL ; bgp = bgp->p )
			agp_Rgb_Y_bgp( bgp );


		for( hunch = gp->hunch ; hunch != NULL ; hunch = hunch->nh )
			agp_Rgb_Y_hunch( hunch );


		for( link = gp->link ; link != NULL ; link = link->nl ){

			switch( link->type ){
			case NF_N3:
				agp_Rgb_Ymax_n3( link->pr[0].n3 );
				if( LINK_NEXT( link ) == NULL )
					agp_Rgb_Ymax_n3( link->pr[1].n3 );
			    break;

			case NF_EDGE:
				agp_Rgb_Ymax_edge( link->pr[0].edge );
				if( LINK_NEXT( link ) == NULL )
					agp_Rgb_Ymax_edge( link->pr[1].edge );
			    break;
			case NF_N3R:
			case NF_RIDGE:
				agp_Rgb_Y_ridge( link->pr[0].ridge );
				if( LINK_NEXT( link ) == NULL )
					agp_Rgb_Y_ridge( link->pr[1].ridge);
			    break;
			}
		}
	    }
}


static void
agp_Rgb_Ymax_edge( edge_type *e )
{
int	dA;
edge_type e_old;

	e_old = *e;

	color_Rgb_Y( e->side[F_RIGHT].val, 0 );

	color_Rgb_Y( e->side[F_LEFT].val, 0 );


	color_Rgb_Y( e->side[F_RIGHT].bval, 0 );

	color_Rgb_Y( e->side[F_LEFT].bval, 0 );

	dA = ABS( e->side[F_RIGHT].val[SP_R] -
			  e->side[F_LEFT].val[SP_R] );

	if ( dA > 32*256 )
		return;

	if ( ABS( e_old.side[F_RIGHT].val[SP_R] - 
		      e_old.side[F_LEFT].val[SP_R] ) > dA ||
	     ABS( e_old.side[F_RIGHT].val[SP_G] - 
		      e_old.side[F_LEFT].val[SP_G] ) > dA ||
	     ABS( e_old.side[F_RIGHT].val[SP_B] - 
		      e_old.side[F_LEFT].val[SP_B] ) > dA )
	{
		agp_Rgb_Ymax_edge_2( e );
	}
}

static void
agp_Rgb_Ymax_edge_1( edge_type *e )
{
qcolor_type	dA;
qcolor_type	A_RIGHT, A_LEFT;
qcolor_type	Ab_RIGHT, Ab_LEFT;
		
	
	dA = ABS( e->side[F_RIGHT].val[SP_R]- 
			  e->side[F_LEFT].val[SP_R] );
	A_RIGHT = e->side[F_RIGHT].val[SP_R];
	A_LEFT  = e->side[F_LEFT].val[SP_R];
	Ab_RIGHT= e->side[F_RIGHT].bval[SP_R];
	Ab_LEFT = e->side[F_LEFT].bval[SP_R];

	//c[SP_Y] = .29900*R + 0.58700*G + 0.11400*B;

	if ( ABS( e->side[F_RIGHT].val[SP_G]- 
			  e->side[F_LEFT].val[SP_G] ) > dA )
	{
		A_RIGHT = e->side[F_RIGHT].val[SP_G];
		A_LEFT  = e->side[F_LEFT].val[SP_G];
		Ab_RIGHT= e->side[F_RIGHT].bval[SP_G];
		Ab_LEFT = e->side[F_LEFT].bval[SP_G];
	}

	if ( ABS( e->side[F_RIGHT].val[SP_B]- 
			  e->side[F_LEFT].val[SP_B] ) > dA )
	{
		A_RIGHT = e->side[F_RIGHT].val[SP_B];
		A_LEFT  = e->side[F_LEFT].val[SP_B];
		Ab_RIGHT= e->side[F_RIGHT].bval[SP_B];
		Ab_LEFT = e->side[F_LEFT].bval[SP_B];
	}

	e->side[F_RIGHT].val[SP_R] =
	e->side[F_RIGHT].val[SP_G] =
	e->side[F_RIGHT].val[SP_B] = A_RIGHT;
	e->side[F_LEFT].val[SP_R] =
	e->side[F_LEFT].val[SP_G] =
	e->side[F_LEFT].val[SP_B] = A_LEFT;
	e->side[F_RIGHT].bval[SP_R] =
	e->side[F_RIGHT].bval[SP_G] =
	e->side[F_RIGHT].bval[SP_B] = Ab_RIGHT;
	e->side[F_LEFT].bval[SP_R] =
	e->side[F_LEFT].bval[SP_G] =
	e->side[F_LEFT].bval[SP_B] = Ab_LEFT;
}

static void
agp_Rgb_Ymax_edge_2( edge_type *e )
{
int	dA;

	dA = ABS( e->side[F_RIGHT].val[SP_R] -
			  e->side[F_LEFT].val[SP_R] );

	if ( dA < 16*256 )
	{

		if ( e->side[F_RIGHT].val[SP_R] > 
			 e->side[F_LEFT].val[SP_R] )
		{
			//COLOR_SHIFT_UP( e->side[F_RIGHT].val[SP_R], 16 );
			COLOR_SHIFT_DOWN( e->side[F_LEFT].val[SP_R], 16 );
		}
		else
		{
			COLOR_SHIFT_DOWN( e->side[F_RIGHT].val[SP_R], 16 );
			//COLOR_SHIFT_UP( e->side[F_LEFT].val[SP_R], 16 );
		}


		e->side[F_RIGHT].val[SP_B] =
		e->side[F_RIGHT].val[SP_G] =
		e->side[F_RIGHT].val[SP_R];
		
		e->side[F_LEFT].val[SP_B] =
		e->side[F_LEFT].val[SP_G] =
		e->side[F_LEFT].val[SP_R];


/*		e->side[F_RIGHT].bval[SP_R] =
		e->side[F_RIGHT].bval[SP_G] =
		e->side[F_RIGHT].bval[SP_B] = 
			e->side[F_RIGHT].val[SP_R];

		e->side[F_LEFT].bval[SP_R] =
		e->side[F_LEFT].bval[SP_G] =
		e->side[F_LEFT].bval[SP_B] = 
			e->side[F_LEFT].val[SP_R]; */

	}
}


static void
agp_Rgb_Ymax_n3( n3_type *e )
{
int	dA;
n3_type e_old;

	e_old = *e;

	color_Rgb_Y( e->side[F_RIGHT].val, 0 );

	color_Rgb_Y( e->mval, 0 );

	color_Rgb_Y( e->side[F_LEFT].val, 0 );


	color_Rgb_Y( e->side[F_RIGHT].bval, 0 );

	color_Rgb_Y( e->side[F_LEFT].bval, 0 );

	dA = ABS( e->side[F_RIGHT].val[SP_R] -
			  e->side[F_LEFT].val[SP_R] );

	if ( dA > 32*256 )
		return;

	if ( ABS( e_old.side[F_RIGHT].val[SP_R] - 
		      e_old.side[F_LEFT].val[SP_R] ) > dA ||
	     ABS( e_old.side[F_RIGHT].val[SP_G] - 
		      e_old.side[F_LEFT].val[SP_G] ) > dA ||
	     ABS( e_old.side[F_RIGHT].val[SP_B] - 
		      e_old.side[F_LEFT].val[SP_B] ) > dA )
	{
		agp_Rgb_Ymax_n3_2( e );
	}
}



static void
agp_Rgb_Ymax_n3_1( n3_type *e )
{
qcolor_type	dA;
qcolor_type	A_RIGHT, A_LEFT, A_m;
qcolor_type	Ab_RIGHT, Ab_LEFT;
		
	dA = ABS( e->side[F_RIGHT].val[SP_R]- 
			  e->side[F_LEFT].val[SP_R] );

	A_RIGHT = e->side[F_RIGHT].val[SP_R];
	A_m		= e->mval[SP_R];
	A_LEFT  = e->side[F_LEFT].val[SP_R];
	Ab_RIGHT= e->side[F_RIGHT].bval[SP_R];
	Ab_LEFT = e->side[F_LEFT].bval[SP_R];

	if ( ABS( e->side[F_RIGHT].val[SP_G]- 
			  e->side[F_LEFT].val[SP_G] ) > dA )
	{
		A_RIGHT = e->side[F_RIGHT].val[SP_G];
		A_m		= e->mval[SP_G];
		A_LEFT  = e->side[F_LEFT].val[SP_G];
		Ab_RIGHT= e->side[F_RIGHT].bval[SP_G];
		Ab_LEFT = e->side[F_LEFT].bval[SP_G];
	}

	if ( ABS( e->side[F_RIGHT].val[SP_B]- 
			  e->side[F_LEFT].val[SP_B] ) > dA )
	{
		A_RIGHT = e->side[F_RIGHT].val[SP_B];
		A_m		= e->mval[SP_B];
		A_LEFT  = e->side[F_LEFT].val[SP_B];
		Ab_RIGHT= e->side[F_RIGHT].bval[SP_B];
		Ab_LEFT = e->side[F_LEFT].bval[SP_B];
	}

	e->side[F_RIGHT].val[SP_R] =
	e->side[F_RIGHT].val[SP_G] =
	e->side[F_RIGHT].val[SP_B] = A_RIGHT;
	e->mval[SP_R] = 
	e->mval[SP_G] =
	e->mval[SP_B] = A_m;
	e->side[F_LEFT].val[SP_R] =
	e->side[F_LEFT].val[SP_G] =
	e->side[F_LEFT].val[SP_B] = A_LEFT;
	e->side[F_RIGHT].bval[SP_R] =
	e->side[F_RIGHT].bval[SP_G] =
	e->side[F_RIGHT].bval[SP_B] = Ab_RIGHT;
	e->side[F_LEFT].bval[SP_R] =
	e->side[F_LEFT].bval[SP_G] =
	e->side[F_LEFT].bval[SP_B] = Ab_LEFT;
}


static void
agp_Rgb_Ymax_n3_2( n3_type *e )
{
int	dA;

	dA = ABS( e->side[F_RIGHT].val[SP_R] -
			  e->side[F_LEFT].val[SP_R] );

	if ( dA < 16*256 )
	{

		if ( e->side[F_RIGHT].val[SP_R] > 
			 e->side[F_LEFT].val[SP_R] )
		{
			//COLOR_SHIFT_UP( e->side[F_RIGHT].val[SP_R], 16 );
			COLOR_SHIFT_DOWN( e->side[F_LEFT].val[SP_R], 16 );
		}
		else
		{
			COLOR_SHIFT_DOWN( e->side[F_RIGHT].val[SP_R], 16 );
			//COLOR_SHIFT_UP( e->side[F_LEFT].val[SP_R], 16 );
		}

		e->side[F_RIGHT].val[SP_B] =
		e->side[F_RIGHT].val[SP_G] =
		e->side[F_RIGHT].val[SP_R];

		e->side[F_LEFT].val[SP_B] =
		e->side[F_LEFT].val[SP_G] =
		e->side[F_LEFT].val[SP_R];

/*		e->side[F_RIGHT].bval[SP_R] =
		e->side[F_RIGHT].bval[SP_G] =
		e->side[F_RIGHT].bval[SP_B] = 
			e->side[F_RIGHT].val[SP_R];

		e->side[F_LEFT].bval[SP_R] =
		e->side[F_LEFT].bval[SP_G] =
		e->side[F_LEFT].bval[SP_B] = 
			e->side[F_LEFT].val[SP_R]; */
	}

}



static void
agp_Rgb_Y_ridge( ridge_type *e )
{
	color_Rgb_Y( e->mval, 0 );
	color_Rgb_Y( e->rval, 1 );

	color_Rgb_Y( e->lm, 1 );
}


static void
agp_Rgb_Y_bgp( bgp_type *e )
{
	color_Rgb_Y( e->val, 0 );
}


static void
agp_Rgb_Y_hunch( hunch_type *e )
{
	color_Rgb_Y( e->gval, 0 );
	color_Rgb_Y( e->rval, 1 );

	color_Rgb_Y( e->lm1, 1 );
	if( e->type == NF_HUNCH )
		color_Rgb_Y( e->lm2, 1 );
}



static void
color_Rgb_Y( qcolor_type c[], int Frel )
{
#ifdef  QFLOAT
float	R,	G,	B;
float	A;
	A = 0;//( Frel != 0 )? 0 : 128.5;

	R = c[SP_R] - A;
	G = c[SP_G] - A;
	B = c[SP_B] - A;

	c[SP_Y] = .29900*R + 0.58700*G + 0.11400*B;
	c[SP_Q] = c[SP_A] = c[SP_Y];
#endif

#ifdef  QSHORT
qcolor_type	R,	G,	B;
qcolor_type	A;
	A = 0;// ( Frel != 0 )? 0 : 128.5*COLOR_F2FP;

	R = c[SP_R] - A;
	G =  c[SP_G] - A;
	B =  c[SP_B] - A;

	c[SP_Y] = .29900*R + 0.58700*G + 0.11400*B;
	c[SP_Q] = c[SP_A] = c[SP_Y];
#endif

}

