/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**************************
 ***  AgpYCrCbRangeY.c  ***
 **************************/
#include        <stdio.h>   
#include	<math.h>

#include        "AgpColor.h"


static void	agp_YCrCb_RangeY_edge( edge_type *e );
static void	agp_YCrCb_RangeY_ridge( ridge_type *e );

static void	agp_YCrCb_RangeY_edge( edge_type *e );
static void	agp_YCrCb_RangeY_ridge( ridge_type *e );
static void	agp_YCrCb_RangeY_n3( n3_type *e );

static void	agp_YCrCb_RangeY_hunch( hunch_type *e );
static void	agp_YCrCb_RangeY_bgp( bgp_type *e );


static void	color_YCrCb_RangeY( qcolor_type c[], int Frel );


void
agp_YCrCb_RangeY_init()
{
}



void
agp_YCrCb_RangeY( gp_array *agp )
{
gp_type	*gp;
link_type	*link;
hunch_type	*hunch;
bgp_type	*bgp;
 
int	i,	j;


	if( agp->color_type == AGP_RGB )        return;
	agp->color_type = AGP_RGB;

 
	for( i = 0 ; i < agp->Ni ; i++ )
	    for( j = 0 ; j < agp->Nj ; j++ ){
		gp = &agp->gp[i][j];
 

		for( bgp = gp->bgp ; bgp != NULL ; bgp = bgp->p )
			agp_YCrCb_RangeY_bgp( bgp );


		for( hunch = gp->hunch ; hunch != NULL ; hunch = hunch->nh )
			agp_YCrCb_RangeY_hunch( hunch );


		for( link = gp->link ; link != NULL ; link = link->nl ){

			switch( link->type ){
			case NF_N3:
				agp_YCrCb_RangeY_n3( link->pr[0].n3 );
				if( LINK_NEXT( link ) == NULL )
					agp_YCrCb_RangeY_n3( link->pr[1].n3 );
			    break;

			case NF_EDGE:
				agp_YCrCb_RangeY_edge( link->pr[0].edge );
				if( LINK_NEXT( link ) == NULL )
					agp_YCrCb_RangeY_edge( link->pr[1].edge );
			    break;
			case NF_N3R:
			case NF_RIDGE:
				agp_YCrCb_RangeY_ridge( link->pr[0].ridge );
				if( LINK_NEXT( link ) == NULL )
					agp_YCrCb_RangeY_ridge( link->pr[1].ridge);
			    break;
			}
		}
	    }
}



static void
agp_YCrCb_RangeY_edge( edge_type *e )
{
	color_YCrCb_RangeY( e->side[F_RIGHT].val, 0 );

	color_YCrCb_RangeY( e->side[F_LEFT].val, 0 );


	color_YCrCb_RangeY( e->side[F_RIGHT].bval, 0 );

	color_YCrCb_RangeY( e->side[F_LEFT].bval, 0 );
}


static void
agp_YCrCb_RangeY_n3( n3_type *e )
{
	color_YCrCb_RangeY( e->side[F_RIGHT].val, 0 );

	color_YCrCb_RangeY( e->mval, 0 );

	color_YCrCb_RangeY( e->side[F_LEFT].val, 0 );


	color_YCrCb_RangeY( e->side[F_RIGHT].bval, 0 );

	color_YCrCb_RangeY( e->side[F_LEFT].bval, 0 );
}


static void
agp_YCrCb_RangeY_ridge( ridge_type *e )
{
	color_YCrCb_RangeY( e->mval, 0 );
	color_YCrCb_RangeY( e->rval, 1 );

	color_YCrCb_RangeY( e->lm, 1 );
}


static void
agp_YCrCb_RangeY_bgp( bgp_type *e )
{
	color_YCrCb_RangeY( e->val, 0 );
}


static void
agp_YCrCb_RangeY_hunch( hunch_type *h )
{
	color_YCrCb_RangeY( h->gval, 0 );
	color_YCrCb_RangeY( h->rval, 1 );

	color_YCrCb_RangeY( h->lm1, 1 );
	if( h->type == NF_HUNCH )
		color_YCrCb_RangeY( h->lm2, 1 );
}



static void
color_YCrCb_RangeY( qcolor_type c[], int Frel )
{
#ifdef	QFLOAT
float	A;

	A = ( Frel != 0 )? 0 : 128.5;

	c[SP_Y] += A;
	c[SP_Q] = 0;
	c[SP_A] = 0;

#endif

#ifdef	QSHORT
float	A;

	A = ( Frel != 0 )? 0 : 128.5*COLOR_F2FP;

	c[SP_Y] += A;
	c[SP_Q] = 0;
	c[SP_A] = 0;

#endif
}
