/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/************************
 ***	AgpYiqRgb.c   ***
 ************************/
#include        <stdio.h>   
#include	<math.h>

#include        "AgpColor.h"





#define		YIQ_RGB( YR, IG, QB ) \
			{ \
				float Y,	I,	Q; \
				Y = YR; \
				I = IG; \
				Q = QB; \
				YR = Y + 1.3333 * I    + 128.5; \
				IG = Y - 0.6666 * I + Q + 128.5; \
				QB = Y - 0.6666 * I - Q + 128.5; \
			}

#define		YIQ_RGB1( YR, IG, QB ) \
			{ \
				float Y,	I,	Q; \
				Y = YR; \
				I = IG; \
				Q = QB; \
				YR = Y + 1.3333 * I; \
				IG = Y - 0.6666 * I; \
				QB = Y - 0.6666 * I; \
			}




static void	agp_Yiq_Rgb_edge( edge_type *e );
static void	agp_Yiq_Rgb_ridge( ridge_type *e );

static void	agp_Yiq_Rgb_edge( edge_type *e );
static void	agp_Yiq_Rgb_ridge( ridge_type *e );
static void	agp_Yiq_Rgb_n3( n3_type *e );

static void	agp_Yiq_Rgb_hunch( hunch_type *e );
static void	agp_Yiq_Rgb_bgp( bgp_type *e );


static void	color_Yiq_Rgb( qcolor_type c[], int Frel );


void
agp_Yiq_Rgb_init()
{
}



void
agp_Yiq_Rgb( gp_array *agp )
{
gp_type	*gp;
link_type	*link;
hunch_type	*hunch;
bgp_type	*bgp;
 
int	i,	j;


	if( agp->color_type == AGP_RGB )        return;
	agp->color_type = AGP_RGB;

 
	for( i = 0 ; i < agp->Ni ; i++ )
	    for( j = 0 ; j < agp->Nj ; j++ ){
		gp = &agp->gp[i][j];
 

		for( bgp = gp->bgp ; bgp != NULL ; bgp = bgp->p )
			agp_Yiq_Rgb_bgp( bgp );


		for( hunch = gp->hunch ; hunch != NULL ; hunch = hunch->nh )
			agp_Yiq_Rgb_hunch( hunch );


		for( link = gp->link ; link != NULL ; link = link->nl ){

			switch( link->type ){
			case NF_N3:
				agp_Yiq_Rgb_n3( link->pr[0].n3 );
				if( LINK_NEXT( link ) == NULL )
					agp_Yiq_Rgb_n3( link->pr[1].n3 );
			    break;

			case NF_EDGE:
				agp_Yiq_Rgb_edge( link->pr[0].edge );
				if( LINK_NEXT( link ) == NULL )
					agp_Yiq_Rgb_edge( link->pr[1].edge );
			    break;
			case NF_N3R:
			case NF_RIDGE:
				agp_Yiq_Rgb_ridge( link->pr[0].ridge );
				if( LINK_NEXT( link ) == NULL )
					agp_Yiq_Rgb_ridge( link->pr[1].ridge);
			    break;
			}
		}
	    }
}



static void
agp_Yiq_Rgb_edge( edge_type *e )
{
        color_Yiq_Rgb( e->side[F_RIGHT].val, 0 );
 
        color_Yiq_Rgb( e->side[F_LEFT].val, 0 );
 
 
        color_Yiq_Rgb( e->side[F_RIGHT].bval, 0 );
 
        color_Yiq_Rgb( e->side[F_LEFT].bval, 0 );
}


static void
agp_Yiq_Rgb_n3( n3_type *e )
{
        color_Yiq_Rgb( e->side[F_RIGHT].val, 0 );
 
        color_Yiq_Rgb( e->mval, 0 );
 
        color_Yiq_Rgb( e->side[F_LEFT].val, 0 );
 
 
        color_Yiq_Rgb( e->side[F_RIGHT].bval, 0 );
 
        color_Yiq_Rgb( e->side[F_LEFT].bval, 0 );
}


static void
agp_Yiq_Rgb_ridge( ridge_type *e )
{
        color_Yiq_Rgb( e->mval, 0 );
        color_Yiq_Rgb( e->rval, 1 );
 
        color_Yiq_Rgb( e->lm, 1 );
}


static void
agp_Yiq_Rgb_bgp( bgp_type *e )
{
	color_Yiq_Rgb( e->val, 0 );
}


static void
agp_Yiq_Rgb_hunch( hunch_type *e )
{
        color_Yiq_Rgb( e->gval, 0 );
        color_Yiq_Rgb( e->rval, 1 );
 
        color_Yiq_Rgb( e->lm1, 1 );
        if( e->type == NF_HUNCH )
                color_Yiq_Rgb( e->lm2, 1 );
}


static void
color_Yiq_Rgb( qcolor_type c[], int Frel )
{
#ifdef	QFLOAT
float Y,	I,	Q;
float	A;

	Y = c[SP_Y];
	I = c[SP_Q];
	Q = c[SP_A];

	A = ( Frel != 0 )? 0 : 128.5;

	c[SP_R] = Y + 1.3333 * I + A;
	c[SP_G] = Y - 0.6666 * I + Q + A;
	c[SP_B] = Y - 0.6666 * I - Q + A;
#endif

#ifdef	QSHORT
float Y,	I,	Q;
float	A;

	Y = c[SP_Y];
	I = c[SP_Q];
	Q = c[SP_A];

	A = ( Frel != 0 )? 0 : 128.5*COLOR_F2FP;

	c[SP_R] = Y + 1.3333 * I + A;
	c[SP_G] = Y - 0.6666 * I + Q + A;
	c[SP_B] = Y - 0.6666 * I - Q + A;
#endif
}
