/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
#include	<stdio.h>
#include	<math.h>

#include	"Uigp/igp.h"


#include	"GpType/GpType.h"
#include	"AgpTool/AgpTool.h"
#include	"AgpEdit/AgpEditTool.h"




static void	agp_default_n3( gp_array *agp, vec2d *p, n3_type *e );
static void	agp_default_edge( gp_array *agp, vec2d *p, edge_type *e );
static void	agp_default_ridge( gp_array *agp, vec2d *p, ridge_type *r);
static void	agp_default_width( width_type *w );


static qcolor_type	Dcolor[SP_NO];
static float	Dwidth = 1.0;
static float	Dlen = 1.0;



void
agp_default_profile_set( int color[], float width, float len )
{
	Dcolor[0] = F2QCOLOR(color[0]);
	Dcolor[1] = F2QCOLOR(color[1]);
	Dcolor[2] = F2QCOLOR(color[2]);

	if( width > 0 )	Dwidth = width;
	if( len > 0 )	Dlen = len;
}


void
agp_default_profile_setF( float color[], float width, float len )
{
	Dcolor[0] = F2QCOLOR(color[0]);
	Dcolor[1] = F2QCOLOR(color[1]);
	Dcolor[2] = F2QCOLOR(color[2]);

	if( width > 0 )	Dwidth = width;
	if( len > 0 )	Dlen = len;
}




void
agp_default_profile( gp_array *agp,
		vec2d *ctr, link_type *link, int direct, float t, int Falloc )
{
profile_type	pr;
link_type	*l;
/*
jnt_type	*jnt;
*/
vec2d	p;

	l = link;

	if( direct == F_END )
		link_t2xy( ctr, link, link->len - t, &p );
	else	
		link_t2xy( ctr, link, t, &p );

	if( Falloc )
		l->pr[direct] = profile_alloc( l->type );

	pr = l->pr[direct];


	switch( l->type ){
	case NF_RIDGE:
	case NF_N3R:
		agp_default_ridge( agp, &p, pr.ridge );
	    break;

	case NF_N3:
		agp_default_n3( agp, &p, pr.n3 );

		if( Falloc )
			l->width[direct] = width_alloc();

		agp_default_width( l->width[direct] );


	    break;
	case NF_EDGE:
		agp_default_edge( agp, &p, pr.edge );
	    break;
	}
}




static void
agp_default_ridge( gp_array *agp, vec2d *p, ridge_type *r )
{
float	val[SP_NO];
qcolor_type	qval[SP_NO];

	agp_bg_value( agp, p, val );

	qval[SP_R] =  F2QCOLOR(val[SP_R]);
	qval[SP_G] =  F2QCOLOR(val[SP_G]);
	qval[SP_B] =  F2QCOLOR(val[SP_B]);

	r->mval[SP_Y] = Dcolor[SP_Y];
	r->mval[SP_Q] = Dcolor[SP_Q];
	r->mval[SP_A] = Dcolor[SP_A];

	r->rval[SP_Y] = r->mval[SP_Y] - qval[SP_Y];
	r->rval[SP_Q] = r->mval[SP_Q] - qval[SP_Q];
	r->rval[SP_A] = r->mval[SP_A] - qval[SP_A];

	r->lm[SP_Y] = -r->rval[SP_Y] / (Dwidth*Dwidth);
	r->lm[SP_Q] = -r->rval[SP_Q] / (Dwidth*Dwidth);
	r->lm[SP_A] = -r->rval[SP_A] / (Dwidth*Dwidth);

	r->width = Dwidth;
}



static void
agp_default_edge( gp_array *agp, vec2d *p, edge_type *e )
{
float	val[SP_NO];
qcolor_type	qval[SP_NO];

	agp_bg_value( agp, p, val );

	qval[SP_R] =  F2QCOLOR(val[SP_R]);
	qval[SP_G] =  F2QCOLOR(val[SP_G]);
	qval[SP_B] =  F2QCOLOR(val[SP_B]);

	e->side[F_RIGHT].bval[SP_Y] = e->side[F_LEFT].bval[SP_Y] = qval[SP_Y];
	e->side[F_RIGHT].bval[SP_Q] = e->side[F_LEFT].bval[SP_Q] = qval[SP_Q];
	e->side[F_RIGHT].bval[SP_A] = e->side[F_LEFT].bval[SP_A] = qval[SP_A];

	e->side[F_RIGHT].val[SP_Y] = qval[SP_Y]+10;
	e->side[F_LEFT].val[SP_Y] = qval[SP_Y]-10;
	e->side[F_RIGHT].val[SP_Q] = e->side[F_LEFT].val[SP_Q] = qval[SP_Q];
	e->side[F_RIGHT].val[SP_A] = e->side[F_LEFT].val[SP_A] = qval[SP_A];

	e->width = Dwidth;
}


static void
agp_default_n3( gp_array *agp, vec2d *p, n3_type *e )
{
float	val[SP_NO];
qcolor_type	qval[SP_NO];

	agp_bg_value( agp, p, val );

	qval[SP_R] =  F2QCOLOR(val[SP_R]);
	qval[SP_G] =  F2QCOLOR(val[SP_G]);
	qval[SP_B] =  F2QCOLOR(val[SP_B]);

	e->side[F_RIGHT].bval[SP_Y] = e->side[F_LEFT].bval[SP_Y] = qval[SP_Y];
	e->side[F_RIGHT].bval[SP_Q] = e->side[F_LEFT].bval[SP_Q] = qval[SP_Q];
	e->side[F_RIGHT].bval[SP_A] = e->side[F_LEFT].bval[SP_A] = qval[SP_A];

	e->side[F_RIGHT].val[SP_Y] = e->side[F_LEFT].val[SP_Y] = qval[SP_Y];
	e->side[F_RIGHT].val[SP_Q] = e->side[F_LEFT].val[SP_Q] = qval[SP_Q];
	e->side[F_RIGHT].val[SP_A] = e->side[F_LEFT].val[SP_A] = qval[SP_A];

	e->mval[SP_Y] = Dcolor[SP_Y];
	e->mval[SP_Q] = Dcolor[SP_Q];
	e->mval[SP_A] = Dcolor[SP_A];

	e->width[F_RIGHT] = e->width[F_LEFT] = Dwidth;

}


static void
agp_default_width( width_type *w )
{
	w->w[F_RIGHT] = Dwidth;
	w->w[F_LEFT] = -Dwidth;

	w->depth[F_RIGHT] = w->depth[F_LEFT] = 1.0;
}



void
agp_default_hunch( gp_array *agp, vec2d *p, vec2d *v )
{
hunch_type	*h;
float	val[SP_NO];
qcolor_type	qval[SP_NO];
int     i,      j;
gp_type *gp;

	h = HUNCH_ALLOC(1);

	agp_bg_value( agp, p, val );

	qval[SP_R] =  F2QCOLOR(val[SP_R]);
	qval[SP_G] =  F2QCOLOR(val[SP_G]);
	qval[SP_B] =  F2QCOLOR(val[SP_B]);

	h->gval[SP_Y] = Dcolor[SP_Y];
	h->gval[SP_Q] = Dcolor[SP_Q];
	h->gval[SP_A] = Dcolor[SP_A];

	h->rval[SP_Y] = h->gval[SP_Y] - qval[SP_Y];
	h->rval[SP_Q] = h->gval[SP_Q] - qval[SP_Q];
	h->rval[SP_A] = h->gval[SP_A] - qval[SP_A];

	h->lm1[SP_Y] = -h->rval[SP_Y] / (Dwidth*Dwidth);
	h->lm1[SP_Q] = -h->rval[SP_Q] / (Dwidth*Dwidth);
	h->lm1[SP_A] = -h->rval[SP_A] / (Dwidth*Dwidth);

	h->lm2[SP_Y] = -h->rval[SP_Y] / (Dlen*Dlen);
	h->lm2[SP_Q] = -h->rval[SP_Q] / (Dlen*Dlen);
	h->lm2[SP_A] = -h->rval[SP_A] / (Dlen*Dlen);

	h->g.width = Dwidth;
	h->g.len = Dlen;

/*
	h->g.theta = 0;
*/
	h->g.theta = atan2( v->x, -v->y );

	h->g.u.x = cos( (double)(h->g.theta) );
	h->g.u.y = sin( (double)(h->g.theta) );

	h->ctr = *p;
	h->type = NF_HUNCH;


        i = (h->ctr.x+agp->bgp) * agp->inv_bgp;
        j = (h->ctr.y+agp->bgp) * agp->inv_bgp;
 
 
        gp = &agp->gp[i][j];
 
 
        h->ctr.x -= gp->ctr.x;
        h->ctr.y -= gp->ctr.y;
        h->nh = gp->hunch;
        gp->hunch = h;
}
