/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/******************************
 ***    AgpLinkN3ToEdge.c   ***
 ******************************/
#include	<stdio.h>
#include	<math.h>

#include	"Uigp/igp.h"

#include	"GpType/GpType.h"


#include	"N3Prime/N3PrimeIn.h"

#include	"AgpEditTool.h"



static void	agp_link_n3_to_edge_jnt( gp_array *agp,
				vec2d *ctr, link_type *link, int direct,
				link_type *lnR[2], link_type *lnL[2],
				jnt_type **rjnt, int *rbno,
				jnt_type **ljnt, int *lbno );


static void	agp_link_n3_to_edge_link( gp_array *agp,
				vec2d *ctr, link_type *link, int side,
				link_type *ln[2],
				jnt_type *jnt0, int bno0,
				jnt_type *jnt1, int bno1 );

static void	agp_link_n3_to_edge_jnt_link( gp_array *agp,
				jnt_type *jnt, jnt_type *jnt0, jnt_type *jnt1 );

static void	agp_link_n3_to_edge_jnt_final( gp_array *agp, jnt_type *jnt );


/*
static void	agp_link_find_in( link_type *link, int side, link_type *ln[2]);
*/

void
agp_link_n3_to_edge( gp_array *agp, vec2d *ctr, link_type *link )
{
jnt_type	*rjnt[2],	*ljnt[2];
int	rbno[2],	lbno[2];
link_type	*lnR[2],	*lnL[2];


	link_in_get( link, F_RIGHT, lnR );
	link_in_get( link, F_LEFT, lnL );
	


	agp_link_n3_to_edge_jnt( agp, ctr, link, F_BEGIN, lnR, lnL,
				&rjnt[0], &rbno[0], &ljnt[0], &lbno[0] );


	agp_link_n3_to_edge_jnt( agp, ctr, link, F_END, lnR, lnL,
				&rjnt[1], &rbno[1], &ljnt[1], &lbno[1] );



	agp_link_n3_to_edge_link( agp, ctr, link, F_RIGHT, lnR,
				rjnt[0], rbno[0], rjnt[1], rbno[1] );

	agp_link_n3_to_edge_link( agp, ctr, link, F_LEFT, lnL,
				ljnt[0], lbno[0], ljnt[1], lbno[1] );




	agp_lnf_clear_link_1( ctr, link, agp );
	link_1_free( link );
 

	agp_link_n3_to_edge_jnt_final( agp, rjnt[0] );
	agp_link_n3_to_edge_jnt_final( agp, rjnt[1] );
	agp_link_n3_to_edge_jnt_final( agp, ljnt[0] );
	agp_link_n3_to_edge_jnt_final( agp, ljnt[1] );
}



static void
agp_link_n3_to_edge_jnt_final( gp_array *agp, jnt_type *jnt )
{
	if( jnt->brn_no == 0 ){ 
		agp_jnt_untied( agp, jnt );
		free( jnt );
		return;
	}

	if( jnt->brn_no == 2 ){
		agp_jnt_connect( agp, jnt );
		return;
	}

	jnt_set_aux( jnt );
}



static void
agp_link_n3_to_edge_jnt( gp_array *agp,
				vec2d *ctr, link_type *link, int direct,
				link_type *lnR[2], link_type *lnL[2],
				jnt_type **rjnt, int *rbno,
				jnt_type **ljnt, int *lbno )
{
jnt_type	*jnt;
brn_type	*b;
width_type	*width;
link_type	*l;
vec2d	gctr;
float	w;
int	bno;


	if( direct == F_BEGIN ){
		if( ( l = LINK_PREV(link)) != NULL ){

			l->pr[1] = profile_make_copy( l->pr[1], l->type );

			if( l->type == NF_N3 )
				l->width[1] = width_make_copy( l->width[1] );


			jnt = jnt_alloc_1( JNT_N3, ctr );
			jnt_connet_link( jnt, 0, l, F_END );
			jnt_n3_load_from_link( jnt );
			agp_jnt_insert( agp, jnt );

			link->width[0]->bes = l->width[1]->bes;


			if( lnR[0] != NULL ){
				jnt->brn[1]->type = BRN_IN;
				jnt->brn[1]->link = lnR[0];
				jnt_n3_set_in_direct( jnt, 1 );
			}
 
			if( lnL[0] != NULL ){
				jnt->brn[2]->type = BRN_IN;
				jnt->brn[2]->link = lnL[0];
				jnt_n3_set_in_direct( jnt, 2 );
			}



			*rjnt = *ljnt = jnt;

			*rbno = 1;
			*lbno = 2;
			return;
		}

		else {
			jnt = link->p[direct].p.jnt;
			if( jnt->type == JNT_RN3 && jnt->brn_no > 1  ){
				jnt_untied_brn( jnt, link->p[direct].bno, 0 );

				jnt_n3_from_jnt_rn3( jnt );


				*rjnt = *ljnt = jnt;

				*rbno = 1;
				*lbno = 2;

				return;
			}
		}
	}
	else	{
		if( (l = LINK_NEXT(link)) != NULL ){

			gctr.x = ctr->x + link->v.x;	
			gctr.y = ctr->y + link->v.y;
			

			jnt = jnt_alloc_1( JNT_N3, &gctr );
			jnt_connet_link( jnt, 0, l, F_BEGIN );
			jnt_n3_load_from_link( jnt );
			agp_jnt_insert( agp, jnt );


			if( lnL[1] != NULL ){
				jnt->brn[1]->type = BRN_IN;
				jnt->brn[1]->link = lnL[1];
				jnt_n3_set_in_direct( jnt, 1 );
			}
 
			if( lnR[1] != NULL ){
				jnt->brn[2]->type = BRN_IN;
				jnt->brn[2]->link = lnR[1];
				jnt_n3_set_in_direct( jnt, 2 );
			}

 



			*rjnt = *ljnt = jnt;

			*rbno = 2;
			*lbno = 1;

			return;
		}
		else {
			jnt = link->p[direct].p.jnt;
			if( jnt->type == JNT_RN3 && jnt->brn_no > 1  ){
				jnt_untied_brn( jnt, link->p[direct].bno, 0 );

				jnt_n3_from_jnt_rn3( jnt );


				*rjnt = *ljnt = jnt;

				*rbno = 2;
				*lbno = 1;

				return;
			}
		}
	}







	jnt = link->p[direct].p.jnt;
	width = link->width[direct];

	w = width->w[F_RIGHT];
	gctr.x = jnt->gctr.x + w* width->bes.y;
	gctr.y = jnt->gctr.y - w* width->bes.x;

	*rjnt = jnt_alloc_1( JNT_EDGE, &gctr );
	agp_jnt_insert( agp, *rjnt );
	*rbno = -1;



	w = width->w[F_LEFT];
	gctr.x = jnt->gctr.x + w* width->bes.y;
	gctr.y = jnt->gctr.y - w* width->bes.x;

	*ljnt = jnt_alloc_1( JNT_EDGE, &gctr );
	agp_jnt_insert( agp, *ljnt );
	*lbno = -1;



	if( direct == F_END )
		agp_link_n3_to_edge_jnt_link( agp, jnt, *rjnt, *ljnt );
	else	agp_link_n3_to_edge_jnt_link( agp, jnt, *ljnt, *rjnt );

	if( jnt->type == JNT_N3 ){

		bno = ( direct == F_BEGIN )? 2 : 1 ;

		if( jnt->brn[bno]->type == BRN_EDGE ){
			b = (*rjnt)->brn[(*rjnt)->brn_no++] = jnt->brn[bno];

			b->link->p[b->direction].p.jnt = *rjnt;
			b->link->p[b->direction].bno = (*rjnt)->brn_no-1;
			b->ctr.x = b->ctr.y = 0;
		}
		else	{
			if( jnt->brn[bno]->type == BRN_IN )
				jnt_n3_in_delete( jnt, bno );
			free( jnt->brn[bno] );
		}

		bno = 3 - bno;

		if( jnt->brn[bno]->type == BRN_EDGE ){
			b = (*ljnt)->brn[(*ljnt)->brn_no++] = jnt->brn[bno];

			b->link->p[b->direction].p.jnt = *ljnt;
			b->link->p[b->direction].bno = (*ljnt)->brn_no-1;
			b->ctr.x = b->ctr.y = 0;
		}
		else	{
			if( jnt->brn[bno]->type == BRN_IN )
				jnt_n3_in_delete( jnt, bno );
			free( jnt->brn[bno] );
		}
	}


	agp_jnt_untied( agp, jnt );
	free( jnt->brn[0] );
	free( jnt );
}






static void
agp_link_n3_to_edge_link( gp_array *agp,
			vec2d *ctr, link_type *link, int side,
				link_type *ln[2],
				jnt_type *jnt0, int bno0,
				jnt_type *jnt1, int bno1 )
{
link_type	*nl;
in_type	*in0,	*in1;
vec2d	ctr0,	ctr1;



	if( ln[0] != NULL && link->in[side] == NULL )	return;

	ctr0 = jnt0->gctr;
	if( bno0 != -1 ){
		ctr0.x += jnt0->brn[bno0]->ctr.x;
		ctr0.y += jnt0->brn[bno0]->ctr.y;
	}

/*
	ctr1 = jnt1->gctr;
	if( bno1 != -1 ){
		ctr1.x += jnt1->brn[bno1]->ctr.x;
		ctr1.y += jnt1->brn[bno1]->ctr.y;
	}
*/

		
	in0 = NULL;

	do {
		in1 = link->in[side];

		if( in1 != NULL ){

			ctr1.x = in1->jnt->gctr.x +
					in1->jnt->brn[in1->bno]->ctr.x;
			ctr1.y = in1->jnt->gctr.y +
					in1->jnt->brn[in1->bno]->ctr.y;

			in1->jnt->brn[in1->bno]->in = NULL;
			in1->jnt->brn[in1->bno]->type = BRN_NULL;

			link->in[side] = in1->p;

			if( in1->jnt->brn[in1->bno]->direction == F_BEGIN ){
				if( in0 != NULL )	free( in0 );

				in0 = in1;
				ctr0 = ctr1;
				continue;
			}
		}
		else	{
			if( ln[1] != NULL ){
				in0 = in1;
				continue;
			}

			ctr1 = jnt1->gctr;
			if( bno1 != -1 ){
				ctr1.x += jnt1->brn[bno1]->ctr.x;
				ctr1.y += jnt1->brn[bno1]->ctr.y;
			}
		}
		


		nl = n3_prime_link_in( ctr, link, side, &ctr0, in0,
						&ctr1, in1 );

		nl->type = NF_EDGE;



		if( in0 == NULL ){
			nl->pr[0].edge = n3_prime_profile( link, side,F_BEGIN);
			jnt_connet_link( jnt0, bno0, nl, F_BEGIN );
		}
		else	{
			nl->pr[0].edge = n3_prime_profile_in( link,
								side, in0->t);
			jnt_connet_link( in0->jnt, in0->bno, nl, F_BEGIN );
		}



		if( in1 == NULL ){
			nl->pr[1].edge = n3_prime_profile( link, side, F_END );
			jnt_connet_link( jnt1, bno1, nl, F_END );
		}
		else	{
			nl->pr[1].edge = n3_prime_profile_in( link,
								side, in1->t);
			jnt_connet_link( in1->jnt, in1->bno, nl, F_END );
		}


		agp_lnf_link( &ctr0, nl, agp );


		if( in0 != NULL )       free( in0 );
		in0 = in1;
		ctr0 = ctr1;

	}
	while( in0 != NULL );
}




static void
agp_link_n3_to_edge_jnt_link( gp_array *agp,
		jnt_type *jnt, jnt_type *jnt0, jnt_type *jnt1 )
{
link_type	*link;
float	len_inv;
int	flag;

	if( jnt->type == JNT_RN3 && jnt->brn_no == 2 )	return;

	if( jnt->type == JNT_N3 && jnt->brn[1]->type != BRN_NULL &&
				jnt->brn[2]->type != BRN_NULL &&
				jnt->brn[0]->type == BRN_N3 )
		return;


	link = link_1_alloc();


	link->type = NF_EDGE;
	link->ctr = jnt0->gctr;
	link->v.x = jnt1->gctr.x - jnt0->gctr.x;
	link->v.y = jnt1->gctr.y - jnt0->gctr.y;
	link->len = hypot( link->v.x, link->v.y );

	len_inv = 1.0/link->len;
	link->u.x =  link->v.y * len_inv;
	link->u.y = -link->v.x * len_inv;

	link->a = 0;
	link->c_prb = 0;


	jnt_connet_link( jnt0, -1, link, F_BEGIN );
	jnt_connet_link( jnt1, -1, link, F_END );

	agp_lnf_link( &link->ctr, link, agp );


	if( jnt->brn[0]->link != NULL ){
		link->br = jnt->brn[0]->link->br;
		link->Fbr = ( jnt->brn[0]->link->Fbr == 0x03 )? 1 : 0;
	}


	if( jnt->type == JNT_RN3 ){
		link->pr[0].edge = n3_prime_profile1( jnt, 0, F_LEFT, 0 );
		link->pr[1].edge = n3_prime_profile1( jnt, 0, F_RIGHT, 0 );
		return;
	}

	if( jnt->brn[0]->type == BRN_N3R ){
		link->pr[0].edge = n3_prime_profile11( jnt, 1, F_END );
		link->pr[1].edge = n3_prime_profile11( jnt, 2, F_BEGIN );
		return;
	}


	flag = jnt->brn[1]->type != BRN_NULL;
	link->pr[0].edge = n3_prime_profile1( jnt, 0, F_LEFT, flag );
	flag = jnt->brn[2]->type != BRN_NULL;
	link->pr[1].edge = n3_prime_profile1( jnt, 0, F_RIGHT, flag );
}




/*
static void
agp_link_find_in( link_type *link, int side, link_type *ln[2] ) 
{ 
jnt_type	*jnt;
link_type	*l,	*pl;
in_type	*in;
int	bno;



        ln[0] = ln[1] = NULL;


	pl = link;
	while( ( l = LINK_PREV(pl)) != NULL ){
		if( l->in[side] != NULL )	break;
		pl = l;
	}


	if( l != NULL ){
                for( in = l->in[side] ; in->p != NULL ; in = in->p ); 
		if( in->direct == F_END )
			ln[0] = in->jnt->brn[0]->link;
	}
	else	{
               	jnt = pl->p[0].p.jnt; 
               	bno = (side == F_RIGHT)? 2: 1; 
               	if( jnt->type == JNT_N3 && jnt->brn[bno]->type == BRN_IN ) 
                       	ln[0] = jnt->brn[bno]->link;
	}




	if( link->in[side] == NULL ){
		ln[1] = ln[0];
		return;
	}

	for( in = link->in[side] ; in->p != NULL ; in = in->p ); 

	if( in->direct == F_END )
		ln[1] = in->jnt->brn[0]->link;
}
*/
