/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
#include	<stdio.h>
#include	<math.h>

#include	"Uigp/igp.h"

#include	"GpType/GpType.h"

#include	"AgpEdit/AgpEditTool.h"
#include	"AgpEdit/AgpSelect.h"

#include	"N3Prime/N3Prime.h"


static void agp_polylink_select_1( gp_array *agp, link_type *clink,
			link_type **BLink, link_type **ELink, int Ldirect,
			vec2d *ctr, link_type *link, int direct );

static void agp_polylink_select_layer_1( link_type *clink,
			link_type **BLink, link_type **ELink, int Ldirect,
			vec2d *ctr, link_type *link, int direct , int layer);


static void	add_link( link_type **BLink, link_type **ELink, int Ldirect,
			vec2d *ctr, link_type *link, int direct );

static int	agp_polylink_select_next_brn( gp_array *agp, jnt_type *jnt, int bno );

static int	agp_polylink_select_is_JNTN3_BRN( gp_array *agp, jnt_type *jnt0, int bno );





link_type *
agp_polylink_select( gp_array *agp, vec2d *p, float d )
{
gp_type	*gp;
jnt_type	*jnt;
link_type	*Blink,	*Elink;
link_type	*link,	*l;
int	Fn3_prime,	i,	j;
vec2d	ctr;
float	dis;


               
	if( (Fn3_prime = agp->Fn3_prime) == 0 ) n3_prime_agp( agp); 

	dis = agp_select_link( agp, p, 0, &ctr, &link );

	
	if( link == NULL || ABS(dis) > d ){
		if( Fn3_prime == 0 )    
			n3_prime_free( agp); 
	      return( NULL );
	}

	if( link->type != NF_EDGE && link->type != NF_N3_P ){
		if( Fn3_prime == 0 )    
			n3_prime_free( agp); 
		return( NULL );
	}



	for( i = 0 ; i < agp->Ni ; i++ )
		for( j = 0 ; j < agp->Nj ; j++ )
		{
			gp = &agp->gp[i][j];
			for( jnt = gp->jnt ; jnt != NULL ; jnt = jnt->nj )
				jnt->jnt_no = 0;
		}


	while( LINK_PREV(link) != NULL )
	{
		link = LINK_PREV(link);
		ctr.x -= link->v.x;
		ctr.y -= link->v.y;
	}

	for( l = link ; LINK_NEXT(l) != NULL ; l = LINK_NEXT(l) );
	if( link->p[0].p.jnt == l->p[1].p.jnt )
		link->p[0].p.jnt->jnt_no = 1;



	Blink = Elink = NULL;

	agp_polylink_select_1( agp, link, &Blink, &Elink, F_END, &ctr, link, F_BEGIN );

	
	agp_polylink_select_1( agp, link, &Blink, &Elink, F_BEGIN, &ctr, link, F_END );

	if( Fn3_prime == 0 )    
		n3_prime_free( agp); 

	return( Blink );
	
}



int
agp_polylink_select_jnt( gp_array *agp, vec2d *p, float d, link_type *alink[] , int EMjnt, jnt_type *ignore_jnt)
{
gp_type	*gp;
jnt_type	*jnt,	*sjnt;
brn_type	*b;
link_type	*Blink,	*Elink;
int	Fn3_prime,	i,	j,	bno,	no,	sbno;
vec2d	ctr;
float	dis;

               
	if( (Fn3_prime = agp->Fn3_prime) == 0 ) 
		n3_prime_agp( agp); 

	if ( EMjnt != 0 )
		dis = agp_select_jntN( agp, p, EMjnt, ignore_jnt, &sjnt, &bno);
	else
		dis = agp_select_jnt( agp, p, 0, &sjnt, &bno );


	
	if( sjnt == NULL || ABS(dis) > d || sjnt->type == JNT_N3)
	{
		//change....get the neighbour junction...
		if( Fn3_prime == 0 )    
			n3_prime_free( agp); 
		return( 0 );
	}


	for( i = 0 ; i < agp->Ni ; i++ )
		for( j = 0 ; j < agp->Nj ; j++ )
		{
			gp = &agp->gp[i][j];
			for( jnt = gp->jnt ; jnt != NULL ; jnt = jnt->nj )
				jnt->jnt_no = 0;
		}



	sbno = -1;
	if(sjnt->type == JNT_N3_P && sjnt->brn_no >= 3 ){
		for( i = 0 ; i < sjnt->brn_no ; i++ ){
			if(  agp_polylink_select_is_JNTN3_BRN( agp, sjnt, i ) > 0 )
				break;
		}
		sbno = i;
	}



	no = 0;
	sjnt->jnt_no = 1;
	for( i = 0 ; i < sjnt->brn_no ; i++ ){
		if( i == sbno )	continue;

		b = sjnt->brn[i];
		if( b->link->type != NF_EDGE && b->link->type != NF_N3_P )	continue;


		ctr.x = sjnt->gctr.x + b->ctr.x;
		ctr.y = sjnt->gctr.y + b->ctr.y;

		Blink = Elink = NULL;
		agp_polylink_select_1( agp, NULL, &Blink, &Elink, F_END, &ctr, b->link, b->direction );

		alink[no++] = Blink;
	}


	if( Fn3_prime == 0 )    
		n3_prime_free( agp); 

	return( no );
	
}







/**** agp_polylink_select_1 ****/
/*!
 * Select a polylink starting at "link", and ends at a joint where there are more than 2 branches
 *
 * @param *Clink : the link we started from
 * @param **BLink : begin link
 * @param **ELink : end link
 * @param Ldirect : direction of link
 * @param *ctr : link's center
 * @param *link : the link to expand from
 * @param direct : direction for expansion
 *
 * @return void : 
 */
static void agp_polylink_select_1( gp_array *agp, link_type *Clink,link_type **BLink, 
					    link_type **ELink, int Ldirect,vec2d *ctr, 
					    link_type *link, int direct )
{
jnt_type	*jnt;
brn_type	*b;
int	bno,	idirect;
vec2d	v;
link_type	*l;



	if( direct == F_BEGIN )
		for( l = link, v = *ctr ; l != NULL ; l = LINK_NEXT(l) ){

//			if( Clink == link && *Blink != NULL )	return;

			add_link( BLink, ELink, Ldirect, &v, l, F_END );

			v.x += l->v.x;
			v.y += l->v.y;

			link = l;
		}
	else
	{

		for( l = (Clink == link)?LINK_PREV(link):link, v = *ctr ;
						l != NULL ; l = LINK_PREV(l) ){


			v.x -= l->v.x;
			v.y -= l->v.y;

			add_link( BLink, ELink, Ldirect, &v, l, F_BEGIN );

			link = l;
		}

	}
	idirect = INV_FLAG( direct );
	jnt = link->p[idirect].p.jnt;
	bno = link->p[idirect].bno;



	if( jnt->jnt_no == 1 )	// the joint was visited already
		return ;
/*
	if( jnt->brn_no != 2 )	// the joint contains more than 2 branches
		return ;


  	bno = jnt->brn_no - 1 - bno;
*/
// 22.7
	if( (bno = agp_polylink_select_next_brn( agp, jnt, bno )) < 0 )	return;
// end 22.7

	jnt->jnt_no = 1;// mark as visited


	b = jnt->brn[bno];

	if( b->link == Clink )	
		return ;

	// this was a joint with two branches - continue selecting
	v.x = jnt->gctr.x + b->ctr.x;
	v.y = jnt->gctr.y + b->ctr.y;

	// continue selecting
	agp_polylink_select_1( agp, Clink, BLink, ELink, Ldirect,
				&v, b->link, b->direction );
}


static int
agp_polylink_select_next_brn( gp_array *agp, jnt_type *jnt, int bno )
{
//jnt_type	*njnt;
//brn_type	*b;
int	i,	nbno,	ret;


	if( jnt->brn_no == 1 )	return( -1 );


	if( jnt->brn_no == 2 )
		return( jnt->brn_no - 1 - bno );

	if( jnt->type != JNT_N3_P )	return( -1 );

/*
	for( i = 0 ; i < jnt->brn_no ; i++ ){
		if( i == bno )	continue;

		b = jnt->brn[i];

		if( b->direction == F_BEGIN )
			njnt = LINK_NEXT_JNT(b->link);
		else	njnt = LINK_PREV_JNT(b->link);
	
		if( njnt == NULL || njnt->type != JNT_N3_P )	return( i );

		if( b->link->len > 4.0 )	return( i );
	}
*/

	for( i = 0 ; i < jnt->brn_no ; i++ ){
		ret = agp_polylink_select_is_JNTN3_BRN( agp,  jnt, i );
		if( ret > 0 )
			break;
	}

	if( i >= jnt->brn_no )	return( -1 );

	if(  i == bno )	return( -1 );

	nbno = ( bno == jnt->brn_no -1 )? 0 : bno+1;
	if( i != nbno )	return( nbno );

	nbno = ( nbno == jnt->brn_no -1 )? 0 : nbno+1;

	return( nbno );
}


static int
agp_polylink_select_is_JNTN3_BRN( gp_array *agp, jnt_type *jnt0, int bno )
{
jnt_type	*jnt;
link_type	*link;
brn_type	*b;
vec2d	ctr,	dv;
float	u,	t;
int	i,	j,	i0,	j0;

	b = jnt0->brn[bno];
	if( b->type == NF_EDGE )	return( -1 );

//	if( b->direction == F_BEGIN )
//		jnt1 = LINK_NEXT_JNT(b->link);
//	else	jnt1 = LINK_PREV_JNT(b->link);

//	if( jnt1 == NULL )	return( -1 );

	if( b->link->len > 6 )	return( -1 );


	i0 = (jnt0->gctr.x+agp->bgp) * agp->inv_bgp;
	j0 = (jnt0->gctr.y+agp->bgp) * agp->inv_bgp;
  
	link = b->link;
	ctr = jnt0->gctr;
	if( b->direction != F_BEGIN ){
		ctr.x -= link->v.x;
		ctr.y -= link->v.y;
	}
  
	for( i = i0-1 ; i <= i0+1 ; i++ ){
		if( i < 0 || i >= agp->Ni )	continue;

		for( j = j0-1  ; j <= j0+1 ; j++ ){

			if( j < 0 || j >= agp->Nj )	continue;
		
			for( jnt = agp->gp[i][j].jnt ; jnt != NULL ; jnt = jnt->nj ){
				if( jnt->type != JNT_N3 )	continue;

				dv.x = jnt->gctr.x - ctr.x;
				dv.y = jnt->gctr.y - ctr.y;

				t = -dv.x *link->u.y + dv.y *link->u.x;
				if( t < 0 || t > link->len )	continue;

				u = link->u.x * dv.x + link->u.y * dv.y;

				if( ABS(u ) < 0.01 )
					return( 1 );
			}
		}
	}

	return( -1 );
}






static void
add_link( link_type **BLink, link_type **ELink, int Ldirect,
			vec2d *ctr, link_type *link, int direct )
{
link_type	*l;

	l = link_1_alloc();

	if( Ldirect == direct ){

		l->ctr = *ctr;
		l->v = link->v;
		l->a = link->a;
	}
	else	{
		l->ctr.x = ctr->x + link->v.x;
		l->ctr.y = ctr->y + link->v.y;

		l->v.x = -link->v.x;
		l->v.y = -link->v.y;
		l->a = - link->a;
	}

	l->type = NF_EDGE;

	link_set_aux( l );


	if( *BLink == NULL ){
		*BLink = *ELink = l ;
		return;
	}


	if( Ldirect == F_END ){
		(*ELink)->p[1].p.l = l;
		l->p[0].p.l = *ELink;
		(*ELink)->p[1].bno = l->p[0].bno = -1;
		*ELink = l;
	}
	else	{
		
		(*BLink)->p[0].p.l = l;
		l->p[1].p.l = *BLink;
		(*BLink)->p[1].bno = l->p[0].bno = -1;
		*BLink = l;
	}
}
