/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
#include	<stdio.h>
#include	<math.h>

#include	"Uigp/igp.h"

#include	"GpType/GpType.h"



#include	"AgpEdit/AgpEditTool.h"




int
agp_profile_load_from_jnt( link_type *link, int direct )
{
jnt_type	*jnt;
edge_type	*e,	*se;
int	side,	sside;
int	bno,	sbno;
int	ret;

	if( link->type != NF_EDGE )	return( 0 );

	jnt = link->p[direct].p.jnt;
	bno = link->p[direct].bno;


	if( jnt->type == JNT_N3 ){
		ret = agp_profile_load_from_jnt_n3( link, direct );
		return( ret );
	}

	
	if( jnt->type != JNT_EDGE )	return( 0 );
	if( jnt->brn_no == 1 )	return( 0 );


	

	sbno = BRN_RIGHT( jnt, bno );

	if( jnt->brn[sbno]->direction == F_BEGIN ){
		se = jnt->brn[sbno]->link->pr[0].edge;
		sside = F_RIGHT;
	}
	else	{
		se = jnt->brn[sbno]->link->pr[1].edge;
		sside = F_LEFT;
	}


	e = jnt->brn[bno]->link->pr[direct].edge;
	side = (direct == F_BEGIN)? F_LEFT : F_RIGHT;

	e->side[side].bval[SP_Y] = se->side[sside].bval[SP_Y];
	e->side[side].val[SP_Y] = se->side[sside].bval[SP_Y];

	e->side[side].bval[SP_Q] = se->side[sside].bval[SP_Q];
	e->side[side].val[SP_Q] = se->side[sside].bval[SP_Q];

	e->side[side].bval[SP_A] = se->side[sside].bval[SP_A];
	e->side[side].val[SP_A] = se->side[sside].bval[SP_A];


	sbno = BRN_LEFT( jnt, bno );

	if( jnt->brn[sbno]->direction == F_END ){
		se = jnt->brn[sbno]->link->pr[1].edge;
		sside = F_RIGHT;
	}
	else	{
		se = jnt->brn[sbno]->link->pr[0].edge;
		sside = F_LEFT;
	}


	e = jnt->brn[bno]->link->pr[direct].edge;
	side = (direct == F_END)? F_LEFT : F_RIGHT;

	e->side[side].bval[SP_Y] = se->side[sside].bval[SP_Y];
	e->side[side].val[SP_Y] = se->side[sside].bval[SP_Y];

	e->side[side].bval[SP_Q] = se->side[sside].bval[SP_Q];
	e->side[side].val[SP_Q] = se->side[sside].bval[SP_Q];

	e->side[side].bval[SP_A] = se->side[sside].bval[SP_A];
	e->side[side].val[SP_A] = se->side[sside].bval[SP_A];

	return( 1 );
}



int
agp_profile_load_from_jnt_n3( link_type *link, int direct )
{
jnt_type	*jnt;
brn_type	*b;
n3_type	*n3;
edge_type	*e;
int	side,	sp,	bno;
int	side0,	side1;

	if( link->type != NF_EDGE )	return( 0 );

	jnt = link->p[direct].p.jnt;
	bno = link->p[direct].bno;

	if( jnt->type != JNT_N3 )	return( 0 );


	if( jnt->brn[0]->type != BRN_N3 )	return( 0 );


	b = jnt->brn[0];

	if( b->direction == F_END )
		side = ( bno == 1 )? F_RIGHT : F_LEFT;
	else	side = ( bno == 1 )? F_LEFT : F_RIGHT;


	n3 = b->link->pr[b->direction].n3;


	if( direct == F_BEGIN )
		side0 = ( bno == 1 )? F_RIGHT : F_LEFT;
	else	side0 = ( bno == 1 )? F_LEFT : F_RIGHT;
	side1 = INV_FLAG(side0);


	e = link->pr[direct].edge;
	for( sp = 0 ; sp < SP_NO ; sp++ ){
		e->side[side0].val[sp] = n3->side[side].val[sp];
		e->side[side0].bval[sp] = n3->side[side].bval[sp];

		e->side[side1].val[sp] = n3->mval[sp];
		e->side[side1].bval[sp] = n3->mval[sp];
	}

	e->width = n3->width[side];




	return( 1 );
}
