/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**************************
 ***	AgpSelect.c	***
 **************************/

#include          <stdio.h>
#include          <math.h>

#include	"Uigp/igp.h" 

#include	"GpType/GpType.h"
#include	"AgpSelect.h"



static float	dis_form_link( vec2d *ctr, link_type *link, vec2d *v );


#define		L_SIZE		5
#define		HSIZE		1



float
agp_select_hunch( gp_array *agp, vec2d *v, vec2d *s_ctr, hunch_type **s_hunch )
{
gp_type		*gp;
hunch_type	*h;

int	i0,	j0,	i,	j;
float	u,	cu;
vec2d	ctr;
  
	u = 1000;
	*s_hunch = NULL;


	i0 = (v->x+agp->bgp) * agp->inv_bgp;
	j0 = (v->y+agp->bgp) * agp->inv_bgp;
  
  
	for( i = i0-HSIZE ; i <= i0+HSIZE ; i++ ){
		if( i < 0 || i >= agp->Ni )	continue;

		for( j = j0-HSIZE  ; j <= j0+HSIZE ; j++ ){
			if( j < 0 || j >= agp->Nj )	continue;
      
			gp = &agp->gp[i][j];
			for( h = gp->hunch ; h != NULL ; h = h->nh ){

				ctr.x = gp->ctr.x + h->ctr.x;
				ctr.y = gp->ctr.y + h->ctr.y;

				cu = hypot( ctr.x - v->x , ctr.y - v->y );
				if( ABS(u) < ABS(cu) )	continue;

				u = cu;

				*s_ctr = ctr;
				*s_hunch = h;
			}
		}
	}

	return( u );
}






float
agp_select_link( gp_array *agp, vec2d *v, int Fridge,
				vec2d *s_ctr, link_type **s_link )
{
gp_type		*gp;
link_type	*l;
jnt_type	*jnt;
brn_type	*b;

int	i0,	j0,	i,	j,	k;
float	u,	cu;
vec2d	ctr;
  
	u = 1000;
	*s_link = NULL;


	i0 = (v->x+agp->bgp) * agp->inv_bgp;
	j0 = (v->y+agp->bgp) * agp->inv_bgp;
  
  
	for( i = i0-L_SIZE ; i <= i0+L_SIZE ; i++ ){
		if( i < 0 || i >= agp->Ni )	continue;

		for( j = j0-L_SIZE  ; j <= j0+L_SIZE ; j++ ){
			if( j < 0 || j >= agp->Nj )	continue;
      
			gp = &agp->gp[i][j];

			for( l = gp->link ; l != NULL ; l = l->nl ){

				if( l->type == NF_RIDGE && Fridge == 0 )
					continue;

				ctr.x = gp->ctr.x + l->ctr.x;
				ctr.y = gp->ctr.y + l->ctr.y;

				cu = dis_form_link( &ctr, l, v );

				if( ABS(u) < ABS(cu) )	continue;

				u = cu;

				*s_ctr = ctr;
				*s_link = l;
			}


			for( jnt = gp->jnt ; jnt != NULL ; jnt = jnt->nj )
			    for( k = 0 ; k < jnt->brn_no ; k++ ){
				b = jnt->brn[k];

				if( b->direction == F_BEGIN )
					continue;

				if( b->type == BRN_NULL || b->type == BRN_IN )
					continue;

				l = b->link;


				if( l->type == NF_RIDGE && Fridge == 0 )
					continue;

				ctr.x = jnt->gctr.x + b->ctr.x - l->v.x;
				ctr.y = jnt->gctr.y + b->ctr.y - l->v.y;


				cu = dis_form_link( &ctr, l, v );

				if( ABS(u) < ABS(cu) )	continue;

				u = cu;

				*s_ctr = ctr;
				*s_link = l;
			    }
		}
	}

	return( u );
}




float
agp_select_linkN( gp_array *agp, vec2d *v, int Mnf, link_type *ignore_link, 
				vec2d *s_ctr, link_type **s_link )
{
gp_type		*gp;
link_type	*l;
jnt_type	*jnt;
brn_type	*b;

int	i0,	j0,	i,	j,	k;
float	u,	cu;
vec2d	ctr;
  
	u = 1000;
	*s_link = NULL;


	i0 = (v->x+agp->bgp) * agp->inv_bgp;
	j0 = (v->y+agp->bgp) * agp->inv_bgp;
  
  
	for( i = i0-L_SIZE ; i <= i0+L_SIZE ; i++ ){
		if( i < 0 || i >= agp->Ni )	continue;

		for( j = j0-L_SIZE  ; j <= j0+L_SIZE ; j++ ){
			if( j < 0 || j >= agp->Nj )	continue;
      
			gp = &agp->gp[i][j];

			for( l = gp->link ; l != NULL ; l = l->nl ){

				if( (Mnf & (1<<l->type)) == 0 )	continue;
				if( l == ignore_link )	continue;


				ctr.x = gp->ctr.x + l->ctr.x;
				ctr.y = gp->ctr.y + l->ctr.y;

				cu = dis_form_link( &ctr, l, v );

				if( ABS(u) < ABS(cu) )	continue;

				u = cu;

				*s_ctr = ctr;
				*s_link = l;
			}


			for( jnt = gp->jnt ; jnt != NULL ; jnt = jnt->nj )
			    for( k = 0 ; k < jnt->brn_no ; k++ ){
				b = jnt->brn[k];

				if( b->direction == F_BEGIN )
					continue;

				if( b->type == BRN_NULL || b->type == BRN_IN )
					continue;

				l = b->link;


				if( (Mnf & (1<<l->type)) == 0 )	continue;
				if( l == ignore_link )	continue;

				ctr.x = jnt->gctr.x + b->ctr.x - l->v.x;
				ctr.y = jnt->gctr.y + b->ctr.y - l->v.y;


				cu = dis_form_link( &ctr, l, v );

				if( ABS(u) < ABS(cu) )	continue;

				u = cu;

				*s_ctr = ctr;
				*s_link = l;
			    }
		}
	}

	return( u );
}


float
agp_select_plink( gp_array *agp, vec2d *v, int Fridge, link_type *link0,
		vec2d *s_ctr, link_type **s_link, int *s_direct )
{
gp_type		*gp;
link_type	*l;
jnt_type	*jnt;
brn_type	*b;

int	i0,	j0,	i,	j,	k;
float	u,	cu;
vec2d	ctr,	dv;
  
	u = 1000;
	*s_link = NULL;


	i0 = (v->x+agp->bgp) * agp->inv_bgp;
	j0 = (v->y+agp->bgp) * agp->inv_bgp;
  
  
	for( i = i0-L_SIZE ; i <= i0+L_SIZE ; i++ ){
		if( i < 0 || i >= agp->Ni )	continue;

		for( j = j0-L_SIZE  ; j <= j0+L_SIZE ; j++ ){
			if( j < 0 || j >= agp->Nj )	continue;
      
			gp = &agp->gp[i][j];

			for( l = gp->link ; l != NULL ; l = l->nl ){

				if( l->type == NF_RIDGE && Fridge == 0 )
					continue;

				if( l == link0 )	continue;
				if( link0 != NULL && LINK_PREV(l) == link0 )
					continue;

				ctr.x = gp->ctr.x + l->ctr.x;
				ctr.y = gp->ctr.y + l->ctr.y;

				dv.x = v->x - ctr.x;
				dv.y = v->y - ctr.y;
				cu = dv.x*dv.x + dv.y*dv.y;

				if( cu < u ){
					u = cu;

					*s_ctr = ctr;
					*s_link = l;

					*s_direct = F_BEGIN;
				}
			}


			for( jnt = gp->jnt ; jnt != NULL ; jnt = jnt->nj )
			    for( k = 0 ; k < jnt->brn_no ; k++ ){
				b = jnt->brn[k];

				if( b->direction == F_BEGIN )
					continue;

				if( b->type == BRN_NULL || b->type == BRN_IN )
					continue;

				l = jnt->brn[k]->link;

				if( l->type == NF_RIDGE && Fridge == 0 )
					continue;

				if( l == link0 )	continue;


				ctr.x = jnt->gctr.x + b->ctr.x;
				ctr.y = jnt->gctr.y + b->ctr.y;


				dv.x = v->x - ctr.x;
				dv.y = v->y - ctr.y;
				cu = dv.x*dv.x + dv.y*dv.y;

				if( cu < u ){
					u = cu;

					*s_ctr = ctr;
					*s_link = l;

					*s_direct = F_END;
				}
			    }
		}
	}


	if( *s_direct == F_BEGIN )
		jnt = LINK_PREV_JNT( *s_link );
	else	jnt =  LINK_NEXT_JNT( *s_link );

	if( jnt == NULL || jnt->brn_no == 1 || jnt->type == JNT_N3 )	return( u );

	*s_link = NULL;
	u = 1000;
	for( i = 0 ; i < jnt->brn_no ; i++ ){
		b = jnt->brn[i];
		ctr.x = jnt->gctr.x + b->ctr.x;
		ctr.y = jnt->gctr.y + b->ctr.y ;
		if( b->direction == F_END ){
			ctr.x -= b->link->v.x;
			ctr.y -= b->link->v.y;
		}

		cu = dis_form_link( &ctr, b->link, v );

		if( *s_link == NULL || ABS(cu) < ABS(u) ){
			u = cu;
			*s_link = b->link;
			*s_direct = b->direction;
			*s_ctr = ctr;
		}
	}


	return( u );
}


float
agp_select_point( gp_array *agp, vec2d *v,
					int Fridge, link_type *elink, jnt_type *ejnt,
					vec2d *s_ctr, link_type **s_link, jnt_type **s_jnt )
{
gp_type		*gp;
link_type	*l;
jnt_type	*jnt;

int	i0,	j0,	i,	j;
float	u,	cu;
vec2d	ctr,	dv;
  
	u = 1000;
	*s_link = NULL;
	*s_jnt = NULL;


	i0 = (v->x+agp->bgp) * agp->inv_bgp;
	j0 = (v->y+agp->bgp) * agp->inv_bgp;
  
  
	for( i = i0-L_SIZE ; i <= i0+L_SIZE ; i++ ){
		if( i < 0 || i >= agp->Ni )	continue;

		for( j = j0-L_SIZE  ; j <= j0+L_SIZE ; j++ ){
			if( j < 0 || j >= agp->Nj )	continue;
      
			gp = &agp->gp[i][j];

			for( l = gp->link ; l != NULL ; l = l->nl ){

				if( l->type == NF_RIDGE && Fridge == 0 )
					continue;

				if( l == elink )	continue;

				if( LINK_PREV(l) == NULL )	continue;

				ctr.x = gp->ctr.x + l->ctr.x;
				ctr.y = gp->ctr.y + l->ctr.y;

				dv.x = v->x - ctr.x;
				dv.y = v->y - ctr.y;
				cu = dv.x*dv.x + dv.y*dv.y;

				if( cu > u )	continue;

				u = cu;

				*s_ctr = ctr;
				*s_link = l;
			}


			for( jnt = gp->jnt ; jnt != NULL ; jnt = jnt->nj ){

				if( jnt == ejnt )	continue;

				if( Fridge == 0 && jnt->type == JNT_RIDGE )	continue;


				ctr.x = jnt->gctr.x;
				ctr.y = jnt->gctr.y;

				dv.x = v->x - ctr.x;
				dv.y = v->y - ctr.y;
				cu = dv.x*dv.x + dv.y*dv.y;

				if( cu > u )	continue;


				u = cu;
				*s_ctr = ctr;
				*s_jnt = jnt;

				*s_link = NULL;
			}
		}
	}

	return( u );
}


int
agp_select_find_link( gp_array *agp, vec2d *ctr0, link_type *link,
				vec2d *s_ctr )
{
gp_type		*gp;
link_type	*l;

int	i0,	j0,	i,	j;
  
	if( link == NULL )	return( 0 );


	i0 = (ctr0->x+agp->bgp) * agp->inv_bgp;
	j0 = (ctr0->y+agp->bgp) * agp->inv_bgp;
  
  
	for( i = i0-L_SIZE ; i <= i0+L_SIZE ; i++ ){
		if( i < 0 || i >= agp->Ni )	continue;

		for( j = j0-L_SIZE  ; j <= j0+L_SIZE ; j++ ){
			if( j < 0 || j >= agp->Nj )	continue;
      
			gp = &agp->gp[i][j];

			for( l = gp->link ; l != NULL ; l = l->nl ){

				if( l != link )	continue;

				s_ctr->x = gp->ctr.x + l->ctr.x;
				s_ctr->y = gp->ctr.y + l->ctr.y;

				return( 1 );
			}
		}
	}

	return( 0 );
}



float
agp_select_jnt( gp_array *agp, vec2d *v, int Fridge,
					jnt_type **s_jnt, int *s_bno )
{
gp_type		*gp;
jnt_type	*jnt;
brn_type	*b;

int	i0,	j0,	i,	j,	bno;
float	u,	cu;
vec2d	ctr,	dv;
  
	u = 1000;
	*s_jnt = NULL;


	i0 = (v->x+agp->bgp) * agp->inv_bgp;
	j0 = (v->y+agp->bgp) * agp->inv_bgp;
  
  
	for( i = i0-L_SIZE ; i <= i0+L_SIZE ; i++ ){
	    if( i < 0 || i >= agp->Ni )	continue;

	    for( j = j0-L_SIZE  ; j <= j0+L_SIZE ; j++ ){
		if( j < 0 || j >= agp->Nj )	continue;
      
		gp = &agp->gp[i][j];

		for( jnt = gp->jnt ; jnt != NULL ; jnt = jnt->nj ){

			if( jnt->type == JNT_RIDGE && Fridge == 0 )
				continue;


			if( jnt->brn_no == 0 ){
				dv.x = v->x - jnt->gctr.x;
				dv.y = v->y - jnt->gctr.y;
				cu = dv.x*dv.x + dv.y*dv.y;

				if( u <= cu )	continue;

				u = cu;

				*s_jnt = jnt;
				*s_bno = -1;

				continue;
			}


			for( bno = 0 ; bno < jnt->brn_no ; bno++ ){
				b = jnt->brn[bno];
				ctr.x = jnt->gctr.x + b->ctr.x;
				ctr.y = jnt->gctr.y + b->ctr.y;

				dv.x = v->x - ctr.x;
				dv.y = v->y - ctr.y;
				cu = dv.x*dv.x + dv.y*dv.y;

				if( u <= cu )	continue;

				u = cu;

				*s_jnt = jnt;
				*s_bno = bno;
			}


			if( jnt->type == JNT_RN3 && jnt->brn_no == 1 &&
						jnt->brn[0]->type == NF_N3 ){
				
				ctr.x = jnt->gctr.x + jnt->rwidth *jnt->u.x;
				ctr.y = jnt->gctr.y + jnt->rwidth *jnt->u.y;

				dv.x = v->x - ctr.x;
				dv.y = v->y - ctr.y;
				cu = dv.x*dv.x + dv.y*dv.y;
				if( cu < u ){
					u = cu;

					*s_jnt = jnt;
					*s_bno = 1;
				}


				ctr.x = jnt->gctr.x - jnt->lwidth *jnt->u.x;
				ctr.y = jnt->gctr.y - jnt->lwidth *jnt->u.y;

				dv.x = v->x - ctr.x;
				dv.y = v->y - ctr.y;
				cu = dv.x*dv.x + dv.y*dv.y;
				if( cu < u ){
					u = cu;

					*s_bno = 2;
				}
			}
		}
	    }
	}

	if( u > 2 )	*s_jnt = NULL;

	return( u );
}



float	agp_select_jntN( gp_array *agp, vec2d *v, int EMjnt, jnt_type *ignore_jnt,
					jnt_type **s_jnt, int *s_bno )
{
gp_type		*gp;
jnt_type	*jnt;
brn_type	*b;

int	i0,	j0,	i,	j,	bno;
float	u,	cu;
vec2d	ctr,	dv;
  
	u = 1000;
	*s_jnt = NULL;


	i0 = (v->x+agp->bgp) * agp->inv_bgp;
	j0 = (v->y+agp->bgp) * agp->inv_bgp;
  
  
	for( i = i0-L_SIZE ; i <= i0+L_SIZE ; i++ ){
	    if( i < 0 || i >= agp->Ni )	continue;

	    for( j = j0-L_SIZE  ; j <= j0+L_SIZE ; j++ ){
		if( j < 0 || j >= agp->Nj )	continue;
      
		gp = &agp->gp[i][j];

		for( jnt = gp->jnt ; jnt != NULL ; jnt = jnt->nj ){

//			if( jnt->type == JNT_RIDGE && Fridge == 0 )
			if( (EMjnt & 1<<jnt->type) == 0 )
				continue;

			if( jnt == ignore_jnt )	continue;


			if( jnt->brn_no == 0 ){
				dv.x = v->x - jnt->gctr.x;
				dv.y = v->y - jnt->gctr.y;
				cu = dv.x*dv.x + dv.y*dv.y;

				if( u <= cu )	continue;

				u = cu;

				*s_jnt = jnt;
				*s_bno = -1;

				continue;
			}


			for( bno = 0 ; bno < jnt->brn_no ; bno++ ){
				b = jnt->brn[bno];
				ctr.x = jnt->gctr.x + b->ctr.x;
				ctr.y = jnt->gctr.y + b->ctr.y;

				dv.x = v->x - ctr.x;
				dv.y = v->y - ctr.y;
				cu = dv.x*dv.x + dv.y*dv.y;

				if( u <= cu )	continue;

				u = cu;

				*s_jnt = jnt;
				*s_bno = bno;
			}


			if( jnt->type == JNT_RN3 && jnt->brn_no == 1 &&
						jnt->brn[0]->type == NF_N3 ){
				
				ctr.x = jnt->gctr.x + jnt->rwidth *jnt->u.x;
				ctr.y = jnt->gctr.y + jnt->rwidth *jnt->u.y;

				dv.x = v->x - ctr.x;
				dv.y = v->y - ctr.y;
				cu = dv.x*dv.x + dv.y*dv.y;
				if( cu < u ){
					u = cu;

					*s_jnt = jnt;
					*s_bno = 1;
				}


				ctr.x = jnt->gctr.x - jnt->lwidth *jnt->u.x;
				ctr.y = jnt->gctr.y - jnt->lwidth *jnt->u.y;

				dv.x = v->x - ctr.x;
				dv.y = v->y - ctr.y;
				cu = dv.x*dv.x + dv.y*dv.y;
				if( cu < u ){
					u = cu;

					*s_bno = 2;
				}
			}
		}
	    }
	}

	if( u > 2 )	*s_jnt = NULL;

	return( u );
}

float agp_select_bgp(gp_array *agp, vec2d *v,vec2d *s_ctr, bgp_type **s_bgp)
{
gp_type		*gp;
bgp_type	*b;

int	i0,	j0,	i,	j;
float	u,	cu;
vec2d	ctr;
  
	u = 1000;
	*s_bgp = NULL;


	i0 = (v->x+agp->bgp) * agp->inv_bgp;
	j0 = (v->y+agp->bgp) * agp->inv_bgp;
  
  
	for( i = i0-HSIZE ; i <= i0+HSIZE ; i++ ){
		if( i < 0 || i >= agp->Ni )	continue;

		for( j = j0-HSIZE  ; j <= j0+HSIZE ; j++ ){
			if( j < 0 || j >= agp->Nj )	continue;
      
			gp = &agp->gp[i][j];
			for( b = gp->bgp ; b != NULL ; b = b->p ){

				ctr.x = gp->ctr.x + b->ctr.x;
				ctr.y = gp->ctr.y + b->ctr.y;

				cu = hypot( ctr.x - v->x , ctr.y - v->y );
				
				if( ABS(u) < ABS(cu) )	continue;

				u = cu;

				*s_ctr = ctr;
				*s_bgp = b;
			}
		}
	}

	return( u );


}













static float
dis_form_link( vec2d *ctr, link_type *l, vec2d *v )
{
vec2d	dv;
float	u,	t;

	dv.x = v->x - ctr->x;
	dv.y = v->y - ctr->y;

	t = -dv.x *l->u.y + dv.y *l->u.x;

	if( t < 0 ){
		u = hypot( dv.x, dv.y );
		return( u );
	}

	if( t > l->len ){
		u = hypot( dv.x - l->v.x, dv.y - l->v.y );
		return( u );
	}

	u = l->u.x * dv.x + l->u.y * dv.y;
	u -= l->c_prb*t *(t - l->len );

	return( u );
}
