/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/***************************
 ***   AgpCopyBranch.c   ***
 ***************************/

#include	<stdio.h>
#include	<math.h>

#include	"Uigp/igp.h"
#include	"Uprm/prm.h"


#include	"EpType/EpType.h"



static void	link_update_box( vec2d *ctr, link_type *link, box2d *box );



void
agp_layer_box( gp_array *agp, int mbr, box2d *box )
{
gp_type	*gp;
hunch_type	*h;
bgp_type	*bp;
link_type	*link;
vec2d	ctr;
int	i,	j,	flag;


	BOX2D_INIT( *box, 0, 0 );


	flag = 0;
	for( i = 0 ; i < agp->Ni ; i++ )
		for( j = 0 ; j < agp->Nj ; j++ ){

			gp = &agp->gp[i][j];

			for( bp = gp->bgp ; bp != NULL ; bp = bp->p ){
				if( !( mbr & (1<<bp->br )) )	continue;
				ctr.x = gp->ctr.x + bp->ctr.x;
				ctr.y = gp->ctr.y + bp->ctr.y;
				if( flag == 0 ){
					BOX2D_INIT( *box, ctr.x, ctr.y );
					flag = 1;
				}
				else	BOX2D_UPDATE( *box, ctr.x, ctr.y );
			}


			for( h = gp->hunch ; h != NULL ; h = h->nh ){
				if( !( mbr & (1<<h->br )) )	continue;
				ctr.x = gp->ctr.x + h->ctr.x;
				ctr.y = gp->ctr.y + h->ctr.y;
				if( flag == 0 ){
					BOX2D_INIT( *box, ctr.x, ctr.y );
					flag = 1;
				}
				else	BOX2D_UPDATE( *box, ctr.x, ctr.y );
			}



			for( link = gp->link ; link != NULL ; link = link->nl ){
				if( !( mbr & (1<<link->br )) )	continue;

				ctr.x = gp->ctr.x + link->ctr.x;
				ctr.y = gp->ctr.y + link->ctr.y;
				if( flag == 0 ){
					BOX2D_INIT( *box, ctr.x, ctr.y );
					flag = 1;
				}
				else	BOX2D_UPDATE( *box, ctr.x, ctr.y );


				link_update_box( &ctr, link, box );
			}
		}
}


void
agp_layer_alink_box( link_type *alink[], int no, box2d *box )
{
link_type	*link;
vec2d	ctr;
int	i;
int	flag = 0;
	
	for( i = 0 ;  i < no ; i++ ){

		for( link = alink[i], ctr = alink[i]->ctr ; link != NULL ; link = LINK_NEXT(link) ){

			if( flag == 0 ){
				BOX2D_INIT( *box, ctr.x, ctr.y );
				flag = 1;
			}
			else	BOX2D_UPDATE( *box, ctr.x, ctr.y );


			link_update_box( &ctr, link, box );

			ctr.x += link->v.x;
			ctr.y += link->v.y;
		}
	}
}



static void
link_update_box( vec2d *ctr, link_type *link, box2d *box )
{
vec2d	v,	p;
float	t,	dt,	w,	w0,	w1;

/*
	BOX2D_UPDATE( *box, ctr->x, ctr->y );
*/


	v.x = ctr->x + link->v.x;
	v.y = ctr->y + link->v.y;
	BOX2D_UPDATE( *box, v.x, v.y );



	if( link_tanget_point_10( link, ctr, &t, &p ) ){
		BOX2D_UPDATE( *box, p.x, p.y );

		if( link->type == NF_N3 ){
			dt = t/link->len;

			w0 = link->width[F_BEGIN]->w[F_RIGHT];
			w1 = link->width[F_END]->w[F_RIGHT];
			w = t*w0 + (1-t)*w1;

			v.x = p.x + w*link->u.x;
			v.y = p.y + w*link->u.y;
			BOX2D_UPDATE( *box, v.x, v.y );



			w0 = link->width[F_BEGIN]->w[F_LEFT];
			w1 = link->width[F_END]->w[F_LEFT];
			w = t*w0 + (1-t)*w1;

			v.x = p.x + w*link->u.x;
			v.y = p.y + w*link->u.y;
			BOX2D_UPDATE( *box, v.x, v.y );
		}
	}


	if( link_tanget_point_01( link, ctr, &t, &p ) ){
		BOX2D_UPDATE( *box, p.x, p.y );

		if( link->type == NF_N3 ){
			dt = t/link->len;

			w0 = link->width[F_BEGIN]->w[F_RIGHT];
			w1 = link->width[F_END]->w[F_RIGHT];
			w = t*w0 + (1-t)*w1;

			v.x = p.x + w*link->u.x;
			v.y = p.y + w*link->u.y;
			BOX2D_UPDATE( *box, v.x, v.y );



			w0 = link->width[F_BEGIN]->w[F_LEFT];
			w1 = link->width[F_END]->w[F_LEFT];
			w = t*w0 + (1-t)*w1;

			v.x = p.x + w*link->u.x;
			v.y = p.y + w*link->u.y;
			BOX2D_UPDATE( *box, v.x, v.y );
		}
	}
}
