/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**************************
 ***	AgpLayerClear.c	***
 *************************/
#include	"Uigp/igp.h"


#include        "GpType/GpType.h"


static void	agp_layer_clear_hunch( vec2d *v, hunch_type *h, int br );

static void	agp_layer_clear_link( vec2d *ctr, link_type *link, int br );

static void	agp_layer_clear_link_n3( vec2d *ctr, link_type *link );

static void	agp_layer_clear_bgp( vec2d *ctr, bgp_type *bgp, int br );





void
agp_layer_clear( gp_array *agp, int br )
{
gp_type		*gp;
bgp_type	*bp;
link_type	*link;
hunch_type	*h;
vec2d	v;
int	i,	j;


	agp->ad->Mbr &= ~(1<<br);
	agp->layer[0]->Mbr &= ~(1<<br);


	for( i = 0 ; i < agp->Ni ; i++ )
		for( j = 0 ; j < agp->Nj ; j++ ){
			gp = &agp->gp[i][j];

			for( bp = gp->bgp ; bp != NULL ; bp = bp->p )
				agp_layer_clear_bgp( &gp->ctr, bp, br );


			for( h = gp->hunch ; h != NULL ; h = h->nh){
				v.x = gp->ctr.x + h->ctr.x;
				v.y = gp->ctr.y + h->ctr.y;

				agp_layer_clear_hunch( &v, h, br );
			}


			for( link = gp->link ; link != NULL ; link = link->nl )
				agp_layer_clear_link( &gp->ctr, link, br );

		}
}


static void
agp_layer_clear_bgp( vec2d *v, bgp_type *bp, int br )
{
	if( bp->br != br )	return;

	bp->depth = 0;
	bp->br = 0;
}



static void
agp_layer_clear_hunch( vec2d *vm, hunch_type *h, int br )
{
	if( h->br != br )	return;

	h->depth = 0;
	h->br = 0;
}




static void
agp_layer_clear_link( vec2d *ctr, link_type *link, int br )
{
vec2d	v;

	if( link->br != br )	return;

	if( link->type == NF_N3 ){
		agp_layer_clear_link_n3( ctr, link );
		return;
	}


	v.x = ctr->x + link->ctr.x;
	v.y = ctr->y + link->ctr.y;

	link->br = 0;
	link->Fbr = 0;

	link->pr[0].p->depth = 0;

	v.x += link->v.x;
	v.y += link->v.y;

	if( LINK_NEXT(link) != NULL )	return;

	link->pr[1].p->depth = 0;
}



static void
agp_layer_clear_link_n3( vec2d *ctr, link_type *link )
{
width_type	*width;


	link->br = 0;
	link->Fbr = 0;

	link->pr[0].p->depth = 0;

	width = link->width[F_BEGIN];
	width->depth[F_RIGHT] = 0;

	width->depth[F_LEFT] = 0;
		


	if( LINK_NEXT(link) != NULL )	return;


	link->pr[1].p->depth = 0;

	width = link->width[F_END];
	width->depth[F_RIGHT] = 0;

	width->depth[F_LEFT] = 0;
}
