/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/***************************
 ***	AgpCopyContour.c   ***
 ***************************/
#include	<stdio.h>   
#include	<math.h>

#include	"AgpTool/AgpTool.h"


#include	"AgpLayer.h"

#include	"EpType/EpType.h"
#include	"Ep/Ep.h"
#include	"EpBgp/EpBgp.h"


static adepth_type *	agp_layer_copy_ad( adepth_type *sad, int mbr,
				int row0, int col0, int row, int col );

static int	agp_copy_jnt_test_br_Fbr( jnt_type *jnt, int mbr );

static jnt_type *	agp_copy_jnt( jnt_type *jnt );

static void	agp_copy_jnt_link( gp_array *agp, jnt_type *jnt, int mbr );
static void	agp_copy_brn_in_n3( jnt_type *jnt, int bno );


static void	set_profile_edge( link_type *link, int side );
static void	set_profile_n3( link_type *link, int side );

static void	agp_copy_jnt_free_br( jnt_type *jnt, int mbr );








/**** *agp_layer_copy_contour ****/
/*!
 * The method copies the selected edges and joints of a layer 
 *
 * @param *agp : the source agp
 * @param branch : the branch number
 * @param Fbox : // flag -  1 : only in the box, 2: the whole agp
 *
 * @return gp_array  : the contour in a layer in an agp
 */
gp_array *agp_layer_copy_contour( gp_array *agp, int branch, int Fbox )
{
gp_array	*nagp;
box2d   box;

gp_type		*gp,	*ngp;
//bgp_type	*bgp,	**nbgp;
//hunch_type	*h,	**nh;
link_type	*link;
jnt_type	*jnt,	**njnt;
brn_type	*b;
int	row,	column;
int	i,	j,	k;
int	i0,	i1,	j0,	j1;
int mbr;
layer_type	*layer;

	if( agp->ad == NULL )	
		return( NULL );
	mbr = 1 << branch;


	switch( Fbox ){
	case 1:
		agp_layer_box( agp, mbr, &box );

		i0 = (box.x0 + agp->bgp )*agp->inv_bgp;
		i1 = (box.x1 + agp->bgp )*agp->inv_bgp;
		if( i0 < 1 )	
			i0 = 1;
		if( i1 >= agp->Ni-1 )	
			i1 = agp->Ni-2;

		j0 = (box.y0 + agp->bgp )*agp->inv_bgp;
		j1 = (box.y1 + agp->bgp )*agp->inv_bgp;
		if( j0 < 1 )	
			j0 = 1;
		if( j1 >= agp->Nj-1 )	
			j1 = agp->Nj-2;
	    break;
	case 2:
		i0 = j0 = 1;
		i1 = agp->Ni-2;
		j1 = agp->Nj-2;
	    break;
	}


	row = (i1-i0+1)*agp->bgp;
	column = (j1-j0+1)*agp->bgp;

	//prepare the gp_array
	//nagp = agp_alloc();
	//agp_clear( nagp, agp->bgp, row, column, agp->bgp*agp->Lni, agp->bgp*agp->Lnj );


	nagp = agp_const( row, column, agp->bgp, 255, 255, 255 );


	nagp->color_type = agp->color_type;
	nagp->Lcoding = agp->Lcoding;
//	nagp->type = AGP_TRANSPARENT;//agp->type;//
	nagp->type = agp->type;
	nagp->layer_no = 0;

	// prepare the layer
	layer = nagp->layer[nagp->layer_no++] = LAYER_ALLOC();
	layer->Mbr = mbr;
	layer->i0 = 1;
	layer->j0 = 1;
	layer->i1 = nagp->Ni-1;
	layer->j1 = nagp->Nj-1;
	layer->Fdepth_delta = 0;

	

	// copy adepth
	nagp->ad = agp_layer_copy_ad( agp->ad, mbr, (i0-1)*agp->bgp, (j0-1)*agp->bgp,
		row, column );

 


	/* copy jnt  */
	for( i = i0-1 ; i <= i1+1 ; i++ )
	{
		for( j = j0-1 ; j <= j1+1 ; j++ )
		{
			gp = &agp->gp[i][j];
			ngp = &nagp->gp[i-i0+1][j-j0+1];

			ngp->hunch = NULL;
			ngp->bgp = NULL;

			njnt = &ngp->jnt;
			for( jnt = gp->jnt ; jnt != NULL ; jnt = jnt->nj )
			{				
				if( agp_copy_jnt_test_br_Fbr( jnt, mbr ) == 0 )
					continue;

				jnt->cjnt = agp_copy_jnt( jnt );

				*njnt = jnt->cjnt;

				(*njnt)->gctr.x = (*njnt)->ctr.x + ngp->ctr.x;
				(*njnt)->gctr.y = (*njnt)->ctr.y + ngp->ctr.y;
				njnt = &(*njnt)->nj;
			}
			*njnt = NULL;
			
		}// for j
	}// for i


	/* copy links - only those who are connected to joints that were copied*/
	for( i = 0 ; i < nagp->Ni ; i++ )
	{
		for( j = 0 ; j < nagp->Nj ; j++ )
		{
			gp = &nagp->gp[i][j];
			for( jnt = gp->jnt ; jnt != NULL ; jnt = jnt->nj )
				agp_copy_jnt_link( nagp, jnt, mbr );
		}// for j
	}// for i


	for( i = 0 ; i < nagp->Ni ; i++ )
		for( j = 0 ; j < nagp->Nj ; j++ ){
			gp = &nagp->gp[i][j];
			for( jnt = gp->jnt ; jnt != NULL ; jnt = jnt->nj )
				agp_copy_jnt_free_br( jnt, mbr );
		}
	
	for( i = 0 ; i < nagp->Ni ; i++ )
	{
		for( j = 0 ; j < nagp->Nj ; j++ )
		{
			gp = &nagp->gp[i][j];

			for( jnt = gp->jnt ; jnt != NULL ; jnt = jnt->nj )
			{

				if( jnt->type != JNT_N3 )	
					continue;

				for( k = 0 ; k < jnt->brn_no ; k++ ){
					b = jnt->brn[k];
					if( b->type == BRN_IN )
					   agp_copy_brn_in_n3( jnt, k );
				}

				jnt_n3_set_aux( jnt );
			}

		}// for j
	}// for i


//	agp_connect_jnt( nagp );



	/* profile value */
	for( i = 0 ; i < nagp->Ni ; i++ )
	{
		for( j = 0 ; j < nagp->Nj ; j++ ){
			gp = &nagp->gp[i][j];

			for( link = gp->link ; link != NULL ; link = link->nl ){
				if( link->Fbr == 0 )	continue;

				switch( link->type )
				{
				case NF_EDGE:
					if( link->Fbr & 0x1 )
					    set_profile_edge( link, F_RIGHT );
					if( link->Fbr & 0x2 )
					    set_profile_edge( link, F_LEFT );
				    break;
				case NF_N3:
					if( link->Fbr & 0x1 )
					    set_profile_n3( link, F_RIGHT );
					if( link->Fbr & 0x2 )
					    set_profile_n3( link, F_LEFT );
				    break;
				}//switch
			}// for link
		}// for j
	}// for i


	return( nagp );
}



static adepth_type *
agp_layer_copy_ad( adepth_type *sad, int mbr,
			int row0, int col0, int row, int col )
{
adepth_type	*ad;
vec2d	p,	vp;
int	i;

	ad = ADEPTH_ALLOC();
	ad->P = sad->P;
	ad->N = sad->N;
	ad->br_no = 0;
	ad->Mbr = mbr;


	ad->br_no = 0;
	for( i = 0 ; i < sad->br_no ; i++ ){
		if( !( mbr & (1<<sad->abr[i].no)) )	continue;

		ad->abr[ad->br_no++] = sad->abr[i];

		if( sad->abr[i].Fplane != 0 ){
			ad->P = sad->abr[i].P;
			ad->N = sad->abr[i].N;
		}
	}



	ad->vw = VIEW_ALLOC();
	*ad->vw = *sad->vw;


	p.x = row0;
	p.y = col0;
	view_pixel2vp( sad->vw, &p, &vp );
	ad->vw->uv.x0 = vp.x;
	ad->vw->uv.y1 = vp.y;

	p.x = row0 + row;
	p.y = col0 + col;
	view_pixel2vp( sad->vw, &p, &vp );
	ad->vw->uv.x1 = vp.x;
	ad->vw->uv.y0 = vp.y;


	view_set_pixel( ad->vw, row, col );


	return( ad );
}


static int
agp_copy_jnt_test_br_Fbr( jnt_type *jnt, int mbr )
{
int	k;

	for( k = 0 ; k < jnt->brn_no ; k++ ){
		if( jnt->brn[k]->link == NULL )	continue;
		if(	jnt->brn[k]->link->Fbr != 0 )
			return 1;
	}

	return( 0 );
}




static void
agp_copy_jnt_free_br( jnt_type *jnt, int mbr )
{
brn_type	*b;
int	k,	i;

	if( jnt->type == JNT_N3 )	return;

	for( k = 0, i = 0 ; k < jnt->brn_no ; k++ ){

		if( jnt->brn[k]->link->Fbr == 0 ){
			free( jnt->brn[k] );
			jnt->brn[k] = NULL;
			continue;
		}

		if( i < k ){
			b = jnt->brn[i] = jnt->brn[k];
			b->link->p[b->direction].bno = i;
		}

		i++;
	}
	jnt->brn_no = i;
}




static jnt_type *
agp_copy_jnt( jnt_type *jnt )
{
jnt_type	*njnt;
int	k;

	njnt = jnt_alloc();
	*njnt = *jnt;

	for( k = 0 ; k < jnt->brn_no ; k++ ){
		njnt->brn[k] = brn_alloc();
		*njnt->brn[k] = *jnt->brn[k];
	}

	return( njnt );
}



static void
agp_copy_jnt_link( gp_array *agp, jnt_type *jnt, int mbr )
{
jnt_type	*njnt;
brn_type	*b,	*nb;
link_type	*link,	*l;
vec2d   ctr;
int     k;

	for( k = 0 ; k < jnt->brn_no ; k++ ){
		b = jnt->brn[k];
		if( b->type != NF_EDGE && b->type != NF_N3 &&
				b->type != NF_RIDGE && b->type != NF_N3R )
			continue;
		if( b->direction != F_BEGIN )   // only those who get out of the joint
			continue;

		if( b->link->Fbr == 0 )		continue;
//		if( !(mbr & (1<<b->link->br)) )	// with the right branch number
//			continue;

		link = link_make_copy( b->link, b->type );

		b->link = link;
		b->link->p[F_BEGIN].p.jnt = jnt;
		
//		link->br = b->link->br; // eyal

		for( l = link ; LINK_NEXT(l ) != NULL ; l = LINK_NEXT(l ) );

		njnt = l->p[F_END].p.jnt->cjnt;
		nb = njnt->brn[l->p[F_END].bno];

		nb->link = l;
		nb->link->p[F_END].p.jnt = njnt;


		ctr.x = jnt->gctr.x + b->ctr.x;
		ctr.y = jnt->gctr.y + b->ctr.y;
		agp_lnf_link( &ctr, b->link, agp );
	}
}




static void
agp_copy_brn_in_n3( jnt_type *jnt, int bno )
{
jnt_type	*njnt,	*pjnt;
brn_type	*b;
int	nbno;

	b = jnt->brn[bno];

	link_get_jnt_and_brn_no( b->link, b->direction, &pjnt, &nbno );

	njnt = pjnt->cjnt;

	b->link = njnt->brn[nbno]->link;

	b->in = NULL;
}


static void
set_profile_edge( link_type *link, int side )
{
edge_type *edge;

	edge = link->pr[0].edge;

	edge->side[side].val[SP_Y] = 128.5;
	edge->side[side].bval[SP_Y] = 128.5;

	edge->side[side].val[SP_Q] = 128.5;
	edge->side[side].bval[SP_Q] = 128.5;

	edge->side[side].val[SP_A] = 128.5;
	edge->side[side].bval[SP_A] = 128.5;

	if( LINK_NEXT(link) != NULL )	return;


	edge = link->pr[1].edge;

	edge->side[side].val[SP_Y] = 128.5;
	edge->side[side].bval[SP_Y] = 128.5;

	edge->side[side].val[SP_Q] = 128.5;
	edge->side[side].bval[SP_Q] = 128.5;

	edge->side[side].val[SP_A] = 128.5;
	edge->side[side].bval[SP_A] = 128.5;
}


static void
set_profile_n3( link_type *link, int side )
{
n3_type *n3;

	n3 = link->pr[0].n3;

	n3->side[side].val[SP_Y] = 128.5;
	n3->side[side].bval[SP_Y] = 128.5;

	n3->side[side].val[SP_Q] = 128.5;
	n3->side[side].bval[SP_Q] = 128.5;

	n3->side[side].val[SP_A] = 128.5;
	n3->side[side].bval[SP_A] = 128.5;

	if( LINK_NEXT(link) != NULL )	return;


	n3 = link->pr[1].n3;

	n3->side[side].val[SP_Y] = 128.5;
	n3->side[side].bval[SP_Y] = 128.5;

	n3->side[side].val[SP_Q] = 128.5;
	n3->side[side].bval[SP_Q] = 128.5;

	n3->side[side].val[SP_A] = 128.5;
	n3->side[side].bval[SP_A] = 128.5;
}


