/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/***************************
 *** AgpLayerCountor.c   ***
 ***************************/

#include	"EmbedLink/EmbedLink.h"
#include	"AgpEdit/AgpEditTool.h"
#include	"AgpDepth/AgpDepth.h"
#include	"AgpLayer/AgpLayer.h"
#include	"AgpTool/AgpTool.h"
#include	"LinkTool/LinkLayer.h"

#include	"N3Prime/N3Prime.h"
#include	"Quantize/Quantize.h"
#include	"GpType/LinkType.h"

#include	"RlLink/RlLink.h"
#include	"LinkTool/LinkTool.h"



int agp_layer_contour( gp_array *agp, vec2d *ctr, link_type *link, 
			     int layer, float depth, vec2d *mp, int step )
{
link_type	*l;
vec3d	P;
vec2d	v,	mp0;


	if( agp->ad == NULL )
		agp_depth_default( agp, 100.0F );

	agp_layer_init( agp );



		

	link = agp_polylink_insertA( agp, ctr, link, 1/*Fclose*/ );


	if( step < 1 )	return(1);



	if( mp == NULL )
//		link_get_point_inside( ctr, link, &mp0 );
		polylink_inside_point( ctr, link, &mp0 );
	else	mp0 = *mp;



	
	for( l = link ; l != NULL ; l = LINK_NEXT(l ) ){
		l->br = layer;
		l->Fbr = 0x10;
	}


	 embed_link( ctr, link, agp );

/* new 13.11.02 */
	 embed_link_fit_curve( agp );
/* end 13.11.02 */

	if( step < 2 )	return(1);



	if( agp_layer_hole( agp, layer, &v ) ==  1 )
		return( -1 );



	agp_layer_FC( agp, layer, &mp0, 0 );


	agp->ad->Mbr |= (1 << layer );
	agp->layer[0]->Mbr = agp->ad->Mbr;


 
	P.x = agp->ad->vw->cp.x + depth * agp->ad->vw->N.x;
	P.y = agp->ad->vw->cp.y + depth * agp->ad->vw->N.y;
	P.z = agp->ad->vw->cp.z + depth * agp->ad->vw->N.z;
 
	adepth_set_plane( agp->ad, layer, &agp->ad->vw->N, &P );
 
	agp_layer_depth( agp, layer );

	return( 1 );
}




gp_array *
agp_layer_contour_copy( gp_array *agp, vec2d *ctr, link_type *link, 
			     int layer, float depth, vec2d *mp )
{
int	Fn3_prime;
gp_array *tagp,	*nagp;
coding_type	*cprm;
	
	if( ( Fn3_prime = agp->Fn3_prime) == 1 )
		n3_prime_free( agp );

	tagp = agp_make_copy( agp );
	if( Fn3_prime == 1 )	n3_prime_agp( agp );


//	agp_layer_init( tagp );



	if( agp_layer_contour( tagp, ctr, link, layer, depth, mp, 2 ) <= 0 ){
		agp_destroy( tagp );
		return( NULL );
	}


	embed_link_fit_bgp( tagp, 0.35 );


	
	nagp = agp_layer_copy( tagp, 1<<layer, 1 );
	agp_destroy( tagp );



	agp_connect_jnt( nagp );

	embed_link_fit_jnt( nagp );

	embed_link_fit( nagp );
	embed_link_fit( nagp );


	rl_link_countor_agp( nagp );

	

//  9.6.2002
	cprm = coding_init( nagp->Lcoding, nagp->bgp );
	quantize_agp( nagp, cprm );
	coding_free( cprm );

	embed_link_fit( nagp );
//	agp_connect_jnt( nagp );

	embed_link_fit_hunch( nagp, 0.75 );
	embed_link_fit_bgp( nagp, 0.35 );


	if( agp_bgp_test( nagp, -1 ) < 0 ){
		agp_destroy( nagp );
		return( NULL );
	}


//	end 9.6.2002

	return( nagp );
}







gp_array *
agp_layer_contour_copyT( gp_array *agp, vec2d *ctr, link_type *link, 
			     int layer, float depth, vec2d *mp )
{
int	Fn3_prime;
gp_array *tagp,	*nagp;
coding_type	*cprm;
	
	if( ( Fn3_prime = agp->Fn3_prime) == 1 )
		n3_prime_free( agp );

	tagp = agp_make_copy( agp );
	if( Fn3_prime == 1 )	n3_prime_agp( agp );


	agp_layer_init( tagp );


	if( agp_layer_contour( tagp, ctr, link, layer, depth, mp, 2 ) <= 0 ){
		agp_destroy( tagp );
		return( NULL );
	}

	
	embed_link_fit_bgp( tagp, 0.35 );

	
	nagp = agp_layer_copy( tagp, 1<<layer, 1 );
	agp_destroy( tagp );


	agp_connect_jnt( nagp );




	agp_connect_jnt( nagp );

	embed_link_fit( nagp );
	embed_link_fit( nagp );




//	agp_bgp_fill( nagp, 0 );

	return( nagp );
	
//  9.6.2002
	cprm = coding_init( nagp->Lcoding, nagp->bgp );
	quantize_agp( nagp, cprm );
	coding_free( cprm );

	embed_link_fit( nagp );
	agp_connect_jnt( nagp );

//	end 9.6.2002

	return( nagp );
}