/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/*! Time-stamp: <@(#)AgpLayerF.c   3/6/2002 - 10:59:21 AM   Yoram>
 *********************************************************************
 *  @file   : AgpLayerF.c
 *
 *  Project : Agp
 *
 *  Package : Core
 *
 *  Company : Vimatix
 *
 *  Author  : Yoram                              Date: 3/6/2002
 *
 *  Purpose : Manage the flow of ep cubes in an Agp, in order to fill a layer in the agp
 *
 *********************************************************************
 * Version History:
 *
 * V 0.10  3/6/2002  BN : First Revision
 *
 *********************************************************************
 */


#include	<stdio.h>
#include	<math.h>

#include	"Uigp/igp.h"
#include	"Uprm/prm.h"
 
#include	"GpType/GpType.h"
#include	"EpType/EpType.h"

#include	"N3Prime/N3Prime.h"
 
#include	"Ep/Ep.h"
 
 
#include	"AgpLayer.h"
#include	"AgpLayerF.h"


static	char	*Fblock;
#define		BST_SIZE	4000

static struct BLOCK_ST {
		int	i,	j;
}	Bst[BST_SIZE];

static int      Bst_b,   Bst_e;

#define BST_ADD( n )     (( n == BST_SIZE-1)? 0 : n++)

static	int Brow,	Bcol;

static void	agp_layer_F_from_point( gp_array *agp,
					ep_array *aep, int br, vec2d *p );

static void	agp_layer_F_from_boundary( gp_array *agp,
					ep_array *aep, int branch );

static void	agp_layer_F_from_boundary_one( gp_type *gp,
						ep_type *ep, int br );

static void	block_st_add( int i, int j );


static void	agp_layer_F_from_boundary_add( ep_type *ep, int bno );




void
agp_layer_F( gp_array *agp, int branch, vec2d *p, int step_no )
{
ep_array	*aep;
int	i0,	j0,	n,	k;
int	Fn3_prime,	step;


	if( (Fn3_prime = agp->Fn3_prime) == 0 ) n3_prime_agp( agp );


	Brow = agp->row /( EP_N * agp->bgp ) + 1;
	Bcol = agp->column /( EP_N * agp->bgp ) + 1;

	Fblock = calloc( Brow*Bcol , 1 );

	Bst_b = Bst_e = 0;


	aep = aep_alloc();



	i0 = p->x /( EP_N * agp->bgp );
	j0 = p->y /( EP_N * agp->bgp );
	if( i0 < 0 )	i0 = 0;
	if( j0 < 0 )	j0 = 0;

	Fblock[ i0 * Bcol + j0 ] = 1;

	i0 = i0 * EP_N + 1;
	j0 = j0 * EP_N + 1;

	// create the structure to hold the layer information
	ep_create( agp, i0, j0, EP_N, EP_N, 0, aep );


	// expand from point, and leave boundaries unvisited
	agp_layer_F_from_point( agp, aep, branch, p );



	for( step = 1 ; 
		Bst_b != Bst_e &&	( step_no <= 0 || step < step_no ) ; 
		step++ ){

		i0 = Bst[Bst_b].i;
		j0 = Bst[Bst_b].j;

		n = i0/EP_N;
		k = j0/EP_N;
	

		// if that block is out or range
		if( i0 <= 0 || i0 >= agp->Ni-1 ||
				j0 <= 0 || j0 >= agp->Nj-1 ){
			BST_ADD( Bst_b );
			continue;
		}

		// if visited in the block already
		if( Fblock[ n * Bcol + k ] == 1 ){
			BST_ADD( Bst_b );
			continue;
		}

		// set block as visited
		Fblock[ n * Bcol + k ] = 1;

		// create corresponding ep
		ep_create( agp, i0, j0, EP_N, EP_N, 0, aep );

		// 
		agp_layer_F_from_boundary( agp, aep, branch );

		// fill part of the layer according to current ep
		agp_layer_F1( agp, aep, branch );

		// advance pointer in block array
		BST_ADD( Bst_b );
	}

	free( Fblock );
	Fblock = NULL;

	aep_destroy( aep );



	agp_layer_F_n3_prime_load( agp );

	if( Fn3_prime == 0 )    n3_prime_free( agp );
}




int
agp_layer_F_test_boundary( ep_array *aep, ep_type *ep )
{
int	i,	j;


	i = ( ep->ctr.x + LSIZE*aep->bgp) * aep->bgp_inv;
	j = ( ep->ctr.y + LSIZE*aep->bgp) * aep->bgp_inv;

	if( i < LSIZE ){
		block_st_add ( aep->Ni0 - EP_N, aep->Nj0 ); 
		return( 0 );
	}

	if( i >= aep->Ni - LSIZE ){
		block_st_add ( aep->Ni0 + EP_N, aep->Nj0 ); 
		return( 0 );
	}


	if( j < LSIZE ){
		block_st_add ( aep->Ni0, aep->Nj0 - EP_N ); 
		return( 0 );
	}

	if( j >= aep->Nj - LSIZE ){
		block_st_add ( aep->Ni0, aep->Nj0 + EP_N ); 
		return( 0 );
	}

	return( 1 );
}


static void
block_st_add( int i, int j )
{
int	n,	k;
	if( Fblock == NULL )	return;

	n = i/EP_N;
	k = j/EP_N;
	
	if( Fblock[ n * Bcol + k ] == 1 )	return;

	Bst[ Bst_e ].i = i;
	Bst[ Bst_e ].j = j;

	BST_ADD( Bst_e );
}






void
agp_layer_F_from_point( gp_array *agp, ep_array *aep, int branch, vec2d *p )
{
ep_type *ep;
vec2d   v;
int     i,      j,      bno;
float	dis;


	i = ( p->x - aep->V0.x + LSIZE*aep->bgp) * aep->bgp_inv;
	j = ( p->y - aep->V0.y + LSIZE*aep->bgp) * aep->bgp_inv;
	ep = &aep->ep[i][j];

	v.x = p->x - aep->V0.x - ep->ctr.x;
	v.y = p->y - aep->V0.y - ep->ctr.y;
	bno = ep_bound_point( ep, &v, &dis );
	if( bno == -1 ) return;
 

	agp_layer_F1_st_clear();
	agp_layer_F1_st_add( ep, ( 1<<bno) );
 
 
	agp_layer_F1( agp, aep, branch );
}




void
agp_layer_F_from_boundary( gp_array *agp, ep_array *aep, int br )
{
gp_type	*gp;
ep_type	*ep;
int     i,      j;
int	i0,	j0;
int	i1,	j1;

	agp_layer_F1_st_clear();

	for( j = JSIZE ; j < aep->Nj-JSIZE ; j++ ){
		ep = &aep->ep[JSIZE][j];

		i1 = aep->Ni0 - 1;
		j1 = aep->Nj0 + j - LSIZE;
		if( i1 == 0 || i1 == agp->Ni-1 || j1 == 0 || j1 == agp->Nj-1 )
			continue;

		gp = &agp->gp[aep->Ni0 - 1][aep->Nj0 + j - LSIZE];
		agp_layer_F_from_boundary_one( gp, ep, br );
	}


	i0 = aep->Ni-1-JSIZE;
	for( j = JSIZE ; j < aep->Nj-JSIZE ; j++ ){
		ep = &aep->ep[i0][j];

		i1 = aep->Ni0 + i0 - LSIZE;
		j1 = aep->Nj0 + j - LSIZE;
		if( i1 == 0 || i1 == agp->Ni-1 || j1 == 0 || j1 == agp->Nj-1 )
			continue;

		gp = &agp->gp[aep->Ni0 + i0 - LSIZE][aep->Nj0 + j - LSIZE];
		agp_layer_F_from_boundary_one( gp, ep, br );
	}

	for( i = JSIZE ; i < aep->Ni-JSIZE ; i++ ){
		ep = &aep->ep[i][JSIZE];

		i1 = aep->Ni0 + i - LSIZE;
		j1 = aep->Nj0 - 1;
		if( i1 == 0 || i1 == agp->Ni-1 || j1 == 0 || j1 == agp->Nj-1 )
			continue;

		gp = &agp->gp[aep->Ni0 + i - LSIZE][aep->Nj0 - 1];
		agp_layer_F_from_boundary_one( gp, ep, br );
	}


	j0 = aep->Nj-1-JSIZE;
	for( i = JSIZE ; i < aep->Ni-JSIZE ; i++ ){
		ep = &aep->ep[i][aep->Nj-1-JSIZE];

		i1 = aep->Ni0 + i - LSIZE;
		j1 = aep->Nj0 + j0 - LSIZE;
		if( i1 == 0 || i1 == agp->Ni-1 || j1 == 0 || j1 == agp->Nj-1 )
			continue;

		gp = &agp->gp[aep->Ni0 + i - LSIZE][aep->Nj0 + j0 - LSIZE];

		agp_layer_F_from_boundary_one( gp, ep, br );
	}
}



static void
agp_layer_F_from_boundary_one( gp_type *gp, ep_type *ep, int br )
{
al_type	*al;

	if( ep->b_no == 1 && ep->bound[0]->type & BOUND_ALL ){
		if( gp->bgp == NULL )	return;

		if( gp->bgp->br != br )	return;
		agp_layer_F_from_boundary_add( ep, 0 );
		return;
	}


	for( al = ep->al ; al != NULL ; al = al->nal ){


		if( al->link->Fbr == 0x10 )      continue;

		if( al->link->Fbr == 0x04 ){
			if( al->link->br == br )
				agp_layer_F_from_boundary_add( ep,
							al->bno[F_RIGHT] );
			if( al->link->br1 == br )
				agp_layer_F_from_boundary_add( ep,
							al->bno[F_LEFT] );
			continue;
		}


		if( al->link->br != br ) continue;


		if( ( al->link->Fbr & ( 1<< F_RIGHT) ) == 0 )
			agp_layer_F_from_boundary_add( ep, al->bno[F_RIGHT] );

		if( ( al->link->Fbr & ( 1<< F_LEFT) ) == 0 )
			agp_layer_F_from_boundary_add( ep, al->bno[F_LEFT] );
	}
}

static void
agp_layer_F_from_boundary_add( ep_type *ep, int bno )
{
bound_type	*b;
int	n;

	b = ep->bound[bno];


	for( n = 0 ; n < 9 ; n++ ){
		if( n == 4 )    continue;

		if( b->Fcont[n] == 0 )      continue;

		agp_layer_F1_st_add( ep->ng[n], b->Fcont[n] );
	}
}
