/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/



#include	<stdio.h>
#include	<math.h>

#include    "Uigp/igp.h"
#include    "Uprm/prm.h"

#include    "GpType/GpType.h"
#include    "EpType/EpType.h"


#include    "AgpLayer/AgpLayer.h"
#include    "AgpLayer/AgpLayerF.h"

#include    "Ep/Ep.h"


#define		ST_SIZE		4000

static struct		ST {
		ep_type	*ep;
		int	bmask;
}	St[ST_SIZE];

static int	St_b,	St_e;

#define	ST_ADD( n )	(( n == ST_SIZE-1)? 0 : n++)





static void	agp_layer_F1_ep( ep_array *aep, ep_type *ep,
					int bmask, int branch );


static void	agp_layer_F1_ep_bound( ep_type *ep, int bno,
			int branch, int nb[], int *nb_no, int cont[9] );



static void	agp_layer_F1_bound_nieg( ep_type *ep, int bno,
					int branch, int nb[], int *nb_no );

static void	sp_nieg_bound_add( ep_type *ep, int bno,
					int branch, int nb[], int *nb_no );


static void	br_fill_link( link_type *link, int branch, int fbr, int Fside );




	/* Local nf */
static void	agp_layer_F1_nf( gp_type *gp, int branch, ep_type *ep );

static void	agp_layer_F1_ridge( vec2d *ctr, link_type *link,
						int branch, ep_type *ep );

static void	agp_layer_F1_hunch( vec2d *ctr, hunch_type *h,
						int branch, ep_type *ep );

static void	agp_layer_F1_bgp( vec2d *ctr, bgp_type *bgp,
						int branch, ep_type *ep );



 




/**** agp_layer_F1 ****/
/*!
 * The method attaches the elements in the gp_array, according to aep's information.
 *
 * @param *agp : the gp_array to fill
 * @param *aep : current cube
 * @param branch : branch number
 *
 * @return void  : 
 */
void agp_layer_F1( gp_array *agp, ep_array *aep, int branch )
{
ep_type	*ep;
gp_type	*gp;
int	i,	j,	k;

	// clear br.no 
	for( i = JSIZE ; i < aep->Ni-JSIZE ; i++ )
		for( j = JSIZE ; j < aep->Nj-JSIZE ; j++ )
		{
			ep = &aep->ep[i][j];
			for( k = 0 ; k < ep->b_no ; k++ )
				ep->bound[k]->br.no = 255;
		}


	while( St_b != St_e )
	{

		agp_layer_F1_ep( aep, St[St_b].ep,St[St_b].bmask, branch );


	      i = ( aep->V0.x + St[St_b].ep->ctr.x + agp->bgp) * agp->inv_bgp;
        	j = ( aep->V0.y + St[St_b].ep->ctr.y + agp->bgp) * agp->inv_bgp;

		gp = &agp->gp[i][j];
		agp_layer_F1_nf( gp, branch, St[St_b].ep );

		ST_ADD( St_b );
	}	
}






/**** agp_layer_F1_ep ****/
/*!
 * 
 *
 * @param *aep : 
 * @param *ep : 
 * @param bmask : 
 * @param branch : 
 *
 * @return void  : 
 */
static void agp_layer_F1_ep( ep_array *aep, ep_type *ep, int bmask, int branch )
{
int	cont[9],	nb[MAX_BOUND],	nb_no;
int	i,	n;

	for( i = 0 ; i < 9 ; i++ )	cont[i] = 0;

	for( i = 0, nb_no = 0 ; i < ep->b_no ; i++ )
		if( bmask & (1<<i) )	nb[nb_no++] = i;


	for( i = 0 ; i < nb_no ; i++ )
		agp_layer_F1_ep_bound( ep, nb[i], branch, nb, &nb_no, cont);



	if( agp_layer_F_test_boundary( aep, ep ) == 0 )	
		return;


	for( n = 0 ; n < 9 ; n++ )
	{
		if( n == 4 )	
			continue;
		if( cont[n] == 0 )	
			continue;

		agp_layer_F1_st_add( ep->ng[n], cont[n] );
	}
}


void
agp_layer_F1_st_clear()
{
	St_b = St_e = 0;
}


void
agp_layer_F1_st_add( ep_type *ep, int cont )
{
int	i;

	if( St_b == St_e )
	{
		St[St_e].ep = ep;
		St[St_e].bmask = cont;

		ST_ADD( St_e );
	}

	i = St_b;
	ST_ADD( i );

	while( i != St_e )
	{

		if( St[i].ep == ep )
		{
			St[i].bmask |= cont;
			return;
		}

		ST_ADD( i );
	}

	St[St_e].ep = ep;
	St[St_e].bmask = cont;

	ST_ADD( St_e );
}


void
agp_layer_F1_ep_bound( ep_type *ep, int bno,
		int branch, int nb[], int *nb_no, int cont[9] )
{
ep_type	*nep;
bound_type	*b;
int	i,	n;

	b = ep->bound[bno];
	if( b->br.no == branch )	return;

	b->br.no = branch;


	agp_layer_F1_bound_nieg( ep, bno, branch, nb, nb_no );

	for( n = 0 ; n < 9 ; n++ )
	{
		if( n == 4 )	
			continue;
		if( b->Fcont[n] == 0 )	
			continue;
		nep = ep->ng[n];

		for( i = 0 ; i < nep->b_no ; i++ ){
			if( !( b->Fcont[n] & (1<<i) ))	
				continue;
			if( nep->bound[i]->br.no == branch )	
				continue;
			cont[n] |= (1<<i);
		}
	}
}








static void
agp_layer_F1_bound_nieg( ep_type *ep, int bno, int branch, int nb[], int *nb_no )
{
bound_type	*b;
al_type	*al;
link_type	*l;
int	side,	iside,	nbno,	fbr;

	b = ep->bound[bno];


	for( al = ep->al ; al != NULL ; al = al->nal )
	{

		if( al->bno[F_RIGHT] != bno && al->bno[F_LEFT] != bno )
			continue;


		side = ( al->bno[F_RIGHT] == bno )? F_RIGHT : F_LEFT;
		iside = INV_FLAG(side);

		if( (l = al->link) == NULL )	
			continue;



            if( l->Fbr == 0x04 )
		{
                  br_fill_link( l,  branch, 0x04, 1<<side );
			continue;
		} 




		if( l->Fbr == 0 )
		{
			nbno = al->bno[ iside ];

			sp_nieg_bound_add( ep, nbno, branch, nb, nb_no);

			br_fill_link( l,  branch, 0, 0x03 );

			continue;
		}





		if( l->Fbr != 0x10 )
		{
			if(  l->type != NF_N3_P )	
				continue;

			if( l->side == side )
				br_fill_link( l,  branch, 0, 0x03 );

			continue;
		}








/*
		if( l->Fbr != 0x10 )	continue;
*/



		if( l->type == NF_EDGE )
		{

			fbr = 1 << iside;
			br_fill_link( l,  branch, fbr, 0x03 );
			continue;
		}




		if( l->side != side )
		{

			fbr = 1 << iside;
			br_fill_link( l,  branch, fbr, 0x03 );

			continue; 
		}



		nbno = al->bno[ iside ];
		sp_nieg_bound_add( ep, nbno, branch, nb, nb_no);

/*
		fbr = 1 << iside;
		br_fill_link( l,  branch, fbr, 0x03 );
*/
		br_fill_link( l,  branch, 0, 0x03 );
	}//for
}





static void
sp_nieg_bound_add( ep_type *ep, int bno, int branch, int nb[], int *nb_no )
{
int	k;

	if( ep->bound[bno]->br.no == branch )	
		return;

	for( k = 0 ; k < *nb_no ; k++ )
		if( nb[k] == bno )	
			return;

	nb[ (*nb_no)++ ] = bno;
}



static void
br_fill_link( link_type *link, int br, int fbr, int Fside )
{
link_type       *l,	*l0;

/*
	if( link->br == br )	return;
*/

        for( l0 = link ; LINK_PREV(l0) != NULL ; l0 = LINK_PREV(l0) );

        for( l = l0 ; l != NULL ; l = LINK_NEXT(l) )
	  {

                l->Fbr = fbr;

		if( Fside & ( 1<<F_RIGHT) )     l->br = br;

		if( Fside & ( 1<<F_LEFT) )      l->br1 = br;

        }
}




static void
agp_layer_F1_nf( gp_type *gp, int branch, ep_type *ep )
{
bgp_type	*bgp;
link_type       *link; 
hunch_type      *h;

	for( bgp = gp->bgp ; bgp != NULL ; bgp = bgp->p )
		agp_layer_F1_bgp( &gp->ctr, bgp, branch, ep );

	for( h = gp->hunch ; h != NULL ; h = h->nh)
		agp_layer_F1_hunch( &gp->ctr, h, branch, ep );


	for( link = gp->link ; link != NULL ; link = link->nl )
	{
		if( link->type != NF_RIDGE && link->type != NF_N3R )
			continue;

		agp_layer_F1_ridge( &gp->ctr, link, branch, ep );
	}
}


static void
agp_layer_F1_bgp( vec2d *ctr, bgp_type *bgp, int branch, ep_type *ep )
{
bound_type	*b;
vec2d	v;
int	bno;
float	dis;

	v.x = ctr->x + bgp->ctr.x;
	v.y = ctr->y + bgp->ctr.y;

	if( (bno = ep_bound_point( ep, &bgp->ctr, &dis ) ) == -1 )
		return;

	b = ep->bound[bno];

	if( b->br.no == branch )
		bgp->br = branch;
}


static void
agp_layer_F1_hunch( vec2d *ctr, hunch_type *h, int branch, ep_type *ep )
{
bound_type	*b;
int	bno;
float	dis;

	if( (bno = ep_bound_point( ep, &h->ctr, &dis ) ) == -1 )
		return;

	b = ep->bound[bno];

	if( b->br.no == branch )
		h->br = branch;
}

static void
agp_layer_F1_ridge( vec2d *ctr, link_type *link, int branch, ep_type *ep )
{
bound_type	*b;
link_type	*l;
vec2d	v;
int	bno;
float	dis;

        v.x = ctr->x + link->ctr.x;
        v.y = ctr->y + link->ctr.y;

	if( (bno = ep_bound_point( ep, &link->ctr, &dis ) ) == -1 )
		return;

	b = ep->bound[bno];
	if( b->br.no != branch )	
		return;

	link->br = branch;
	link->Fbr = 0;

	for( l = LINK_PREV(link) ; l != NULL ; l = LINK_PREV(l) )
	{
		l->br = branch;
		l->Fbr = 0;
	}

	for( l = LINK_NEXT(link) ; l != NULL ; l = LINK_NEXT(l) )
	{
		l->br = branch;
		l->Fbr = 0;
	}
}
