/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/*! Time-stamp: <@(#)AgpLayerFC1.c   3/6/2002 - 10:57:30 AM   Yoram>
 *********************************************************************
 *  @file   : AgpLayerFC1.c
 *
 *  Project : Agp
 *
 *  Package : Core
 *
 *  Company : Vimatix
 *
 *  Author  : Eyal Peleg				     Date: 3/6/2002
 *
 *  Purpose : Fill part of a layer in an agp according to info of a ep_array structure.
 *
 *********************************************************************
 * Version History:
 *
 * V 0.10  3/6/2002  BN : First Revision
 *
 *********************************************************************
 */


#include	<stdio.h>
#include	<assert.h>
#include	<math.h>

#include    "Uigp/igp.h"
#include    "Uprm/prm.h"

#include    "GpType/GpType.h"
#include    "EpType/EpType.h"


#include    "AgpLayer/AgpLayer.h"
#include    "AgpLayer/AgpLayerF.h"

#include    "Ep/Ep.h"


#define		ST_SIZE		8000

static struct		ST {
		ep_type	*ep;
		int	bmask;
}	St[ST_SIZE];

static int	St_b,	St_e;

#define	ST_ADD( n )	(( n == ST_SIZE-1)? 0 : n++)





static void	agp_layer_FC1_ep( ep_array *aep, ep_type *ep,
					int bmask, int branch );


static void	agp_layer_FC1_ep_bound( ep_type *ep, int bno,
			int branch, int nb[], int *nb_no, int cont[9] );



static void	agp_layer_FC1_bound_nieg( ep_type *ep, int bno,
					int branch, int nb[], int *nb_no );

static void	sp_nieg_bound_add( ep_type *ep, int bno,
					int branch, int nb[], int *nb_no );


static void	br_fill_link( link_type *link, int branch, int fbr, int Fside );
static void br_fill_linkFBR( link_type *link, int direct, int br, int fbr, int Fside );





	/* Local nf */
static void	agp_layer_FC1_nf( gp_type *gp, gp_type *baseGp, int branch, ep_type *ep );

static void	agp_layer_FC1_ridge( vec2d *ctr, link_type *link, int branch, ep_type *ep );

static void	agp_layer_FC1_edge( gp_type *gp, vec2d *ctr, link_type *link, int branch, ep_type *ep );

static void	agp_layer_FC1_hunch( vec2d *ctr, hunch_type *h, int branch, ep_type *ep );

static void	agp_layer_FC1_bgp( vec2d *ctr, bgp_type *bgp,int branch, ep_type *ep );



 




/**** agp_layer_FC1 ****/
/*!
 * The method attaches the elements in the gp_array, according to aep's information.
 *
 * @param *agp : the gp_array to fill
 * @param *aep : current cube
 * @param branch : branch number
 *
 * @return void  : 
 */
void agp_layer_FC1( gp_array *agp, gp_array *baseAgp, ep_array *aep, int branch )
{
ep_type	*ep;
gp_type	*gp, *baseGp;
int	i,	j,	k;

	// clear br.no 
	for( i = JSIZE ; i < aep->Ni-JSIZE ; i++ )
	{
		for( j = JSIZE ; j < aep->Nj-JSIZE ; j++ )
		{
			ep = &aep->ep[i][j];
			for( k = 0 ; k < ep->b_no ; k++ )
				ep->bound[k]->br.no = 255;
		}// for j
	}// for i
	
	while( St_b != St_e )
	{

		// split the ep to bounds (according to contour-agp).
		agp_layer_FC1_ep( aep, St[St_b].ep,St[St_b].bmask, branch );


	      i = ( aep->V0.x + St[St_b].ep->ctr.x + agp->bgp) * agp->inv_bgp;
        	j = ( aep->V0.y + St[St_b].ep->ctr.y + agp->bgp) * agp->inv_bgp;

		gp = &agp->gp[i][j];
		baseGp = &baseAgp->gp[i][j];
		// set all normal forms in the baseGp to have correct bound number
		agp_layer_FC1_nf( gp, baseGp, branch, St[St_b].ep );

		ST_ADD( St_b );
	}
	
	//agp_layer_F2_fill_data(baseAgp, aep, branch, i, j);	
	
}






/**** agp_layer_FC1_ep ****/
/*!
 * build the aep structure, and set connection between aep and it's neighbours
 *
 * @param *aep : the aep
 * @param *ep : the ep to start from
 * @param bmask : the bound mask
 * @param branch : branch number
 *
 * @return void  : 
 */
static void agp_layer_FC1_ep( ep_array *aep, ep_type *ep, int bmask, int branch )
{
int	cont[9],	nb[MAX_BOUND],	nb_no;
int	i,	n;

	for( i = 0 ; i < 9 ; i++ )	cont[i] = 0;

	for( i = 0, nb_no = 0 ; i < ep->b_no ; i++ )
		if( bmask & (1<<i) )	nb[nb_no++] = i;


	for( i = 0 ; i < nb_no ; i++ )
		agp_layer_FC1_ep_bound( ep, nb[i], branch, nb, &nb_no, cont);



	if( agp_layer_FC_test_boundary( aep, ep ) == 0 )	
		return;


	for( n = 0 ; n < 9 ; n++ )
	{
		if( n == 4 )	
			continue;
		if( cont[n] == 0 )	
			continue;

		agp_layer_FC1_st_add( ep->ng[n], cont[n] );
	}
}



/**** agp_layer_FC1_st_clear ****/
/*!
 * clear the stack
 *
 * @param none
 *
 * @return void  : 
 */
void agp_layer_FC1_st_clear()
{
	St_b = St_e = 0;
}



/**** agp_layer_FC1_st_add ****/
/*!
 * The method adds an ep to the stack.
 *
 * @param *ep : the ep to add
 * @param cont : the bound mask (1<<bno)
 *
 * @return void  : 
 */
void agp_layer_FC1_st_add( ep_type *ep, int cont )
{
int	i;

	if( St_b == St_e )
	{
		St[St_e].ep = ep;
		St[St_e].bmask = cont;

		ST_ADD( St_e );
	}

	i = St_b;
	ST_ADD( i );

	while( i != St_e )
	{

		if( St[i].ep == ep )
		{
			St[i].bmask |= cont;
			return;
		}

		ST_ADD( i );
	}

	St[St_e].ep = ep;
	St[St_e].bmask = cont;

	ST_ADD( St_e );
}



/**** agp_layer_FC1_ep_bound ****/
/*!
 * set bound in an ep and all it's neighbours
 *
 * @param *ep : the ep to start from
 * @param bno : bound number
 * @param branch : branch number
 * @param nb[] : neighbours array
 * @param *nb_no : how many neighbours are accessible
 * @param cont[9] : array of masks
 *
 * @return void  : 
 */
void agp_layer_FC1_ep_bound( ep_type *ep, int bno, int branch, 
				    int nb[], int *nb_no, int cont[9] )
{
ep_type	*nep;
bound_type	*b;
int	i,	n;

	b = ep->bound[bno];
	if( b->br.no == branch )	return;

	b->br.no = branch;


	agp_layer_FC1_bound_nieg( ep, bno, branch, nb, nb_no );

	for( n = 0 ; n < 9 ; n++ )
	{
		if( n == 4 )	
			continue;
		if( b->Fcont[n] == 0 )	
			continue;
		nep = ep->ng[n];

		for( i = 0 ; i < nep->b_no ; i++ ){
			if( !( b->Fcont[n] & (1<<i) ))	
				continue;
			if( nep->bound[i]->br.no == branch )	
				continue;
			cont[n] |= (1<<i);
		}
	}
}



/**** static agp_layer_FC1_bound_nieg ****/
/*!
 * 
 *
 * @param *ep : 
 * @param bno : bound number
 * @param branch : 
 * @param nb[] : 
 * @param *nb_no : 
 *
 * @return void  : 
 */
static void agp_layer_FC1_bound_nieg( ep_type *ep, int bno, int branch, 
						 int nb[], int *nb_no )
{
bound_type	*b;
al_type	*al;
link_type	*l;
int	side,	iside,	nbno,	fbr;

	b = ep->bound[bno];


	for( al = ep->al ; al != NULL ; al = al->nal )
	{

		if( al->bno[F_RIGHT] != bno && al->bno[F_LEFT] != bno )
			continue;


		side = ( al->bno[F_RIGHT] == bno )? F_RIGHT : F_LEFT;
		iside = INV_FLAG(side);

		if( (l = al->link) == NULL )	
			continue;



            if( l->Fbr == 0x04 )
		{
                  br_fill_link( l,  branch, 0x04, 1<<side );
			continue;
		} 




		if( l->Fbr == 0 )
		{
			nbno = al->bno[ iside ];

			sp_nieg_bound_add( ep, nbno, branch, nb, nb_no);

			br_fill_link( l,  branch, 0, 0x03 );

			continue;
		}





		if( l->Fbr != 0x10 )
		{
			if(  l->type != NF_N3_P )	
				continue;

			if( l->side == side )
				br_fill_link( l,  branch, 0, 0x03 );

			continue;
		}








/*
		if( l->Fbr != 0x10 )	continue;
*/



		if( l->type == NF_EDGE )
		{

			fbr = 1 << iside;
			br_fill_link( l,  branch, fbr, 0x03 );
			continue;
		}




		if( l->side != side )
		{

			fbr = 1 << iside;
			br_fill_link( l,  branch, fbr, 0x03 );

			continue; 
		}



		nbno = al->bno[ iside ];
		sp_nieg_bound_add( ep, nbno, branch, nb, nb_no);

/*
		fbr = 1 << iside;
		br_fill_link( l,  branch, fbr, 0x03 );
*/
		br_fill_link( l,  branch, 0, 0x03 );
	}//for
}





static void
sp_nieg_bound_add( ep_type *ep, int bno, int branch, int nb[], int *nb_no )
{
int	k;

	if( ep->bound[bno]->br.no == branch )	
		return;

	for( k = 0 ; k < *nb_no ; k++ )
		if( nb[k] == bno )	
			return;

	nb[ (*nb_no)++ ] = bno;
}




/**** static br_fill_link ****/
/*!
 * fill a link with layer data.
 *
 * @param *link : the link
 * @param br : branch number
 * @param fbr : the branch flag to set
 * @param Fside : side flag
 *
 * @return void  : 
 */
static void 
br_fill_link( link_type *link, int br, int fbr, int Fside )
{
link_type       *l,	*l0;



	for( l0 = link ; LINK_PREV(l0) != NULL ; l0 = LINK_PREV(l0) );

	if( fbr != 0 ){
		br_fill_linkFBR( link, F_BEGIN, br, fbr, Fside );
		return;
	}

	for( l = l0 ; l != NULL ; l = LINK_NEXT(l) ){

		l->Fbr = fbr;

		if( Fside & ( 1<<F_RIGHT) )     l->br = br;

		if( Fside & ( 1<<F_LEFT) )      l->br1 = br;
	}
}


static void 
br_fill_linkFBR( link_type *link, int direct, int br, int fbr, int Fside )
{
link_type	*l;
jnt_type	*njnt;
int	i,	nbno,	ndirect;

	if( direct == F_BEGIN ){
		for( l = link ; l != NULL ; l = LINK_NEXT(l) ){
			l->Fbr = fbr;
			l->br = br;

			if( LINK_NEXT(l) == NULL )	break;
		}
	}
	else	{
				for( l = link ; l != NULL ; l = LINK_PREV(l) ){
					l->Fbr = fbr;
					l->br = br;

					if( LINK_PREV(l) == NULL )	break;
				}
	}


	ndirect = INV_FLAG( direct );
	njnt = l->p[ndirect].p.jnt;
	nbno = l->p[ndirect].bno;

	for( i = 0 ; i < njnt->brn_no ; i++ ){
		if( i == nbno )	continue;
		if( njnt->brn[i]->link->Fbr == 0x10 )
			break;
	}

	if( i == njnt->brn_no )	return;

	if( njnt->brn[i]->direction != direct )
		fbr = ( fbr == 0x1)? 0x02 : 0x01;

	br_fill_linkFBR( njnt->brn[i]->link, njnt->brn[i]->direction, br, fbr, Fside );
}






/**** static agp_layer_FC1_nf ****/
/*!
 * The method fills all the structures in the gp_type for this ep
 *
 * @param *gp : the contour gp_type 
 * @param *baseGp : the original gp_type
 * @param branch : branch number
 * @param *ep : the ep structure with bound information
 *
 * @return void  : 
 */
static void agp_layer_FC1_nf( gp_type *gp, gp_type *baseGp, int branch, ep_type *ep )
{
bgp_type	*bgp;
link_type       *link; 
hunch_type      *h;


	// fill bgps
	for( bgp = baseGp->bgp ; bgp != NULL ; bgp = bgp->p )
		agp_layer_FC1_bgp( &baseGp->ctr, bgp, branch, ep );

	// fill hunches
	for( h = baseGp->hunch ; h != NULL ; h = h->nh)
		agp_layer_FC1_hunch( &baseGp->ctr, h, branch, ep );

	//fill links
	for( link = baseGp->link ; link != NULL ; link = link->nl ){

		if( link->br == branch && link->Fbr != 0x10 )	continue;

		// copy all elements! not only ridges and n3
		if( link->type == NF_EDGE )
			agp_layer_FC1_edge( gp, &baseGp->ctr, link, branch, ep );
		else
			agp_layer_FC1_ridge( &baseGp->ctr, link, branch, ep );
	}	
}


static void
agp_layer_FC1_bgp( vec2d *ctr, bgp_type *bgp, int branch, ep_type *ep )
{
bound_type	*b;
vec2d	v;
int	bno;
float	dis;

	v.x = ctr->x + bgp->ctr.x;
	v.y = ctr->y + bgp->ctr.y;

	if( (bno = ep_bound_point( ep, &bgp->ctr, &dis ) ) == -1 )
		return;

	b = ep->bound[bno];

	if( b->br.no == branch )
		bgp->br = branch;
}


static void
agp_layer_FC1_hunch( vec2d *ctr, hunch_type *h, int branch, ep_type *ep )
{
bound_type	*b;
int	bno;
float	dis;

	if( (bno = ep_bound_point( ep, &h->ctr, &dis ) ) == -1 )
		return;

	b = ep->bound[bno];

	if( b->br.no == branch )
		h->br = branch;
}


/**** static agp_layer_FC1_ridge ****/
/*!
 * The method sets the polylink to have layer number, if it is in the ep cube.
 *
 * @param *ctr : global center
 * @param *link : the polylink to check
 * @param branch : layer number
 * @param *ep : the ep struct
 *
 * @return void  : 
 */
static void agp_layer_FC1_ridge( vec2d *ctr, link_type *link, int branch, ep_type *ep )
{
bound_type	*b;
link_type	*l;
vec2d	v;
int	bno ;
float	dis;


	v.x = ctr->x + link->ctr.x;
	v.y = ctr->y + link->ctr.y;


	if( (bno = ep_bound_point( ep, &link->ctr, &dis ) ) == -1 )
		return;

	
	// is it the right bound?
	b = ep->bound[bno];
	if( b->br.no != branch )	
		return;


	link->br = branch;
	link->Fbr = 0;

	for( l = LINK_PREV(link) ; l != NULL ; l = LINK_PREV(l) )
	{
		l->br = branch;
		l->Fbr = 0;
	}

	for( l = LINK_NEXT(link) ; l != NULL ; l = LINK_NEXT(l) )
	{
		l->br = branch;
		l->Fbr = 0;
	}
}







/**** static agp_layer_FC1_edge ****/
/*!
 * The method tries to bound an edge. 
 * If it's center is on the contour, a special traatment is considered. 
 *
 * @param *gp : the contour gp.
 * @param *ctr : center of the link
 * @param *link : the link
 * @param branch : branch number
 * @param *ep : the ep structure
 *
 * @return void  : 
 */
static void agp_layer_FC1_edge( gp_type *gp, vec2d *ctr, link_type *link, int branch, ep_type *ep )
{
link_type	 *nl, *pl;
vec2d	v, p;
int	bno, next_bno;
float	dis;
static int ass = 0;

	v.x = ctr->x + link->ctr.x;
	v.y = ctr->y + link->ctr.y;




	if ( link->Fbr != 0x0){
		// find the corresponding link in gp, and copy data to all polylink
		for( nl = gp->link ; nl != NULL ; nl = nl->nl ){
			if ( VEC2D_EQUAL(nl->ctr, link->ctr) &&
						(nl->a == link->a) && VEC2D_EQUAL(nl->v ,link->v)){
	
				br_fill_link( link,  branch, nl->Fbr, 0/*side*/ );

				return;
			}
		}

		/*problem : didn't find the corresponding link!!!*/
		assert(nl != NULL);		
//		br_fill_link( link,  branch, 1/*Fbr*/, 0/*side*/ );
		//link->Fbr = 0;

		return;
	}



	// check if has prev. If not - it comes out of a joint
	if ( ( pl = LINK_PREV(link) ) == NULL ){

		if ( (nl = LINK_NEXT(link)) != NULL )		
			return;


		link_t2xy( &link->ctr, link, (float)(link->len * 0.1), &p );
		if( (next_bno = ep_bound_point( ep, &p, &dis ) ) == -1 )
			return;					
		if( ep->bound[next_bno]->br.no != branch )	
			return;
		br_fill_link(link,branch,0,1<<F_RIGHT);
		return;
				
	}
	
	

	
	// check if that link is in the bound anyway
	if( (bno = ep_bound_point( ep, &link->ctr, &dis ) ) == -1 )
		return;
	
	if( ep->bound[bno]->br.no != branch )	
		return;


	// the link is ok, and we can fill the whole polylink
	br_fill_link(link,branch,0,1<<F_RIGHT);

}
