/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/*****************************
 ***	AgpLayerFillN3.c   ***
 *****************************/

#include	<math.h>

#include	"GpType/GpType.h"

#include	"AgpLayer.h"


static void	n3_prime_load_jnt( jnt_type *jnt, int brn_no );


static void	agp_n3_prime_load_link( jnt_type *jnt, int bno, int side, int *br, int *Fbr );


static int	get_next_in( vec2d *ctr0, link_type *link0, in_type *in0,
			int side,
			vec2d *ctr1, link_type **link1, in_type **in1 );

static jnt_type *	get_null_jnt( jnt_type *jnt, int bno, int side );



static int	n3_prime_load_find_br( jnt_type *jnt0,
					jnt_type *jnt1, int *bno1 );


void
agp_layer_F_n3_prime_load( gp_array *agp )
{
gp_type	*gp;
jnt_type	*jnt;
int	i,	j;


	if( agp->Fn3_prime == 0 )	return;



	for( i = 0 ; i < agp->Ni ; i++ )
		for( j = 0 ; j < agp->Nj ; j++ ){
			gp = &agp->gp[i][j];

			for( jnt = gp->jnt ; jnt != NULL ; jnt = jnt->nj ){
				if( jnt->type == JNT_N3 ){
					n3_prime_load_jnt( jnt, 1 );
					continue;
				}

				if( jnt->type == JNT_RN3 ){
					n3_prime_load_jnt( jnt, jnt->brn_no );
					continue;
				}
			}
		}
}



static void
n3_prime_load_jnt( jnt_type *jnt, int brn_no )
{
brn_type	*b;
link_type	*l;
int	i,	br,	fbr,	br0,	fbr0;

	for( i = 0 ; i < brn_no ; i++ ){
		b = jnt->brn[i];
		if( b->type != NF_N3 || b->direction != F_BEGIN )
			continue;

		agp_n3_prime_load_link( jnt, i, F_LEFT, &br, &fbr );
		agp_n3_prime_load_link( jnt, i, F_RIGHT, &br0, &fbr0 );

		fbr |= fbr0;

		for( l = b->link ; l != NULL ; l = LINK_NEXT(l) ){
			l->br = br;
			l->Fbr = fbr;
		}
	}
}





static void
agp_n3_prime_load_link( jnt_type *jnt, int bno, int side, int *br, int *Fbr )
{
jnt_type	*jnt0,	*jnt1,	*ejnt;
brn_type	*b;
link_type	*link0,	*link1;
in_type	*in0,	*in1;
in_type	*in00,	*in11;
vec2d	ctr0,	ctr1;
int	i,	bno1;

	b = jnt->brn[bno];

        i = ( bno == 0 )? 1 - side : side;
        jnt0 = jnt->pj[i];

        if( jnt0 == NULL && jnt->type == JNT_N3 )
                jnt0 = jnt->brn[i+1]->pj;


	if( jnt0 == NULL )
		jnt0 = get_null_jnt( jnt, (side == F_RIGHT)?2:1, side );

	in0 = NULL;

	in00 = ( jnt->type == JNT_N3 )? jnt->brn[i+1]->in : NULL;

	link0 = b->link;
	ctr0 = jnt->gctr;

	while( get_next_in( &ctr0, link0, in0, side, &ctr1, &link1, &in1 ) ){

		jnt1 = in1->jnt->brn[in1->bno]->pj;

		if( jnt0 == jnt1 ){
			ctr0 = ctr1;
			link0 = link1;
			in0 = in1;
			jnt0 = jnt1;

			continue;
		}


		bno = n3_prime_load_find_br( jnt0, jnt1, &bno1 );

		b = jnt0->brn[bno];
		
		*br = b->link->br;
		*Fbr = b->link->Fbr;
		if( *Fbr != 0 && b->direction == F_END ) *Fbr = 3 - *Fbr;
		if( in00 != 0 )	*Fbr = 0;

		return;
	}
		


	ejnt = link1->p[F_END].p.jnt;
	bno = link1->p[F_END].bno;

        i = ( bno == 0 )? side :  1 - side;
        jnt1 = ejnt->pj[i];

        if( jnt1 == NULL && ejnt->type == JNT_N3 )
                jnt1 = ejnt->brn[i+1]->pj;


	if( jnt1 == NULL )
		jnt1 = get_null_jnt( ejnt, (side == F_RIGHT)?1:2, side );


	if( jnt0 == jnt1 )	return;


	in11 = ( ejnt->type == JNT_N3 )? ejnt->brn[i+1]->in : NULL;



	bno = n3_prime_load_find_br( jnt0, jnt1, &bno1 );

	b = jnt0->brn[bno];
		
	*br = b->link->br;
	*Fbr = b->link->Fbr;
	if( *Fbr != 0 && b->direction == F_END ) *Fbr = 3 - *Fbr;
	if( in00 != 0 )	*Fbr = 0;
}



static jnt_type *
get_null_jnt( jnt_type *jnt, int bno, int side )
{
jnt_type	*jnt0,	*tjnt;
brn_type	*b;
link_type	*l;
in_type	*in,	*p;
int	tside,	tbno;
int	i;


	b = jnt->brn[bno];


        side = ( jnt->brn[0]->direction == F_BEGIN )? 2-bno : bno-1;
        if( jnt->brn[0]->direction != b->direction )
                tside = INV_FLAG( side );
        else    tside = side;


	if( b->direction == F_END ){
		p = NULL;

		for( l = b->link ; l != NULL ; l = LINK_PREV(l) ){
			for( in = l->in[tside] ;
				in != NULL && in != b->in ; in = in->p ){
				if( in->type != IN_NULL )
					p = in;
			}

			if( p != NULL )
				return( p->jnt->brn[p->bno]->pj );

			if( LINK_PREV(l) == NULL ) break;
		}

		tbno = l->p[0].bno;
		tjnt = l->p[0].p.jnt;

/*
		if( tbno == 0 )
			jnt0 = ( tside == F_RIGHT )? tjnt->pj[1] : tjnt->pj[0];
		else	jnt0 = ( tside == F_RIGHT )? tjnt->pj[0] : tjnt->pj[1];
*/

		i = ( tbno == 0 )? 1 - tside : tside;
		jnt0 = tjnt->pj[i];

		if( jnt0 == NULL && jnt->type == JNT_N3 )
			jnt0 = tjnt->brn[i+1]->pj;


		return( jnt0 );
	}



	for( l = b->link ; l != NULL ; l = LINK_NEXT(l) ){
		in = ( l == b->link )? b->in : l->in[tside];
		for(  ; in != NULL ; in = in->p )
			if( in->type != IN_NULL )
				return( in->jnt->brn[in->bno]->pj );

		if( LINK_NEXT(l) == NULL ) break;
	}

	tbno = l->p[1].bno;
	tjnt = l->p[1].p.jnt;

/*
	if( tbno == 0 )
		jnt0 = ( tside == F_RIGHT )? tjnt->pj[0] : tjnt->pj[1];
	else	jnt0 = ( tside == F_RIGHT )? tjnt->pj[1] : tjnt->pj[0];
*/

	i = ( tbno == 0 )? tside :  1 - tside;
	jnt0 = tjnt->pj[i];

	if( jnt0 == NULL && tjnt->type == JNT_N3 )
		jnt0 = tjnt->brn[i+1]->pj;

	return( jnt0 );
}






static int
get_next_in( vec2d *ctr0, link_type *link0, in_type *in0, int side,
		vec2d *ctr1, link_type **link1, in_type **in1 )
{
link_type	*l;
in_type	*in;

	for( l = link0, *ctr1 = *ctr0 ; l != NULL  ; l = LINK_NEXT(l) ){

		for( in = (l == link0 && in0 != NULL  )? in0->p : l->in[side] ;
			in != NULL && in->type == IN_NULL ; in = in->p ) ;

		if( in != NULL ){
			*in1 = in;
			*link1 = l;
			return( 1 );
		}


		ctr1->x += l->v.x;
		ctr1->y += l->v.y;
		*link1 = l;
	}

	*in1 = NULL;
	return( 0 );

}









static int
n3_prime_load_find_br( jnt_type *jnt0, jnt_type *jnt1, int *bno1 )
{
brn_type	*b;
link_type	*l;
int	i,	direct;


	for( i = 0 ; i < jnt0->brn_no ; i++ ){
		b = jnt0->brn[i];
		if( b->link->type != NF_N3_P )	continue;

		direct = INV_FLAG(b->direction);

		for( l = b->link ; l->p[direct].bno < 0 ;
						l = l->p[direct].p.l ) ;

		if( l->p[direct].p.jnt == jnt1 ){
			*bno1 = l->p[direct].bno;
			return( i );
		}
	}

	return( -1 );
}
