/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/*******************************
 ***	AgpMiddleCountor.c   ***
 *******************************/

#include        <stdio.h>   
#include	<math.h>

#include        "AgpLayer/AgpLayer.h"

#include	"LinkTool/TlType.h"


static float	Mu = 0;



static void	agp_distance_bgp( gp_type *gp, bgp_type *bgp,
					tl_type link[], int no );

static void	agp_distance_hunch( gp_type *gp, hunch_type *h,
					tl_type link[], int no );

static void	agp_distance_link( gp_type *gp, link_type *l,
					tl_type link[], int no );


static void get_distance( tl_type link[], int no,
						  vec2d *v, int br, float *depth );



static void	agp_countor_links( gp_array *agp, tl_type tlink[], int *no,int mbr);



float
agp_layer_middle_distance( gp_array *agp, int mbr)
{
gp_type	*gp;
link_type	*l;
hunch_type	*hunch;
bgp_type	*bgp;


tl_type	link[10000];
int	i,	j,	no;

	

	Mu = 0;


	agp_countor_links( agp, link, &no,mbr);


	if( no == 0 )	return -1;

 
	for( i = 0 ; i < agp->Ni ; i++ )
	    for( j = 0 ; j < agp->Nj ; j++ ){
		gp = &agp->gp[i][j];
 
		for( bgp = gp->bgp ; bgp != NULL ; bgp = bgp->p ){
			if( !(mbr & (1<<bgp->br)) )	continue;
			agp_distance_bgp( gp, bgp, link, no );
		}

		for( hunch = gp->hunch ; hunch != NULL ; hunch = hunch->nh ){
			if( !(mbr & (1<<hunch->br)) )	continue;
			agp_distance_hunch( gp, hunch, link, no );
		}

		for( l = gp->link ; l != NULL ; l = l->nl ){
			if( !(mbr & (1<<l->br)) )	continue;
			agp_distance_link( gp, l, link, no);
		}
	}	

		return Mu;
}


static void
agp_countor_links( gp_array *agp, tl_type tlink[], int *no,int mbr)
{
gp_type *gp;
tl_type *tl;
link_type       *l;
int     i,      j;
 
 
	*no = 0;
	for( i = 0 ; i < agp->Ni ; i++ )
		for( j = 0 ; j < agp->Nj ; j++ ){
			gp = &agp->gp[i][j];
 
			for( l = gp->link ; l != NULL ; l = l->nl ){
				if( ( l->Fbr == 0 ) || !(mbr & (1<<l->br)))
  continue;
 
				tl = &tlink[(*no)];
				(*no)++;
 
				tl->ctr.x = gp->ctr.x + l->ctr.x;
				tl->ctr.y = gp->ctr.y + l->ctr.y;
				tl->l = l;
 
				tl->mbr = ( 1<<l->br) | ( 1<<l->br1);
 
			}
		}
}

//=================================================================================================

static void agp_distance_bgp( gp_type *gp, bgp_type *bgp, tl_type link[], int no )
{
	vec2d	v;
	v.x = gp->ctr.x + bgp->ctr.x;
	v.y = gp->ctr.y + bgp->ctr.y;

	get_distance( link, no, &v, bgp->br, &bgp->depth);
	
}

//=================================================================================================

static void agp_distance_hunch( gp_type *gp, hunch_type *hunch,	tl_type link[], int no )
{
	vec2d	v;
	v.x = gp->ctr.x + hunch->ctr.x;
	v.y = gp->ctr.y + hunch->ctr.y;

	get_distance( link, no, &v, hunch->br, &hunch->depth );
	
}


//=================================================================================================

/*
static void agp_distance_jnt( gp_type *gp, jnt_type *jnt, tl_type link[], int no )
{
	vec2d	v;
	int	br;

	br = jnt->brn[0]->link->br;


	v.x = gp->ctr.x + jnt->ctr.x;
	v.y = gp->ctr.y + jnt->ctr.y;

	get_distance( link, no, &v, br, &jnt->depth);
}
*/

//=================================================================================================

static void agp_distance_link( gp_type *gp, link_type *l, tl_type link[], int no )
{
	width_type	*w;
	vec2d	v,	p;
	int	br;


	br = l->br;

	v.x = gp->ctr.x + l->ctr.x;
	v.y = gp->ctr.y + l->ctr.y;

	get_distance( link, no, &v, br, &l->pr[0].p->depth );


	if( ( w = l->width[0] ) != NULL ){
		p.x = v.x + w->w[F_RIGHT] * w->bes.y;
		p.y = v.y - w->w[F_RIGHT] * w->bes.x;
		get_distance( link, no, &p, br, &w->depth[F_RIGHT] );

		p.x = v.x + w->w[F_LEFT] * w->bes.y;
		p.y = v.y - w->w[F_LEFT] * w->bes.x;
		get_distance( link, no, &p, br, &w->depth[F_LEFT] );
	}

	if( LINK_NEXT(l) != NULL )	return;

	v.x += l->v.x;
	v.y += l->v.y;

	get_distance( link, no, &v, br, &l->pr[1].p->depth );

	if( ( w = l->width[1] ) != NULL ){
		p.x = v.x + w->w[F_RIGHT] * w->bes.y;
		p.y = v.y - w->w[F_RIGHT] * w->bes.x;
		get_distance( link, no, &p, br, &w->depth[F_RIGHT] );

		p.x = v.x + w->w[F_LEFT] * w->bes.y;
		p.y = v.y - w->w[F_LEFT] * w->bes.x;
		get_distance( link, no, &p, br, &w->depth[F_LEFT] );
	}
}

//=================================================================================================

static void get_distance( tl_type link[], int no, vec2d *v, int br, float *depth )
{
	tl_type	*tl;
	float	t,	u;

	tl_array_select( link, no, v, 1<<br, &tl, &u, &t );
	if( tl == NULL )	return;

	if( u < 0 )	u = -u;

	if( u > Mu )	Mu = u;	
}

//=================================================================================================



