/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/*****************************
 ***   AgpLayerSelect.c    ***
 *****************************/

#include	<stdio.h>
#include	<math.h>

#include	"Uigp/igp.h"

#include	"GpType/GpType.h"

#include	"AgpLayer.h"


#define		 HSIZE		1


int
agp_layer_select( gp_array *agp, mbr_type Lmbr, vec2d *p, float d,
				vec2d *s_ctr, float *s_depth )
{
gp_type	*gp;
hunch_type	*h;
bgp_type	*bp;
link_type	*link;
int	i0,	j0,	i,	j;
int	layer;
vec2d	v,	ctr;
float	dis,	tmp;


	dis = d*d;

	layer = -1;
	*s_depth = 1000;

	i0 = (p->x+agp->bgp) * agp->inv_bgp;
	j0 = (p->y+agp->bgp) * agp->inv_bgp;


	for( i = i0-HSIZE ; i <= i0+HSIZE ; i++ ){
		if( i < 0 || i >= agp->Ni )     continue;

		for( j = j0-HSIZE  ; j <= j0+HSIZE ; j++ ){
			if( j < 0 || j >= agp->Nj )     continue;

			gp = &agp->gp[i][j];

			for( bp = gp->bgp ; bp != NULL ; bp = bp->p ){

//				if( Lmbr & (1<<bp->br) )	continue;
				if( MBR_IS_SET( Lmbr, bp->br ) )	continue;

				ctr.x = gp->ctr.x + bp->ctr.x;
				ctr.y = gp->ctr.y + bp->ctr.y;
				v.x = ctr.x - p->x;
				v.y = ctr.y - p->y;

				tmp = v.x*v.x + v.y*v.y;
				if( tmp > dis )	continue;
				if( bp->depth >= *s_depth )	continue;

				layer = bp->br;
				*s_ctr = ctr;
				*s_depth = bp->depth;
			}
				
				


			for( h = gp->hunch ; h != NULL ; h = h->nh ){

//				if( Lmbr & (1<<h->br) )	continue;
				if( MBR_IS_SET( Lmbr, h->br ) )	continue;

				ctr.x = gp->ctr.x + h->ctr.x;
				ctr.y = gp->ctr.y + h->ctr.y;
				v.x = ctr.x - p->x;
				v.y = ctr.y - p->y;

				tmp = v.x*v.x + v.y*v.y;
				if( tmp > dis )	continue;
				if( h->depth >= *s_depth )	continue;

				layer = h->br;
				*s_ctr = ctr;
				*s_depth = h->depth;
			}
				


			for( link = gp->link ; link != NULL ; link = link->nl ){

				if( Lmbr & (1<<link->br) )	continue;
				if( MBR_IS_SET( Lmbr, link->br ) )	continue;

				ctr.x = gp->ctr.x + link->ctr.x;
				ctr.y = gp->ctr.y + link->ctr.y;
				v.x = ctr.x - p->x;
				v.y = ctr.y - p->y;

				tmp = v.x*v.x + v.y*v.y;
				if( tmp > dis )	continue;
				if( link->pr[0].p->depth >= *s_depth )
					continue;

				layer = link->br;
				*s_ctr = ctr;
				*s_depth = link->pr[0].p->depth;
			}
		}
	}

	return( layer );
}
