/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/

/**************************
 ***   AgpBgValue.c  	***
 **************************/

#include <stdio.h>
#include <math.h>

#include	"Uigp/igp.h"
#include	"Uprm/prm.h"


#include	"EpType/EpType.h"


#include	"Ep/Ep.h"
#include	"EpBgp/EpBgp.h"
#include	"EpValue/EpValue.h"
#include	"N3Prime/N3Prime.h"







int
agp_bg_value( gp_array *agp, vec2d *p, float val[SP_NO] )
{
ep_array	*aep;
int     i0,     j0,     bno;
float   dis;
ep_type *ep;
vec2d   ctr;
int	Fn3_prime;

	val[SP_Y] = val[SP_Q] = val[SP_A] = 0;

	i0 = ( p->x + agp->bgp )* agp->inv_bgp;
	j0 = ( p->y + agp->bgp )* agp->inv_bgp;

	if( i0 < 1 || i0 >= agp->Ni-1 || j0 < 1 || j0 >= agp->Nj-1 )
		return(0);

	if( (Fn3_prime = agp->Fn3_prime) == 0 )	n3_prime_agp( agp);

	aep = aep_alloc();

	ep_create( agp, i0, j0, 1, 1, 0, aep );
	ep_bgp_get( agp, aep );



	ep = &aep->ep[LSIZE][LSIZE];

	ctr.x = p->x - (aep->V0.x + ep->ctr.x);
	ctr.y = p->y - (aep->V0.y + ep->ctr.y);

	bno = ep_bound_point( ep, &ctr, &dis );

	if( bno != -1 ){
		ep_bound_value3( &aep->epValue, ep, bno, &ctr, val );

		aep_destroy( aep );

		if( Fn3_prime == 0 )	n3_prime_free( agp);

		return( 1 );
	}

	aep_destroy( aep );

	if( Fn3_prime == 0 )	n3_prime_free( agp);
	return( -1 );
}


int
agp_bg_valueL( gp_array *agp, int layer, vec2d *p, float val[SP_NO] )
{
ep_array	*aep;
int     i0,     j0,     bno;
float   dis;
ep_type *ep;
vec2d   ctr;
int	Fn3_prime;

	val[SP_Y] = val[SP_Q] = val[SP_A] = 0;

	i0 = ( p->x + agp->bgp )* agp->inv_bgp;
	j0 = ( p->y + agp->bgp )* agp->inv_bgp;

	if( i0 < 1 || i0 >= agp->Ni-1 || j0 < 1 || j0 >= agp->Nj-1 )
		return(0);

	if( (Fn3_prime = agp->Fn3_prime) == 0 )	n3_prime_agp( agp);

	aep = aep_alloc();


	ep_create( agp, i0, j0, 1, 1, 1<<layer, aep );

	aep_br_set( aep );
	ep_bgp_get_br( agp, 1<<layer, aep );
	aep_br_fill( aep );


	ep = &aep->ep[LSIZE][LSIZE];

	ctr.x = p->x - (aep->V0.x + ep->ctr.x);
	ctr.y = p->y - (aep->V0.y + ep->ctr.y);

	bno = ep_bound_point( ep, &ctr, &dis );

	if( bno != -1 && ep->bound[bno]->br.no == layer ){
		ep_bound_value3( &aep->epValue, ep, bno, &ctr, val );

		aep_destroy( aep );

		if( Fn3_prime == 0 )	n3_prime_free( agp);

		return( 1 );
	}

	aep_destroy( aep );

	if( Fn3_prime == 0 )	n3_prime_free( agp);
	return( -1 );
}




/*
int
agp_bg_value_depth( gp_array *agp, vec2d *p, float val[SP_NO], float *depth )
{
int     i0,     j0,     bno;
float   dis;
ep_type *ep;
bound_type	*b;
vec2d   ctr;
int	i,	br;

	val[SP_Y] = val[SP_Q] = val[SP_A] = 0;
	*depth = 0;

	i0 = ( p->x + agp->bgp )* agp->inv_bgp;
	j0 = ( p->y + agp->bgp )* agp->inv_bgp;

	if( i0 < 1 || i0 >= agp->Ni-1 || j0 < 1 || j0 >= agp->Nj-1 )
		return(0);

	depth_ep_create( agp, i0, j0, 1, 1, &Aep, 1 );

	ep = &Aep.ep[LSIZE][LSIZE];

	ctr.x = p->x - (Aep.V0.x + ep->ctr.x);
	ctr.y = p->y - (Aep.V0.y + ep->ctr.y);

	br = -1;
	for( i = 0 ; i < ep->b_no ; i++ ){
		b = ep->bound[i];
		if( b->br.no == br )    continue;

		br = b->br.no;

		bno = ep_bound_point_br( ep, br, &ctr, &dis );

		if( ep->bound[bno]->br.flag != 0 )	continue;


		if( bno != -1 ){
			ep_bg_value_bound_all( ep, bno, &ctr, val );
			*depth = ep->bound[bno]->br.depth;

			return( 1 );
		}
		return(0);
	}

	return(0);
}
*/
