/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**************************
 ***	AgpBgpFill.c	***
 **************************/
#include	<stdio.h>   
#include	<math.h>

#include	"AgpTool.h"

#include	"EpType/EpType.h"
#include	"Ep/Ep.h"
#include	"EpBgp/EpBgp.h"
#include	"EpValue/EpValue.h"
#include	"N3Prime/N3Prime.h"



void
agp_bgp_fill( gp_array *agp, int layer_no )
{
ep_array	*aep;
int	i,	j,	k;
int	Fn3_prime;



	if( agp->ad == NULL ){

		if( (Fn3_prime = agp->Fn3_prime) == 0 ) n3_prime_agp( agp);

		aep = aep_alloc();

		for( i = 1 ; i < agp->Ni ; i += EP_N )
			for( j = 1 ; j < agp->Nj ; j += EP_N ){
				ep_create( agp, i, j, EP_N, EP_N, 0, aep );

				ep_bgp_get( agp, aep );

				aep_bgp_fill( agp, aep );
			}

		aep_destroy( aep );

		if( Fn3_prime == 0 )    n3_prime_free( agp );

		return;
	}

	if( layer_no >= 0 ){
		agp_bgp_fill_layer( agp, layer_no );
		return;
	}



	if( (Fn3_prime = agp->Fn3_prime) == 0 ) n3_prime_agp( agp);

	for( k = 0 ; k < agp->layer_no ; k++ )
		agp_bgp_fill_layer( agp, k );



	if( Fn3_prime == 0 )    n3_prime_free( agp );
	
}



void
agp_bgp_fill_layer( gp_array *agp, int layer_no )
{
ep_array	*aep;
layer_type	*layer;
int	mbr,	Fn3_prime,	i,	j;

	if( (Fn3_prime = agp->Fn3_prime) == 0 ) n3_prime_agp( agp);

	layer = agp->layer[layer_no];
	mbr = layer->Mbr;

	aep = aep_alloc();

	for( i = layer->i0 ; i < layer->i1 ; i += EP_N ){
		for( j = layer->j0 ; j < layer->j1 ; j += EP_N ){

			ep_create( agp, i, j, EP_N, EP_N, mbr, aep );

			aep_br_set( aep );
			ep_bgp_get_br( agp, mbr, aep );
			aep_br_fill( aep );

			aep_bgp_fill_br( agp, aep, mbr );
		}
	}

	aep_destroy( aep );

	if( Fn3_prime == 0 )    n3_prime_free( agp );
}
