/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**********************************
 ***	AgpBgpFillSmooth.c	***
 **********************************/
#include	<stdio.h>   
#include	<math.h>

#include	"AgpTool.h"

#include	"EpType/EpType.h"
#include	"Ep/Ep.h"
#include	"EpBgp/EpBgp.h"




static int	ep_bgp_fill_smooth( gp_array *agp, ep_array *aep, int *k );

static int	ep_bgp_fill_smooth_one( gp_array *agp, gp_type *gp,
					ep_type *ep );

static void	gp_bgp_add_middle_bgp( gp_array *agp, gp_type *gp,
				float x, float y, float val[], int layer );

static float	depth_plane_value( gp_array *agp, int layer, vec2d *p );


void
agp_bgp_fill_smooth( gp_array *agp, box2d *box )
{
ep_array	*aep;

int     i0,     j0,     i1,     j1;
int     i,      j,	no,	k;


	if( box != NULL ){
		i0 = ( box->x0 + agp->bgp )*agp->inv_bgp;
		j0 = ( box->y0 + agp->bgp )*agp->inv_bgp;

		i1 = ( box->x1 + agp->bgp )*agp->inv_bgp;
		j1 = ( box->y1 + agp->bgp )*agp->inv_bgp;


		if( i0 < 1 )	i0 = 1;
		if( j0 < 1 )	j0 = 1;
		if( i1 > agp->Ni-1 )	i1 = agp->Ni-1;
		if( j1 > agp->Nj-1 )	j1 = agp->Nj-1;
	}
	else	{
		i0 = 1;
		j0 = 1;
		i1 = agp->Ni-1;
		j1 = agp->Nj-1;
	}




	aep = aep_alloc();

	no = 1;
	k = 2;

	while( no > 0 && no < k ){


		no = 0;
		k = 0;

		for( i = i0 ; i < i1 ; i += EP_N )
			for( j = j0 ; j < j1 ; j += EP_N ){

				
				ep_create( agp, i, j, EP_N, EP_N, 0, aep );

				ep_bgp_get( agp, aep );


				no += ep_bgp_fill_smooth( agp, aep, &k );

			}


	}

	aep_destroy( aep );
}



static int
ep_bgp_fill_smooth( gp_array *agp, ep_array *aep, int *k )
{
gp_type         *gp;
ep_type         *ep;

int     i,      j,      i1,     j1;
int	no;


	no = 0;
        for( i = LSIZE, i1 = aep->Ni0; i < aep->Ni-LSIZE ; i++, i1++ )
                for( j = LSIZE, j1 = aep->Nj0; j < aep->Nj-LSIZE ; j++, j1++){

                        gp = &agp->gp[i1][j1];

                        ep = &aep->ep[i][j];

			if( ep->b_no > 1 || ep->al != NULL || gp->bgp != NULL )
				continue;

			(*k)++;

			no += ep_bgp_fill_smooth_one( agp, gp, ep );
                }

	return( no );
}




static int
ep_bgp_fill_smooth_one( gp_array *agp, gp_type *gp, ep_type *ep )
{
ep_type	*nep;
int	i,	n,	br;
bound_type	*b;
ebgp_type	*bgp;
float	val[SP_NO],	w,	hb;


	w = 0;
	val[SP_Y] = val[SP_Q] = val[SP_A] = w = 0;

	b = ep->bound[0];
	for( n = 0 ; n < 9 ; n++ ){

		if( n == 4 )	continue;
		if( b->Fcont[n] == 0 )  continue;

		nep = ep->ng[n];

		for( i = 0 ; i < nep->b_no ; i++ ){

			if( !(b->Fcont[n] & (1<<i ) ) ) continue;
			if( (bgp = nep->bound[i]->bgp) == NULL)	continue;
			val[SP_Y] += bgp->val[SP_Y];
			val[SP_Q] += bgp->val[SP_Q];
			val[SP_A] += bgp->val[SP_A];
/*
			br = bgp->br;
*/
			br = nep->bound[i]->br.no;
			w++;
		}
	}

	if( w == 0 )	return( 0 );
	val[SP_Y] /= w;
	val[SP_Q] /= w;
	val[SP_A] /= w;

	hb = 0.25*agp->bgp;
	gp_bgp_add_middle_bgp( agp, gp, -hb, -hb, val, br );
	gp_bgp_add_middle_bgp( agp, gp, -hb,  hb, val, br );
	gp_bgp_add_middle_bgp( agp, gp,  hb, -hb, val, br );
	gp_bgp_add_middle_bgp( agp, gp,  hb,  hb, val, br );

	return( 1 );
}


static void
gp_bgp_add_middle_bgp( gp_array *agp, gp_type *gp,
			float x, float y, float val[], int layer )
{
bgp_type	*bp;
vec2d	p;
 
 
	bp = BGP_ALLOC();

	bp->ctr.x = x;
	bp->ctr.y = y;

	bp->val[0] = val[0];
	bp->val[1] = val[1];
	bp->val[2] = val[2];

	bp->br = layer;

	p.x = gp->ctr.x + bp->ctr.x;
	p.y = gp->ctr.y + bp->ctr.y;

	bp->depth = depth_plane_value( agp, layer, &p );

	bp->p = gp->bgp;
	gp->bgp = bp;
}





static float
depth_plane_value( gp_array *agp, int layer, vec2d *p )
{
view_type	*vw;
vec3d	N;
vec2d	v;
float	t,	value,	D;
vec3d	n,	p3;
 
	if( agp->ad == NULL )   return( 0 );
 
	if( adepth_get_plane( agp->ad, layer, &N, &D ) == 0 )   return( 0 );

	vw = agp->ad->vw;


	v.x = vw->Ua_inv*(p->y - vw->Ub);
	v.y = vw->Va_inv*(p->x - vw->Vb);
 
 
	view_3d_ray( vw, v.x, v.y, &p3, &n );
	t = 1.0 / VEC3D_NORM( n );
	n.x *= t;
	n.y *= t;
	n.z *= t;
 
	t = ( D - VEC3D_INNER( vw->cp, N ) ) / VEC3D_INNER( n, N );
 
	value = t* VEC3D_INNER( n, vw->N );
 
	return( value );
}
