/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**************************
 ***	AgpTestBgp.c	***
 **************************/
#include	<stdio.h>   
#include	<math.h>

#include	"AgpTool.h"

#include	"EpType/EpType.h"
#include	"Ep/Ep.h"
#include	"EpBgp/EpBgp.h"
#include	"EpValue/EpValue.h"
#include	"N3Prime/N3Prime.h"

static int	agp_bgp_test_layer( gp_array *agp, int iLayer );

static int	agp_bgp_test_layer_aep1( ep_array *aep );

static int	agp_bgp_test_layer_ep1( ep_type *ep );



int
agp_bgp_test( gp_array *agp, int iLayer )
{
int	k;
int	Fn3_prime;

	if( (agp->type & AGP_TRANSPARENT ) == 0 )	return( 1 );

	if( iLayer >= 0 ){
		if( agp_bgp_test_layer( agp, iLayer ) < 0 )
			return( -1 );
		return( 1 );
	}



	if( (Fn3_prime = agp->Fn3_prime) == 0 ) n3_prime_agp( agp);

	for( k = 0 ; k < agp->layer_no ; k++ )
		if( agp_bgp_test_layer( agp, k ) < 0 ){
			if( Fn3_prime == 0 )    n3_prime_free( agp );
			return( -1 );
		}


	if( Fn3_prime == 0 )    n3_prime_free( agp );

	return( 1 );
}



static int
agp_bgp_test_layer( gp_array *agp, int iLayer )
{
ep_array	*aep;
layer_type	*layer;
int	mbr,	Fn3_prime,	i,	j;

	if( (Fn3_prime = agp->Fn3_prime) == 0 ) n3_prime_agp( agp);

	layer = agp->layer[iLayer];
	mbr = layer->Mbr;

	aep = aep_alloc();

	for( i = layer->i0 ; i < layer->i1 ; i += EP_N ){
		for( j = layer->j0 ; j < layer->j1 ; j += EP_N ){

			ep_create( agp, i, j, EP_N, EP_N, mbr, aep );

			aep_br_set( aep );
			ep_bgp_get_br( agp, mbr, aep );

			if( agp_bgp_test_layer_aep1( aep ) < 0 ){
				aep_destroy( aep );
				if( Fn3_prime == 0 )    n3_prime_free( agp );
				return( -1 );
			}
		}
	}

	aep_destroy( aep );

	if( Fn3_prime == 0 )    n3_prime_free( agp );

	return( 1 );
}



static int
agp_bgp_test_layer_aep1( ep_array *aep )
{
ep_type		*ep;

int	i,	j;


	for( i = LSIZE ; i < aep->Ni-LSIZE ; i++ )
		for( j = LSIZE ; j < aep->Nj-LSIZE ; j++ ){

			ep = &aep->ep[i][j];

			if( agp_bgp_test_layer_ep1( ep ) < 0 )
				return( -1 );
		}

	return( 1 );
}

static int
agp_bgp_test_layer_ep1( ep_type *ep )
{
bound_type	*b;
al_type	*al;
int	i;

	for( al = ep->al ; al != NULL ; al = al->nal )
		if( al->link->Fbr != 0 )	break;
	

	if( al == NULL )	return( 1 );


	for( i = 0 ; i < ep->b_no ; i++ ){
		b = ep->bound[i];
		if( b->br.flag != 0 )	continue;


		if( b->bgp != NULL )
			return( -1 );
	}

	return( 1 );
}



/*
static void
ep_br_fill( ep_type *ep )
{
ep_type	*nep;
bound_type	*b;
int	i,	n,	k;


	for( i = 0 ; i < ep->b_no ; i++ ){
		b = ep->bound[i];
		if( b->br.no != 255 )	continue;

		for( n = 0 ; n < 9 ; n++ ){

			if( b->Fcont[n] == 0 )  continue;

			nep = ep->ng[n];

			for( k = 0 ; k < nep->b_no ; k++ ){

				if( !(b->Fcont[n] & (1<<k ) ) )
					continue;
				if( nep->bound[k]->br.no == 255 )
					continue;

				b->br.no = nep->bound[k]->br.no;
				break;
			}

			if( b->br.no != 255 )	break;
		}
	}
}
*/