/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/*************************
 ***	AgpCheckIn.c   ***
 *************************/
#include	<stdio.h>

#include	"GpType/GpType.h"
#include	"GpType/JntType.h"
#include	"GpType/LinkType.h"



static int	agp_check_in_one( jnt_type *jnt, int bno );

static int	link_find_in( link_type *link, int direction, int side, in_type *in0, in_type *fin );



int
agp_check_in( gp_array *agp, vec2d *p )
{
int     i,      j;
gp_type *gp;   
jnt_type	*jnt;

	for( i = 0 ; i < agp->Ni ; i++ ){

		for( j = 0 ; j < agp->Nj ; j++ ){

			gp = &agp->gp[i][j];

			for( jnt = gp->jnt ; jnt != NULL ; jnt = jnt->nj ){

				if( jnt->type != JNT_N3 )    continue;

				if( jnt->brn[1]->type == BRN_IN ){
					if( agp_check_in_one( jnt, 1 ) < 0 ){
						if( p != NULL )
							*p = jnt->gctr;
						return( -1 );
					}
				}
				
				if( jnt->brn[2]->type == BRN_IN ){
					if( agp_check_in_one( jnt, 2 ) < 0 ){
						if( p != NULL )
							*p = jnt->gctr;
						return( -1 );
					}
				}
			}
		}
	}

	return( 1 );
}



static int
agp_check_in_one( jnt_type *jnt, int bno )
{
jnt_type *jnt1,	*jnt2;
int	bno1,	bno2,	side,	nside;
int	ret;

	side = ( jnt->brn[0]->direction == F_BEGIN )? 2 - bno : bno - 1;

	nside = ( jnt->brn[0]->direction == jnt->brn[bno]->direction )? side : 1 - side;


	// first chcek
	link_get_jnt_and_brn_no( jnt->brn[0]->link, INV_FLAG(jnt->brn[0]->direction), &jnt1, &bno1 );
	bno1 = 3 - bno;
	if( jnt1->type == JNT_N3 && jnt1->brn[bno1]->type == BRN_IN ){
		ret = link_find_in( jnt->brn[bno]->link, INV_FLAG(jnt->brn[bno]->direction), nside,
				jnt->brn[bno]->in,
				jnt1->brn[bno1]->in );

		if( ret > 0 )
			return( ret );
	}



	// second check;
	link_get_jnt_and_brn_no( jnt->brn[bno]->link, jnt->brn[bno]->direction, &jnt2, &bno2 );

	bno2 = ( jnt2->brn[0]->direction == F_BEGIN )? 2 - nside : 1 + nside;

	if( jnt2->type == JNT_N3 && jnt2->brn[bno2]->type == BRN_IN ){
		ret = link_find_in( jnt->brn[0]->link, jnt->brn[0]->direction, side,
								NULL,
								jnt2->brn[bno2]->in );

		if( ret > 0 )
			return( ret );
	}

	return( -1 );
}



static int
link_find_in( link_type *link, int direction, int side, in_type *in0, in_type *fin )
{
link_type	*l;
in_type	*in;

	if( direction == F_BEGIN ){
		if ( in0 == NULL)
			in0 = link->in[side];
		for( in = in0 ; in != NULL ; in = in->p )
			if( in == fin )	return( 1 );

		for( l = LINK_NEXT(link) ; l != NULL ; l = LINK_NEXT(l) ){
			for( in = l->in[side] ; in != NULL ; in = in->p )
				if( in == fin )	return( 1 );
		}

		return( -1 );
	}



	for( l = link ; l != NULL ; l = LINK_PREV(l) ){
		for( in = l->in[side] ; in != NULL && in != in0; in = in->p )
			if( in == fin )	return( 1 );
	}

	return( -1 );
}

