/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/*************************
 ***	AgpCleare.c   ***
 *************************/
#include	<stdio.h>   
#include	<math.h>

#include	"GpType/GpType.h"

#include	"N3Prime/N3Prime.h"

#include	"Ep/Ep.h"
#include	"EpBgp/EpBgp.h"
#include	"EpValue/EpValue.h"

#include	"AgpTool.h"




static void	agp_copy_layer( gp_array *agp, gp_array *nagp );

static void	agp_mold_jnt( vec2d *ctr, jnt_type *jnt, gp_array *agp );



static jnt_type *	jnt_copy_spr( jnt_type *jnt );
static void	agp_cleare_jnt_link( gp_array *agp, jnt_type *jnt );


static void	agp_cleare_link_color( link_type *link );


static void	agp_cleare_layer( gp_array *agp,
				gp_array *nagp, int layer_no );

static void	agp_cleare_bgp( gp_array *agp, ep_array *aep, int mbr,
						ep_array *baep );

static void	agp_cleare_bgp_one( gp_array *agp, gp_type *gp, ep_type *ep,
					int mbr, ep_type *bep, float hb, epValue_type *epValue );

static void	agp_cleare_bgp_add( gp_array *agp, gp_type *gp, int mbr,
					float x, float y,
					ep_type *ep,
					ep_type *bep, epValue_type *epValue );

static float	depth_plane_value( gp_array *agp, int layer, vec2d *p );


//Grisha 03.01.00...
static int agp_cleare_middle_bgp( gp_array *agp, int mbr, ep_array *aep, float valMid[3] );
static int agp_cleare_bgp_val_add( int mbr, float x, float y, ep_type *ep,
								  float valMid[3], epValue_type *epValue );
static void agp_cleare_layer_color( gp_array *agp, int layer_no );
static void agp_cleare_link_color1( gp_array *agp, vec2d *gctr, link_type *l );

float *g_ColorR, *g_ColorG, *g_ColorB;
int g_Layer_cur = 0, g_Layers_No = 0;

#define MOLD_COLOR_SAVE_ALL_BGP 1 //save all bgp
#define MOLD_COLOR_MONO_LAYERS  2 //mono layer
int g_MoldColorInLayers = MOLD_COLOR_SAVE_ALL_BGP;
//...Grisha 03.01.00

gp_array *
agp_make_cleare( gp_array *agp, int moldColorInLayers )
{
gp_array	*nagp;
gp_type		*gp;
jnt_type	*jnt;
link_type	*link;
int	i,	j;
int	Fn3_prime;

	if ( !agp || !agp->layer_no )
		return NULL;

	//Grisha 03.01.00...
	g_MoldColorInLayers = moldColorInLayers;
	g_Layers_No = agp->layer_no;
	g_ColorR = malloc( sizeof(float) * agp->layer_no); 
	g_ColorG = malloc( sizeof(float) * agp->layer_no);
	g_ColorB = malloc( sizeof(float) * agp->layer_no); 
	//...Grisha 03.01.00

	nagp = agp_alloc();


	agp_clear( nagp, agp->bgp, agp->row, agp->column, agp->Lni, agp->Lnj );

	nagp->type = agp->type;
	nagp->color_type = agp->color_type;
	nagp->Lcoding = agp->Lcoding;


	nagp->ad = adepth_copy( agp->ad );
	agp_copy_layer( agp, nagp );



	if( (Fn3_prime = agp->Fn3_prime) == 0 )
		n3_prime_agp( agp);

  
	//Grisha 03.01.00...
	if( g_MoldColorInLayers == MOLD_COLOR_MONO_LAYERS )
	for( i = 0 ; i < agp->layer_no ; i++ )
		agp_cleare_layer_color( agp, i ); //Calc middle color of layers g_ColorR, g_ColorG, g_ColorB
	//...Grisha 03.01.00


	/* copy nf and jnt  */
	for( i = 0 ; i < agp->Ni ; i++ )
		for( j = 0 ; j < agp->Nj ; j++ ){
			gp = &agp->gp[i][j];

			for( jnt = gp->jnt ; jnt != NULL ; jnt = jnt->nj )
				agp_mold_jnt( &gp->ctr, jnt, nagp );
		}


	for( i = 0 ; i < nagp->Ni ; i++ )
		for( j = 0 ; j < nagp->Nj ; j++ ){
			gp = &nagp->gp[i][j];
			for( jnt = gp->jnt ; jnt != NULL ; jnt = jnt->nj )
				agp_cleare_jnt_link( nagp, jnt );
		}


	for( i = 0 ; i < agp->layer_no ; i++ )
		agp_cleare_layer( agp, nagp, i );

	//Grisha 03.01.00	
	//Set color of Links eqwil color of near bgp
	if ( g_MoldColorInLayers == MOLD_COLOR_SAVE_ALL_BGP )
	for( i = 0 ; i < nagp->Ni ; i++ )
		for( j = 0 ; j < nagp->Nj ; j++ ){
			gp = &nagp->gp[i][j];
			for( link = gp->link ; link != NULL ; link = link->nl )
				agp_cleare_link_color1( nagp, &gp->ctr, link );
		}
	//Grisha 03.01.00	

	if( Fn3_prime == 0 )    n3_prime_free( agp);


	agp_connect_jnt( nagp );

	//Grisha 03.01.00...
	free( g_ColorR ); 
	free( g_ColorG ); 
	free( g_ColorB ); 
	//...Grisha 03.01.00

	return( nagp );
}






static void
agp_copy_layer( gp_array *agp, gp_array *nagp )
{
layer_type	*layer,	*nlayer;
int	i;

	nagp->layer_no = 0;
	for( i = 0 ; i < agp->layer_no ; i++ ){
		layer = agp->layer[i];
		nlayer = nagp->layer[nagp->layer_no++] = LAYER_ALLOC();

		*nlayer = *layer;
	}
}








static void
agp_mold_jnt( vec2d *ctr, jnt_type *jnt, gp_array *agp )
{
gp_type	*gp;
jnt_type	*njnt;
int	Fbr;
int	i,	j;

	if( jnt->type != JNT_EDGE && jnt->type != JNT_N3_P )
		return;

	for( i = 0, Fbr = 0 ; i < jnt->brn_no ; i++ )
		if( (Fbr = jnt->brn[i]->link->Fbr) != 0 )	break;

	if( Fbr == 0 )	return;


	njnt = jnt->cjnt = jnt_copy_spr( jnt );

	njnt->type = JNT_EDGE;


	i = (int)((njnt->gctr.x+agp->bgp) * agp->inv_bgp);
	j = (int)((njnt->gctr.y+agp->bgp) * agp->inv_bgp);

	gp = &agp->gp[i][j];


	njnt->ctr.x = njnt->gctr.x - gp->ctr.x;
	njnt->ctr.y = njnt->gctr.y - gp->ctr.y;
	njnt->nj = gp->jnt;
	gp->jnt = njnt;
}


static jnt_type *
jnt_copy_spr( jnt_type *jnt )
{
jnt_type	*njnt;
int	k;

	njnt = jnt_alloc();
	*njnt = *jnt;

	for( k = 0, njnt->brn_no = 0 ; k < jnt->brn_no ; k++ ){
		if( jnt->brn[k]->link->Fbr == 0 )	continue;
		njnt->brn[njnt->brn_no] = brn_alloc();
		*njnt->brn[njnt->brn_no] = *jnt->brn[k];
		njnt->brn_no++;
	}

	return( njnt );
}






static void
agp_cleare_jnt_link( gp_array *agp, jnt_type *jnt )
{
jnt_type	*njnt;
brn_type	*b,     *nb;
link_type       *link,  *l,	*l1;
vec2d   ctr;
int     k,	j;

	for( k = 0 ; k < jnt->brn_no ; k++ ){
		b = jnt->brn[k];

		if( b->direction != F_BEGIN )   continue;

		for( l1 = b->link ; LINK_NEXT(l1 ) != NULL ; l1 =LINK_NEXT(l1));


		link = link_make_copy( b->link, b->type );
		//Grisha 03.01.00 
		if( g_MoldColorInLayers == MOLD_COLOR_MONO_LAYERS )
			agp_cleare_link_color( link );


		b->type = NF_EDGE;
		b->link = link;
		b->link->p[F_BEGIN].p.jnt = jnt;
		b->link->p[F_BEGIN].bno = k;


		for( l = link ; LINK_NEXT(l ) != NULL ; l = LINK_NEXT(l ) )
			l->type = NF_EDGE;

		l->type = NF_EDGE;

		njnt = l->p[F_END].p.jnt->cjnt;

		for( j = 0 ; j < njnt->brn_no ; j ++ ){
			nb = njnt->brn[j];
			if( nb->link == l1 )	break;
		}

		nb->link = l;
		nb->link->p[F_END].p.jnt = njnt;
		nb->link->p[F_END].bno = j;
		nb->type = NF_EDGE;


		ctr.x = jnt->gctr.x + b->ctr.x;
		ctr.y = jnt->gctr.y + b->ctr.y;
		agp_lnf_link( &ctr, b->link, agp );
	}
}


/*Grisha 02.01.00*/
static void
agp_cleare_link_color( link_type *link )
{
link_type	*l;
edge_type	*e;
int			side, layer, layer1;

	if( link->Fbr != 0x04 ){
		side = ( link->Fbr & 0x01 )? F_LEFT : F_RIGHT;
		for( l = link ; l != NULL ; l = LINK_NEXT(l) ){

			layer = (l->br-1)%g_Layers_No;
			layer1 = (l->br1-1)%g_Layers_No;
			e = l->pr[0].edge;

			e->side[side].bval[SP_R] = g_ColorR[layer];
			e->side[side].bval[SP_G] = g_ColorG[layer];
			e->side[side].bval[SP_B] = g_ColorB[layer];

			if( LINK_NEXT(l) == NULL )	break;
		}

		e = l->pr[1].edge;

		e->side[side].bval[SP_R] = g_ColorR[layer];
		e->side[side].bval[SP_G] = g_ColorG[layer];
		e->side[side].bval[SP_B] = g_ColorB[layer];

		return;
	}


	for( l = link ; l != NULL ; l = LINK_NEXT(l) ){
		layer = (l->br-1)%g_Layers_No;
		layer1 = (l->br1-1)%g_Layers_No;
		e = l->pr[0].edge;

		e->side[F_RIGHT].bval[SP_R] = g_ColorR[layer];
		e->side[F_RIGHT].bval[SP_G] = g_ColorG[layer];
		e->side[F_RIGHT].bval[SP_B] = g_ColorB[layer];

		e->side[F_LEFT].bval[SP_R] = g_ColorR[layer1];
		e->side[F_LEFT].bval[SP_G] = g_ColorG[layer1];
		e->side[F_LEFT].bval[SP_B] = g_ColorB[layer1];
	}

	e = l->pr[1].edge;

	e->side[F_RIGHT].bval[SP_R] = g_ColorR[layer];
	e->side[F_RIGHT].bval[SP_G] = g_ColorG[layer];
	e->side[F_RIGHT].bval[SP_B] = g_ColorB[layer];


	e->side[F_LEFT].bval[SP_R] = g_ColorR[layer1];
	e->side[F_LEFT].bval[SP_G] = g_ColorG[layer1];
	e->side[F_LEFT].bval[SP_B] = g_ColorB[layer1];
}
/*Grisha 02.01.00*/



static void
agp_cleare_layer( gp_array *agp, gp_array *nagp, int layer_no )
{
ep_array	*aep,	*naep;
layer_type  *layer;
int			mbr,    i,      j;

	g_Layer_cur = layer_no;
	layer = agp->layer[layer_no];
	mbr = layer->Mbr;

	aep = aep_alloc();
	naep = aep_alloc();

	for( i = layer->i0 ; i < layer->i1 ; i += EP_N ){
		for( j = layer->j0 ; j < layer->j1 ; j += EP_N ){

			ep_create( agp, i, j, EP_N, EP_N, mbr, aep );


			aep_br_set( aep );
			ep_bgp_get_br( agp, mbr, aep );
			aep_br_fill( aep );


			ep_create( nagp, i, j, EP_N, EP_N, mbr, naep );

			agp_cleare_bgp( nagp, naep, mbr, aep );

		}
	}

	aep_destroy( aep );
	aep_destroy( naep );

}


static void
agp_cleare_bgp( gp_array *agp, ep_array *aep, int mbr, ep_array *baep )
{
int     i,      j,      i1,     j1;
gp_type *gp;
ep_type *ep,	*bep;


	for( i = LSIZE, i1 = aep->Ni0 ; i < aep->Ni-LSIZE ; i++, i1++ ){
		if( i1 < 0 || i1 >= agp->Ni-1 )       continue;

		for( j = LSIZE, j1 = aep->Nj0 ; j < aep->Nj-LSIZE ; j++, j1++ ){			
			if( j1 < 0 || j1 >= agp->Nj-1 )   continue;

			gp = &agp->gp[i1][j1];
			ep = &aep->ep[i][j];
			bep = &baep->ep[i][j];


			agp_cleare_bgp_one( agp, gp, ep, mbr, bep, aep->hbgp, &aep->epValue );
		}
	}
}




static void
agp_cleare_bgp_one( gp_array *agp, gp_type *gp,
				ep_type *ep, int mbr, ep_type *bep, float hbgp, epValue_type *epValue )
{
int			bno,    bgp_in[4];
float   hb = hbgp *0.5F;

	bno = ep_middle_bgp( bep, bgp_in, hbgp );
	if( bno < 0 )   return;



	if( bgp_in[0] == 1 )
		agp_cleare_bgp_add( agp, gp, mbr, -hb, -hb, ep, bep, epValue );


	if( bgp_in[1] == 1 )
		agp_cleare_bgp_add( agp, gp, mbr, -hb, +hb, ep, bep, epValue );


	if( bgp_in[3] == 1 )
		agp_cleare_bgp_add( agp, gp, mbr, +hb, -hb, ep, bep, epValue );


	if( bgp_in[2] == 1 )
		agp_cleare_bgp_add( agp, gp, mbr, +hb, +hb, ep, bep, epValue );

}


static void
agp_cleare_bgp_add( gp_array *agp, gp_type *gp, int mbr,
					float x, float y,
					ep_type *ep,
					ep_type *bep, epValue_type *epValue )
{
bgp_type	*bp;
int	bno,	br;
float	dis;
vec2d   p;
float	val[SP_NO];


	p.x = x;
	p.y = y;

	bno = ep_bound_point( bep, &p, &dis );
	br = bep->bound[bno]->br.no;

	if(  255 || !(mbr & (1<< br ) ) )
	if( br == 255 || !(mbr & (1<< br ) ) )	return;
	
	

	bp = BGP_ALLOC();
	bp->ctr.x = x;
	bp->ctr.y = y;

	bp->br = br;

	//Grisha 02.01.00 ...
	if ( bno != -1 && g_MoldColorInLayers == MOLD_COLOR_SAVE_ALL_BGP ){
		ep_bound_value3( epValue, bep, bno, &bp->ctr, val );
		bp->val[0] = F2QCOLOR(val[0]);
		bp->val[1] = F2QCOLOR(val[1]);
		bp->val[2] = F2QCOLOR(val[2]);
	}
	else
	{
		bp->val[SP_R] = g_ColorR[g_Layer_cur];
		bp->val[SP_G] = g_ColorG[g_Layer_cur];
		bp->val[SP_B] = g_ColorB[g_Layer_cur];
	}
	//... Grisha 02.01.00

	p.x = gp->ctr.x + bp->ctr.x;
	p.y = gp->ctr.y + bp->ctr.y;
	bp->depth = depth_plane_value( agp, br, &p );


	bp->p = gp->bgp;
	gp->bgp = bp;
}



static float
depth_plane_value( gp_array *agp, int layer, vec2d *p )
{
view_type       *vw;
vec3d   N;
vec2d   v;
float   t,      value,  D;
vec3d   n,      p3;
 
	if( agp->ad == NULL )   return( 0 );
 
	if( adepth_get_plane( agp->ad, layer, &N, &D ) == 0 )   return( 0 );
 
	vw = agp->ad->vw;
	 
 
	v.x = vw->Ua_inv*(p->y - vw->Ub);
	v.y = vw->Va_inv*(p->x - vw->Vb);
 
 
	view_3d_ray( vw, v.x, v.y, &p3, &n );
	t = (float)(1.0 / VEC3D_NORM( n ));
	n.x *= t;
	n.y *= t;
	n.z *= t;
 
	t = ( D - VEC3D_INNER( vw->cp, N ) ) / VEC3D_INNER( n, N );
 
	value = t* VEC3D_INNER( n, vw->N );
 
	return( value );
}

static void
agp_cleare_layer_color( gp_array *agp, int layer_no )
{
ep_array	*aep;
layer_type  *layer;
int			mbr, i, j, n = 0;
float		valMid[SP_NO] = { 0.0, 0.0, 0.0 };

	layer = agp->layer[layer_no];
	mbr = layer->Mbr;

	aep = aep_alloc();

	for( i = layer->i0 ; i < layer->i1 ; i += EP_N ){
		for( j = layer->j0 ; j < layer->j1 ; j += EP_N ){

			ep_create( agp, i, j, EP_N, EP_N, mbr, aep );


			aep_br_set( aep );
			ep_bgp_get_br( agp, mbr, aep );
			aep_br_fill( aep );


			n += agp_cleare_middle_bgp( agp, mbr, aep, valMid ); //Grisha 03.01.00
		}
	}

	aep_destroy( aep );

	g_ColorR[layer_no] = valMid[SP_R]/(n ? n : 1);
	g_ColorG[layer_no] = valMid[SP_G]/(n ? n : 1);
	g_ColorB[layer_no] = valMid[SP_B]/(n ? n : 1);
}

static int
agp_cleare_middle_bgp( gp_array *agp, int mbr, ep_array *aep, float valMid[SP_NO] )
{
ep_type *ep;
float   hb = agp->bgp *0.25F;
int     i, j, i1, j1, n = 0;
int		bno,    bgp_in[4];

	for( i = LSIZE, i1 = aep->Ni0 ; i < aep->Ni-LSIZE ; i++, i1++ ){
		if( i1 < 0 || i1 >= agp->Ni-1 )       continue;

		for( j = LSIZE, j1 = aep->Nj0 ; j < aep->Nj-LSIZE ; j++, j1++ ){			
			if( j1 < 0 || j1 >= agp->Nj-1 )   continue;

			//gp = &agp->gp[i1][j1];
			ep = &aep->ep[i][j];


			//agp_mold_bgp_one( agp, gp, ep, mbr, bep, hb );

			bno = ep_middle_bgp( ep, bgp_in, aep->hbgp );
			if( bno < 0 )   continue;

			if( bgp_in[0] == 1 )
				n += agp_cleare_bgp_val_add( mbr, -hb, -hb, ep, valMid, &aep->epValue );

			if( bgp_in[1] == 1 )
				n += agp_cleare_bgp_val_add( mbr, -hb, +hb, ep, valMid, &aep->epValue );

			if( bgp_in[3] == 1 )
				n += agp_cleare_bgp_val_add( mbr, +hb, -hb, ep, valMid, &aep->epValue );

			if( bgp_in[2] == 1 )
				n += agp_cleare_bgp_val_add( mbr, +hb, +hb, ep, valMid, &aep->epValue );
		}
	}
	return n;
}

static int
agp_cleare_bgp_val_add( int mbr, float x, float y,
					   ep_type *ep, float valMid[SP_NO], epValue_type *epValue )
{
int	bno,	br;
float		dis;
vec2d		p;
float		val[SP_NO];


	p.x = x;
	p.y = y;

	bno = ep_bound_point( ep, &p, &dis );
	br = ep->bound[bno]->br.no;

	if(  255 || !(mbr & (1<< br ) ) )
	if( br == 255 || !(mbr & (1<< br ) ) )	
		return 0;

	if ( bno == -1 )
		return 0;

	ep_bound_value3( epValue, ep, bno, &p, val );

	valMid[SP_R] += val[SP_R];
	valMid[SP_G] += val[SP_G];
	valMid[SP_B] += val[SP_B];

	return 1;
}



static void
link_br_color( gp_array *agp, int br, vec2d *ctr, float val[SP_NO] )
{//get color of near bgp
gp_type		*gp;
bgp_type	*bp;
int			i, j;
float		dd = 1E6, dist;
vec2d		bpCtr;

	val[SP_R] = val[SP_G] = val[SP_B] = 0.0F;
	for( i = 0 ; i < agp->Ni ; i++ )
		for( j = 0 ; j < agp->Nj ; j++ )
		{
			gp = &agp->gp[i][j];
			for( bp = gp->bgp ; bp != NULL ; bp = bp->p )
			if ( bp->br == br )
			{
				bpCtr.x = gp->ctr.x + bp->ctr.x;
				bpCtr.y = gp->ctr.y + bp->ctr.y;
				dist = (bpCtr.x-ctr->x)*(bpCtr.x-ctr->x) + (bpCtr.y-ctr->y)*(bpCtr.y-ctr->y);
				if ( dist < dd )
				{
					dd = dist;
					val[SP_R] = bp->val[SP_R];
					val[SP_G] = bp->val[SP_G];
					val[SP_B] = bp->val[SP_B];
				}
			}
		}

}

static void
agp_cleare_link_color1( gp_array *agp, vec2d *gctr, link_type *l )
{
edge_type	*e;
int			side;
float		linkRGB[SP_NO], linkRGB1[SP_NO];
vec2d		ctr;


	ctr.x = gctr->x + l->ctr.x;
	ctr.y = gctr->y + l->ctr.y;
	link_br_color( agp, l->br, &ctr, linkRGB );//get color of near bgp

	if( l->Fbr != 0x04 ){

		side = ( l->Fbr & 0x01 )? F_LEFT : F_RIGHT;
		e = l->pr[0].edge;
		e->side[side].bval[SP_R] = linkRGB[SP_R]; 
		e->side[side].bval[SP_G] = linkRGB[SP_G];
		e->side[side].bval[SP_B] = linkRGB[SP_B];

		if ( LINK_NEXT(l) == NULL )
		{
			e = l->pr[1].edge;
			e->side[side].bval[SP_R] = linkRGB[SP_R];
			e->side[side].bval[SP_G] = linkRGB[SP_G];
			e->side[side].bval[SP_B] = linkRGB[SP_B];
		}
		return;
	}


	ctr.x = gctr->x + l->ctr.x + l->v.x;
	ctr.y = gctr->y + l->ctr.y + l->v.y;
	link_br_color( agp, l->br1, &ctr, linkRGB1 );//get color of near bgp

	e = l->pr[0].edge;
	e->side[F_RIGHT].bval[SP_R] = linkRGB[SP_R];
	e->side[F_RIGHT].bval[SP_G] = linkRGB[SP_G];
	e->side[F_RIGHT].bval[SP_B] = linkRGB[SP_B];

	e->side[F_LEFT].bval[SP_R] = linkRGB1[SP_R];
	e->side[F_LEFT].bval[SP_G] = linkRGB1[SP_G];
	e->side[F_LEFT].bval[SP_B] = linkRGB1[SP_B];
	
	if ( LINK_NEXT(l) == NULL )
	{
		e = l->pr[1].edge;
		e->side[F_RIGHT].bval[SP_R] = linkRGB[SP_R];
		e->side[F_RIGHT].bval[SP_G] = linkRGB[SP_G];
		e->side[F_RIGHT].bval[SP_B] = linkRGB[SP_B];

		e->side[F_LEFT].bval[SP_R] = linkRGB1[SP_R];
		e->side[F_LEFT].bval[SP_G] = linkRGB1[SP_G];
		e->side[F_LEFT].bval[SP_B] = linkRGB1[SP_B]; 
	}
}

