/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
#include	<math.h>

#include	"Uigp/igp.h"
#include	"GpType/GpType.h"
#include	"AgpEdit/AgpEditTool.h"

#include	"AgpTool/AgpTool.h"

#define	GOOD_DELTA_PROFILE	16
#define	L_SIZE		5


static float	agp_find_jnt_near_jnt( gp_array *agp, jnt_type *jnt0, 
					   jnt_type **s_jnt, float *delta_profile );

static float	agp_find_link_near_jnt( gp_array *agp, jnt_type *jnt0, vec2d *s_ctr, 
					    link_type **s_link, vec2d *s_pointOnLink );

static link_type *	agp_link_from_2jnt( gp_array *agp, jnt_type *jnt1, jnt_type *jnt2);

static void		agp_close_hole_split_link( gp_array *agp, vec2d *jnt2_gctr, link_type **s_link, 
					link_type *link, vec2d ctr, vec2d pointOnLink );


int	agp_close_hole_jnt( gp_array *agp, jnt_type	*jnt1, float max_u );



///////////////////////////////////////////////////////////
/////////////     AgpCloseHoleGr.c    /////////////////////
///////////////////////////////////////////////////////////
int
agp_close_hole( gp_array *agp, float max_u )
{
	// TODO: Add your command handler code here
	int			i,	j;
	gp_type		*gp;
	jnt_type	*jnt1;//, *jnt2;
	int			rez = 0;



	if( agp == NULL )	return 0;
	for( i = 0 ; i < agp->Ni ; i++ )
	    for( j = 0 ; j < agp->Nj ; j++ )
	{
		gp = &agp->gp[i][j];

		for( jnt1 = gp->jnt; jnt1 != NULL; jnt1 = jnt1->nj ) 
		{
			if ( jnt1->type != JNT_EDGE || jnt1->brn_no != 1 )	
				continue;
			rez |= agp_close_hole_jnt( agp, jnt1, max_u );
		}
	}

	return ( rez );
}

int
agp_close_hole_jnt( gp_array *agp, jnt_type	*jnt1, float max_u )
{
	// TODO: Add your command handler code here
	//int			i,	j;
	//gp_type		*gp;
	jnt_type	*jnt2; //*jnt1,
	link_type	*link, *link2;
	vec2d       jnt2_gctr;
	vec2d       ctr, pointOnLink;
	float		u, delta_profile, d_gd;
	//int			rez = 0;

	link = NULL;	
	if ( jnt1->type != JNT_EDGE || jnt1->brn_no != 1 )	
		return 0;

	u =	agp_find_jnt_near_jnt( agp, jnt1, &jnt2, &delta_profile );

	d_gd = (float)((delta_profile/GOOD_DELTA_PROFILE)*(delta_profile/GOOD_DELTA_PROFILE));
	//if ( u <= max_u && jnt2 != NULL && 
	if ( u*d_gd <= max_u && 
		 jnt2 != NULL && jnt2->type == JNT_EDGE && jnt2->brn_no == 1 )	
		link = agp_link_from_2jnt( agp, jnt1, jnt2);
	else {
		u = agp_find_link_near_jnt( agp, jnt1, &ctr, &link, &pointOnLink );
		u = u*u;
		//u = find_link_near_jnt_and_split_link( agp, &jnt2_gctr, &link2, jnt1 );
		if ( u <= max_u && link ) {
			agp_close_hole_split_link( agp, &jnt2_gctr, &link2, link, ctr, pointOnLink );
			if ( link2 && link2->type == NF_EDGE &&
				 (jnt2 = agp_link_to_jnt( agp, &jnt2_gctr, link2 )) != NULL ) 
			link = agp_link_from_2jnt( agp, jnt1, jnt2 );
		}
	}
	return ( link ? 1 : 0);
}

///////////////////////////////////////////////////////////
/////////////     EdgeToolGr.c        /////////////////////
///////////////////////////////////////////////////////////
float
edge_max_delta_val( edge_type *e1, int s1, edge_type *e2, int s2 )
{
	int iSP;
	float max_delta = 0;
	for ( iSP = SP_Y; iSP <= SP_A; iSP++ ) 
	if ( ABS( e1->side[s1].val[iSP] - e2->side[s2].val[iSP] ) > max_delta )
		max_delta = ABS( e1->side[s1].val[iSP] - e2->side[s2].val[iSP] );
	return max_delta;
}

///////////////////////////////////////////////////////////
/////////////     ProfileToolGr.c     /////////////////////
///////////////////////////////////////////////////////////
float 
profile_max_delta_val( int type, profile_type *pr1, int s1, profile_type *pr2, int s2 )
{
	switch( type ){
	case NF_EDGE:
	case NF_N3_P:
		return edge_max_delta_val( pr1->edge, s1, pr2->edge, s2 );
	    break;
	case NF_N3R:
	case NF_RIDGE:
	    break;
	case NF_N3:
	    break;
	}
	return 256.0;
}

///////////////////////////////////////////////////////////
/////////////     JntToolGr.c         /////////////////////
///////////////////////////////////////////////////////////
float jnt_max_delta_profile( jnt_type *jnt1, jnt_type *jnt2 )
{
	int	direct1, direct2;
	float delta1, delta2, max_delta;
    profile_type *pr1, *pr2;

	if ( jnt1->type != JNT_EDGE || jnt1->brn_no != 1 || 
		 jnt2->type != JNT_EDGE || jnt2->brn_no != 1 ||
		 jnt1->brn[0]->link->type != NF_EDGE || 
		 jnt2->brn[0]->link->type != NF_EDGE )
		return 256.0;
	direct1 = jnt1->brn[0]->direction;
	pr1 = &(jnt1->brn[0]->link->pr[direct1]);
	direct2 = jnt2->brn[0]->direction;
	pr2 = &(jnt2->brn[0]->link->pr[direct2]);

	if ( direct1 == direct2 ) {
		delta1 = profile_max_delta_val( NF_EDGE, pr1, F_RIGHT, pr2, F_LEFT );
		delta2 = profile_max_delta_val( NF_EDGE, pr1, F_LEFT, pr2, F_RIGHT );
	}
	else {
		delta1 = profile_max_delta_val( NF_EDGE, pr1, F_RIGHT, pr2, F_RIGHT );
		delta2 = profile_max_delta_val( NF_EDGE, pr1, F_LEFT, pr2, F_LEFT );
	}
	max_delta = MAX( delta1, delta2 );
	return max_delta;
}

///////////////////////////////////////////////////////////
/////////////     AgpFindGr.c    //////////////////////////
///////////////////////////////////////////////////////////
float
agp_find_jnt_near_jnt( gp_array *agp, jnt_type *jnt0, 
					   jnt_type **s_jnt, float *delta_profile )
{
	gp_type		*gp;
	jnt_type	*jnt;
	link_type	*l0;//, *l1;
	brn_type	*b;

	int			i0,	j0,	i, j;
	float		u,	cu;
	float		u_calc,	cu_calc, cdelta_profile, cd_gd;

	vec2d		ctr,	dv;
	int			cLine, direct0;

	vec2d		*v = &jnt0->gctr;
	
	u_calc = u = 1E5;
	*s_jnt = NULL;

	if ( !jnt0 || jnt0->brn_no != 1 || !jnt0->brn[0]->link) 
		return u; 

	i0 = (int)((v->x+agp->bgp) * agp->inv_bgp);
	j0 = (int)((v->y+agp->bgp) * agp->inv_bgp);
  
	for( i = i0-L_SIZE ; i <= i0+L_SIZE ; i++ ){
	    if( i < 0 || i >= agp->Ni )	continue;

	    for( j = j0-L_SIZE  ; j <= j0+L_SIZE ; j++ ){
			if( j < 0 || j >= agp->Nj ) continue;
		  
			gp = &agp->gp[i][j];

			for( jnt = gp->jnt; jnt != NULL; jnt = jnt->nj )
			if ( jnt != jnt0 && jnt->brn_no == 1 &&
				 jnt->brn[0]->link && 
				 jnt->brn[0]->link != jnt0->brn[0]->link &&
				 jnt->type == jnt0->type &&
				 jnt->brn[0]->link->type == jnt0->brn[0]->link->type )
			{

				l0 = jnt0->brn[0]->link;
				cLine = 0;
				direct0 = LINK_NEXT(l0) ? F_END : F_BEGIN;
				while ( l0 && cLine < 2 && 
					    (direct0==F_END ? LINK_NEXT(l0) : LINK_PREV(l0)) ) {
					l0 = direct0==F_END ? LINK_NEXT(l0) : LINK_PREV(l0);
					cLine++;
				}
				if ( l0 && l0->p[direct0].bno >= 0 && 
					 l0->p[direct0].p.jnt == jnt && cLine < 2 )
					continue;
				
				cdelta_profile = jnt_max_delta_profile( jnt0, jnt );
				cd_gd = (float)((cdelta_profile/GOOD_DELTA_PROFILE)*(cdelta_profile/GOOD_DELTA_PROFILE));


				b = jnt->brn[0];
				ctr.x = jnt->gctr.x + b->ctr.x;
				ctr.y = jnt->gctr.y + b->ctr.y;

				dv.x = v->x - ctr.x;
				dv.y = v->y - ctr.y;
				cu = dv.x*dv.x + dv.y*dv.y;

				cu_calc = cu*cd_gd;

//				if( u > cu ) {
				if( u_calc > cu_calc ) {
					u_calc = cu_calc;
					u = cu;
					*delta_profile = cdelta_profile;
					*s_jnt = jnt;
				}

			}
	    }
	}

	return( u );
}

float
dis_and_point_form_link( vec2d *ctr, link_type *l, 
						 vec2d *v, vec2d *pointOnLink, float *t)
{
	vec2d	dv;
	float	u;

	if ( ABS(l->len) < 1E-5 || ((ABS(l->u.x)<1E-5)&&(ABS(l->u.y)<1E-5)) )
		return 1000;
	dv.x = v->x - ctr->x;
	dv.y = v->y - ctr->y;
	pointOnLink->x = ctr->x; 
	pointOnLink->y = ctr->y;

	*t = -dv.x *l->u.y + dv.y *l->u.x;

	if( (*t) < 0.0 ){
		u = (float)hypot( dv.x, dv.y );
		return( u );
	}

	if( (*t) > l->len ){
		/*u = hypot( dv.x - l->v.x, dv.y - l->v.y );*/
		pointOnLink->x += l->v.x; 
		pointOnLink->y += l->v.y; 
		return 1000;
	}

	u = l->u.x * dv.x + l->u.y * dv.y;
	u -= l->c_prb*(*t) *((*t) - l->len );
	link_t2xy( ctr, l, (*t), pointOnLink );

	return( ABS(u) ); 
}

float
agp_find_link_near_jnt( gp_array *agp, jnt_type *jnt0, vec2d *s_ctr, 
					    link_type **s_link, vec2d *s_pointOnLink )
{
	gp_type		*gp;
	link_type	*l;
	link_type	*l0;
	//jnt_type	*jnt;
	//brn_type	*b;

	int		i0,	j0,	i,	j;//,	k;
	float	u, cu, t;
	vec2d	*v = &jnt0->gctr;
	vec2d	ctr;
	vec2d   pointOnLink;
	int		cLine, direct0;
  
	u = 1E5;
	*s_link = NULL;

	if ( !jnt0 || jnt0->brn_no != 1 || !jnt0->brn[0]->link) 
		return u; 

	i0 = (int)((v->x+agp->bgp) * agp->inv_bgp);
	j0 = (int)((v->y+agp->bgp) * agp->inv_bgp);
  
  
	for( i = i0-L_SIZE ; i <= i0+L_SIZE ; i++ ){
		if( i < 0 || i >= agp->Ni )	continue;

		for( j = j0-L_SIZE  ; j <= j0+L_SIZE ; j++ ){
			if( j < 0 || j >= agp->Nj )	continue;
      
			gp = &agp->gp[i][j];

			for( l = gp->link ; l != NULL ; l = l->nl )
			if ( l != jnt0->brn[0]->link &&
				 l->type == jnt0->brn[0]->link->type )
			{

				l0 = jnt0->brn[0]->link;
				cLine = 0;
				direct0 = LINK_NEXT(l0) ? F_END : F_BEGIN;
				while ( l0 && l0 != l && cLine < 3 && 
					    (direct0==F_END ? LINK_NEXT(l0) : LINK_PREV(l0)) ) {
					l0 = direct0==F_END ? LINK_NEXT(l0) : LINK_PREV(l0);
					cLine++;
				}
				if ( l0 == l && cLine < 3 )
					continue;

				ctr.x = gp->ctr.x + l->ctr.x;
				ctr.y = gp->ctr.y + l->ctr.y;

				cu = dis_and_point_form_link( &ctr, l, v, &pointOnLink, &t );

				if( u > cu+0.1 || 
					( ABS(t) < 0.5 && ABS(cu-u) < 0.5 ) ) {
					u = cu;
					*s_ctr = ctr;
					*s_pointOnLink = pointOnLink;
					*s_link = l;
				}
			}


			/*for( jnt = gp->jnt ; jnt != NULL ; jnt = jnt->nj )
			    for( k = 0 ; k < jnt->brn_no ; k++ )
			{
				b = jnt->brn[k];

				if( b->direction == F_BEGIN )
					continue;

				if( b->type == BRN_NULL || b->type == BRN_IN )
					continue;

				l = b->link;


				ctr.x = jnt->gctr.x + b->ctr.x - l->v.x;
				ctr.y = jnt->gctr.y + b->ctr.y - l->v.y;


				cu = dis_and_point_form_link( &ctr, l, v, &pointOnLink, &t );

				if( u > cu+0.1 || 
					( ABS(t) < 0.5 && ABS(cu-u) < 0.5 ) ) {
					u = cu;
					*s_ctr = ctr;
					*s_pointOnLink = pointOnLink;
					*s_link = l;
				}
			}*/
		}
	}

	return( u );
}


///////////////////////////////////////////////////////////
/////////////     LinkToolGr.c    /////////////////////
///////////////////////////////////////////////////////////

link_type *
agp_link_from_2jnt( gp_array *agp, jnt_type *jnt1, jnt_type *jnt2)
{
	link_type	*link = NULL, *link1;
	//vec2d ctr;

	if ( !jnt1 || !jnt2 || !jnt1->brn[0]->link )
		return NULL;

	link = link_1_alloc();
	link->type = jnt1->brn[0]->link->type;

	link->v.x = jnt2->gctr.x - jnt1->gctr.x;
	link->v.y = jnt2->gctr.y - jnt1->gctr.y;
	link->a = 0;
	link_set_aux( link );

	link1 = jnt1->brn[0]->link;
	link->br = link1->br;
	link->Fbr = link1->Fbr;


	agp_default_profile( agp, &jnt1->gctr, link, 0, 0.25, TRUE);
	agp_default_profile( agp, &jnt1->gctr, link, 1, 0.25, TRUE);

	jnt_connet_link( jnt1, -1, link, F_BEGIN );
	jnt_set_aux( jnt1 );
	jnt_connet_link( jnt2, -1, link, F_END );
	jnt_set_aux( jnt2 );

	agp_lnf_link( &jnt1->gctr, link, agp );

	/*ctr = jnt1->gctr;
	link1 = link;
	if( link1->type == NF_EDGE ) {
		while( LINK_PREV(link1) != NULL ){
			link1 = LINK_PREV(link1);
			ctr.x -= link1->v.x;
			ctr.y -= link1->v.y;
		}
		embed_link( &ctr, link1, agp );
	}*/
	
	return link;
}

float
link_dis_sign_point( vec2d gctr, link_type *link, vec2d point )
{
	float dis = 0.0;
	float A, B, C; 
	if ( link->len == 0.0 ) 
		return 0.0;
	
	A = link->u.x;//link->v.x/link->len;
	B = link->u.y;//link->v.y/link->len;
	C = -A*(gctr.x+link->ctr.x) - B*(gctr.y+link->ctr.y);
	if ( C > 0 ) {
		A = -A; B = -B;  C = -C;
	}

	dis = A*point.x + B*point.y +C;
	return dis;
}

float
points2_dis_sign_point( vec2d p1, vec2d p2, vec2d point )
{
	float dis = 0.0;
	float A, B, C; 
	float dx, dy, dd;
	dx = p2.x - p1.x;
	dy = p2.y - p1.y;
	dd = (float)hypot( dx, dy );
	if ( dd == 0.0 )
		return 0.0;
	
	A = dy/dd;
	B = -dx/dd;
	C = -A*p1.x - B*p1.y;
	if ( C > 0 ) {
		A = -A; B = -B;  C = -C;
	}

	dis = A*point.x + B*point.y +C;
	return dis;
}

int
cross_points2_point2( vec2d line1_p1, vec2d line1_p2, vec2d line2_p1, vec2d line2_p2 )
{
	float dis_line1_from_l2p1, dis_line1_from_l2p2;
	float dis_line2_from_l1p1, dis_line2_from_l1p2;
	dis_line1_from_l2p1 = points2_dis_sign_point( line1_p1, line1_p2, line2_p1);
	dis_line1_from_l2p2 = points2_dis_sign_point( line1_p1, line1_p2, line2_p2);
	if ( dis_line1_from_l2p1*dis_line1_from_l2p2 <= 0.0 )
	{
		dis_line2_from_l1p1 = points2_dis_sign_point( line2_p1, line2_p2, line1_p1);
		dis_line2_from_l1p2 = points2_dis_sign_point( line2_p1, line2_p2, line1_p2);
		if ( dis_line2_from_l1p1*dis_line2_from_l1p2 <= 0.0 )
			return 1;
	}
	return 0;
}

int 
link_cross_link( vec2d gctr1, link_type *link1, vec2d gctr2, link_type *link2 )
{
	vec2d link1_p1, link1_pc, link1_p2;
	vec2d link2_p1, link2_pc, link2_p2;

	link1_p1.x = link1->ctr.x + gctr1.x;
	link1_p1.y = link1->ctr.y + gctr1.y;
	link1_p2.x = link1_p1.x + link1->v.x;
	link1_p2.y = link1_p1.y + link1->v.y;
	link1_pc.x = link1_p1.x + link1->v.x/2.0F+ link1->u.x*link1->a;
	link1_pc.y = link1_p1.y + link1->v.y/2.0F+ link1->u.y*link1->a;

	link2_p1.x = link2->ctr.x + gctr2.x;
	link2_p1.y = link2->ctr.y + gctr2.y;
	link2_p2.x = link2_p1.x + link2->v.x;
	link2_p2.y = link2_p1.y + link2->v.y;
	link2_pc.x = link2_p1.x + link2->v.x/2.0F+ link2->u.x*link2->a;
	link2_pc.y = link2_p1.y + link2->v.y/2.0F+ link2->u.y*link2->a;

	if ( cross_points2_point2(link1_p1, link1_pc, link2_p1, link2_pc) )
		return 1;
	if ( cross_points2_point2(link1_p1, link1_pc, link2_pc, link2_p2) )
		return 1;
	if ( cross_points2_point2(link1_pc, link1_p2, link2_p1, link2_pc) )
		return 1;
	if ( cross_points2_point2(link1_pc, link1_p2, link2_pc, link2_p2) )
		return 1;

	return 0;
}

///////////////////////////////////////////////////////////
///////////   agp_split_link      /////////////////////////
///////////////////////////////////////////////////////////
static void 
agp_close_hole_split_link( gp_array *agp, vec2d *jnt2_gctr, link_type **s_link, 
			    link_type *link, vec2d ctr, vec2d pointOnLink )
{
	if ( hypot( ctr.x-pointOnLink.x, ctr.y-pointOnLink.y ) < 0.5 )
	{
		*s_link = link;
		*jnt2_gctr = ctr;
	}

	agp_link_split( agp, &ctr, link, &pointOnLink );

	*s_link = LINK_NEXT( link );
	*jnt2_gctr = pointOnLink;
}
