/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
#include        <stdio.h>
#include        <math.h>

#include        "Uigp/igp.h"

#include        "GpType/GpType.h"

#include        "AgpEdit/AgpEditTool.h"

#include        "AgpTool/AgpTool.h"




static int	link_in_box( vec2d *ctr, link_type *link, box2d *box );




void
agp_delete( gp_array *agp, int Mnf, box2d *box )
{
int	i0,	j0,	i1,	j1;
int	i,	j;
gp_type	*gp;
link_type	*link;
hunch_type	*h,	*ph;
vec2d	ctr;


	if( box != NULL ){
		i0 = ( box->x0 + agp->bgp )*agp->inv_bgp;
		j0 = ( box->y0 + agp->bgp )*agp->inv_bgp;

		i1 = ( box->x1 + agp->bgp )*agp->inv_bgp;
		j1 = ( box->y1 + agp->bgp )*agp->inv_bgp;

		if( i0 < 0 )	i0 = 0;
		if( i1 >= agp->Ni )	i1 = agp->Ni -1;
		if( j0 < 0 )	j0 = 0;
		if( j1 >= agp->Nj )	j1 = agp->Nj -1;
	}
	else	{
		i0 = 0;
		i1 = agp->Ni-1;
		j0 = 0;
		j1 = agp->Nj-1;
	}


	for( i = i0 ; i <= i1 ; i++ )
	    for( j = j0 ; j <= j1 ; j++ ){
		gp = &agp->gp[i][j];


		if( Mnf & (1<<NF_HUNCH) ){
		    for( h = gp->hunch, ph = NULL ; h != NULL ; ){
			ctr.x = gp->ctr.x + h->ctr.x;
			ctr.y = gp->ctr.y + h->ctr.y;

			if( box != NULL && !BOX2D_IS_IN( *box, ctr.x, ctr.y ) ){
				ph = h;
				h = h->nh;
				continue;
			}

			if( ph == NULL ){
				gp->hunch = h->nh;
				free( h);
				h = gp->hunch;
				continue;
			}
			ph->nh = h->nh;
			free( h);
			h = ph->nh;
		    }
		}


		for( link = gp->link ; link != NULL ; ){

			if( link->Fbr != 0 ){
				link = link->nl;
				continue;
			}

			if( LINK_PREV(link) != NULL ){
				link = link->nl;
				continue;
			}


			if( !( Mnf &(1<<link->type)) ){
				link = link->nl;
				continue;
			}


			ctr.x = gp->ctr.x + link->ctr.x;
			ctr.y = gp->ctr.y + link->ctr.y;


			if( link_in_box( &ctr, link, box ) == 0 ){
				link = link->nl;
				continue;
			}


			if( agp_polylink_delete( agp, link ) == 0 )
				link = link->nl;
			else	link = gp->link;
		}
	}

}





static int
link_in_box( vec2d *ctr, link_type *link, box2d *box )
{
link_type	*l;
vec2d	v;

	if( box == NULL )	return( 1  );

	for( l = link, v = *ctr ; l != NULL ; l = LINK_NEXT(l) ){
		if( !BOX2D_IS_IN( *box, v.x, v.y ) )	return( 0 );

		v.x += l->v.x;
		v.y += l->v.y;
	}

	if( !BOX2D_IS_IN( *box, v.x, v.y ) )	return( 0 );

	return( 1 );
}
