/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/***************************
 ***	AgpMakeUnion.c   ***
 **************************/
#include	<stdio.h>   
#include	<math.h>

#include	"AgpTool/AgpTool.h"


static adepth_type *	agp_make_union_ad( gp_array *Agp[], int no );

static void	agp_make_union_get_uv( gp_array *agp, box2d *uv );


gp_array *
agp_make_union2( gp_array *agp0, gp_array *agp1 )
{
gp_array	*tagp[2],	*agp;

	tagp[0] = agp0;

	tagp[1] = agp1;

	agp = agp_make_union( tagp, 2 );

	return( agp );
}




gp_array *
agp_make_union( gp_array *Agp[], int no )
{
gp_array	*agp;
adepth_type	*ad;
layer_type	*layer;
vec2d	ctr,	vp,	gctr;
int	i,	k,	i0,	j0;

//	if( no == 1 )	return( Agp[0]);
 
	if( Agp[0]->ad == NULL )	return( NULL );


	agp = agp_alloc();

	ad = agp_make_union_ad( Agp, no );

	agp_init( agp, Agp[0]->bgp, ad->vw->Vpixel, ad->vw->Upixel, 48, 48 );

	agp->ad = ad;

        agp->color_type = Agp[0]->color_type;

	agp->type = AGP_TRANSPARENT_U;


	agp->Lcoding = Agp[0]->Lcoding;


	ctr.x = ctr.y = 0;
	for( i = 0 ; i < no ; i++ ){

		if( Agp[i]->layer_no == 0 )	continue;


        view_pixel2vp( Agp[i]->ad->vw, &ctr, &vp );
        view_vp2pixel( agp->ad->vw, &vp, &gctr );

		if( gctr.x < 0 )
				gctr.x = ((int)(gctr.x*1024-0.5))/1024.0;
		else	gctr.x = ((int)(gctr.x*1024+0.5))/1024.0;
		if( gctr.y< 0 )
				gctr.y = ((int)(gctr.y*1024-0.5))/1024.0;
		else	gctr.y = ((int)(gctr.y*1024+0.5))/1024.0;

        agp_union( Agp[i], &gctr, agp );

		i0 = (gctr.x + agp->bgp)* agp->inv_bgp -1;
		j0 = (gctr.y + agp->bgp)* agp->inv_bgp -1;

		for( k = 0 ; k < Agp[i]->layer_no ; k++ ){
			layer = agp->layer[agp->layer_no++] = LAYER_ALLOC();

			*layer = *Agp[i]->layer[k];

			layer->i0 += i0;
			layer->i1 += i0;
			layer->j0 += j0;
			layer->j1 += j0;
		}




		agp_destroy( Agp[i] );
	}


	return( agp );
}




static adepth_type *
agp_make_union_ad( gp_array *Agp[], int no )
{
adepth_type	*nad,	*ad;
view_type       *vw;
int     row,    column,    i,	k;

box2d	uv;
//vec2d	v,	v1,	vp;

	
	nad = ADEPTH_ALLOC();
	nad->P = Agp[0]->ad->P;
	nad->N = Agp[0]->ad->N;
	nad->br_no = 0;
	nad->Mbr = 0;

	nad->vw = VIEW_ALLOC();
	*nad->vw = *Agp[0]->ad->vw;


	for( i = 0 ; i < no ; i++ ){

		if( Agp[i]->layer_no == 0 )	continue;

		ad = Agp[i]->ad;
		vw = ad->vw;

		agp_make_union_get_uv( Agp[i], &uv );
		if( i > 0 ){
			BOX2D_UPDATE( nad->vw->uv, uv.x0, uv.y0 );
			BOX2D_UPDATE( nad->vw->uv, uv.x1, uv.y1 );
		}
		else	nad->vw->uv = uv;
//		BOX2D_UPDATE( nad->vw->uv, vw->uv.x0, vw->uv.y0 );
//		BOX2D_UPDATE( nad->vw->uv, vw->uv.x1, vw->uv.y1 );


		for( k = 0 ; k < ad->br_no ; k++ )
			nad->abr[ nad->br_no++ ] = ad->abr[k];

		nad->Mbr |= ad->Mbr;
    }
 
 
 
	vw = Agp[0]->ad->vw;
 
	row = ( nad->vw->uv.y1 - nad->vw->uv.y0 ) *
				vw->Vpixel /( vw->uv.y1 - vw->uv.y0 ) + 0.5;
	column = ( nad->vw->uv.x1 - nad->vw->uv.x0 ) *
				vw->Upixel /( vw->uv.x1 - vw->uv.x0 ) + 0.5;

/*
	vp.x = nad->vw->uv.x0;
	vp.y = nad->vw->uv.y1;
	view_vp2pixel( vw, &vp, &v );

	vp.x = nad->vw->uv.x1;
	vp.y = nad->vw->uv.y0;
	view_vp2pixel( vw, &vp, &v1 );

	row = v1.x - v.x+0.5;
	column = v1.y - v.y+0.5;
 */

	view_set_pixel( nad->vw, row, column );


	return( nad );
}


static void
agp_make_union_get_uv( gp_array *agp, box2d *uv )
{
layer_type	*layer;
int	i,	i0,	j0,	i1,	j1;
vec2d	v,	vp;

	layer = agp->layer[0];
	i0 = layer->i0;
	i1 = layer->i1;
	j0 = layer->j0;
	j1 = layer->j1;

	for( i = 1 ; i < agp->layer_no ; i++ ){
		layer = agp->layer[i];
		if( i0 > layer->i0 )	i0 = layer->i0;
		if( i1 < layer->i1 )	i1 = layer->i1;
		if( j0 > layer->j0 )	j0 = layer->j0;
		if( j1 < layer->j1 )	j1 = layer->j1;
	}

	v.x = agp->bgp*(i0-1);
	v.y = agp->bgp*(j0-1);
	view_pixel2vp( agp->ad->vw, &v, &vp );
	uv->x0 = vp.x;
	uv->y1 = vp.y;

	v.x = agp->bgp*i1;
	v.y = agp->bgp*j1;
	view_pixel2vp( agp->ad->vw, &v, &vp );
	uv->x1 = vp.x;
	uv->y0 = vp.y;
}

		

