/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/*************************
 ***	AgpCopySpr.c   ***
 *************************/
#include	<stdio.h>   
#include	<math.h>

#include	"GpType/GpType.h"

#include	"N3Prime/N3Prime.h"

#include	"Ep/Ep.h"
#include	"EpBgp/EpBgp.h"

#include	"AgpTool.h"




static void	agp_copy_layer( gp_array *agp, gp_array *nagp );

static void	agp_mold_jnt( vec2d *ctr, jnt_type *jnt, gp_array *agp );



static jnt_type *	jnt_copy_spr( jnt_type *jnt );
static void	agp_mold_jnt_link( gp_array *agp, jnt_type *jnt );


static void	agp_mold_link_color( link_type *link );


static void	agp_mold_layer( gp_array *agp,
				gp_array *nagp, int layer_no );

static void	agp_mold_bgp( gp_array *agp, ep_array *aep, int mbr,
						ep_array *baep );

static void	agp_mold_bgp_one( gp_array *agp, gp_type *gp, ep_type *ep,
					int mbr, ep_type *bep, float hb );

static void	agp_mold_bgp_add( gp_array *agp, gp_type *gp, int mbr,
					float x, float y,
					ep_type *ep,
					ep_type *bep );

static float	depth_plane_value( gp_array *agp, int layer, vec2d *p );


#define COLORS	6
static int Color[] = { 0x0000ff, 0x00ff00, 0xff0000,
				0xffff00, 0xff00ff, 0x00ffff };

gp_array *
agp_make_mold( gp_array *agp )
{
gp_array	*nagp;
gp_type		*gp;
jnt_type	*jnt;
int	i,	j;
int	Fn3_prime;

	nagp = agp_alloc();


	agp_clear( nagp, agp->bgp, agp->row, agp->column, agp->Lni, agp->Lnj );

	nagp->type = agp->type;
	nagp->color_type = agp->color_type;
	nagp->Lcoding = agp->Lcoding;


	nagp->ad = adepth_copy( agp->ad );
	agp_copy_layer( agp, nagp );



	if( (Fn3_prime = agp->Fn3_prime) == 0 )
		n3_prime_agp( agp);

  


	/* copy nf and jnt  */
	for( i = 0 ; i < agp->Ni ; i++ )
		for( j = 0 ; j < agp->Nj ; j++ ){
			gp = &agp->gp[i][j];

			for( jnt = gp->jnt ; jnt != NULL ; jnt = jnt->nj )
				agp_mold_jnt( &gp->ctr, jnt, nagp );
		}



	for( i = 0 ; i < nagp->Ni ; i++ )
		for( j = 0 ; j < nagp->Nj ; j++ ){
			gp = &nagp->gp[i][j];
			for( jnt = gp->jnt ; jnt != NULL ; jnt = jnt->nj )
				agp_mold_jnt_link( nagp, jnt );
		}




	for( i = 0 ; i < agp->layer_no ; i++ )
		agp_mold_layer( agp, nagp, i );

	if( Fn3_prime == 0 )    n3_prime_free( agp);



	agp_connect_jnt( nagp );


	return( nagp );
}






static void
agp_copy_layer( gp_array *agp, gp_array *nagp )
{
layer_type	*layer,	*nlayer;
int	i;

	nagp->layer_no = 0;
	for( i = 0 ; i < agp->layer_no ; i++ ){
		layer = agp->layer[i];
		nlayer = nagp->layer[nagp->layer_no++] = LAYER_ALLOC();

		*nlayer = *layer;
	}
}








static void
agp_mold_jnt( vec2d *ctr, jnt_type *jnt, gp_array *agp )
{
gp_type	*gp;
jnt_type	*njnt;
int	Fbr;
int	i,	j;

	if( jnt->type != JNT_EDGE && jnt->type != JNT_N3_P )
		return;

	for( i = 0, Fbr = 0 ; i < jnt->brn_no ; i++ )
		if( (Fbr = jnt->brn[i]->link->Fbr) != 0 )	break;

	if( Fbr == 0 )	return;


	njnt = jnt->cjnt = jnt_copy_spr( jnt );

	njnt->type = JNT_EDGE;


	i = (njnt->gctr.x+agp->bgp) * agp->inv_bgp;
	j = (njnt->gctr.y+agp->bgp) * agp->inv_bgp;

	gp = &agp->gp[i][j];


	njnt->ctr.x = njnt->gctr.x - gp->ctr.x;
	njnt->ctr.y = njnt->gctr.y - gp->ctr.y;
	njnt->nj = gp->jnt;
	gp->jnt = njnt;
}


static jnt_type *
jnt_copy_spr( jnt_type *jnt )
{
jnt_type	*njnt;
int	k;

	njnt = jnt_alloc();
	*njnt = *jnt;

	for( k = 0, njnt->brn_no = 0 ; k < jnt->brn_no ; k++ ){
		if( jnt->brn[k]->link->Fbr == 0 )	continue;
		njnt->brn[njnt->brn_no] = brn_alloc();
		*njnt->brn[njnt->brn_no] = *jnt->brn[k];
		njnt->brn_no++;
	}

	return( njnt );
}






static void
agp_mold_jnt_link( gp_array *agp, jnt_type *jnt )
{
jnt_type	*njnt;
brn_type	*b,     *nb;
link_type       *link,  *l,	*l1;
vec2d   ctr;
int     k,	j;

	for( k = 0 ; k < jnt->brn_no ; k++ ){
		b = jnt->brn[k];

		if( b->direction != F_BEGIN )   continue;

		for( l1 = b->link ; LINK_NEXT(l1 ) != NULL ; l1 =LINK_NEXT(l1));


		link = link_make_copy( b->link, b->type );
		agp_mold_link_color( link );


		b->type = NF_EDGE;
		b->link = link;
		b->link->p[F_BEGIN].p.jnt = jnt;
		b->link->p[F_BEGIN].bno = k;


		for( l = link ; LINK_NEXT(l ) != NULL ; l = LINK_NEXT(l ) )
			l->type = NF_EDGE;

		l->type = NF_EDGE;

		njnt = l->p[F_END].p.jnt->cjnt;

		for( j = 0 ; j < njnt->brn_no ; j ++ ){
			nb = njnt->brn[j];
			if( nb->link == l1 )	break;
		}

		nb->link = l;
		nb->link->p[F_END].p.jnt = njnt;
		nb->link->p[F_END].bno = j;
		nb->type = NF_EDGE;


		ctr.x = jnt->gctr.x + b->ctr.x;
		ctr.y = jnt->gctr.y + b->ctr.y;
		agp_lnf_link( &ctr, b->link, agp );
	}
}



static void
agp_mold_link_color( link_type *link )
{
link_type	*l;
edge_type	*e;
int	side,	color,	color1;

	if( link->Fbr != 0x04 ){
		color = Color[ link->br%COLORS ];

		side = ( link->Fbr & 0x01 )? F_LEFT : F_RIGHT;
		for( l = link ; l != NULL ; l = LINK_NEXT(l) ){
			e = l->pr[0].edge;

			e->side[side].val[SP_R] = color &0xff;
			e->side[side].val[SP_G] = (color>>8) &0xff;
			e->side[side].val[SP_B] = (color>>16) &0xff;

			e->side[side].bval[SP_R] = color &0xff;
			e->side[side].bval[SP_G] = (color>>8) &0xff;
			e->side[side].bval[SP_B] = (color>>16) &0xff;

			if( LINK_NEXT(l) == NULL )	break;
		}

		e = l->pr[1].edge;

		e->side[side].val[SP_R] = color &0xff;
		e->side[side].val[SP_G] = (color>>8) &0xff;
		e->side[side].val[SP_B] = (color>>16) &0xff;

		e->side[side].bval[SP_R] = color &0xff;
		e->side[side].bval[SP_G] = (color>>8) &0xff;
		e->side[side].bval[SP_B] = (color>>16) &0xff;

		return;
	}


	color = Color[ link->br%COLORS ];
	color1 = Color[ link->br1%COLORS ];

	for( l = link ; l != NULL ; l = LINK_NEXT(l) ){
		e = l->pr[0].edge;

		e->side[F_RIGHT].val[SP_R] = color &0xff;
		e->side[F_RIGHT].val[SP_G] = (color>>8) &0xff;
		e->side[F_RIGHT].val[SP_B] = (color>>16) &0xff;

		e->side[F_RIGHT].bval[SP_R] = color &0xff;
		e->side[F_RIGHT].bval[SP_G] = (color>>8) &0xff;
		e->side[F_RIGHT].bval[SP_B] = (color>>16) &0xff;


		e->side[F_LEFT].val[SP_R] = color1 &0xff;
		e->side[F_LEFT].val[SP_G] = (color1>>8) &0xff;
		e->side[F_LEFT].val[SP_B] = (color1>>16) &0xff;

		e->side[F_LEFT].bval[SP_R] = color1 &0xff;
		e->side[F_LEFT].bval[SP_G] = (color1>>8) &0xff;
		e->side[F_LEFT].bval[SP_B] = (color1>>16) &0xff;
	}

	e = l->pr[1].edge;

	e->side[F_RIGHT].val[SP_R] = color &0xff;
	e->side[F_RIGHT].val[SP_G] = (color>>8) &0xff;
	e->side[F_RIGHT].val[SP_B] = (color>>16) &0xff;

	e->side[F_RIGHT].bval[SP_R] = color &0xff;
	e->side[F_RIGHT].bval[SP_G] = (color>>8) &0xff;
	e->side[F_RIGHT].bval[SP_B] = (color>>16) &0xff;


	e->side[F_LEFT].val[SP_R] = color1 &0xff;
	e->side[F_LEFT].val[SP_G] = (color1>>8) &0xff;
	e->side[F_LEFT].val[SP_B] = (color1>>16) &0xff;

	e->side[F_LEFT].bval[SP_R] = color1 &0xff;
	e->side[F_LEFT].bval[SP_G] = (color1>>8) &0xff;
	e->side[F_LEFT].bval[SP_B] = (color1>>16) &0xff;
}



static void
agp_mold_layer( gp_array *agp, gp_array *nagp, int layer_no )
{
ep_array	*aep,	*naep;
layer_type      *layer;
int     mbr,    i,      j;

	layer = agp->layer[layer_no];
	mbr = layer->Mbr;

	aep = aep_alloc();
	naep = aep_alloc();

	for( i = layer->i0 ; i < layer->i1 ; i += EP_N ){
		for( j = layer->j0 ; j < layer->j1 ; j += EP_N ){

			ep_create( agp, i, j, EP_N, EP_N, mbr, aep );


			aep_br_set( aep );
			ep_bgp_get_br( agp, mbr, aep );
			aep_br_fill( aep );


			ep_create( nagp, i, j, EP_N, EP_N, mbr, naep );

			agp_mold_bgp( nagp, naep, mbr, aep );
		}
	}


	aep_destroy( aep );
	aep_destroy( naep );

}


static void
agp_mold_bgp( gp_array *agp, ep_array *aep, int mbr, ep_array *baep )
{
int     i,      j,      i1,     j1;
gp_type *gp;
ep_type *ep,	*bep;
float	hbgp = aep->hbgp;

	for( i = LSIZE, i1 = aep->Ni0 ; i < aep->Ni-LSIZE ; i++, i1++ ){
		if( i1 < 0 || i1 >= agp->Ni-1 )       continue;

		for( j = LSIZE, j1 = aep->Nj0 ; j < aep->Nj-LSIZE ; j++, j1++ ){			if( j1 < 0 || j1 >= agp->Nj-1 )   continue;

			gp = &agp->gp[i1][j1];
			ep = &aep->ep[i][j];
			bep = &baep->ep[i][j];


			agp_mold_bgp_one( agp, gp, ep, mbr, bep, hbgp );
		}
	}
}




static void
agp_mold_bgp_one( gp_array *agp, gp_type *gp,
				ep_type *ep, int mbr, ep_type *bep, float hbgp )
{
int     bno,    bgp_in[4];
float   hb = hbgp *0.5;

	bno = ep_middle_bgp( bep, bgp_in, hbgp );
	if( bno < 0 )   return;



	if( bgp_in[0] == 1 )
		agp_mold_bgp_add( agp, gp, mbr, -hb, -hb, ep, bep );


	if( bgp_in[1] == 1 )
		agp_mold_bgp_add( agp, gp, mbr, -hb, +hb, ep, bep );


	if( bgp_in[3] == 1 )
		agp_mold_bgp_add( agp, gp, mbr, +hb, -hb, ep, bep );


	if( bgp_in[2] == 1 )
		agp_mold_bgp_add( agp, gp, mbr, +hb, +hb, ep, bep );
}


static void
agp_mold_bgp_add( gp_array *agp, gp_type *gp, int mbr,
					float x, float y,
					ep_type *ep,
					ep_type *bep )
{
bgp_type	*bp;
int	bno,	br;
float	dis;
vec2d   p;
int	color;


	p.x = x;
	p.y = y;

	bno = ep_bound_point( bep, &p, &dis );
	br = bep->bound[bno]->br.no;

	if(  255 || !(mbr & (1<< br ) ) )
	if( br == 255 || !(mbr & (1<< br ) ) )	return;
	
	

	bp = BGP_ALLOC();
	bp->ctr.x = x;
	bp->ctr.y = y;

/*
	ep_bg_value_bound_all( ep, bno, &bp->ctr, bp->val );
*/
	bp->br = br;
	color = Color[ br%COLORS ];
	bp->val[SP_R] = color &0xff;
	bp->val[SP_G] = (color>>8) &0xff;
	bp->val[SP_B] = (color>>16) &0xff;

	p.x = gp->ctr.x + bp->ctr.x;
	p.y = gp->ctr.y + bp->ctr.y;
	bp->depth = depth_plane_value( agp, br, &p );


	bp->p = gp->bgp;
	gp->bgp = bp;
}



static float
depth_plane_value( gp_array *agp, int layer, vec2d *p )
{
view_type       *vw;
vec3d   N;
vec2d   v;
float   t,      value,  D;
vec3d   n,      p3;
 
	if( agp->ad == NULL )   return( 0 );
 
	if( adepth_get_plane( agp->ad, layer, &N, &D ) == 0 )   return( 0 );
 
	vw = agp->ad->vw;
	 
 
	v.x = vw->Ua_inv*(p->y - vw->Ub);
	v.y = vw->Va_inv*(p->x - vw->Vb);
 
 
	view_3d_ray( vw, v.x, v.y, &p3, &n );
	t = 1.0 / VEC3D_NORM( n );
	n.x *= t;
	n.y *= t;
	n.z *= t;
 
	t = ( D - VEC3D_INNER( vw->cp, N ) ) / VEC3D_INNER( n, N );
 
	value = t* VEC3D_INNER( n, vw->N );
 
	return( value );
}
