/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**************************
 ***	AgpNormalView.c	***
 **************************/
#include	<stdio.h>

#include	"GpType/GpType.h"
#include	"AgpTool/AgpTool.h"


static void	agp_2Nview_vec( view_type *cvw, vec3d *p, view_type *vw );
static void	agp_2Nview_point( view_type *ovw, vec3d *P, view_type *vw );


void
agp_normal_view( gp_array *agp )
{
view_type	ovw;
adepth_type     *ad;
abr_type        *br;
int	i;

	ad = agp->ad;
	ovw = *ad->vw;

	view_create_noraml( ad->vw, 1.0/512.0, agp->row, agp->column );


	for( i = 0 ; i < ad->br_no ; i++ ){
		br = &ad->abr[i];

		agp_2Nview_vec( &ovw, &br->N, ad->vw );
		agp_2Nview_point( &ovw, &br->P, ad->vw );
	}
}



static void
agp_2Nview_vec( view_type *ovw, vec3d *p, view_type *vw )
{
vec3d	dp;

	dp.x = ovw->U.x * p->x + ovw->U.y * p->y + ovw->U.z * p->z;
	dp.y = ovw->V.x * p->x + ovw->V.y * p->y + ovw->V.z * p->z;
	dp.z = ovw->N.x * p->x + ovw->N.y * p->y + ovw->N.z * p->z;

	p->x = dp.x * vw->U.x + dp.y * vw->V.x + dp.z * vw->N.x;
	p->y = dp.x * vw->U.y + dp.y * vw->V.y + dp.z * vw->N.y;
	p->z = dp.x * vw->U.z + dp.y * vw->V.z + dp.z * vw->N.z;
}


static void
agp_2Nview_point( view_type *ovw, vec3d *P, view_type *vw )
{
vec3d	dp,	p;

	p.x = P->x - ovw->cp.x;
	p.y = P->y - ovw->cp.y;
	p.z = P->z - ovw->cp.z;

	dp.x = ovw->U.x * p.x + ovw->U.y * p.y + ovw->U.z * p.z;
	dp.y = ovw->V.x * p.x + ovw->V.y * p.y + ovw->V.z * p.z;
	dp.z = ovw->N.x * p.x + ovw->N.y * p.y + ovw->N.z * p.z;

	p.x = dp.x * vw->U.x + dp.y * vw->V.x + dp.z * vw->N.x;
	p.y = dp.x * vw->U.y + dp.y * vw->V.y + dp.z * vw->N.y;
	p.z = dp.x * vw->U.z + dp.y * vw->V.z + dp.z * vw->N.z;

	P->x = p.x + vw->cp.x;
	P->y = p.y + vw->cp.y;
	P->z = p.z + vw->cp.z;

}
