/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/*************************
 ***	AgpProfile.c   ***
 *************************/
#include        <stdio.h>   
#include	<math.h>

#include        "AgpTool.h"
#include        "AgpEdit/AgpEditTool.h"


static void	agp_profile_n3( n3_type *e );
static void	agp_profile_ln3( n3_type *e, int sp );

static void	agp_profile_edge( edge_type *e );
static void	agp_profile_ledge( edge_type *e, int sp );

static void	agp_profile_n3_width( width_type *w );
static void	agp_profile_jnt_n3( gp_array *agp, jnt_type *jnt );


int	Fprofile;
static float	Wk,	Dh_a,	Dh_b;


static GP_prm params[] = {
	{"P-F", (char *)&Fprofile, PRM_BOOLEAN, "%d", "No" },
        {"P-W-k  ", (char *)&Wk,      PRM_FLOAT, "%.2f", "1.0"},
        {"P-Dh-a  ", (char *)&Dh_a,      PRM_FLOAT, "%.2f", "1.0"},
        {"P-Dh-b  ", (char *)&Dh_b,      PRM_FLOAT, "%.2f", "0.0"},
};



void
agp_profile_get_prm( GP_prm **prm, int *prm_no, void (**check)() )
{
        *prm    = params;
        *prm_no  = PRM_NUMBER(params);
        *check = NULL ;
}


void
agp_profile_init( char *file )
{
        gp_prm_init( file, agp_profile_get_prm );

}



void
agp_profile( gp_array *agp )
{
gp_type	*gp;
link_type	*link;
jnt_type	*jnt;
 
int	i,	j;

	if( Fprofile == 0 )	return;
 
	for( i = 0 ; i < agp->Ni ; i++ )
	    for( j = 0 ; j < agp->Nj ; j++ ){
		gp = &agp->gp[i][j];
 
		for( link = gp->link ; link != NULL ; link = link->nl ){

			if( link->type == NF_N3 ){
				agp_profile_n3( link->pr[0].n3 );
				agp_profile_n3_width( link->width[0] );

				if( LINK_NEXT( link ) == NULL ){
					agp_profile_n3( link->pr[1].n3 );
					agp_profile_n3_width( link->width[1] );
				}
				continue;
			}

			if( link->type == NF_EDGE ){
				agp_profile_edge( link->pr[0].edge );

				if( LINK_NEXT( link ) == NULL )
					agp_profile_edge( link->pr[1].edge );
				continue;
			}
		}
	    }


	if( Wk == 1.0 )	return;

	for( i = 0 ; i < agp->Ni ; i++ )
	    for( j = 0 ; j < agp->Nj ; j++ ){
		gp = &agp->gp[i][j];
 
		for( jnt = gp->jnt ; jnt != NULL ; jnt = jnt->nj ){
			if( jnt->type != JNT_N3 )	continue;
			agp_profile_jnt_n3( agp, jnt );
		}
	    }
			

}



static void
agp_profile_edge( edge_type *e )
{

	e->width *= Wk;

	agp_profile_ledge( e, SP_Y );
	agp_profile_ledge( e, SP_Q );
	agp_profile_ledge( e, SP_A );
}

static void
agp_profile_ledge( edge_type *e, int sp )
{
float	dh,	tmp;
edge_ltype	*elr,*ell;

	elr = &e->side[F_RIGHT];
	ell = &e->side[F_LEFT];

	tmp = dh =  0.5*( elr->val[sp] - ell->val[sp] );
	if( dh > 0 )	dh = ( Dh_a - 1 )*dh + Dh_b;
	else	dh = ( Dh_a - 1 )*dh - Dh_b;

	elr->val[sp] += dh;
	ell->val[sp] -= dh;


	if( tmp > 0 ){
		if( elr->val[sp] > 380 )	elr->val[sp] = 380;
		if( ell->val[sp] < -60 )	ell->val[sp] = -60;

		if( elr->val[sp] < elr->bval[sp] )
			elr->val[sp] = elr->bval[sp];
		if( ell->val[sp] > ell->bval[sp] )
			ell->val[sp] = ell->bval[sp];
	}
	else	{
		if( elr->val[sp] < -60 )	elr->val[sp] = -60;
		if( ell->val[sp] > 380 )	ell->val[sp] = 380;

		if( elr->val[sp] > elr->bval[sp] )
			elr->val[sp] = elr->bval[sp];
		if( ell->val[sp] < ell->bval[sp] )
			ell->val[sp] = ell->bval[sp];
	}
}



static void
agp_profile_n3( n3_type *e )
{
	agp_profile_ln3( e, SP_Y );
	agp_profile_ln3( e, SP_Q );
	agp_profile_ln3( e, SP_A );


/*
float	dh;

	dh =  0.5*e->mval[SP_Y] -
			0.25*( e->side[F_RIGHT].val[SP_Y]+e->side[F_LEFT].val[SP_Y] );
	if( dh > 0 )	dh = ( Dh_a - 1 )*dh + Dh_b;
	else	dh = ( Dh_a - 1 )*dh - Dh_b;

	e->mval[SP_Y] += dh;
	e->side[F_RIGHT].val[SP_Y] -= dh;
	e->side[F_LEFT].val[SP_Y] -= dh;



	dh =  0.5*e->mval[SP_Q] -
			0.25*( e->side[F_RIGHT].val[SP_Q]+e->side[F_LEFT].val[SP_Q] );
	if( dh > 0 )	dh = ( Dh_a - 1 )*dh + Dh_b;
	else	dh = ( Dh_a - 1 )*dh - Dh_b;

	e->mval[SP_Q] += dh;
	e->side[F_RIGHT].val[SP_Q] -= dh;
	e->side[F_LEFT].val[SP_Q] -= dh;



	dh =  0.5*e->mval[SP_A] -
			0.25*( e->side[F_RIGHT].val[SP_A]+e->side[F_LEFT].val[SP_A] );
	if( dh > 0 )	dh = ( Dh_a - 1 )*dh + Dh_b;
	else	dh = ( Dh_a - 1 )*dh - Dh_b;

	e->mval[SP_A] += dh;
	e->side[F_RIGHT].val[SP_A] -= dh;
	e->side[F_LEFT].val[SP_A] -= dh;
*/
}


static void
agp_profile_ln3( n3_type *e, int sp )
{
float	dh,	tmp;
n3_ltype	*elr,*ell;

	elr = &e->side[sp];
	ell = &e->side[sp];

	tmp = dh =  0.5*e->mval[sp] - 0.25*( elr->val[sp] + ell->val[sp] );
	if( dh > 0 )	dh = ( Dh_a - 1 )*dh + Dh_b;
	else	dh = ( Dh_a - 1 )*dh - Dh_b;

	e->mval[sp] += dh;
	elr->val[sp] -= dh;
	ell->val[sp] -= dh;


	if( tmp > 0 ){
		if( e->mval[sp] > 380 )	e->mval[sp] = 380;
		if( elr->val[sp] < -60 )	elr->val[sp] = -60;
		if( ell->val[sp] < -60 )	ell->val[sp] = -60;

		if( elr->val[sp] > elr->bval[sp] )
			elr->val[sp] = elr->bval[sp];
		if( ell->val[sp] > ell->bval[sp] )
			ell->val[sp] = ell->bval[sp];
	}
	else	{
		if( e->mval[sp] < -60 )	e->mval[sp] = -60;
		if( elr->val[sp] > 380 )	elr->val[sp] = 380;
		if( ell->val[sp] > 380 )	ell->val[sp] = 380;

		if( elr->val[sp] < elr->bval[sp] )
			elr->val[sp] = elr->bval[sp];
		if( ell->val[sp] < ell->bval[sp] )
			ell->val[sp] = ell->bval[sp];
	}
}






static void
agp_profile_n3_width( width_type *width )
{
	width->w[F_LEFT] *= Wk;
	width->w[F_RIGHT] *= Wk;
}


static void
agp_profile_jnt_n3( gp_array *agp, jnt_type *jnt )
{
	jnt->rwidth *= Wk;
	jnt->lwidth *= Wk;

	agp_jnt_n3_reposition( agp, jnt );
}
