/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/************************
 *	ImagePSDFile.c	*
 ************************/
#include	<memory.h>
#include	<malloc.h>

#include	"Uigp/igp.h"
#include	"Ugio/GioType.h"
#include    "UGeoApp/GeoApp.h"

#include	"ImageType/ImageType.h"
#include	"ImageIo/ImageIo.h"


#include	"GpType/GpType.h"
#include	"GpType/NfType.h"
#include	"GpType/LinkType.h"
#include	"LinkTool/LinkTool.h"
#include	"LinkTool/LinkLayer.h"
#include	"AgpLayer/AgpLayer.h"
#include	"AgpDepth/AgpDepth.h"
#include	"AgpTool.h"

#include	"CorTransform/CorTransform.h"

#include        "PSD/ImageReadPsd.h"


#define MAX_LAYERS		256




///////////////////////////////////////
// placed in ImageReadPSD
psdLayer_type *	image_read_PSDlayers( gio_type *gio, int *number_layers, 
					   int *row, int *column);


//gp_array *	agp_read_PSD( gio_type *gio );


link_type *	layer_image2polylink( image_type  *image, vec2d *p );

static link_type *	points2polylink( int n_points, vec2d *points );

static void		points2gapp( int n_points, vec2d *points, gapp_type *gapp);

void	image4_copy_in_transparent( image_type *sim, image_type *im, 
						   int row0, int col0, u_int opacity );

void	image4_clear_transparent( image_type *image );

static u_int *	image4_find_border_first( image_type *image, int *ir, int *jr );

static int	image4_find_num_points( image_type *image );

static vec2d *	image4_find_border( image_type *image, int *n_points );


void	set_view( adepth_type *ad, int row0, int col0, int row, int col );

static int	image4_test_in_out( image_type *image );

void	image4_filter( image_type *image );

static int	image4_filter1( image_type *image );

static int	image4_filter2( image_type *image );




gp_array *
agp_from_PSD_file( char *psd_file )
{
int n_images, i;
psdLayer_type	*arr_layer_info;

gp_array	*tagp[MAX_LAYERS], *agp;

int row, column;
int i_layer;

view_type *vw;


	if( (n_images = image_read_PSDLayer_file( psd_file, &arr_layer_info, &row, &column )) <= 0 )
		return( NULL );



	vw = VIEW_ALLOC();
	view_create_noraml( vw, (float)(1.0/512.0), row, column );

	for ( i = 0, i_layer = 0; i < n_images; i++ )
	{
		tagp[i_layer] = agp_from_psdLayer( &(arr_layer_info[i]), 
							i_layer+1, 6.5-0.05*i_layer, vw );

		if ( tagp[i_layer] == NULL )
			continue;

		i_layer++;
	}

	for (i = 0; i < n_images; i++)
		image_destroy( arr_layer_info[i].image, 1 );
	free( arr_layer_info );

	free( vw );

	if ( i_layer == 0 )
		return NULL;

	if ( i_layer == 1 )
		return tagp[0]; 
	
	agp = agp_make_union( tagp, i_layer );

	return agp;
}


gp_array *
agp_from_PSD_file_layer( char *psd_file, int iLayer )
{
int n_images,	i;
psdLayer_type	*arr_layer_info;

gp_array	*agp;

int row, column;
view_type *vw;


	if( (n_images = image_read_PSDLayer_file( psd_file, &arr_layer_info, &row, &column )) <= 0 )
		return( NULL );


	if( n_images <= iLayer )	return( NULL );


	vw = VIEW_ALLOC();
	view_create_noraml( vw, (float)(1.0/512.0), row, column );


	agp = agp_from_psdLayer( &arr_layer_info[iLayer], 
							iLayer+1, 6.5-0.05*iLayer, vw );



	for (i = 0; i < n_images; i++)
		image_destroy( arr_layer_info[i].image, 1 );
	free( arr_layer_info );

	free( vw );



	return agp;
}




int
agp_from_psd_extrct( psdLayer_type *psdL, image_type **im, link_type **link )
{
vec2d	p;
const int	sh = 2;



	*im = image_create( psdL->image->row+2*sh, 
					psdL->image->column+2*sh, 4, 1, NULL );
	image4_const( *im, 0xFF000000 );
	image4_copy_in_transparent( psdL->image, *im, sh, sh, 0 );

	image4_filter( *im );

	*link = layer_image2polylink( *im, &p );

	image4_clear_transparent( *im );

	return( 1 );
}




gp_array *
agp_from_psdLayer( psdLayer_type *layer_info, int i_layer, float depth,
				 view_type *vw)
{
gp_array	*agp;
gp_array	*pagp;

link_type	*link;
vec2d		ctr, p;

image_type  *im_bas;
const int	sh = 2;

	if ( layer_info->image == NULL )
		return NULL;

	im_bas = image_create( layer_info->image->row+sh*2, 
					layer_info->image->column+sh*2, 4, 1, NULL );
	image4_const( im_bas, 0xFF000000 );
	image4_copy_in_transparent( layer_info->image, im_bas, sh, sh, 0 );

	image4_filter( im_bas );

	link = layer_image2polylink( im_bas, &p );

	if ( link == NULL )
	{
		image_destroy( im_bas, 1 );
		return NULL;
	}

	image4_clear_transparent( im_bas );

	agp = cor_transform( im_bas, 0, 6, "D", 0, NULL);

	image_destroy( im_bas, 1 );

	if ( agp == NULL )
		return NULL;

	agp_depth_default( agp, depth );

	ctr = link->ctr;
	polylink_filter_short_link( &ctr, link, 0.5 );

	ctr = link->ctr;
//	pagp = agp_layer_contour_copy( agp, &ctr, link, 
//			     i_layer, depth, &p );

	pagp = agp_layer_contour_copy( agp, &ctr, link, 
			     i_layer, depth, NULL );

	if ( pagp == NULL )
		return NULL;

	agp_destroy( agp );

	*pagp->ad->vw = *vw;
	set_view( pagp->ad, layer_info->y, layer_info->x, 
					   layer_info->height, layer_info->width );

	return pagp;
}


link_type *
layer_image2polylink( image_type *image, vec2d *p )
{
link_type	*link;

vec2d		*points;
int			n_points;

	points = image4_find_border( image, &n_points );

	if ( points == NULL || n_points == 0)
		return NULL;

/*	
double		x, y;

	p->x = points[0].x + 0.5;
	p->y = points[0].y + 0.5;

	if ( image4_find_point_in( image, &x, &y ) )
	{
		p->x = x;
		p->y = y;
	}
*/

	link = points2polylink( n_points, points );

	if ( link )
		link_get_point_inside(&link->ctr, link, p);
	
	free( points );

	return link;
}


static void 
set_Qgprm(gapp_prm *Qgprm)
{
	Qgprm->dwight = 0.0;
	Qgprm->err	 = 0.01;

	Qgprm->min_len = 5.0;
	Qgprm->Min_pnumber = 12; 
	Qgprm->max_len = 24.0;

	Qgprm->Max_dev = 2.0;
	Qgprm->Min_dev = 0.5;

}

static link_type *
points2polylink( int n_points, vec2d *points )
{
gapp_type *gapp;
link_type *l, *last;
gapp_prm  Qgprm;

	gapp = gapp_alloc( n_points + 1 );
	points2gapp( n_points, points, gapp);

	set_Qgprm( &Qgprm );
	geo_approximate( gapp, &Qgprm );
		
	l= link_from_gapp( gapp, NF_EDGE, &last );

	polylink_set_ctr( &gapp->v[0], l);

	gapp_destroy( gapp );


	last->v.x = l->ctr.x - last->ctr.x;
	last->v.y = l->ctr.y - last->ctr.y;

	return l;
}


static void 
points2gapp( int n_points, vec2d *points, gapp_type *gapp)
{
int i;

	gapp->no = n_points;
	
	for(i = 0; i < n_points; i++)
	{
		gapp->v[i].x = points[i].x;
		gapp->v[i].y = points[i].y;

		gapp->tang[i].x = 0;
		gapp->tang[i].y = 0;

		gapp->flag[i] = 0;
	}
}


void
image4_copy_in_transparent( image_type *sim, image_type *im, 
						   int row0, int col0, u_int opacity )
{
///////////////////////////////////////
// Error if row<0 || col<0 || im<sim
///////////////////////////////////////

u_int	*p,	*sp;//, trans;
int	i,	j,	algin;
//u_int R, G, B, Rb, Gb, Bb;

	
	algin = im->column - sim->column;
	sp = (u_int *)sim->data;
	p = IMAGE4_PIXEL( im, row0, col0 );

	for( i = 0 ; i < sim->row ; i++, p += algin ){
		for( j = 0 ; j < sim->column ; j++, p++, sp++ )
			if ( !(*sp & 0xFF000000 ) && (opacity==0) )
				*p = *sp;
			else if ( ((*sp & 0xFF000000 ) < 0x4F000000) &&
						(opacity==0) )
				*p = *sp;

/*
			else if ( (*sp & 0xFF000000 ) != 0xFF000000 &&
				      (opacity!=0xFF) )
			{
				R = ((*sp) >> 16) & 0xFF; 
				G = ((*sp) >> 8) & 0xFF;
				B = (*sp) & 0xFF;

				Rb = ((*p) >> 16) & 0xFF; 
				Gb = ((*p) >> 8) & 0xFF;
				Bb = (*p) & 0xFF;

				trans = ((*sp) >> 24) + opacity;
				if ( trans > 255 ) trans = 255;

				R = ( R - ((R-Rb)*trans)/255) & 0xFF;
				G = ( G - ((G-Gb)*trans)/255) & 0xFF;
				B = ( B - ((B-Bb)*trans)/255) & 0xFF;

				*p = IMAGE4_RGB(R,G,B);
			}
*/
	}
}


void
image4_clear_transparent( image_type *image )
{
int x, y;
u_int *pData;

	pData = (u_int *)image->data;

    for ( y=0; y < image->row; y++)
    for ( x=0; x < image->column; x++, pData++ )
		*pData &= 0x00FFFFFF;
}


static u_int *
image4_find_border_first( image_type *image, int *ir, int *jr )
{
u_int	*p, *pb, *pr;
int		i, j;

	pr = NULL;
	for( i = 0; i < image->row && pr == NULL; i++ )
	{
		p = IMAGE4_PIXEL( image, i, 1 );
		for( j = 1 ; j < image->column && pr == NULL; j++, p++ )
		{
			pb = p - 1;
			if ( (*pb == 0xFF000000) && (*p != 0xFF000000) )
			{
				pr = p;
				*ir = i;
				*jr = j;
			}
		}
	}

	return pr;
}


static int
image4_find_num_points( image_type *image )
{
u_int	*p, Fp;
int		i, j, num;
	
	num = 0;

	for( i = 1; i < image->row-1; i++ )
	{
		p = IMAGE4_PIXEL( image, i, 1 );
		for( j = 1 ; j < image->column-1; j++, p++ )
		{
			Fp = *p & 0xFF000000;
			if ( ( Fp != (*(p-1) & 0xFF000000) ) ||
				 ( Fp != (*(p+1) & 0xFF000000) ) ||
				 ( Fp != (*(p-image->column) & 0xFF000000) ) ||
				 ( Fp != (*(p+image->column) & 0xFF000000) ) )
				num++;
		}
	}

	return num;
}


static vec2d *
image4_find_border( image_type *image, int *n_points )
{
double deltaX[4] = { 0.5, 1.0, 0.5, 0.0 };
double deltaY[4] = { 0.0, 0.5, 1.0, 0.5 };

int deltaI1[4] = { 1, 0, -1, 0 };
int deltaJ1[4] = { 0, 1, 0, -1 };
int deltaI2[4] = { 1, 1, -1, -1 };
int deltaJ2[4] = { -1, 1, 1, -1 };

u_int	*pr, *p;
u_int	*p1, *p2;
int		i, j;
int		i1, j1, i2, j2;
int     fl1, fl2;

vec2d   *points;
int		dir;
int		max_points;
	
	*n_points = 0;

	if ( (pr = image4_find_border_first( image, &i, &j )) == NULL )
		return NULL;
	dir = 0;

	max_points = image4_find_num_points( image )*4;
		//((image->row+1)*(image->column+1))/2;
	points = (vec2d *)malloc( max_points*sizeof( vec2d ) );
	points[*n_points].x = (double)i + deltaX[dir];
	points[*n_points].y = (double)j + deltaY[dir];
	(*n_points)++;

	do {
		fl1 = fl2 = 0;
		p = p1 = p2 = NULL;

		i1 = i + deltaI1[dir];
		j1 = j + deltaJ1[dir];
		if ( i1 < 0 || i1 >= image->row || j1 < 0 || j1 >= image->column )
			fl1 = 0;
		else
		{
			p1 = IMAGE4_PIXEL( image, i1, j1 );
			fl1 = (*p1) != 0xFF000000;
		}

		i2 = i + deltaI2[dir];
		j2 = j + deltaJ2[dir];
		if ( i2 < 0 || i2 >= image->row || j2 < 0 || j2 >= image->column )
			fl2 = 0;
		else
		{
			p2 = IMAGE4_PIXEL( image, i2, j2 );
			fl2 = (*p2) != 0xFF000000;
		}

		if ( !fl1 && fl2 )
		{
			//fl2 = 0;
			*n_points = 0;
			free( points );
			return NULL;
		}

		if ( fl1 && !fl2 )
		{
			i = i1;
			j = j1;
			p = p1;
		}
		else if ( fl1 && fl2 )
		{
			i = i2;
			j = j2;
			dir = dir == 0 ? 3 : dir-1;
			p = p2;
		}
		else if ( !fl1 && !fl2 )
			dir = dir == 3 ? 0 : dir+1;

		points[*n_points].x = (double)i + deltaX[dir];
		points[*n_points].y = (double)j + deltaY[dir];
		(*n_points)++;

	} while ( p != pr && *n_points < max_points);

	if ( *n_points == max_points )
	{
		*n_points = 0;
		free( points );
		return NULL;
	}

	return points;
}



void 
set_view( adepth_type *ad,
				int row0, int col0, int row, int col )
{
vec2d	p,	vp1, vp2;

	p.x = row0;
	p.y = col0;
	view_pixel2vp( ad->vw, &p, &vp1 );

	p.x = row0 + row;
	p.y = col0 + col;
	view_pixel2vp( ad->vw, &p, &vp2 );

	ad->vw->uv.x0 = vp1.x;
	ad->vw->uv.y1 = vp1.y;
	ad->vw->uv.x1 = vp2.x;
	ad->vw->uv.y0 = vp2.y;

	view_set_pixel( ad->vw, row, col );
}



void
image4_filter( image_type *image )
{
	while( image4_filter1( image ) );
	while( image4_filter2( image ) );
}

static int
image4_filter1( image_type *image )
{
u_int	*p;
int		i, j, n;
int Fl_i, Fl1_i, Fl2_i, Fl3_i, Fl4_i, Fl14_i;
int Fl_b, Fl1_b, Fl2_b, Fl3_b, Fl4_b, Fl14_b;

	n = 0;

	for( i = 1; i < image->row-1; i++ )
	{
		p = IMAGE4_PIXEL( image, i, 1 );
		for( j = 1 ; j < image->column-1; j++, p++ )
		{
			Fl_i= Fl1_i= Fl2_i= Fl3_i= Fl4_i= Fl14_i= 0;
			Fl_b= Fl1_b= Fl2_b= Fl3_b= Fl4_b= Fl14_b= 0;

			if (*p == 0xFF000000) 
				Fl_b = 1;
			else
				Fl_i = 1;

			if ( *(p-1) == 0xFF000000 )
				Fl1_b = 1;
			else
				Fl1_i = 1;

			if ( *(p+1) == 0xFF000000 )
				Fl2_b = 1;
			else
				Fl2_i = 1;

			if ( *(p-image->column) == 0xFF000000 ) 
				Fl3_b = 1;
			else
				Fl3_i = 1;
				
			if ( *(p+image->column) == 0xFF000000 ) 
				Fl4_b = 1;
			else
				Fl4_i = 1;
			
			Fl14_b = Fl1_b+ Fl2_b+ Fl3_b+ Fl4_b;
			Fl14_i = Fl1_i+ Fl2_i+ Fl3_i+ Fl4_i;
			
			if ( Fl_i && Fl14_b >= 3 )
			{
				*p = 0xFF000000;
				n++;
			}
		}
	}

	return 0;
}

static int
image4_filter2( image_type *image )
{
u_int	*p;
int		i, j, n;
int Fl_i, Fl1_i, Fl2_i, Fl3_i, Fl4_i, Fl14_i;
int Fl_b, Fl1_b, Fl2_b, Fl3_b, Fl4_b, Fl14_b;

	n = 0;

	for( i = 1; i < image->row-1; i++ )
	{
		p = IMAGE4_PIXEL( image, i, 1 );
		for( j = 1 ; j < image->column-1; j++, p++ )
		{
			Fl_i= Fl1_i= Fl2_i= Fl3_i= Fl4_i= Fl14_i= 0;
			Fl_b= Fl1_b= Fl2_b= Fl3_b= Fl4_b= Fl14_b= 0;

			if (*p == 0xFF000000) 
				Fl_b = 1;
			else
				Fl_i = 1;

			if ( *(p-1) == 0xFF000000 )
				Fl1_b = 1;
			else
				Fl1_i = 1;

			if ( *(p+1) == 0xFF000000 )
				Fl2_b = 1;
			else
				Fl2_i = 1;

			if ( *(p-image->column) == 0xFF000000 ) 
				Fl3_b = 1;
			else
				Fl3_i = 1;
				
			if ( *(p+image->column) == 0xFF000000 ) 
				Fl4_b = 1;
			else
				Fl4_i = 1;
			
			Fl14_b = Fl1_b+ Fl2_b+ Fl3_b+ Fl4_b;
			Fl14_i = Fl1_i+ Fl2_i+ Fl3_i+ Fl4_i;
				
			if ( Fl_b && Fl14_i >= 3 )
			{
				if ( Fl1_i )
					*p = *(p-1);
				else
					*p = *(p+1);
				n++;
			}

		}
	}

	return 0;
}
