/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/***************************
 ***	AgpMoveLayer.c   ***
 **************************/
#include	<stdio.h>   
#include	<math.h>

#include	"AgpTool/AgpTool.h"
#include	"AgpLayer/AgpLayer.h"



static adepth_type *	agp_reposition_layer_ad( gp_array *agp, box2d *b );

static void	agp_reposition_layer_NF( gp_array *agp, vec2d av[], gp_array *uagp );



gp_array *
agp_reposition_layer( gp_array *sagp, int iLayer, vec2d *dp )
{
box2d	ab[MAX_LAYER],	b;
gp_array	*agp;
layer_type	*layer;
vec2d	ctr,	vp,	gctr,	av[256];
int	i,	j;
int	row,	column,	mbr;

	for( i = 0 ; i < sagp->layer_no ; i++ )
		agp_layer_box( sagp, sagp->layer[i]->Mbr, &ab[i] );


	if( iLayer >= 0 ){
		ab[iLayer].x0 += dp->x;
		ab[iLayer].x1 += dp->x;
		ab[iLayer].y0 += dp->y;
		ab[iLayer].y1 += dp->y;
	}


	box2d_union( ab, sagp->layer_no, &b );
	box2d_round( &b );


	if( iLayer >= 0 ){
		ab[iLayer].x0 -= dp->x;
		ab[iLayer].x1 -= dp->x;
		ab[iLayer].y0 -= dp->y;
		ab[iLayer].y1 -= dp->y;
	}


	row = b.x1 - b.x0;
	column = b.y1 - b.y0;


	agp = agp_alloc();
	agp_init( agp, sagp->bgp, row, column, 48, 48 );

	agp->ad = agp_reposition_layer_ad( sagp, &b );




	agp->color_type = sagp->color_type;

	agp->type = AGP_TRANSPARENT_U;


	agp->Lcoding = sagp->Lcoding;


	ctr.x = ctr.y = 0;
	for( i = 0 ; i < sagp->layer_no ; i++ ){

		if( iLayer != i )
				view_pixel2vp( sagp->ad->vw, &ctr, &vp );
		else	view_pixel2vp( sagp->ad->vw, dp, &vp );

        view_vp2pixel( agp->ad->vw, &vp, &gctr );


		for( j = 0, mbr = sagp->layer[i]->Mbr ; mbr != 0 ; j++, mbr >>= 1 )
			if( mbr & 0x1 )
				av[j] = gctr;


//       agp_union1( sagp, &gctr, sagp->layer[i]->Mbr, agp );


		layer = layer_alloc();
		*layer = *sagp->layer[i];


		layer->i0 = ( ab[i].x0 + gctr.x + agp->bgp )* agp->inv_bgp;
		layer->j0 = ( ab[i].y0 + gctr.y + agp->bgp )* agp->inv_bgp;

		layer->i1 = ( ab[i].x1 + gctr.x + agp->bgp )* agp->inv_bgp;
		layer->j1 = ( ab[i].y1 + gctr.y + agp->bgp )* agp->inv_bgp;


		agp->layer[ agp->layer_no++] = layer;

	}


	agp_reposition_layer_NF( sagp, av, agp );

	agp_destroy( sagp );


	return( agp );
}





static adepth_type *
agp_reposition_layer_ad( gp_array *agp, box2d *b )
{
adepth_type	*nad;
int     row,    column;
vec2d	p,	vp;
int	k;


	
	nad = ADEPTH_ALLOC();
	nad->P = agp->ad->P;
	nad->N = agp->ad->N;
	nad->br_no = 0;
	nad->Mbr = 0;

	nad->vw = VIEW_ALLOC();
	*nad->vw = *agp->ad->vw;


	for( k = 0 ; k < agp->ad->br_no ; k++ )
		nad->abr[ nad->br_no++ ] = agp->ad->abr[k];

	nad->Mbr = agp->ad->Mbr;



	p.x = b->x0;
	p.y = b->y0;
	view_pixel2vp( agp->ad->vw, &p, &vp );

	nad->vw->uv.x0 = vp.x;
	nad->vw->uv.y1 = vp.y;



	p.x = b->x1;
	p.y = b->y1;
	view_pixel2vp( agp->ad->vw, &p, &vp );

	nad->vw->uv.x1 = vp.x;
	nad->vw->uv.y0 = vp.y;

 
	row = b->x1 - b->x0;
				
	column = b->y1 - b->y0;
			
	view_set_pixel( nad->vw, row, column );


	return( nad );
}




static void
agp_reposition_layer_NF( gp_array *agp, vec2d av[], gp_array *uagp )
{
gp_type	*gp,	*ugp;
int	i,	j;
int	ui,	uj;
vec2d	ctr,	*gctr;

bgp_type	*bgp;
hunch_type	*h;
link_type	*l;
jnt_type	*jnt;


	for( i = 0 ; i < agp->Ni ; i++ )
		for( j = 0, gp = agp->gp[i] ; j < agp->Nj ; j++, gp++ ){

			while( (bgp = gp->bgp ) != NULL ){
				gp->bgp = gp->bgp->p;

				gctr = &av[ bgp->br];

				ctr.x = gp->ctr.x + bgp->ctr.x + gctr->x;
				ctr.y = gp->ctr.y + bgp->ctr.y + gctr->y;

				ui = (ctr.x+ uagp->bgp) * uagp->inv_bgp;
				uj = (ctr.y+ uagp->bgp) * uagp->inv_bgp;

				ugp = &uagp->gp[ui][uj];

				bgp->ctr.x = ctr.x - ugp->ctr.x;
				bgp->ctr.y = ctr.y - ugp->ctr.y;
				bgp->p = ugp->bgp;
				ugp->bgp = bgp;
			}



			while( (h = gp->hunch ) != NULL ){
				gp->hunch = gp->hunch->nh;

				gctr = &av[ h->br];

				ctr.x = gp->ctr.x + h->ctr.x + gctr->x;
				ctr.y = gp->ctr.y + h->ctr.y + gctr->y;

				ui = (ctr.x+ uagp->bgp) * uagp->inv_bgp;
				uj = (ctr.y+ uagp->bgp) * uagp->inv_bgp;

				ugp = &uagp->gp[ui][uj];

				h->ctr.x = ctr.x - ugp->ctr.x;
				h->ctr.y = ctr.y - ugp->ctr.y;
				h->nh = ugp->hunch;
				ugp->hunch = h;
			}


			while( (l = gp->link ) != NULL ){
				gp->link = gp->link->nl;

				gctr = &av[ l->br];

				ctr.x = gp->ctr.x + l->ctr.x + gctr->x;
				ctr.y = gp->ctr.y + l->ctr.y + gctr->y;

				ui = (ctr.x+ uagp->bgp) * uagp->inv_bgp;
				uj = (ctr.y+ uagp->bgp) * uagp->inv_bgp;

				ugp = &uagp->gp[ui][uj];
				l->ctr.x = ctr.x - ugp->ctr.x;
				l->ctr.y = ctr.y - ugp->ctr.y;
				l->nl = ugp->link;
				ugp->link = l;
			}


			while( (jnt = gp->jnt ) != NULL ){
				gp->jnt = gp->jnt->nj;

				gctr = &av[ jnt->brn[0]->link->br];

				ctr.x = gp->ctr.x + jnt->ctr.x + gctr->x;
				ctr.y = gp->ctr.y + jnt->ctr.y + gctr->y;

				ui = (ctr.x+ uagp->bgp) * uagp->inv_bgp;
				uj = (ctr.y+ uagp->bgp) * uagp->inv_bgp;

				ugp = &uagp->gp[ui][uj];
				jnt->gctr = ctr;
				jnt->ctr.x = ctr.x - ugp->ctr.x;
				jnt->ctr.y = ctr.y - ugp->ctr.y;
				jnt->nj = ugp->jnt;
				ugp->jnt = jnt;
			}
		}
}
