/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**************************
 ***  AgpScriptRead.c  ***
 **************************/
#include    <string.h>
#include    <stdio.h>   
#include	<math.h>

#include    "AgpTool.h"
#include    "AgpEdit/AgpEditTool.h"


//#define FLOAT_SHORT_PLACES 4		//1 << 4
#define FLOAT_SHORT 16.0F
//#define FLOAT_LONG_PLACES 16		//1 << 16
#define FLOAT_LONG 65536.0F

#define NL 48

enum FILE_DUMP_TYPE {
	FILE_DUMP_TEXT = 0,
	FILE_DUMP_BINARY,
	FILE_DUMP_TEXT_F2INT,
	FILE_DUMP_BINARY_F2INT
};

enum ENUM_OBJECT_SCRIPT {
	OBJECT_AGP = 0,
	OBJECT_JOINT,
	OBJECT_POLYLINK,
	OBJECT_LINK,
	OBJECT_LINK_PROFILE,
	OBJECT_HUNCH,
	OBJECT_BGP
};

static char *str_object_vim[7] = {
	"TEXTURE", 
	"TP", 
	"LN", 
	"LS", 
	"LC", 
	"PA", 
	"AC" };

static FILE *agp_script_read_init( char *path, char *name, 
						enum FILE_DUMP_TYPE *e_dump_type );

static void agp_script_read_agp( FILE *flDump, 
						enum FILE_DUMP_TYPE e_dump_type,
						gp_array *agp, int bgp, int *npolylink, int *nhunch, 
						int *nbgp, int *njnt );
static void agp_script_read_agp_numbers( FILE *flDump, 
						enum FILE_DUMP_TYPE e_dump_type,
						int *npolylink, int *nhunch, 
						int *nbgp, int *njnt, int *layer_no );

static void agp_script_read_all_polylink( 
				FILE *flDump, enum FILE_DUMP_TYPE e_dump_type,
				link_type **links, int npolylink );
static void agp_script_read_all_hunch( 
				FILE *flDump, enum FILE_DUMP_TYPE e_dump_type,
				hunch_type **hunchs, int nhunch);
static void agp_script_read_all_bgp( 
				FILE *flDump, enum FILE_DUMP_TYPE e_dump_type,
				bgp_type **bgps, int nbgp );
static void agp_script_read_all_joint( FILE *flDump, enum FILE_DUMP_TYPE e_dump_type,
							    jnt_type **joints, int njnt );

static link_type *agp_script_read_polylink( 
				FILE *flDump, enum FILE_DUMP_TYPE e_dump_type );
static link_type *agp_script_read_link( 
				FILE *flDump, enum FILE_DUMP_TYPE e_dump_type );
static void agp_script_read_link_profile( 
				FILE *flDump, enum FILE_DUMP_TYPE e_dump_type,
				link_type *link, int dir );
static profile_type agp_script_read_edge( 
				FILE *flDump, enum FILE_DUMP_TYPE e_dump_type );
static profile_type agp_script_read_ridge( 
				FILE *flDump, enum FILE_DUMP_TYPE e_dump_type );

static hunch_type *agp_script_read_hunch( 
				FILE *flDump, enum FILE_DUMP_TYPE e_dump_type );
static bgp_type *agp_script_read_bgp( 
				FILE *flDump, enum FILE_DUMP_TYPE e_dump_type );
static jnt_type *agp_script_read_joint( 
				FILE *flDump, enum FILE_DUMP_TYPE e_dump_type );

static void agp_script_read_insert_joints( gp_array *agp, 
							jnt_type **joints, int njnt );
static void agp_script_read_insert_polylinks( gp_array *agp, 
			link_type **links, int npolylink, jnt_type **joints );
static void agp_script_read_insert_hunchs( gp_array *agp, 
							hunch_type **hunchs, int nhunch );
static void agp_script_read_insert_bgps( gp_array *agp, 
							bgp_type **bgps, int nbgp );

static void	agp_script_read_color( 
				FILE *flDump, enum FILE_DUMP_TYPE e_dump_type, 
				qcolor_type *c );
static void agp_script_read_object_index( 
				FILE *flDump, enum FILE_DUMP_TYPE e_dump_type,
				enum ENUM_OBJECT_SCRIPT *object_vim, int *index );
static void agp_script_read_int( FILE *flDump, 
				enum FILE_DUMP_TYPE e_dump_type, int *i );
static void agp_script_read_float_short( FILE *flDump, 
				enum FILE_DUMP_TYPE e_dump_type, float *f );
static void agp_script_read_hex( FILE *flDump, 
			    enum FILE_DUMP_TYPE e_dump_type, int *i );
static void agp_script_read_comma(  FILE *flDump, 
				enum FILE_DUMP_TYPE e_dump_type );


gp_array *
agp_script_read( char *path, char *name, int bgp )
{
gp_array	*agp;
FILE		*flDump;
enum FILE_DUMP_TYPE e_dump_type;
link_type	**links;
hunch_type  **hunchs;
bgp_type	**bgps;
jnt_type	**joints;
int npolylink, nhunch, nbgp, njnt;

	flDump = agp_script_read_init( path, name, &e_dump_type );
	if ( flDump == NULL )
		return NULL;

	if ( (agp = agp_alloc()) == NULL )
		return NULL;

	agp_script_read_agp( flDump, e_dump_type, agp, bgp, 
					&npolylink, &nhunch, &nbgp, &njnt );

	links = (link_type **)malloc( sizeof(link_type *)*npolylink );
	hunchs= (hunch_type **)malloc( sizeof(hunch_type *)*nhunch );
	bgps  = (bgp_type **)malloc( sizeof(bgp_type *)*nbgp );
	joints = (jnt_type **)malloc( sizeof(jnt_type *)*njnt );

	agp_script_read_all_polylink( flDump, e_dump_type, links, npolylink );
	agp_script_read_all_hunch( flDump, e_dump_type, hunchs, nhunch );
	agp_script_read_all_bgp( flDump, e_dump_type, bgps, nbgp );
	agp_script_read_all_joint( flDump, e_dump_type, joints, njnt );

	agp_script_read_insert_joints( agp, joints, njnt );
	agp_script_read_insert_polylinks( agp, links, npolylink, joints );
	agp_script_read_insert_hunchs( agp, hunchs, nhunch );
	agp_script_read_insert_bgps( agp, bgps, nbgp );

	free( joints );
	free( bgps );
	free( hunchs );
	free( links );

	agp_geomtric_aux( agp );
	agp_link_in_reorder( agp );

	fclose( flDump );

	return agp;
}
  
static FILE *
agp_script_read_init( char *path, char *name, 
					 enum FILE_DUMP_TYPE *e_dump_type )
{
FILE *flDump;
char ss[128];
char c_dump_type_binary;

	sprintf( ss, "%s/%s.srp", path, name );
	flDump = fopen( ss, "rb" );

	if ( !flDump )
		return NULL;

	fread( &c_dump_type_binary, sizeof( char ), 1, flDump );
	*e_dump_type = (enum FILE_DUMP_TYPE)(c_dump_type_binary - '0');

	if ( *e_dump_type == FILE_DUMP_TEXT ||
		 *e_dump_type == FILE_DUMP_TEXT_F2INT )
	{
		fclose( flDump );
		flDump = fopen( ss, "rt" );
		if ( !flDump )
			return NULL;
		fgets( ss, 128, flDump );
	}

	return flDump;
}

static void
agp_script_read_agp( FILE *flDump, enum FILE_DUMP_TYPE e_dump_type,
						gp_array *agp, int bgp, int *npolylink, int *nhunch, 
						int *nbgp, int *njnt )
{
enum ENUM_OBJECT_SCRIPT	object;
int  index;
int	 ni, nj;
int	 i;
int  type;

	agp_script_read_object_index( flDump, e_dump_type, &object, &index );
	agp_script_read_int( flDump, e_dump_type, &type );
	agp_script_read_int( flDump, e_dump_type, &ni );
	agp_script_read_comma( flDump, e_dump_type );
	agp_script_read_int( flDump, e_dump_type, &nj );
	agp_script_read_agp_numbers( flDump, e_dump_type, 
			npolylink, nhunch, nbgp, njnt, &agp->layer_no );

	agp_init( agp, bgp, ni, nj, NL, NL );
	agp->type = type;
	agp->Ni0 = agp->Nj0 = 1;
	agp->Lcoding = 1;
	agp->color_type = AGP_RGB;

	agp->ad = adepth_default( 8.0F, ni, nj );

	for ( i = 0; i < agp->layer_no; i++ )
	{
		agp->layer[i] = LAYER_ALLOC();
		agp->layer[i]->Fdepth_delta = 0;
		agp->layer[i]->Mbr = 1 << (i+1);
		agp->layer[i]->i0 = 1;
		agp->layer[i]->j0 = 1;
		agp->layer[i]->i1 = agp->Ni-1;
		agp->layer[i]->j1 = agp->Nj-1;

		adepth_set_plane( agp->ad, i+1, &agp->ad->N, &agp->ad->P );
	}
}

static void
agp_script_read_agp_numbers( FILE *flDump, enum FILE_DUMP_TYPE e_dump_type,
		int *npolylink, int *nhunch, int *nbgp, int *njnt, int *layer_no )
{
int ncontour, nskl, n, i;

	*npolylink = *nhunch = *nbgp = *njnt = *layer_no = 0;

	agp_script_read_int( flDump, e_dump_type, npolylink );
	agp_script_read_int( flDump, e_dump_type, nhunch );
	agp_script_read_int( flDump, e_dump_type, nbgp );
	agp_script_read_int( flDump, e_dump_type, njnt );

	//Number of Sub-Textures in this texture:
	agp_script_read_int( flDump, e_dump_type, layer_no );

	//corresponds to "Contour" property of the texture
	agp_script_read_int( flDump, e_dump_type, &ncontour );
	for ( i = 0; i < ncontour; i++ )
		agp_script_read_int( flDump, e_dump_type, &n );

	//control points - skl
	agp_script_read_int( flDump, e_dump_type, &nskl );
	for ( i = 0; i < nskl; i++ )
		agp_script_read_int( flDump, e_dump_type, &n );

}

static void 
agp_script_read_all_polylink( FILE *flDump, enum FILE_DUMP_TYPE e_dump_type,
							  link_type **links, int npolylink )
{
int		i;

	for ( i = 0; i < npolylink; i++ )
		links[i] = agp_script_read_polylink( flDump, e_dump_type );
}

static void 
agp_script_read_all_hunch( FILE *flDump, enum FILE_DUMP_TYPE e_dump_type,
						   hunch_type **hunchs, int nhunch )
{
int		i;

	for ( i = 0; i < nhunch; i++ )
		hunchs[i] = agp_script_read_hunch( flDump, e_dump_type );
}

static void 
agp_script_read_all_bgp( FILE *flDump, enum FILE_DUMP_TYPE e_dump_type,
						   bgp_type **bgps, int nbgp )
{
int		i;

	for ( i = 0; i < nbgp; i++ )
		bgps[i] = agp_script_read_bgp( flDump, e_dump_type );
}

static void 
agp_script_read_all_joint( FILE *flDump, enum FILE_DUMP_TYPE e_dump_type,
						   jnt_type **joints, int njnt )
{
int		i;

	for ( i = 0; i < njnt; i++ )
		joints[i] = agp_script_read_joint( flDump, e_dump_type );
}

static link_type *
agp_script_read_polylink( FILE *flDump, enum FILE_DUMP_TYPE e_dump_type )
{
enum ENUM_OBJECT_SCRIPT	object;
int			index;
link_type	*link, *link_prev, *link_start, *link_end;
int			link_type, separating_flag;
int			nlinks, nprofiles;
int			jnt_no1, jnt_no2;
int			bno1, bno2;
vec2d		gctr;
int			i;
int			contour_flag, br1, br2;

	agp_script_read_object_index( flDump, e_dump_type, &object, &index );
	agp_script_read_int( flDump, e_dump_type, &link_type );
	agp_script_read_int( flDump, e_dump_type, &separating_flag );
	agp_script_read_int( flDump, e_dump_type, &contour_flag );
	agp_script_read_int( flDump, e_dump_type, &nlinks );
	agp_script_read_int( flDump, e_dump_type, &nprofiles );
	agp_script_read_int( flDump, e_dump_type, &jnt_no1 );
	agp_script_read_comma( flDump, e_dump_type );
	agp_script_read_int( flDump, e_dump_type, &bno1 );
	agp_script_read_int( flDump, e_dump_type, &jnt_no2 );
	agp_script_read_comma( flDump, e_dump_type );
	agp_script_read_int( flDump, e_dump_type, &bno2 );
	agp_script_read_float_short( flDump, e_dump_type, &gctr.x );
	agp_script_read_comma( flDump, e_dump_type );
	agp_script_read_float_short( flDump, e_dump_type, &gctr.y );
	agp_script_read_int( flDump, e_dump_type, &br1 );
	agp_script_read_comma( flDump, e_dump_type );
	agp_script_read_int( flDump, e_dump_type, &br2 );
	if ( br1 && br2 && br1 != br2 )
		contour_flag = 4;

	link_start = NULL;
	for ( i = 0; i < nlinks; i++ )
	{
		link = agp_script_read_link( flDump, e_dump_type );
		link_set_aux( link );
		link->type = link_type == 0 ? NF_EDGE : NF_RIDGE;
		if ( link_start == NULL )
		{
			link_prev = link_start = link;
			continue;
		}
		link_prev->p[F_END].bno = -1;
		link_prev->p[F_END].p.l = link;
		link->p[F_BEGIN].bno = -1;
		link->p[F_BEGIN].p.l = link_prev;
		link_prev = link;
	}

	link_end = link;

	link_start->p[F_BEGIN].bno = bno1;
	link_start->p[F_BEGIN].p.jnt = (jnt_type *)jnt_no1;
	link_start->br1 = jnt_no1;
	link_end->p[F_END].bno = bno2;
	link_end->p[F_END].p.jnt = (jnt_type *)jnt_no2;

	link_prev = 0;
	for ( link = link_start; link; link = LINK_NEXT( link ) )
	{
		agp_script_read_link_profile( flDump, e_dump_type, 
										link, F_BEGIN );
		if ( link_prev != NULL )
			link_prev->pr[F_END] = link->pr[F_BEGIN];

		link->Fbr = contour_flag;
		link->br = br2;
		link->br1 = br1;

		if ( link->Fbr == 1 ) link->br = br1;

		link_prev = link;
	}

	agp_script_read_link_profile( flDump, e_dump_type, 
										link_end, F_END );

	return link_start;
}

static link_type *
agp_script_read_link( FILE *flDump, enum FILE_DUMP_TYPE e_dump_type )
{
enum ENUM_OBJECT_SCRIPT	object;
int			index;
link_type	*link;

	link = link_1_alloc( );
	agp_script_read_object_index( flDump, e_dump_type, &object, &index );
	agp_script_read_float_short( flDump, e_dump_type, &link->v.x );
	agp_script_read_comma( flDump, e_dump_type );
	agp_script_read_float_short( flDump, e_dump_type, &link->v.y );
	agp_script_read_float_short( flDump, e_dump_type, &link->a );

	return link;
}

static void 
agp_script_read_link_profile( FILE *flDump, enum FILE_DUMP_TYPE e_dump_type,
							    link_type *link, int dir )
{
enum ENUM_OBJECT_SCRIPT	object;
int			index;

	agp_script_read_object_index( flDump, e_dump_type, &object, &index );
	switch( link->type )
	{
		case NF_EDGE:
			link->pr[dir] = agp_script_read_edge( flDump, e_dump_type );
		break;
		case NF_RIDGE:
			link->pr[dir] = agp_script_read_ridge( flDump, e_dump_type );
		break;
	}
}

static profile_type 
agp_script_read_edge( FILE *flDump, enum FILE_DUMP_TYPE e_dump_type )
{
int A;
profile_type pr;
edge_type *e;
float width_2;
qcolor_type color0;
	
	pr = profile_alloc( NF_EDGE );
	e = pr.edge;

	agp_script_read_float_short( flDump, e_dump_type, &width_2 );
	agp_script_read_comma( flDump, e_dump_type );
	agp_script_read_float_short( flDump, e_dump_type, &width_2 );

	e->width = width_2*2.0F;
	
	for ( A = 0; A < SP_NO; A++ )
	{
 		agp_script_read_color( flDump, e_dump_type, &e->side[F_LEFT].bval[A] );
		agp_script_read_comma( flDump, e_dump_type );
		agp_script_read_color( flDump, e_dump_type, &e->side[F_LEFT].val[A] );
		agp_script_read_comma( flDump, e_dump_type );
		agp_script_read_color( flDump, e_dump_type, &e->side[F_RIGHT].bval[A] );
		agp_script_read_comma( flDump, e_dump_type );
		agp_script_read_color( flDump, e_dump_type, &e->side[F_RIGHT].val[A] );
		agp_script_read_comma( flDump, e_dump_type );
		agp_script_read_color( flDump, e_dump_type, &color0 );
	}

	agp_script_read_float_short( flDump, e_dump_type, &e->depth );

	return pr;
}


static profile_type 
agp_script_read_ridge( FILE *flDump, enum FILE_DUMP_TYPE e_dump_type )
{
int A;
profile_type pr;
ridge_type *e;
qcolor_type bg[SP_NO];
float w1, w2, width2;

	
	pr = profile_alloc( NF_RIDGE );
	e = pr.ridge;

	agp_script_read_float_short( flDump, e_dump_type, &w1 );
	agp_script_read_comma( flDump, e_dump_type );
	agp_script_read_float_short( flDump, e_dump_type, &w2 );
	e->width = w1+w2;

	for ( A = 0; A < SP_NO; A++ )
	{
 		agp_script_read_color( flDump, e_dump_type, &bg[A] );
		agp_script_read_comma( flDump, e_dump_type );
 		agp_script_read_color( flDump, e_dump_type, &bg[A] );
		agp_script_read_comma( flDump, e_dump_type );
		agp_script_read_color( flDump, e_dump_type, &bg[A] );
		agp_script_read_comma( flDump, e_dump_type );
		agp_script_read_color( flDump, e_dump_type, &bg[A] );
		agp_script_read_comma( flDump, e_dump_type );
		agp_script_read_color( flDump, e_dump_type, &e->mval[A] );
	}

	width2 = e->width*e->width;
	for ( A = 0; A < SP_NO; A++ )
	{
		e->lm[A] = (qcolor_type)((bg[A] - e->mval[A])/ width2);
		e->rval[A] = -(qcolor_type)(e->lm[A]* width2);

		//e->rval[A] = e->mval[A] - bg[A];
		//e->lm[A] = - (qcolor_type)(((float)e->rval[A]) / width2);
	}

	agp_script_read_float_short( flDump, e_dump_type, &e->depth );

	return pr;
}

static hunch_type *
agp_script_read_hunch( FILE *flDump, enum FILE_DUMP_TYPE e_dump_type )
{
enum ENUM_OBJECT_SCRIPT	object;
int	index;
hunch_type *h;
float len_2, width_2;
int alpha;
int ibr;
int A;
qcolor_type bg[SP_NO];

	h = HUNCH_ALLOC( 1 );
	agp_script_read_object_index( flDump, e_dump_type, &object, &index );
	agp_script_read_float_short( flDump, e_dump_type, &h->ctr.x );
	agp_script_read_comma( flDump, e_dump_type );
	agp_script_read_float_short( flDump, e_dump_type, &h->ctr.y );
	agp_script_read_float_short( flDump, e_dump_type, &len_2 );
	agp_script_read_comma( flDump, e_dump_type );
	agp_script_read_float_short( flDump, e_dump_type, &width_2 );
	h->g.len = len_2 * 2.0F;
	h->g.width = width_2 * 2.0F;
	agp_script_read_int( flDump, e_dump_type, &alpha );
	h->g.theta = ((float)alpha)*((float)M_PI)/180.0F;

	for ( A = 0; A < SP_NO; A++ )
	{
		agp_script_read_color( flDump, e_dump_type, &h->gval[A] );
		agp_script_read_comma( flDump, e_dump_type );
		agp_script_read_color( flDump, e_dump_type, &bg[A] );
	}

	for ( A = 0; A < SP_NO; A++ )
		h->rval[A] = h->gval[A] - bg[A];

	agp_script_read_int( flDump, e_dump_type, &ibr );
	h->br = (u_char)ibr;
	agp_script_read_float_short( flDump, e_dump_type, &h->depth );
	h->type = NF_HUNCH;

	hunch_set_aux( h );

	return h;
}


static bgp_type *
agp_script_read_bgp( FILE *flDump, enum FILE_DUMP_TYPE e_dump_type )
{
enum ENUM_OBJECT_SCRIPT	object;
int	index;
bgp_type *bgp;
int A;
int ibr;

	bgp = BGP_ALLOC();

	agp_script_read_object_index( flDump, e_dump_type, &object, &index );
	agp_script_read_float_short( flDump, e_dump_type, &bgp->ctr.x );
	agp_script_read_comma( flDump, e_dump_type );
	agp_script_read_float_short( flDump, e_dump_type, &bgp->ctr.y );
	for ( A = 0; A < SP_NO; A++ )
	{
		agp_script_read_color( flDump, e_dump_type, &bgp->val[A] );
		agp_script_read_comma( flDump, e_dump_type );
	}
	agp_script_read_int( flDump, e_dump_type, &ibr );
	bgp->br = (u_char)ibr;
	agp_script_read_float_short( flDump, e_dump_type, &bgp->depth );

	return bgp;
}


static jnt_type *
agp_script_read_joint( FILE *flDump, enum FILE_DUMP_TYPE e_dump_type )
{
enum ENUM_OBJECT_SCRIPT	object;
jnt_type *jnt;
int i;

	jnt = jnt_alloc();

	agp_script_read_object_index( flDump, e_dump_type, &object, &jnt->jnt_no );
	agp_script_read_float_short( flDump, e_dump_type, &jnt->gctr.x );
	agp_script_read_comma( flDump, e_dump_type );
	agp_script_read_float_short( flDump, e_dump_type, &jnt->gctr.y );
	agp_script_read_int( flDump, e_dump_type, &jnt->type );
	agp_script_read_int( flDump, e_dump_type, &jnt->brn_no );

	for ( i = 0; i < jnt->brn_no; i++)
		jnt->brn[i] = brn_alloc();

	return jnt;
}


/*static void
agp_script_read_insert_bgp( gp_array *agp, bgp_type *bgp )
{
int	i,	j;
gp_type	*gp;

	i = (int)((bgp->ctr.x+agp->bgp) * agp->inv_bgp);
    j = (int)((bgp->ctr.y+agp->bgp) * agp->inv_bgp);

	gp = &agp->gp[i][j];

	bgp->ctr.x -= gp->ctr.x;
	bgp->ctr.y -= gp->ctr.y;
	bgp->p = gp->bgp;
	gp->bgp = bgp;
}*/

static void 
agp_script_read_insert_joints( gp_array *agp, jnt_type **joints, int njnt )
{
int i;
	
	//for ( i = 0; i < njnt; i++ )
	for ( i = njnt-1; i >= 0; i-- )
		agp_jnt_insert( agp, joints[i] );
}

static void 
agp_script_read_insert_polylinks( gp_array *agp, 
				link_type **links, int npolylink, jnt_type **joints )
{
link_type *link_end;
brn_type *brn;
int jnt_no1, jnt_no2;
vec2d gctr;
int i;
	
	for ( i = 0; i < npolylink; i++ )
	{
		jnt_no1 = (int)links[i]->p[F_BEGIN].p.jnt;
		for ( link_end = links[i]; LINK_NEXT( link_end ); 
						link_end = LINK_NEXT( link_end ) );
		jnt_no2 = (int)link_end->p[F_END].p.jnt;

		jnt_connet_link( joints[jnt_no1], links[i]->p[F_BEGIN].bno, 
						 links[i], F_BEGIN );

		jnt_connet_link( joints[jnt_no2], link_end->p[F_END].bno, 
						 link_end, F_END );

		//agp_lnf_link( &joints[jnt_no1]->gctr, links[i], agp );
		
		brn = joints[jnt_no1]->brn[ links[i]->p[F_BEGIN].bno ];
		gctr.x = joints[jnt_no1]->gctr.x + brn->ctr.x;
		gctr.y = joints[jnt_no1]->gctr.y + brn->ctr.y;

		agp_lnf_link( &gctr, links[i], agp );
		link_set_geo_aux( links[i] );

	}
}

static void 
agp_script_read_insert_hunchs( gp_array *agp, hunch_type **hunchs, int nhunch )
{
int	i,	j;
gp_type	*gp;
int k;
	
	for ( k = 0; k < nhunch; k++ )
	{
  		i = (int)((hunchs[k]->ctr.x+agp->bgp) * agp->inv_bgp);
		j = (int)((hunchs[k]->ctr.y+agp->bgp) * agp->inv_bgp);
		gp = &agp->gp[i][j];

		hunchs[k]->ctr.x -= gp->ctr.x;
		hunchs[k]->ctr.y -= gp->ctr.y;
		hunchs[k]->nh = gp->hunch;
		gp->hunch = hunchs[k];
	}
}

static void 
agp_script_read_insert_bgps( gp_array *agp, bgp_type **bgps, int nbgp )
{
int	i,	j;
gp_type	*gp;
int k;
	
	for ( k = 0; k < nbgp; k++ )
	{
  		i = (int)((bgps[k]->ctr.x+agp->bgp) * agp->inv_bgp);
		j = (int)((bgps[k]->ctr.y+agp->bgp) * agp->inv_bgp);
		gp = &agp->gp[i][j];

		bgps[k]->ctr.x -= gp->ctr.x;
		bgps[k]->ctr.y -= gp->ctr.y;
		bgps[k]->p = gp->bgp;
		gp->bgp = bgps[k];
	}
}

static void
agp_script_read_color( FILE *flDump, enum FILE_DUMP_TYPE e_dump_type,
							    qcolor_type *c )
{
#ifdef	QFLOAT
int cc;

	agp_script_read_hex( flDump, e_dump_type, &cc );
	*c = ((float)cc)*256.0F;
#endif

#ifdef	QSHORT
int cc;

	agp_script_read_hex( flDump, e_dump_type, &cc );
	*c = (short)(cc<<4);
#endif
}

static void agp_script_read_object_index( FILE *flDump, enum FILE_DUMP_TYPE e_dump_type,
							    enum ENUM_OBJECT_SCRIPT *object_vim, int *index )
{
int i, i_object_vim;
char str_object[32], *p_index;

	if ( e_dump_type == FILE_DUMP_TEXT ||
		 e_dump_type == FILE_DUMP_TEXT_F2INT )
	{
		fscanf( flDump, "%s", str_object );
		p_index = str_object;
		while ( *p_index < '0' || *p_index > '9' ) 
			p_index++;

		if ( *p_index == 0 )
			return; //error

		sscanf( p_index, "%d", index );
		*p_index = 0;
		i_object_vim = -1;

		for ( i = 0; i < 7; i++ )
		if ( gp_strnicmp( str_object, str_object_vim[i], 31 )==0 )
		{
			i_object_vim = i;
			break; 
		}

		if ( i_object_vim == -1 )
			return; //error

		*object_vim = (enum ENUM_OBJECT_SCRIPT)i_object_vim;
	}
	else
	{
		fread( &i_object_vim, sizeof( int ), 1, flDump );
		*object_vim = ( enum ENUM_OBJECT_SCRIPT ) i_object_vim;
		fread( &index, sizeof( int ), 1, flDump );
	}
}

static void
agp_script_read_int( FILE *flDump, 
				   enum FILE_DUMP_TYPE e_dump_type, int *i )
{
	if ( e_dump_type == FILE_DUMP_TEXT ||
		 e_dump_type == FILE_DUMP_TEXT_F2INT )
		fscanf( flDump, "%d", i );
	else
		fread( i, sizeof( int ), 1, flDump );
}

static void
agp_script_read_float_short( FILE *flDump, 
				enum FILE_DUMP_TYPE e_dump_type, float *f )
{
int i;

	if ( e_dump_type == FILE_DUMP_TEXT )
		fscanf( flDump, "%f", f );
	else if ( e_dump_type == FILE_DUMP_BINARY )
		fread( f, sizeof( float ), 1, flDump );
	else
	{
		agp_script_read_int( flDump, e_dump_type, &i );
		*f = ( (float)i )/FLOAT_SHORT;
	}
}

static void
agp_script_read_hex( FILE *flDump, 
				   enum FILE_DUMP_TYPE e_dump_type, int *i )
{
	if ( e_dump_type == FILE_DUMP_TEXT ||
		 e_dump_type == FILE_DUMP_TEXT_F2INT )
		fscanf( flDump, "%x", i );
	else
		fread( i, sizeof( int ), 1, flDump );
}

static void 
agp_script_read_comma(  FILE *flDump, enum FILE_DUMP_TYPE e_dump_type )
{
char str_object[32];

	if ( e_dump_type == FILE_DUMP_TEXT ||
		 e_dump_type == FILE_DUMP_TEXT_F2INT )
		fscanf( flDump, "%s", str_object );
}
