/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**************************
 ***  AgpScriptWrite.c  ***
 **************************/
#include    <stdio.h>   
#include	<math.h>

#include    "AgpTool.h"

//#define FLOAT_SHORT_PLACES 4		//1 << 4
#define FLOAT_SHORT 16.0F
//#define FLOAT_LONG_PLACES 16		//1 << 16
#define FLOAT_LONG 65536.0F

enum FILE_DUMP_TYPE {
	FILE_DUMP_TEXT = 0,
	FILE_DUMP_BINARY,
	FILE_DUMP_TEXT_F2INT,
	FILE_DUMP_BINARY_F2INT
};

enum ENUM_OBJECT_SCRIPT {
	OBJECT_AGP = 0,
	OBJECT_JOINT,
	OBJECT_POLYLINK,
	OBJECT_LINK,
	OBJECT_LINK_PROFILE,
	OBJECT_HUNCH,
	OBJECT_BGP
};

static char *str_object_vim[7] = {
	"TEXTURE", 
	"TP", 
	"LN", 
	"LS", 
	"LC", 
	"PA", 
	"AC" };

static FILE *agp_script_write_init( char *path, char *name, 
						enum FILE_DUMP_TYPE e_dump_type );

static void	agp_script_write_agp( FILE *flDump, enum FILE_DUMP_TYPE e_dump_type,
							    gp_array *agp );
static void agp_script_write_agp_numbers( FILE *flDump, enum FILE_DUMP_TYPE e_dump_type,
							    gp_array *agp );

static void agp_script_write_all_polylink( FILE *flDump, enum FILE_DUMP_TYPE e_dump_type,
							    gp_array *agp );
static void agp_script_write_all_hunch( FILE *flDump, enum FILE_DUMP_TYPE e_dump_type,
							    gp_array *agp );
static void agp_script_write_all_bgp( FILE *flDump, enum FILE_DUMP_TYPE e_dump_type,
							    gp_array *agp );
static void agp_script_write_all_joint( FILE *flDump, enum FILE_DUMP_TYPE e_dump_type,
							    gp_array *agp );

static void agp_script_write_polylink( FILE *flDump, enum FILE_DUMP_TYPE e_dump_type,
				link_type *link, vec2d *gctr, int jnt_no1, int ipolylink );
static void agp_script_write_link( FILE *flDump, enum FILE_DUMP_TYPE e_dump_type,
				link_type *link, int ilink );
static void agp_script_write_link_profile( FILE *flDump, enum FILE_DUMP_TYPE e_dump_type,
							    link_type *link, int ilink, int dir );
static void	agp_script_write_edge( FILE *flDump, enum FILE_DUMP_TYPE e_dump_type,
							    edge_type *e );
static void	agp_script_write_ridge( FILE *flDump, enum FILE_DUMP_TYPE e_dump_type,
							    ridge_type *e );

static void	agp_script_write_hunch( FILE *flDump, enum FILE_DUMP_TYPE e_dump_type,
					hunch_type *hunch, vec2d *gctr, int ihunch );
static void	agp_script_write_bgp( FILE *flDump, enum FILE_DUMP_TYPE e_dump_type,
					bgp_type *bgp, vec2d *gctr, int ibgp );
static void agp_script_write_joint( FILE *flDump, enum FILE_DUMP_TYPE e_dump_type,
							    jnt_type *jnt );

static void	agp_script_write_color( FILE *flDump, enum FILE_DUMP_TYPE e_dump_type,
							    qcolor_type c );
static void agp_script_write_object_index( FILE *flDump, enum FILE_DUMP_TYPE e_dump_type,
							    enum ENUM_OBJECT_SCRIPT object_vim, int index );

static void agp_script_write_joint_get_in( jnt_type *jnt, int i, 
								int *jnt_in_no, int *brn_in_no );

static void agp_script_write_int( FILE *flDump, 
					enum FILE_DUMP_TYPE e_dump_type, int i );
static void agp_script_write_hex( FILE *flDump, 
				    enum FILE_DUMP_TYPE e_dump_type, int i );
static void agp_script_write_float_short( FILE *flDump, 
				enum FILE_DUMP_TYPE e_dump_type, float f );
static void agp_script_write_comma(  FILE *flDump, 
					enum FILE_DUMP_TYPE e_dump_type );
static void agp_script_write_cr(  FILE *flDump, 
					enum FILE_DUMP_TYPE e_dump_type );


void
agp_script_write( gp_array *agp, char *path, char *name, char dump_type )
{
FILE *flDump;
enum FILE_DUMP_TYPE e_dump_type;

	e_dump_type = (enum FILE_DUMP_TYPE)dump_type;
	flDump = agp_script_write_init( path, name, e_dump_type );
	if ( flDump == NULL )
		return;

	agp_script_write_agp( flDump, e_dump_type, agp );

	agp_script_write_all_polylink( flDump, e_dump_type, agp );

	agp_script_write_all_hunch( flDump, e_dump_type, agp );

	agp_script_write_all_bgp( flDump, e_dump_type, agp );

	agp_script_write_all_joint( flDump, e_dump_type, agp );

	fclose( flDump );
}
  
static FILE *
agp_script_write_init( char *path, char *name, 
					 enum FILE_DUMP_TYPE e_dump_type )
{
FILE *flDump;
char ss[128];
char c_dump_type_binary;

	sprintf( ss, "%s/%s.srp", path, name );

	if ( e_dump_type == FILE_DUMP_TEXT ||
		 e_dump_type == FILE_DUMP_TEXT_F2INT )
		flDump = fopen( ss, "wt" );
	else
		flDump = fopen( ss, "wb" );

	if ( !flDump )
		return NULL;

	c_dump_type_binary = (char)e_dump_type + '0';
	fwrite( &c_dump_type_binary, sizeof( char ), 1, flDump );

	return flDump;
}

static void
agp_script_write_agp( FILE *flDump, enum FILE_DUMP_TYPE e_dump_type,
							    gp_array *agp )
{
	agp_script_write_object_index( flDump, e_dump_type, OBJECT_AGP, 0 );
	agp_script_write_int( flDump, e_dump_type, agp->type );
	agp_script_write_int( flDump, e_dump_type, agp->row );
	agp_script_write_comma( flDump, e_dump_type );
	agp_script_write_int( flDump, e_dump_type, agp->column );
	agp_script_write_agp_numbers( flDump, e_dump_type, agp );
}

static void
agp_script_write_agp_numbers( FILE *flDump, enum FILE_DUMP_TYPE e_dump_type,
							    gp_array *agp )
{
gp_type		*gp;	
jnt_type	*jnt;
hunch_type  *hunch;
bgp_type	*bgp;
int			npolylink, nhunch, nbgp, njnt, ncontour;
int			i, j, k, ip;

	npolylink = nhunch = nbgp = njnt = ncontour = 0;
	for( i = 0 ; i < agp->Ni ; i++ )
	for( j = 0 ; j < agp->Nj ; j++ )
	{
		gp = &agp->gp[i][j];

		for( jnt = gp->jnt ; jnt != NULL ; jnt = jnt->nj )
		{
			jnt->jnt_no = njnt++;
			for( k = 0; k < jnt->brn_no; k++ )
			{
				if( jnt->brn[k]->direction == F_END )
					continue;
				if( jnt->brn[k]->type == BRN_NULL || 
					jnt->brn[k]->type == BRN_IN )
					continue;

				npolylink++;

				if ( jnt->brn[k]->link->Fbr )
					ncontour++;
			}
		}

		for( hunch = gp->hunch ; hunch != NULL ; hunch = hunch->nh )
			nhunch++;

		for( bgp = gp->bgp ; bgp != NULL ; bgp = bgp->p )
			nbgp++;
	}

	agp_script_write_int( flDump, e_dump_type, npolylink );
	agp_script_write_int( flDump, e_dump_type, nhunch );
	agp_script_write_int( flDump, e_dump_type, nbgp );
	agp_script_write_int( flDump, e_dump_type, njnt );

	//Number of Sub-Textures in this texture:
	agp_script_write_int( flDump, e_dump_type, agp->layer_no );

	//corresponds to "Contour" property of the texture
	agp_script_write_int( flDump, e_dump_type, ncontour );
	ip = 0;
	for( i = 0 ; i < agp->Ni ; i++ )
	for( j = 0 ; j < agp->Nj ; j++ )
	{
		gp = &agp->gp[i][j];

		for( jnt = gp->jnt ; jnt != NULL ; jnt = jnt->nj )
		{
			for( k = 0; k < jnt->brn_no; k++ )
			{
				if( jnt->brn[k]->direction == F_END )
					continue;
				if( jnt->brn[k]->type == BRN_NULL || 
					jnt->brn[k]->type == BRN_IN )
					continue;
				
				ip++;

				if ( jnt->brn[k]->link->Fbr )
					agp_script_write_int( flDump, e_dump_type, ip );
			}
		}
	}

	//Control points - skl
	agp_script_write_int( flDump, e_dump_type, 0 );
}

static void 
agp_script_write_all_polylink( FILE *flDump, enum FILE_DUMP_TYPE e_dump_type,
							    gp_array *agp )
{
gp_type		*gp;	
jnt_type	*jnt;
vec2d		gctr;
int			i, j, k;
int			ipolylink;

	ipolylink = 0;

	for( i = 0 ; i < agp->Ni ; i++ )
	for( j = 0 ; j < agp->Nj ; j++ )
	{
		gp = &agp->gp[i][j];

		for( jnt = gp->jnt ; jnt != NULL ; jnt = jnt->nj )
			for( k = 0; k < jnt->brn_no; k++ )
			{
				if( jnt->brn[k]->direction == F_END )
					continue;
				if( jnt->brn[k]->type == BRN_NULL || 
					jnt->brn[k]->type == BRN_IN )
					continue;

				gctr.x = jnt->gctr.x + jnt->brn[k]->ctr.x;
				gctr.y = jnt->gctr.y + jnt->brn[k]->ctr.y;
				agp_script_write_polylink( flDump, e_dump_type, 
					jnt->brn[k]->link, &gctr, jnt->jnt_no, ipolylink );
				ipolylink++;
			}
	}
}

static void 
agp_script_write_all_hunch( FILE *flDump, enum FILE_DUMP_TYPE e_dump_type,
							    gp_array *agp )
{
gp_type		*gp;	
hunch_type	*hunch;
vec2d		gctr;
int			i, j;
int			ihunch;

	ihunch = 0;

	for( i = 0 ; i < agp->Ni ; i++ )
	for( j = 0 ; j < agp->Nj ; j++ )
	{
		gp = &agp->gp[i][j];

		for( hunch = gp->hunch ; hunch != NULL ; hunch = hunch->nh )
		{
			gctr.x = gp->ctr.x + hunch->ctr.x;
			gctr.y = gp->ctr.y + hunch->ctr.y;
			agp_script_write_hunch( flDump, 
				e_dump_type, hunch, &gctr, ihunch );
			ihunch++;
		}
	}
}

static void 
agp_script_write_all_bgp( FILE *flDump, enum FILE_DUMP_TYPE e_dump_type,
							    gp_array *agp )
{
gp_type		*gp;	
bgp_type	*bgp;
vec2d		gctr;
int			i, j;
int			ibgp;

	ibgp = 0;

	for( i = 0 ; i < agp->Ni ; i++ )
	for( j = 0 ; j < agp->Nj ; j++ )
	{
		gp = &agp->gp[i][j];

		for( bgp = gp->bgp ; bgp != NULL ; bgp = bgp->p )
		{
			gctr.x = gp->ctr.x + bgp->ctr.x;
			gctr.y = gp->ctr.y + bgp->ctr.y;
			agp_script_write_bgp( flDump, e_dump_type, bgp, 
								&gctr, ibgp );
			ibgp++;
		}
	}
}

static void 
agp_script_write_all_joint( FILE *flDump, enum FILE_DUMP_TYPE e_dump_type,
							    gp_array *agp )
{
gp_type		*gp;	
jnt_type	*jnt;
int			i, j;

	for( i = 0 ; i < agp->Ni ; i++ )
	for( j = 0 ; j < agp->Nj ; j++ )
	{
		gp = &agp->gp[i][j];
		for( jnt = gp->jnt; jnt; jnt = jnt->nj )
			agp_script_write_joint( flDump, e_dump_type, jnt );
	}
}

static void 
agp_script_write_polylink( FILE *flDump, enum FILE_DUMP_TYPE e_dump_type,
		link_type *link, vec2d *gctr, int jnt_no1, int ipolylink )
{
link_type	*l;
int			ino, ilink;
int			link_type, separating_flag;
int			contour_flag, br1, br2;

	for ( l = link, ino = 1; LINK_NEXT( l ) != NULL; l = LINK_NEXT( l ), ino++ );

	switch( link->type )
	{
		case NF_EDGE:
			link_type = 0;
			separating_flag = 1;
		break;
		case NF_N3R:
		case NF_RIDGE:
			link_type = 1;
			separating_flag = 0;
		break;
	}

	contour_flag = link->Fbr;
	br1 = link->br1;
	br2 = link->br;
	switch ( contour_flag )
	{
	case 0: 
		br1 = br2;
		break;
	case 1: 
		br1 = br2;
		br2 = 0;
		break;
	case 2:
		br1 = 0;
		break;
	default:
		contour_flag = 0;
		break;
	}

	agp_script_write_object_index( flDump, e_dump_type, OBJECT_POLYLINK, ipolylink );
	agp_script_write_int( flDump, e_dump_type, link_type );
	agp_script_write_int( flDump, e_dump_type, separating_flag );
	agp_script_write_int( flDump, e_dump_type, contour_flag );

	agp_script_write_int( flDump, e_dump_type, ino );
	agp_script_write_int( flDump, e_dump_type, ino+1 );

	agp_script_write_int( flDump, e_dump_type, jnt_no1 );
	agp_script_write_comma( flDump, e_dump_type );
	agp_script_write_int( flDump, e_dump_type, link->p[F_BEGIN].bno );
	agp_script_write_int( flDump, e_dump_type, l->p[F_END].p.jnt->jnt_no);
	agp_script_write_comma( flDump, e_dump_type );
	agp_script_write_int( flDump, e_dump_type, l->p[F_END].bno);
	agp_script_write_float_short( flDump, e_dump_type, gctr->x );
	agp_script_write_comma( flDump, e_dump_type );
	agp_script_write_float_short( flDump, e_dump_type, gctr->y );

	agp_script_write_int( flDump, e_dump_type, br1 );
	agp_script_write_comma( flDump, e_dump_type );
	agp_script_write_int( flDump, e_dump_type, br2 );

	ilink = 0;
	for ( l = link; l; l = LINK_NEXT( l ) )
		agp_script_write_link( flDump, e_dump_type, l, ilink++ );
	for ( l = link, ilink = 0; l; l = LINK_NEXT( l ) )
	{
		agp_script_write_link_profile( flDump, e_dump_type, l, ilink++, F_BEGIN );
		if( LINK_NEXT( l ) == NULL )
			agp_script_write_link_profile( flDump, e_dump_type, l, ilink++, F_END );
	}

}

static void 
agp_script_write_link( FILE *flDump, enum FILE_DUMP_TYPE e_dump_type,
						link_type *link, int ilink )
{
	agp_script_write_object_index( flDump, e_dump_type, OBJECT_LINK, ilink );
	agp_script_write_float_short( flDump, e_dump_type, link->v.x );
	agp_script_write_comma( flDump, e_dump_type );
	agp_script_write_float_short( flDump, e_dump_type, link->v.y );
	agp_script_write_float_short( flDump, e_dump_type, link->a );
}

static void 
agp_script_write_link_profile( FILE *flDump, enum FILE_DUMP_TYPE e_dump_type,
							    link_type *link, int ilink, int dir )
{
	agp_script_write_object_index( flDump, e_dump_type, OBJECT_LINK_PROFILE, ilink );
	switch( link->type )
	{
		case NF_EDGE:
			agp_script_write_edge( flDump, e_dump_type, link->pr[dir].edge );
		break;
		case NF_N3R:
		case NF_RIDGE:
			agp_script_write_ridge( flDump, e_dump_type, link->pr[dir].ridge );
		break;
	}
}

static void
agp_script_write_edge( FILE *flDump, enum FILE_DUMP_TYPE e_dump_type,
							    edge_type *e )
{
int A;
float width_2;

	width_2 = e->width/2.0F;

	agp_script_write_float_short( flDump, e_dump_type, width_2 );
	agp_script_write_comma( flDump, e_dump_type );
	agp_script_write_float_short( flDump, e_dump_type, width_2 );
	
	for ( A = 0; A < SP_NO; A++ )
	{
 		agp_script_write_color( flDump, e_dump_type, e->side[F_LEFT].bval[A] );
		agp_script_write_comma( flDump, e_dump_type );
		agp_script_write_color( flDump, e_dump_type, e->side[F_LEFT].val[A] );
		agp_script_write_comma( flDump, e_dump_type );
		agp_script_write_color( flDump, e_dump_type, e->side[F_RIGHT].bval[A] );
		agp_script_write_comma( flDump, e_dump_type );
		agp_script_write_color( flDump, e_dump_type, e->side[F_RIGHT].val[A] );
		agp_script_write_comma( flDump, e_dump_type );
		agp_script_write_color( flDump, e_dump_type, 0 );
	}

	agp_script_write_float_short( flDump, e_dump_type, e->depth );
}


static void
agp_script_write_ridge( FILE *flDump, enum FILE_DUMP_TYPE e_dump_type,
							    ridge_type *e )
{
int A;
qcolor_type bg[SP_NO];

	agp_script_write_float_short( flDump, e_dump_type, e->width/2.0F );
	agp_script_write_comma( flDump, e_dump_type );
	agp_script_write_float_short( flDump, e_dump_type, e->width/2.0F );

	//for ( A = 0; A < SP_NO; A++ )
	//	bg[A] = e->mval[A] - e->rval[A];

	for ( A = 0; A < SP_NO; A++ )
		bg[A] = (qcolor_type)(e->mval[A] + 
					e->lm[A]*e->width*e->width);

	for ( A = 0; A < SP_NO; A++ )
	{
 		agp_script_write_color( flDump, e_dump_type, bg[A] );
		agp_script_write_comma( flDump, e_dump_type );
 		agp_script_write_color( flDump, e_dump_type, bg[A] );
		agp_script_write_comma( flDump, e_dump_type );
		agp_script_write_color( flDump, e_dump_type, bg[A] );
		agp_script_write_comma( flDump, e_dump_type );
		agp_script_write_color( flDump, e_dump_type, bg[A] );
		agp_script_write_comma( flDump, e_dump_type );
		agp_script_write_color( flDump, e_dump_type, e->mval[A] );
	}

	agp_script_write_float_short( flDump, e_dump_type, e->depth );
}

static void
agp_script_write_hunch( FILE *flDump, enum FILE_DUMP_TYPE e_dump_type,
				hunch_type *h, vec2d *gctr, int ihunch )
{
int A;
int alpha;
qcolor_type bg[SP_NO];

	agp_script_write_object_index( flDump, e_dump_type, OBJECT_HUNCH, ihunch );
	agp_script_write_float_short( flDump, e_dump_type, gctr->x );
	agp_script_write_comma( flDump, e_dump_type );
	agp_script_write_float_short( flDump, e_dump_type, gctr->y );
	agp_script_write_float_short( flDump, e_dump_type, h->g.len/2.0F );
	agp_script_write_comma( flDump, e_dump_type );
	agp_script_write_float_short( flDump, e_dump_type, h->g.width/2.0F );
	alpha = (int)(h->g.theta*180.0F/M_PI);
	agp_script_write_int( flDump, e_dump_type, alpha );

	for ( A = 0; A < SP_NO; A++ )
		bg[A] = h->gval[A] - h->rval[A];

	for ( A = 0; A < SP_NO; A++ )
	{
		agp_script_write_color( flDump, e_dump_type, h->gval[A] );
		agp_script_write_comma( flDump, e_dump_type );
		agp_script_write_color( flDump, e_dump_type, bg[A] );
	}

	agp_script_write_int( flDump, e_dump_type, h->br );
	agp_script_write_float_short( flDump, e_dump_type, h->depth );
}


static void
agp_script_write_bgp( FILE *flDump, enum FILE_DUMP_TYPE e_dump_type,
			bgp_type *bgp, vec2d *gctr, int ibgp )
{
int A;

	agp_script_write_object_index( flDump, e_dump_type, OBJECT_BGP, ibgp );
	agp_script_write_float_short( flDump, e_dump_type, gctr->x );
	agp_script_write_comma( flDump, e_dump_type );
	agp_script_write_float_short( flDump, e_dump_type, gctr->y );
	for ( A = 0; A < SP_NO; A++ )
	{
		agp_script_write_color( flDump, e_dump_type, bgp->val[A] );
		agp_script_write_comma( flDump, e_dump_type );
	}
	agp_script_write_int( flDump, e_dump_type, bgp->br );
	agp_script_write_float_short( flDump, e_dump_type, bgp->depth );
}


static void
agp_script_write_joint( FILE *flDump, enum FILE_DUMP_TYPE e_dump_type,
							    jnt_type *jnt )
{
	agp_script_write_object_index( flDump, e_dump_type, OBJECT_JOINT, jnt->jnt_no );
	agp_script_write_float_short( flDump, e_dump_type, jnt->gctr.x );
	agp_script_write_comma( flDump, e_dump_type );
	agp_script_write_float_short( flDump, e_dump_type, jnt->gctr.y );
	agp_script_write_int( flDump, e_dump_type, jnt->type );
	agp_script_write_int( flDump, e_dump_type, jnt->brn_no );
}


static void
agp_script_write_color( FILE *flDump, enum FILE_DUMP_TYPE e_dump_type,
							    qcolor_type c )
{
#ifdef	QFLOAT
	agp_script_write_hex( flDump, e_dump_type, (int)(c/256.0F+0.5F) );
#endif

#ifdef	QSHORT
int cc;
	cc = (int)(((float)c)/16.0F + 0.5F);
	cc = (cc < 0) ? 0 : (( cc > 255) ? 255 : cc );
	agp_script_write_hex( flDump, e_dump_type, cc );
#endif
}

static void agp_script_write_object_index( FILE *flDump, enum FILE_DUMP_TYPE e_dump_type,
							    enum ENUM_OBJECT_SCRIPT object_vim, int index )
{
int i;

	i = (int)object_vim;

	if ( e_dump_type == FILE_DUMP_TEXT ||
		 e_dump_type == FILE_DUMP_TEXT_F2INT )
		fprintf( flDump, "\n%s%05d", str_object_vim[i], index );
	else
	{
		fwrite( &i, sizeof( int ), 1, flDump );
		fwrite( &index, sizeof( int ), 1, flDump );
	}
}

static void
agp_script_write_joint_get_in( jnt_type *jnt, int i, 
								int *jnt_in_no, int *brn_in_no )
{
jnt_type *jnt_in;
brn_type *brn;
link_type *link, *l;
int inv_dir;

	brn = jnt->brn[i];
	link = brn->link;
	inv_dir = INV_FLAG( brn->direction );
	if ( brn->direction == F_BEGIN )
		for ( l = link; LINK_NEXT( l ); l = LINK_NEXT( l ) );
	else //if ( brn->direction = F_END )
		for ( l = link; LINK_PREV( l ); l = LINK_PREV( l ) );

	jnt_in = l->p[inv_dir].p.jnt;
	*brn_in_no = l->p[inv_dir].bno;
	*jnt_in_no = jnt_in->jnt_no;

}

static void
agp_script_write_int( FILE *flDump, 
				   enum FILE_DUMP_TYPE e_dump_type, int i )
{
	if ( e_dump_type == FILE_DUMP_TEXT ||
		 e_dump_type == FILE_DUMP_TEXT_F2INT )
		fprintf( flDump, " %6d", i );
	else
		fwrite( &i, sizeof( int ), 1, flDump );
}

static void
agp_script_write_hex( FILE *flDump, 
				   enum FILE_DUMP_TYPE e_dump_type, int i )
{
	if ( e_dump_type == FILE_DUMP_TEXT ||
		 e_dump_type == FILE_DUMP_TEXT_F2INT )
		fprintf( flDump, " %6X", i );
	else
		fwrite( &i, sizeof( int ), 1, flDump );
}

static void
agp_script_write_float_short( FILE *flDump, 
				enum FILE_DUMP_TYPE e_dump_type, float f )
{
int i;

	if ( e_dump_type == FILE_DUMP_TEXT )
			fprintf( flDump, " %6.2f", f );
	else if ( e_dump_type == FILE_DUMP_BINARY )
		fwrite( &f, sizeof( float ), 1, flDump );
	else
	{
		i = (int)( f*FLOAT_SHORT );
		agp_script_write_int( flDump, e_dump_type, i );
	}
}

static void 
agp_script_write_comma(  FILE *flDump, enum FILE_DUMP_TYPE e_dump_type )
{
	if ( e_dump_type == FILE_DUMP_TEXT ||
		 e_dump_type == FILE_DUMP_TEXT_F2INT )
		fprintf( flDump, " , " );
}

static void 
agp_script_write_cr(  FILE *flDump, enum FILE_DUMP_TYPE e_dump_type )
{
	if ( e_dump_type == FILE_DUMP_TEXT ||
		 e_dump_type == FILE_DUMP_TEXT_F2INT )
		fprintf( flDump, "\n" );
}
