/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
#include        <stdio.h>
#include        <math.h>

#include        "Uigp/igp.h"

#include        "Uprm/prm.h"
 
#include        "GpType/GpType.h"
#include        "Filter.h"



 
#define		DSIZE	0

static float      Hlm_min = 1.5;
static float      Hrval_min = 5;
static float      Hlen_min = 0.1;
static float      Rrval_min = 5;


static GP_prm  P2F_params[]    =    {
        {"H-lm-min", (char*)&Hlm_min, PRM_FLOAT, "%.2f", "1.5"},
        {"H-rval-min", (char*)&Hrval_min, PRM_FLOAT, "%.2f", "5.0"},

        {"H-len-min", (char*)&Hlen_min, PRM_FLOAT, "%.2f", "0.1"},

        
        {"R-rval-min", (char*)&Rrval_min, PRM_FLOAT, "%.2f", "5.0"},
};





void
filter_get_prm( GP_prm **prm, int *prm_no,void (**check)() ) 
{
        *prm    = P2F_params;
        *prm_no  = PRM_NUMBER(P2F_params);
        *check = NULL;
}
 
 
void
filter_init( char *dir, int Lfilter )
{
char	file[256];

	sprintf( file, "%s/L%d.filter", dir, Lfilter );
 
	gp_prm_init( file, filter_get_prm );

	gp_prm_init( file, filter_edge_get_prm );

	gp_prm_init( file, filter_rhunch_to_hunch_get_prm );
 
        filter_set_lprm();
}

void
filter_set_lprm()
{
}


void
filter_agp( gp_array *agp )
{
	filter_hunch_agp( agp, NULL,
			F2QCOLOR(Hrval_min), F2QCOLOR(Hlm_min), Hlen_min );


	filter_ridge_agp( agp, NULL, F2QCOLOR(Rrval_min) );
}





void
filter_percent_agp( gp_array *agp, box2d *box, float Phunch, float Pridge )
{
int	no,	val[256];
int	pno,	sum,	i;


	if( Phunch > 0 ){
		if( Phunch > 100 )	Phunch = 100;
		no = filter_hunch_agp_aux( agp, box, val );

		pno = no * Phunch / 100;
		for( i = 0, sum = val[0] ; sum < pno ; i++, sum += val[i] );

		filter_hunch_agp( agp, box, F2QCOLOR(i+0.5), 0, 0 );
	}


	if( Pridge > 0 ){
		if( Pridge > 100 )	Pridge = 100;
		no = filter_ridge_agp_aux( agp, box, val );

		pno = no * Pridge / 100;
		for( i = 0, sum = val[0] ; sum < pno ; i++, sum += val[i] );

		filter_ridge_agp( agp, box, F2QCOLOR(i+0.5) );
	}
}
