/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/************************************************************************/

/*                            D2GpEdge.c				*/ 

/************************************************************************/
#include        <stdio.h>   
#include	<math.h>

#include        "Uigp/igp.h"
#include        "Uprm/prm.h"

#include        "GpType/GpType.h"


#include	"UGeoApp/GeoApp.h"

#include        "RlLink/RlType.h"

#include		"RlLink/RlLinkFree.h"

static void	filter_edge_one( gp_array *agp, gp_type *gp, jnt_type *jnt );

static int	filter_edge_one_1( gp_array *agp, jnt_type *jnt );


static gapp_prm	FEgprm;
static float	Fedge_len,	Fedge_len1,	Fedge_n3_len1;

static GP_prm params[]  = {
	{"FE-D-weight" , (char *)&FEgprm.dwight, PRM_FLOAT,"%.2f","0.10"},
	{"FE-Ls-error" , (char *)&FEgprm.err, PRM_FLOAT,"%.2f","0.01"},

	{"FE-len-min" , (char *)&FEgprm.min_len,PRM_FLOAT,"%.2f","1.0"},
	{"FE-min-no" , (char *)&FEgprm.Min_pnumber,PRM_INTEGER,"%d","4"},
	{"FE-len-max" , (char *)&FEgprm.max_len,PRM_FLOAT,"%.2f","12.0"},

	{"FE-dev-max" , (char *)&FEgprm.Max_dev,PRM_FLOAT,"%.2f","1.0"},
	{"FE-dev-min" , (char *)&FEgprm.Min_dev,PRM_FLOAT,"%.2f","0.1"},
	{"FE-len" , (char *)&Fedge_len,PRM_FLOAT,"%.4f","0.5"},
	{"FE-len1" , (char *)&Fedge_len1,PRM_FLOAT,"%.4f","3.0"},
	{"FE-n3-len1" , (char *)&Fedge_n3_len1,PRM_FLOAT,"%.4f","1.0"},
};


static rl_type	Rl;

void
filter_edge_get_prm(GP_prm **prm, int *prm_no, void (**check)() )
{
	*prm    = params;
	*prm_no  = PRM_NUMBER(params);
	*check = NULL ;
}


void
filter_edge_set_lprm()
{
}


void
filter_edge( gp_array *agp )
{
gp_type	*gp;
jnt_type	*jnt,	*nj;
int	i,	j;

	for( i = 0 ; i < agp->Ni ; i++ )
		for( j = 0 ; j < agp->Nj ; j++ ){

			gp = &agp->gp[i][j];

			for( jnt = gp->jnt ; jnt != NULL ; ){

				if( jnt->type != JNT_EDGE ){
					jnt = jnt->nj;
					continue;
				}

				if( filter_edge_one_1( agp, jnt ) == 1 ){
					jnt = gp->jnt;
					continue;
				}

				nj = jnt->nj;

				filter_edge_one( agp, gp, jnt );

				jnt = nj;
			}
		}
}


static void
filter_edge_one( gp_array *agp, gp_type *gp, jnt_type *jnt )
{
gapp_type       *gapp;
brn_type	*b;
link_type	*l0,	*l1,	*nl0,	*nl1;
vec2d	ctr,	gctr;
rl_aux	laux[4];
int	lno;
int	i,	j;

	if( jnt->brn_no > 1 )	return;
	b = jnt->brn[0];

/*
	if( LINK_NEXT(b->link) == NULL && LINK_PREV(b->link) == NULL ){
		if( b->link->len > Fedge_len1 )  return;
		filter_edge_one_1( agp, jnt );
		return;
	}
*/

	if( b->link->len > Fedge_len )	return;

	if( b->direction == F_BEGIN ){
		l0 = b->link;
		l1 = LINK_NEXT(b->link);
		gctr = jnt->gctr;
	}
	else	{
		l0 = LINK_PREV(b->link);
		l1 = b->link;

		gctr.x = jnt->gctr.x - l1->v.x - l0->v.x;
		gctr.y = jnt->gctr.y - l1->v.y - l0->v.y;
	}

	ctr.x = ctr.y = 0;
		
	gapp = gapp_alloc( 4096 );
	rl_link_gapp( &ctr, l0, l1, gapp, laux, &lno );
	gapp->ino = 0;
	rl_approximate( gapp, 0, gapp->no-1,
		&laux[0].ctr, &laux[lno].ctr, &FEgprm, &Rl );

	if( gapp->ino == 1 ){
		rl_to_link( NF_EDGE, &Rl, laux, gapp->ino, &nl0, &nl1 );
		link_replace( l0, l1, nl0, nl1 );
		link_2_free( agp, &gctr, l0, l1 );

		agp_lnf_add_link_1( &gctr, nl0, agp );
/*
		link_1_free( l1 );
*/
		gapp_destroy( gapp );
		return;
	}


	gp_untied_jnt( gp, jnt );

	if( b->direction == F_BEGIN ){
		agp_untied_link( agp, &jnt->gctr, b->link );
		jnt->gctr.x += b->link->v.x;
		jnt->gctr.y += b->link->v.y;
		
		l1->p[F_BEGIN] = l0->p[F_BEGIN];
		free( b->link->pr[0].p );
		free( b->link );
		b->link = l1;
	}
	else	{
		jnt->gctr.x -= b->link->v.x;
		jnt->gctr.y -= b->link->v.y;

		agp_untied_link( agp, &jnt->gctr, b->link );

		l0->p[F_END] = l1->p[F_END];
		free( b->link->pr[1].p );
		free( b->link );
		b->link = l0;
	}


	i = ( jnt->gctr.x + agp->bgp ) * agp->inv_bgp;
	j = ( jnt->gctr.y + agp->bgp ) * agp->inv_bgp;

	gp = &agp->gp[i][j];
	jnt->ctr.x = jnt->gctr.x - gp->ctr.x;
	jnt->ctr.y = jnt->gctr.y - gp->ctr.y;

	jnt->nj = gp->jnt;
	gp->jnt = jnt;

	gapp_destroy( gapp );
}




static int
filter_edge_one_1( gp_array *agp, jnt_type *jnt )
{
jnt_type	*jnt1;
int	bno1;
link_type	*l;
int	direct;
vec2d	ctr;

	if( jnt->brn_no > 1 )	return( 0 );

	l = jnt->brn[0]->link;

	if( LINK_NEXT(l) != NULL || LINK_PREV(l) != NULL )
		return( 0 );

	if( l->len > Fedge_len1 )  return( 0 );


	direct = INV_FLAG( jnt->brn[0]->direction );
	jnt1 = l->p[direct].p.jnt;
	bno1 = l->p[direct].bno;

	if( jnt1->type == JNT_EDGE && l->len > Fedge_len1 )  return( 0 );
	if( jnt1->type == JNT_N3 && l->len > Fedge_n3_len1 )  return( 0 );


	/* NEW 30.11.98 */
	if( jnt->brn[0]->direction == F_BEGIN ){
		ctr = jnt->gctr;

		agp_lnf_clear_link( &ctr, l, agp );
		link_free( l );
	}
	else	{
		ctr.x = jnt1->gctr.x + jnt1->brn[bno1]->ctr.x;
		ctr.y = jnt1->gctr.y + jnt1->brn[bno1]->ctr.y;

		agp_lnf_clear_link( &ctr, l, agp );
		link_free( l );
	}
	/* END NEW 30.11.98 */


	jnt_untied_brn( jnt, 0, 1 );
	if( jnt->brn_no == 0 ){
		agp_untied_jnt( agp, jnt );
		free( jnt );
	}


	jnt_untied_brn( jnt1, bno1, 1 );
	if( jnt1->brn_no == 0 ){
		agp_untied_jnt( agp, jnt1 );
		free( jnt1 );
	}

	return( 1 );
}
