/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**************************
 ***	FilterHunch.c   ***
 **************************/
#include        <stdio.h>
#include        <math.h>

#include        "Uigp/igp.h"


#include        "GpType/GpType.h"
#include        "Filter.h"

 

static int	filter_hunch( hunch_type *h,
				float rval_t, float lm_t, float len_t );



void
filter_hunch_agp( gp_array *agp, box2d *box,
			float rval_t, float lm_t, float len_t )
{
int     i0,     j0,     i1,     j1;
int     i,      j;
gp_type *gp;
hunch_type      *h,     **ph;
 
 
	if( box != NULL ){
        	i0 = ( box->x0 + agp->bgp )*agp->inv_bgp;
        	j0 = ( box->y0 + agp->bgp )*agp->inv_bgp;
 
        	i1 = ( box->x1 + agp->bgp )*agp->inv_bgp;
        	j1 = ( box->y1 + agp->bgp )*agp->inv_bgp;
 
        	if( i0 < 0 )    i0 = 0;
        	if( i1 >= agp->Ni )     i1 = agp->Ni -1;
        	if( j0 < 0 )    j0 = 0;
        	if( j1 >= agp->Nj )     j1 = agp->Nj -1;
	}
	else	{
		i0 = j0 = 0;
		i1 = agp->Ni -1;
		j1 = agp->Nj -1;
	}

 
 
	for( i = i0 ; i <= i1 ; i++ )
	    for( j = j0 ; j <= j1 ; j++ ){
		gp = &agp->gp[i][j];          

		ph = &gp->hunch;
		for( h = gp->hunch ; h != NULL ; ){

			if( filter_hunch( h, rval_t, lm_t, len_t ) == 1 ){
				ph = &h->nh;
					
				h = (*ph);
				continue;
			}

			(*ph) = h->nh;
			free( h );

			h = (*ph);
		}
	    }
}



static int
filter_hunch( hunch_type *h, float rval_t, float lm_t, float len_t )
{

	if( h->g.len < len_t )	return( 0 ); 


	if( h->lm1[0] > 0 ){
		if (  h->rval[0] > -rval_t || h->lm1[0] < lm_t  )
			return( 0 );
		return( 1 );
	}


	if(  h->rval[0] < rval_t || h->lm1[0] > -lm_t )
		return( 0 );    

	return( 1 );
}


/*
static int
filter_rhunch( hunch_type *h )
{
	if( h->g.len < RHlen_min )	return( 0 );

	if( h->l[0].lm1 > 0.0 ){
		if( h->l[0].rval > -RHrval_min )
			return( 0 );
		return( 1 );
	}


	if( h->l[0].rval < RHrval_min )
		return (0); 
	else	return (1);
}
*/



int
filter_hunch_agp_aux( gp_array *agp, box2d *box, int val[256] )
{
int     i0,     j0,     i1,     j1;
int     i,      j,	no,	tmp;
float	t;
gp_type *gp;
hunch_type      *h;
 
 
	if( box != NULL ){
        	i0 = ( box->x0 + agp->bgp )*agp->inv_bgp;
        	j0 = ( box->y0 + agp->bgp )*agp->inv_bgp;
 
        	i1 = ( box->x1 + agp->bgp )*agp->inv_bgp;
        	j1 = ( box->y1 + agp->bgp )*agp->inv_bgp;
 
        	if( i0 < 0 )    i0 = 0;
        	if( i1 >= agp->Ni )     i1 = agp->Ni -1;
        	if( j0 < 0 )    j0 = 0;
        	if( j1 >= agp->Nj )     j1 = agp->Nj -1;
	}
	else	{
		i0 = j0 = 0;
		i1 = agp->Ni -1;
		j1 = agp->Nj -1;
	}

 
 
	for( i = 0 ; i < 256 ; i++ )    val[i] = 0;

	no = 0;
	for( i = i0 ; i <= i1 ; i++ )
	    for( j = j0 ; j <= j1 ; j++ ){
		gp = &agp->gp[i][j];          

		for( h = gp->hunch ; h != NULL ; h = h->nh ){

			if( (t = QCOLOR2F(h->rval[0])) < 0 )	t = -t;

			if( t >= 255 )	t = 255;
			tmp = t + 0.5;

			val[ tmp ]++;

			no++;
		}
	    }

	return( no );
}
