/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/********************
 *** FilterN3.c   ***
 ********************/
#include        <stdio.h>   
#include		<math.h>

#include        "Uigp/igp.h"

#include        "GpType/GpType.h"

#include		"AgpEdit/AgpEditTool.h"




static void	filter_n3_one( gp_array *agp, link_type *link );


static int	filter_n3_delete_short( gp_array *agp, link_type *link, float Tlen );



void
filter_n3( gp_array *agp )
{
gp_type	*gp;
link_type	*l,	*nl;
int	i,	j;

	for( i = 0 ; i < agp->Ni ; i++ )
		for( j = 0 ; j < agp->Nj ; j++ ){

			gp = &agp->gp[i][j];


			for( l = gp->link ; l != NULL ; l = nl ){
				nl = l->nl;

				if( l->type != NF_N3 )	continue;

				if( l->len > 4 )	continue;


				if( l->p[0].bno < 0 || l->p[1].bno < 0 )	continue;


				if( filter_n3_delete_short( agp, l, 1.0 ) == 1 )
					continue;



				filter_n3_one( agp, l );
			}

		}
}



static void
filter_n3_one( gp_array *agp, link_type *link )
{
jnt_type	*jnt0,	*jnt1;
brn_type *b0,	*b1;
int	bno0,	bno1;

	bno0 = link->p[0].bno;
	jnt0 = link->p[0].p.jnt;

	bno1 = link->p[1].bno;
	jnt1 = link->p[1].p.jnt;


	if( jnt0->type == JNT_N3 && jnt1->type == JNT_N3 ){
		b0 = jnt0->brn[1];
		b1 = jnt1->brn[2];
		if( b0->type == BRN_IN && b1->type == BRN_IN && b0->link == b1->link && b0->in->t == b1->type ){
			jnt_n3_in_delete( jnt0, 1 );
			jnt_n3_in_delete( jnt1, 2 );
		}


		b0 = jnt0->brn[2];
		b1 = jnt1->brn[1];
		if( b0->type == BRN_IN && b1->type == BRN_IN && b0->link == b1->link && b0->in->t == b1->type ){
			jnt_n3_in_delete( jnt0, 2 );
			jnt_n3_in_delete( jnt1, 1 );
		}
	}




	if( link->len < 2.5 ){

		link->a = 0;

		link_set_aux( link );

		if( jnt0->type == JNT_N3 )
			agp_jnt_n3_reposition( agp, jnt0 );
		else	jnt_rn3_set_aux( jnt0 );

		if( jnt1->type == JNT_N3 )
			agp_jnt_n3_reposition( agp, jnt1 );
		else	jnt_rn3_set_aux( jnt1 );
	}
}



static int
filter_n3_delete_short( gp_array *agp, link_type *link, float Tlen )
{
jnt_type	*jnt0,	*jnt1;
int	bno0,	bno1;
int	Fdelete;


	if( link->len > Tlen )	return( 0 );

	bno0 = link->p[0].bno;
	jnt0 = link->p[0].p.jnt;

	bno1 = link->p[1].bno;
	jnt1 = link->p[1].p.jnt;


	Fdelete = 1;
	if( jnt0->type == JNT_N3 && ( jnt0->brn[1]->type == BRN_EDGE || jnt0->brn[2]->type == BRN_EDGE) )
		Fdelete = 0;

	if( jnt1->type == JNT_N3 && ( jnt1->brn[1]->type == BRN_EDGE || jnt1->brn[2]->type == BRN_EDGE) )
		Fdelete = 0;


	if( Fdelete == 0 )	return( 0 );


	agp_polylink_delete( agp, link );

	return( 1 );
}
