/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**************************
 ***	FilterRidge.c   ***
 **************************/

#include        <stdio.h>
#include        <math.h>

#include        "Uigp/igp.h"

 
#include        "GpType/GpType.h"

#include        "AgpEdit/AgpEditTool.h"


 
static int	filter_ridge_N3R_test( link_type *link );

static float	filter_ridge_rval( link_type *link );




void
filter_ridge_agp( gp_array *agp, box2d *box, float rval_t )
{
int	i0,	j0,	i1,	j1;
int	i,	j;
gp_type *gp;
link_type	*l;
 
 
	if( box != NULL ){
		i0 = ( box->x0 + agp->bgp )*agp->inv_bgp;
		j0 = ( box->y0 + agp->bgp )*agp->inv_bgp;
 
		i1 = ( box->x1 + agp->bgp )*agp->inv_bgp;
		j1 = ( box->y1 + agp->bgp )*agp->inv_bgp;
 
		if( i0 < 0 )    i0 = 0;
		if( i1 >= agp->Ni )     i1 = agp->Ni -1;
		if( j0 < 0 )    j0 = 0;
		if( j1 >= agp->Nj )     j1 = agp->Nj -1;
        }
        else    {
                i0 = j0 = 0;
                i1 = agp->Ni -1;
                j1 = agp->Nj -1;
        }   
 
 
 
	for( i = i0 ; i <= i1 ; i++ )
	    for( j = j0 ; j <= j1 ; j++ ){
		gp = &agp->gp[i][j];
 

		for( l = gp->link ; l != NULL ; ){

			if( LINK_PREV(l) != NULL ){
				l = l->nl;
				continue;
			}

			if( l->type != NF_RIDGE && l->type != NF_N3R ){
				l = l->nl;
				continue;
			}



			if(  l->type == NF_N3R &&
					filter_ridge_N3R_test( l ) == 0 ){
				l = l->nl;
				continue;
			}



			if( filter_ridge_rval( l ) >=  rval_t  ){
				l = l->nl;
				continue;
			}

			agp_polylink_delete( agp, l );

		
			l = gp->link;
		}
	    }
}
				
			

static int
filter_ridge_N3R_test( link_type *link )
{
jnt_type	*jnt;
link_type	*l;

	jnt = link->p[0].p.jnt;

        if( jnt->type == JNT_N3 && ( jnt->brn[1]->type == BRN_EDGE ||
			jnt->brn[2]->type == BRN_EDGE ) )
                return( 0 ); 


	for( l = link ; LINK_NEXT(l) != NULL ; l =  LINK_NEXT(l) );


	jnt = l->p[1].p.jnt;

        if( jnt->type == JNT_N3 && ( jnt->brn[1]->type == BRN_EDGE ||
			jnt->brn[2]->type == BRN_EDGE ) )
                return( 0 ); 

	return( 1 );
}





static float
filter_ridge_rval( link_type *link )
{
link_type	*l;
ridge_type	*r;
float	val,	max_val;


	max_val = 0.0;


	for( l = link ; l != NULL ; l = LINK_NEXT(l) ){

		r = l->pr[0].ridge;

		val = ABS( r->rval[0] );
		if( val > max_val )	max_val = val;


		if( LINK_NEXT(l) != NULL )	continue;


		r = l->pr[1].ridge;

		val = ABS( r->rval[0] );
		if( val > max_val )	max_val = val;
	}



	return( max_val );
}



int
filter_ridge_agp_aux( gp_array *agp, box2d *box, int val[256] )
{
int	i0,	j0,	i1,	j1;
int	i,	j,	no,	tmp;
gp_type *gp;
link_type	*l;
 
 
	if( box != NULL ){
		i0 = ( box->x0 + agp->bgp )*agp->inv_bgp;
		j0 = ( box->y0 + agp->bgp )*agp->inv_bgp;
 
		i1 = ( box->x1 + agp->bgp )*agp->inv_bgp;
		j1 = ( box->y1 + agp->bgp )*agp->inv_bgp;
 
		if( i0 < 0 )    i0 = 0;
		if( i1 >= agp->Ni )     i1 = agp->Ni -1;
		if( j0 < 0 )    j0 = 0;
		if( j1 >= agp->Nj )     j1 = agp->Nj -1;
        }
        else    {
                i0 = j0 = 0;
                i1 = agp->Ni -1;
                j1 = agp->Nj -1;
        }   
 
 

	for( i = 0 ; i < 256 ; i++ )	val[i] = 0;
 
	no = 0;
	for( i = i0 ; i <= i1 ; i++ )
	    for( j = j0 ; j <= j1 ; j++ ){
		gp = &agp->gp[i][j];
 

		for( l = gp->link ; l != NULL ; l = l->nl ){

			if( LINK_PREV(l) != NULL )	continue;

			if( l->type != NF_RIDGE && l->type != NF_N3R )
				continue;


			tmp = filter_ridge_rval( l ) + 0.5;
			tmp = QCOLOR2F(tmp);

			if( tmp >= 255 )	tmp = 255;

			val[ tmp ]++; 
			no++;
			
		}
	}

	return( no );
}
