/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/************************************************************************/

/* inf_agp.c					 */

/************************************************************************/
#include        <stdio.h>
#include	<math.h>


#include        "InfIn.h"

#include        "GpType/GpType.h"

#define		MAX_NF		32

static	inf	Inf,	Ijnt,	Iall;

struct INF_NFM_CTRS {
	int     gp_no[MAX_NF];
};

static struct INF_NFM_CTRS Inf_nfm_ctrs;



static void inf_nfm_ctrs_init();

static void inf_nfm_ctrs_print(FILE * fp);



void
inf_agp_init()
{
	Inf = Ijnt = Iall = Inf_init;

	inf_nfm_ctrs_init();

	inf_jnt_init();

	inf_profile_init();

	inf_hunch_init();

	inf_rhunch_init();

	inf_ridge_init();

	inf_n3_init();

	inf_edge_init();

	inf_bgp_init();

}

void
inf_agp( gp_array *agp )
{
gp_type		*gp;
hunch_type	*h;
jnt_type	*jnt;
int	i,	j,	nf_no,	jnt_no;


	for (i = 0; i < agp->Ni; i++)
		for (j = 0; j < agp->Nj; j++) {

			gp = &agp->gp[i][j];


			nf_no = 0;
			for( h = gp->hunch ; h != NULL ; h = h->nh ){
	

				switch (h->type) {
				case NF_HUNCH:
					inf_hunch( h );
				    break;
				case NF_RH:
					inf_rhunch( h );
				    break;
				}

				nf_no++;
			}


			jnt_no = 0;
			for( jnt = gp->jnt ; jnt != NULL ; jnt = jnt->nj ){
				inf_jnt( jnt, &gp->ctr);
				jnt_no++;
			}


			inf_update( nf_no, &Inf );
			inf_update( jnt_no, &Ijnt );
			inf_update( jnt_no+nf_no, &Iall );

			Inf_nfm_ctrs.gp_no[ nf_no + jnt_no ]++;
		}

	inf_bgp( agp );
}



void
inf_agp_print(FILE * fp)
{
	inf_nfm_ctrs_print(fp);

	inf_hunch_print(fp);

	inf_rhunch_print(fp);

	inf_ridge_print(fp);

	inf_edge_print(fp);

	inf_n3_print(fp);

	inf_profile_print( fp );

	inf_bgp_print(fp);


	inf_jnt_print(fp);


	fprintf(fp, "\n\n\nTotal nf:\n\n");

	inf_hunch_print_total(fp);
	inf_rhunch_print_total(fp);
	inf_ridge_print_total(fp);
	inf_edge_print_total(fp);
	inf_n3_print_total(fp);

	fprintf(fp, "\n\n\nNf in Bgp*Bgp\n\n");

	inf_print( fp, "Nf", &Inf, 1.0 );
	inf_print( fp, "Jnt", &Ijnt, 1.0 );
	inf_print( fp, "All", &Iall, 1.0 );
}


static void
inf_nfm_ctrs_init()
{
	int     i;

	for (i = 0; i < MAX_NF; i++)
		Inf_nfm_ctrs.gp_no[i] = 0;

}


static void
inf_nfm_ctrs_print(FILE * fp)
{
	int     i;

	for (i = 0; i < MAX_NF; i++){
		if( Inf_nfm_ctrs.gp_no[i] == 0 )	continue;
		fprintf(fp, "\n Number of gps with %d NFs = %d \n",
			i, Inf_nfm_ctrs.gp_no[i]);
	}

}
