/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
#include	<stdio.h>
#include        "Uigp/igp.h"
#include        "Uinf/uinf.h"


#include	"InfIn.h" 


static int	Jnt_no,	Jcut_no,	Brn_no,	Brn_in;

typedef struct INF_JNT {
	inf	n3;
	inf	edge;
	inf	ridge;
	inf	rn3;
	inf	brn;
}              INF_JNT;

static INF_JNT Jnt;


void
inf_jnt_init()
{
	Jnt_no = Jcut_no = 0;
	Brn_no = 0;
	Brn_in = 0;

	Jnt.n3 = Inf_init;
	Jnt.edge = Inf_init;
	Jnt.ridge = Inf_init;
	Jnt.rn3 = Inf_init;
	Jnt.brn = Inf_init;
}



void
inf_jnt( jnt_type *jnt, vec2d *vm )
{
int	k,	no;
brn_type	*b;
vec2d	v,	ctr;

	v.x = vm->x + jnt->ctr.x;
	v.y = vm->y + jnt->ctr.y;

	Jnt_no++;
	if( jnt->Fborder == JNT_BRODER )	Jcut_no++;

	no = 0;
	for( k = 0 ; k < jnt->brn_no ; k++ ){
		b = jnt->brn[k];

		if( IS_BRN_NULL( b->type ) )	continue;
		if( b->type == BRN_IN ){
			Brn_in++;
			continue;
		}

                if( b->direction != F_BEGIN )        continue;
		

		ctr.x = v.x + b->ctr.x; 
		ctr.y = v.y + b->ctr.y;
                                                            
		switch( b->type ) {
		case NF_RIDGE:
		case NF_N3R:
			inf_ridge( &ctr, b->link );
		    break;
		case NF_EDGE:
			inf_edge( &ctr, b->link );
		    break;
		case NF_N3:
			inf_n3( &ctr, b->link );
		    break;
		}



		inf_profile( b->link, b->type, F_RIGHT );
		inf_profile( b->link, b->type, F_LEFT );

		Brn_no++;

		no++;
	}


	switch( jnt->type ){
	case JNT_N3:
		inf_update( (float)no, &Jnt.n3 );
	    break;
	case JNT_EDGE:
		inf_update( (float)no, &Jnt.edge );
	    break;
	case JNT_RIDGE:
		inf_update( (float)no, &Jnt.ridge );
	    break;
	case JNT_RN3:
		inf_update( (float)no, &Jnt.rn3 );
	    break;
	}

	inf_update( (float)no, &Jnt.brn );
}	



void inf_jnt_print ( FILE *fp )
{
	fprintf( fp, "\n\n\nJoint: %d( %d )     Brn: %d   Brn-in: %d\n",
				Jnt_no, Jcut_no, Brn_no, Brn_in );
	fprintf( fp, "---------------\n\n" );

	inf_print( fp, "J-n3", &Jnt.n3, 1.0 );
	inf_print( fp, "J-Edge", &Jnt.edge, 1.0 );
	inf_print( fp, "J-Ridge", &Jnt.ridge, 1.0 );
	inf_print( fp, "J-Rn3", &Jnt.rn3, 1.0 );
	fprintf( fp, "\n" );
	inf_print( fp, "J", &Jnt.brn, 1.0 );
}
