/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
#include	<stdio.h>
#include	"InfIn.h"
#include        "Uigp/igp.h"



static int N3_no,	Pno,	Pdup;

static struct INF_N3 {
	inf     interval;
	inf     width;

	inf     len;

	inf     rval[SP_NO];
	inf     mval[SP_NO];
	inf     lval[SP_NO];

	inf     rpval,  lpval;

}       In3;

static int W[10000];


void 
inf_n3_init()
{
int     sp;

	N3_no = Pno = Pdup = 0;
	In3.interval = Inf_init;
	In3.width = Inf_init;

	In3.len = Inf_init;
	In3.rpval = In3.lpval = Inf_init;

	for (sp = 0; sp < SP_NO; sp++) {
		In3.rval[sp] = Inf_init;
		In3.mval[sp] = Inf_init;
		In3.lval[sp] = Inf_init;
	}
}




void 
inf_n3( vec2d *ctr, link_type *link )
{
int	i,	ino,	k,	pr_no;
float   t;
link_type	*l;
width_type	*width;
n3_type *n3;

	N3_no++;

	for( l = link, ino = 0 ; l != NULL ; l = LINK_NEXT(l), ino++ ){

		inf_update( l->len, &In3.len );

		width = l->width[F_BEGIN];
		inf_update(width->w[F_RIGHT], &In3.width);
		inf_update(width->w[F_LEFT], &In3.width);


		k = width->w[F_RIGHT] * 100 + 0.5;
		if (k < 10000)	W[k]++;

		k = -width->w[F_LEFT] * 100 + 0.5;
		if (k < 10000)	W[k]++;

		width = l->width[F_END];


		pr_no = ( LINK_NEXT(l) == NULL )? 2 : 1;
		for( i = 0 ; i < pr_no ; i++ ){
			Pno++;

			n3 = l->pr[i].n3;


                        t = n3->side[F_RIGHT].val[SP_Y] -
                                                n3->side[F_RIGHT].bval[SP_Y];
                        inf_update( t, &In3.rpval );
                        t = n3->side[F_LEFT].val[SP_Y] -
                                                n3->side[F_LEFT].bval[SP_Y];
                        inf_update( t, &In3.lpval );



			inf_update(n3->mval[SP_Y], &In3.mval[SP_Y]);
			inf_update(n3->side[F_RIGHT].val[SP_Y], &In3.rval[SP_Y]);
			inf_update(n3->side[F_LEFT].val[SP_Y], &In3.lval[SP_Y]);
		}
	}

	inf_update(width->w[F_RIGHT], &In3.width);
	inf_update(width->w[F_LEFT], &In3.width);


	k = width->w[F_RIGHT] * 100 + 0.5;
	if (k < 10000)	W[k]++;

	k = -width->w[F_LEFT] * 100 + 0.5;
	if (k < 10000)	W[k]++;
		

	inf_update((float) ino, &In3.interval);
}


void 
inf_n3_print(FILE * fp)
{
	int     sp;

	fprintf(fp, "\n\nN3: %d\n", N3_no);
	fprintf(fp, "---------------\n\n");

	inf_print(fp, "Interval", &In3.interval, 1.0);
	inf_print(fp, "Length  ", &In3.len, 1.0);

	fprintf(fp, "\n");
	inf_print(fp, "Width", &In3.width, 1.0);

        inf_print(fp, "R-pval  ", &In3.rpval, 1.0);
        inf_print(fp, "L-pval  ", &In3.lpval, 1.0);


	for (sp = 0; sp < SP_NO; sp++) {

		fprintf(fp, "\nColor- %d\n", sp);

		inf_print(fp, "R-val  ", &In3.rval[sp], 1.0);

		inf_print(fp, "M-val  ", &In3.mval[sp], 1.0);

		inf_print(fp, "L-val  ", &In3.lval[sp], 1.0);

	}
}

void 
inf_n3_print_total(FILE * fp)
{
	float   w, inf_edge_max_width(int w[10000]);
	w = inf_edge_max_width(W);
	fprintf(fp, "N3    :   %d         wdith: %.4f\n", N3_no, w);
}
