/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
#include	<stdio.h>
#include        "Uigp/igp.h"
#include        "Uinf/uinf.h"

#include	"InfIn.h"


static int Rno;

static struct INF_N_RDG {
	inf     interval;
	inf     width;
	inf     len;
	inf     Lamda;

	inf     gval[SP_NO];
	inf     rval[SP_NO];

	inf	rvalQA;
}       Irdg;


static int	inf_ridge_sgn( link_type *link, int sp);



void 
inf_ridge_init()
{
int     sp;

	Rno = 0;

	Irdg.interval = Inf_init;
	Irdg.width = Inf_init;
	Irdg.len = Inf_init;
	Irdg.Lamda = Inf_init;

	for( sp = 0; sp < SP_NO; sp++ ){
		Irdg.gval[sp] = Inf_init;
		Irdg.rval[sp] = Inf_init;
	}

	Irdg.rvalQA = Inf_init;
}




void
inf_ridge( vec2d *ctr, link_type *link )
{
link_type	*l;
int     ino,	i, sp, sign[SP_NO],	pr_no;
ridge_type	*r;

	Rno++;

	for (sp = 0; sp < SP_NO; sp++)
		sign[sp] = inf_ridge_sgn( link, sp );

	for( l = link, ino = 0 ; l != NULL ; l = LINK_NEXT(l), ino++ ){

		inf_update( l->len, &Irdg.len );

		pr_no = ( LINK_NEXT(l) == NULL )? 2 : 1;
                for( i = 0 ; i < pr_no ; i++ ){
                        r = l->pr[i].ridge;

			inf_update(ABS( r->width), &Irdg.width);

			inf_update(ABS( r->lm[SP_Y]), &Irdg.Lamda);

			for (sp = 0; sp < SP_NO; sp++ ){

				inf_update(r->mval[sp], &Irdg.gval[sp]);

				if( sign[sp] == 1)
				    inf_update( r->rval[sp], &Irdg.rval[sp] );
				else
				    inf_update(-r->rval[sp], &Irdg.rval[sp] );


			}
			if( ABS(r->rval[SP_Q] ) > ABS(r->rval[SP_A]) )
				inf_update( r->rval[SP_Q], &Irdg.rvalQA );
			else	inf_update( r->rval[SP_A], &Irdg.rvalQA );
		}
	}

	inf_update((float) ino, &Irdg.interval);
}


static int
inf_ridge_sgn( link_type *link, int sp)
{
int     i, m, p,	pr_no;
link_type	*l;
ridge_type *r;


	if (sp == SP_Y) {
		r = link->pr[0].ridge;
		if( r->lm[sp] < 0)
			return (1);
		return (0);
	}

	m = p = 0;
	for( l = link ; l != NULL ; l = LINK_NEXT(l) ){
		pr_no = ( LINK_NEXT(l) == NULL )? 2 : 1;

		for( i = 0 ; i < pr_no ; i++ ){
			r = l->pr[i].ridge;
			if( r->rval[sp] < 0 )
				m++;
			else
				p++;
		}
	}

	if (m < p)
		return (1);
	return (0);
}



void
inf_ridge_print(FILE * fp)
{
	int     sp;

	fprintf(fp, "\n\nRidge: %d\n", Rno);
	fprintf(fp, "---------------\n");


	inf_print(fp, "Interval", &Irdg.interval, 1.0);
	inf_print(fp, "Length  ", &Irdg.len, 1.0);
	inf_print(fp, "Lamda", &Irdg.Lamda, 1.0);
	inf_print(fp, "Width", &Irdg.width, 1.0);


	for (sp = 0; sp < SP_NO; sp++) {

		fprintf(fp, "\nColor- %d\n", sp);

		inf_print(fp, "Gval  ", &Irdg.gval[sp], 1.0);
		inf_print(fp, "Bval  ", &Irdg.rval[sp], 1.0);
	}

	inf_print( fp, "\nRval QA", &Irdg.rvalQA, 1.0 );
}


void
inf_ridge_print_total(FILE * fp)
{
	fprintf(fp, "Ridge :  %d\n", Rno);
}
