/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/************************
 ***	Beizer2Link.c ***
 ************************/

#include <math.h>

#include	"GpType/NfType.h"
#include	"GpType/LinkType.h"


#include	"LinkTool.h"





#define STEP 100.0


gapp_prm	Qgprm;

static void		beizer_to_gapp( vec2d*p0,vec2d*p1,vec2d*p2,vec2d*p3,
										gapp_type *gapp);


static link_type *	link_from_conic_bezier_analitic(vec2d *p0,vec2d *p1,vec2d *p2, link_type **last );

static void		conic_beizer_to_gapp( vec2d *p0,vec2d *p1,vec2d *p2 ,gapp_type *gapp );

static int	beizer_is_line(vec2d *p0,vec2d *p1,vec2d *p2,vec2d *p3 );




/*
static GP_prm params[]  = {
	{"RL-D-weight" , (char *)&Qgprm.dwight, PRM_FLOAT,"%.2f","0.10"},
	{"RL-Ls-error" , (char *)&Qgprm.err, PRM_FLOAT,"%.2f","0.01"},

	{"RL-len-min" , (char *)&Qgprm.min_len,PRM_FLOAT,"%.2f","3.0"},
	{"RL-min-no" , (char *)&Qgprm.Min_pnumber,PRM_INTEGER,"%d","10"},
	{"RL-len-max" , (char *)&Qgprm.max_len,PRM_FLOAT,"%.2f","24.0"},

	{"RL-dev-max" , (char *)&Qgprm.Max_dev,PRM_FLOAT,"%.2f","1.0"},
	{"RL-dev-min" , (char *)&Qgprm.Min_dev,PRM_FLOAT,"%.2f","0.1"},
};


void
beizer_to_link_get_prm(GP_prm **prm, int *prm_no, void (**check)() )
{
	*prm    = params;
	*prm_no  = PRM_NUMBER(params);
	*check = NULL ;
}


void
beizer_to_link_init( char *file )
{
	gp_prm_init( file, beizer_to_link_get_prm );
}
*/


link_type *
beizer_curve_to_links(vec2d *p, int nCount )
{
	
link_type *l=NULL;
int	j;
link_type *ll,*poly = NULL;


//	beizer_to_link_init(NULL);


	Qgprm.dwight = 0.10;
	Qgprm.err = 0.01;

	Qgprm.min_len = 3.0;
	Qgprm.Min_pnumber = 10;
	Qgprm.max_len = 24.0;

	Qgprm.Max_dev = 1.0;
	Qgprm.Min_dev = 0.1;


	for( j=0;j<nCount-1;j+=3)
	{
		l = beizer_to_Links(&p[j],&p[j+1],&p[j+2],&p[j+3] );


		if( poly == NULL ){
			poly = l;
			poly->ctr = p[0];
			continue;
		}


		for( ll = poly ; LINK_NEXT(ll ) != NULL ; ll = LINK_NEXT(ll ) );
		
		ll->p[F_END].bno=-1;
		ll->p[F_END].p.l=l;

		free( ll->pr[F_END].p );
		ll->pr[F_END] = l->pr[ F_BEGIN];

		l->p[F_BEGIN].bno =-1;
		l->p[F_BEGIN].p.l =ll;

	}

	return poly;
}




link_type * 
beizer_to_Links(vec2d*p0,vec2d*p1,vec2d*p2,vec2d*p3 )
{
gapp_type *gapp;
link_type *l;

	if( beizer_is_line( p0, p1, p2, p3 ) ){	
		l = links_from_line( p0, p3, 16, NF_EDGE, NULL );
			
		l->ctr = *p0;
		return( l );
	}



	gapp = gapp_alloc( 4096 );
	beizer_to_gapp(p0, p1, p2, p3, gapp);
	

	geo_approximate( gapp, &Qgprm );
		
	l= link_from_gapp( gapp, NF_EDGE, NULL );


	
	l->ctr = gapp->v[0];

	gapp_destroy( gapp );

	return( l );
}



link_type * 
link_from_qubic_bezier(vec2d *p0,vec2d *p1,vec2d *p2,vec2d *p3, link_type **last )
{
gapp_type *gapp;
link_type *l;


	gapp = gapp_alloc( 4096 );

	beizer_to_gapp(p0, p1, p2, p3, gapp);
	

	geo_approximate( gapp, &Qgprm );
		
	l= link_from_gapp( gapp, NF_EDGE, last );


	
	l->ctr = gapp->v[0];

	gapp_destroy( gapp );

	return( l );
}




static void 
beizer_to_gapp( vec2d*p0,vec2d*p1,vec2d*p2,vec2d*p3 ,gapp_type *gapp)
{
float t=0.0;
float t1=0.0;


	double res = 1.0/STEP;
	gapp->no =0;
	
	for(t=0;t<1.0;t+=res,gapp->no++)
	{
		t1 = 1.0-t;
		if(t1<0.01)
		{
			t1=0;
			t=1.0;
		}

		gapp->v[gapp->no].x = t1*t1*t1 *p0->x +
				3.0*t1*t1 * t * p1->x +
				3.0*t1*t*t*p2->x + t*t*t*p3->x;

	  	gapp->v[gapp->no].y = t1*t1*t1 *p0->y +
				3.0*t1*t1 * t * p1->y +
				3.0*t1*t*t*p2->y +
				t*t*t*p3->y;
		
		
		gapp->tang[gapp->no].x = ( -3.0 * p0->x * t1*t1 ) +
					(  3.0*p1->x*t1*t1 ) +
					(-6.0*t*t1*p1->x) +
					(  6.0*p2->x*t*t1) +
					(-3.0*p2->x*t*t ) +
					(  3.0*t*t*p3->x);




		gapp->tang[gapp->no].y = ( -3.0 * p0->y * t1*t1 ) +
					(  3.0*p1->y*t1*t1 ) +
					(-6.0*t*t1*p1->y)  + 
					(  6.0*p2->y*t*t1) + 
					(-3.0*p2->y*t*t ) + 
					(  3.0*t*t*p3->y);

		gapp->flag[gapp->no] = 0;
	}


}



link_type * 
link_from_conic_bezier(vec2d *p0,vec2d *p1,vec2d *p2, link_type **last )
{
gapp_prm	qgprm;
gapp_type *gapp;
link_type *l,	*l1;

//		l = links_from_line( p0, p2, 30, NF_EDGE, last );
			
//		l->ctr = *p0;
//		return( l );


	l1 = link_from_conic_bezier_analitic( p0, p1, p2, last );
	if( l1 != NULL ){
		l = l1;
	}


	qgprm.dwight = 0.10;
	qgprm.err = 0.01;

	qgprm.min_len = 3.0;
	qgprm.Min_pnumber = 10;
	qgprm.max_len = 24.0;

	qgprm.Max_dev = 1.0;
	qgprm.Min_dev = 0.1;



//	gapp = (gapp_type *)malloc( sizeof( gapp_type) );
	gapp = gapp_alloc( 4096 );
	conic_beizer_to_gapp(p0, p1, p2, gapp);
	

	geo_approximate( gapp, &qgprm );
		
	l= link_from_gapp( gapp, NF_EDGE, last );


	
	l->ctr = gapp->v[0];

//	free( gapp );

	gapp_destroy( gapp );

	

	return( l );
}



static link_type * 
link_from_conic_bezier_analitic(vec2d *p0,vec2d *p1,vec2d *p2, link_type **last )
{
vec2d	v,	u;
link_type	*l;
float	a,	t;

	v.x = p2->x - p0->x;
	v.y = p2->y - p0->y;

	if( ( t = 0.5*v.x - p1->x ) < 0 )	t = -t;
	if( t > 0.01 )	return( NULL );

	if( ( t = 0.5*v.y - p1->y ) < 0 )	t = -t;
	if( t > 0.01 )	return( NULL );


	t = hypot( v.x , v.y );

	if( t > 20 )	return( NULL );

	u.x = - t * v.y;
	u.y = t * v.x;


	a = (p1->x - p0->x ) * u.x + (p1->y- p0->y) * u.y;

	a *= 0.5;

	l = *last = link_alloc( NF_EDGE, 1 );

	l->v = v;
	l->ctr = *p0;
	l->a = a;

	link_set_aux( l );

	return( l );
}


static void 
conic_beizer_to_gapp( vec2d *p0,vec2d *p1,vec2d *p2 ,gapp_type *gapp )
{
float t;
float t1;


	double res = 1.0/STEP;
	gapp->no =0;

	t = hypot( p2->x - p0->x, p2->y - p0->y );
	if( t < 1.0 )	res = 0.2;


	
	for( t = 0 ; t < 1.0 ; t += res, gapp->no++ ){
		t1 = 1.0-t;

		if(t1<0.01){
			t1=0;
			t=1.0;
		}

		gapp->v[gapp->no].x = t1*t1 *p0->x +
				2.0*t1 * t * p1->x +
				t*t*p2->x;

		gapp->v[gapp->no].y = t1*t1 *p0->y +
				2.0*t1 * t * p1->y +
				t*t*p2->y;



		gapp->tang[gapp->no].x = -2*t1 *p0->x +
				2.0*( 1 - 2*t) * p1->x +
				2*t*p2->x;

		gapp->tang[gapp->no].y = -2*t1 *p0->y +
				2.0*( 1 - 2*t) * p1->y +
				2*t*p2->y;

		
		

		gapp->flag[gapp->no] = 0;
	}


}



static int
beizer_is_line(vec2d *p0,vec2d *p1,vec2d *p2,vec2d *p3)
{
	vec2d v,u,u1;
	float d=0;

	v.x = p3->x-p0->x; 
	v.y = p3->y-p0->y;
	

	u.x = -v.y;
	u.y = v.x;

	u1.x = u.x/(sqrt((u.x*u.x)+(u.y*u.y)));//cos
	u1.y = u.y/(sqrt((u.x*u.x)+(u.y*u.y)));//sin


	d = (p1->x -p0->x)*u1.x + (p1->y -p0->y)*u1.y;
	if( ABS( d ) > 0.0001 )	return( 0 );


	d = (p2->x -p0->x)*u1.x + (p2->y -p0->y)*u1.y;
	if( ABS( d ) > 0.0001 )	return( 0 );


	return( 1 );
}


