/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/*************************** 
 ***    LinkAppend.c   ***
 ***************************/
#include	<stdio.h>
#include	<math.h>

#include	"Uigp/igp.h"

#include	"GpType/GpType.h"

#include	"AgpEdit/AgpEditTool.h"

#include	"LinkTool.h"







void
polylink_append( vec2d *ctr, link_type *link, vec2d *v2, link_type *l2 )
{
link_type	*l,	*l1,	*sl;
vec2d	v,	v1,	p,	dv;
float	u,	t,	gt;




	for( l = link, v = *ctr ; LINK_NEXT(l) != NULL ; l = LINK_NEXT(l) ){
		v.x += l->v.x;
		v.y += l->v.y;
	}

	v1.x = v.x + l->v.x;
	v1.y = v.y + l->v.y;


	polylink_distance( v2, l2, NULL, &v1, &sl, &u, &t, &gt );



	dv.x = v2->x - v1.x;
	dv.y = v2->y - v1.y;


	if( t < 0 ){

		if( dv.x * dv.x + dv.y * dv.y < 1.0 ){
			p.x = 0.5*(v1.x + v2->x);
			p.y = 0.5*(v1.y + v2->y);


			l->v.x = p.x - v.x;
			l->v.y = p.y - v.y;
			link_set_aux(l);

			l2->v.x += v2->x - p.x;
			l2->v.y += v2->y - p.y;

			link_set_aux( l2 );

			return;
		}


		l1 = link_1_alloc();
		l1->v.x = v2->x - v1.x;
		l1->v.y = v2->y - v1.y;
		l1->a = 0;
		link_set_aux( l1 );

		l->p[1].p.l = l1;
		l1->p[0].p.l = l;
		l->p[1].bno = l1->p[0].bno = -1;


		l1->p[1].p.l = l2;
		l2->p[0].p.l = l1;
		l1->p[1].bno = l2->p[0].bno = -1;

		return;
	}
}





int
polylink_append_point( vec2d *ctr, link_type **link, vec2d *p, float Tlen, int Fsimplify )
{
link_type	*l,	*l1,	*sl;
vec2d	v,	v1;
float	len,	u,	t,	gt;

	v.x = p->x - ctr->x;
	v.y = p->y - ctr->y;

	if( *link == NULL ){

		if( v.x*v.x + v.y *v.y < Tlen ){
			if( Fsimplify == 1 )
				*ctr = *p;

			return( Fsimplify );
		}


		*link = l = link_1_alloc();
		l->ctr = *ctr;
		l->v = v;
		l->a = 0;
		link_set_aux( l );


		return( 0 );
	}





	if( Fsimplify == 1 ){
		len = polylink_distance( ctr, *link, NULL, p, &sl, &u, &t, &gt );
		if( sl != NULL && ABS(u) < 4*Tlen*Tlen && gt < len - 0.1 ){
			l = link_ssplit( sl, t, 0.05 );
			sl = LINK_PREV(l);
			if ( sl != NULL )
				sl->p[1].p.l = NULL;
			link_free(l);
		}
	}




	for( l = *link, v = *ctr ; LINK_NEXT(l) != NULL ; l = LINK_NEXT(l) ){
		v.x += l->v.x;
		v.y += l->v.y;
	}

	v1.x = v.x + l->v.x;
	v1.y = v.y + l->v.y;



	l1 = link_1_alloc();
	l1->ctr = v1;
	l1->v.x = p->x - v1.x;
	l1->v.y = p->y - v1.y;
	l1->a = 0;
	link_set_aux( l1 );

	l->p[1].p.l = l1;
	l1->p[0].p.l = l;
	l->p[1].bno = l1->p[0].bno = -1;
	l1->p[1].p.l = NULL;


	if( Fsimplify == 0 )	return( 0 );


	polylink_distance( ctr, *link, l1, &v1, &sl, &u, &t, &gt );
	if( l1->len > Tlen || ABS(u) > Tlen*Tlen )
		return( 0 );



	if( Fsimplify == 1 ){
		l->v.x = p->x - v.x;
		l->v.y = p->y - v.y;
		link_set_aux( l );
	}


	// Fsimplify == 2
	l->p[1].p.l = NULL;

	link_1_free( l1 );


	return( Fsimplify );
}


void
link_set_end_point( vec2d *ctr, link_type *link, vec2d *p );


void
polylink_close( vec2d *ctr, link_type *link, float Tlen )
{
link_type	*l,	*l0,	*l1,	*sl;
vec2d	v,	v0, v1,	dv;
float	u,	t,	gt,	len;



	for( l = link, v = *ctr ; LINK_NEXT(l) != NULL ; l = LINK_NEXT(l) ){
		v.x += l->v.x;
		v.y += l->v.y;
	}

	v1.x = v.x + l->v.x;
	v1.y = v.y + l->v.y;

	
	v0.x = ctr->x + link->v.x;
	v0.y = ctr->y + link->v.y;
	l0 = LINK_NEXT(link);

	len = polylink_distance( &v0, l0, l, ctr, &sl, &u, &t, &gt );

	if( gt > len && ABS(u) < 1.5 ){
		polylink_split( ctr, link, sl, t, &v1, &l1 );

		if( l1 != NULL )	link_free( l1 );

		for( l = link, v = *ctr ; LINK_NEXT(l) != NULL ; l = LINK_NEXT(l) ){
			v.x += l->v.x;
			v.y += l->v.y;
		}

		v1.x = v.x + l->v.x;
		v1.y = v.y + l->v.y;
	}


	dv.x = v1.x - ctr->x;
	dv.y = v1.y - ctr->y;

	if( dv.x * dv.x + dv.y *dv.y < Tlen *Tlen ){
		link_set_end_point( &v, l, ctr );

		if( l->len < 0.75 ){
			l1 = l;
			l = LINK_PREV(l);
			v.x -= l->v.x;
			v.y -= l->v.y;
			l->p[1].p.l = NULL;
			link_1_free( l1 );

			link_set_end_point( &v, l, ctr );
		}

		return;
	}


	l1 = link_1_alloc();
	l1->v.x = ctr->x - v1.x;
	l1->v.y = ctr->y - v1.y;
	l1->ctr = v1;
	l1->a = 0;
	link_set_aux( l1 );

	l->p[1].p.l = l1;
	l1->p[0].p.l = l;
	l->p[1].bno = l1->p[0].bno = -1;
}



void
link_set_end_point( vec2d *ctr, link_type *link, vec2d *p )
{
	float	len;

	link->v.x = p->x - ctr->x;
	link->v.y = p->y - ctr->y;

	len = link->len;

	link_set_aux( link );

	link->a *= link->len / len;
	link->c_prb *= link->len / len;
}


void
polylink_split( vec2d *ctr, link_type *link, link_type *sl, float t, vec2d *ctr1, link_type **l1 )
{

vec2d	v;
link_type	*l;


	for( l = link, v = *ctr ; l != sl ; l = LINK_NEXT(l) ){
		v.x += l->v.x;
		v.y += l->v.y;
	}

	if( t > sl->len - 0.25 ){
		*l1 = LINK_NEXT(sl);
		ctr1->x = v.x + sl->v.x;
		ctr1->y = v.y + sl->v.y;

		sl->p[1].p.l = NULL;
		if( *l1 != NULL )
			(*l1)->p[0].p.l = NULL;
		return;
	}

	if( t < 0.25 ){
		*l1 = sl;
		*ctr1 = v;

		if( (l = LINK_PREV(sl)) != NULL )
			l->p[1].p.l = NULL;

		(*l1)->p[0].p.l = NULL;
		v.x -= sl->v.x;
		v.y -= sl->v.y;
	}



	*l1 = link_split_t( sl, t );

	ctr1->x = v.x + sl->v.x;
	ctr1->y = v.y + sl->v.y;

	sl->p[1].p.l = NULL;
	if( *l1 != NULL )
		(*l1)->p[0].p.l = NULL;
}
