/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
#include	"GpType/LinkType.h"
#include	"LinkTool.h"

 
int lessThan(float a, float b);
int greaterThan(float a, float b);
float boxValueX0(box2d *b);
float boxValueX1(box2d *b);
float boxValueY0(box2d *b);
float boxValueY1(box2d *b);
float vecValueX(vec2d *v);
float vecValueY(vec2d *v);

typedef float (*vecValue)(vec2d *v);
typedef float (*boxValue)(box2d *b);
typedef int (*compare)(float a, float b);

static link_type *link_connect_gap(link_type *l0, link_type *l1);
//static link_type *link_clip_by_edge(link_type *link, box2d *box, vecValue vVal, boxValue edge, compare outside, compare inside);
static link_type* link_clip_by_edge(vec2d *gCtr, link_type *link, float axisVal, vecValue vVal, compare outside, compare inside);

link_type *newLink(int type1, float x0, float y0, float x1, float y1);



/**** link_clip ****/
/*!
 * clips a link by each of the box's edges.
 *
 * @param gCtr : the global center of the first link.
 * @param *link : the link to clip
 * @param *box : 
 *
 * @return link_type*  : the link after clipping
 */
link_type* link_clip(vec2d *gCtr, link_type *link, box2d *box)
{
	link = link_clip_by_edge(gCtr, link,box->x0,vecValueX,lessThan,greaterThan);
	link = link_clip_by_edge(&link->ctr, link,box->x1,vecValueX,greaterThan,lessThan);
	link = link_clip_by_edge(&link->ctr, link,box->y0,vecValueY,lessThan,greaterThan);
	link = link_clip_by_edge(&link->ctr, link,box->y1,vecValueY,greaterThan,lessThan);
	polylink_filter_short_link(&link->ctr, link, 1.0f);
	link_set_gCtr(link,&link->ctr);
	return link;	
}



/**** link_frame_box ****/
/*!
 * creates a frame around the box
 *
 * @param type1 : type of the polylink
 * @param *box : 
 *
 * @return link_type*  : the frame
 */
link_type* link_frame_box(int type1, box2d *box)
{
	link_type *l0, *l1, *l2, *l3;
	l0 = newLink(type1, box->x0, box->y1, box->x0, box->y0);
	l1 = newLink(type1, box->x0, box->y0, box->x1, box->y0);
	l2 = newLink(type1, box->x1, box->y0, box->x1, box->y1);
	l3 = newLink(type1, box->x1, box->y1, box->x0, box->y1);

	link_set_prev_next(l0,l1);
	link_set_prev_next(l1,l2);
	link_set_prev_next(l2,l3);
	l3->p[1].p.l = NULL;
	
	l0->p[0].p.l = NULL;

	link_set_aux( l0 ) ;
	return l0;	
}





/**** link_clip_by_edge ****/
/*!
 * clips the link so it won't extend out of the box edge (only one edge, for which we have boxValue)
 *
 * @param gCtr : the global center of the first link.
 * @param *link : the link to clip
 * @param val : the value that represents the edge axis
 * @param vVal : vecValue function
 * @param outside : comparison function (determines if the link got out of the line)
 * @param inside : comparison function (determines if the link returned to the line)
 *
 * @return link_type*  : the new link after clipping
 */
//link_type* link_clip_by_edge(link_type *link, box2d *box, vecValue vVal, boxValue edge, compare outside, compare inside)
link_type* link_clip_by_edge(vec2d *gCtr, link_type *link, float axisVal, vecValue vVal, compare outside, compare inside)
{
	link_type  *link0 = NULL, *link1 = NULL;
	link_type  *link0a = NULL, *link1a = NULL;
	link_type *next;
	vec2d p0, p1, ctr, newCtr;	
	float t0, t1, t2;	
	int got_in_again = 0;
	// scan all links
	ctr = *gCtr;
	newCtr = ctr;
	for ( link0 = link; link0 != NULL ; link0 = next)
	{
		got_in_again = 0;
		next = LINK_NEXT(link0);
		ctr = newCtr;
		link0->ctr = newCtr;
		VEC2D_SET(newCtr, ctr.x + link0->v.x, ctr.y + link0->v.y);
		// scan each link with delta of 0.1 to see if it's outside.
		for ( t0 = 0 ; link0 != NULL && t0 < link0->len ; t0 += 0.1 )
		{
			// t0 + 0.1 so if I find it's out, t0 is still inside...
			t2 = t0 + 0.1;
			if (t2 > link0->len)
				t2 = link0->len;
			link_t2xy(&link0->ctr, link0, t0 + 0.1, &p0);			
			if ( outside(vVal(&p0),axisVal) == 1)
			{				
				link_t2xy(&link0->ctr, link0, t0 , &p0);// now I have a point inside while (t0 + 0.1) is outside...
				got_in_again = 0;
				newCtr = ctr; // for the first assignment (ctr = newCtr ...)
				for ( link1 = link0 ; !got_in_again && link1 != NULL ; link1 = LINK_NEXT(link1))
				{
					ctr = newCtr;
					link1->ctr = newCtr;
					VEC2D_SET(newCtr, ctr.x + link1->v.x, ctr.y + link1->v.y);
					t1 = t2;
					while ( !got_in_again && t1 <= link1->len ) 
					{
						link_t2xy(&link1->ctr, link1, t1 , &p1);						
						if ( inside(vVal(&p1),axisVal) == 1)
						{// we have a link that got out and in...
							if ( t0 == 0 )
							{
								if (  link0 == link)// means the begin point is out of box
								{

									link1a = link_split(&link1->ctr, link1, &p1);
									// set ctr for next link
									newCtr = link1a->ctr; // set by split.
									link1->p[1].p.l = NULL ;
									link_free(link);
									link = link1a;
									link->p[0].p.l = NULL;
									link0 = NULL;
								}// if begin point is outside of the line
								else 
								{// a link (not first) starts outside.
									//if the same link that got out is also the one that enters back in...
									if ( link1 == link0 ) 
									{
										link1->a = 0;
										link1a = LINK_NEXT(link1);
										link_set_aux(link1);
										VEC2D_SET(newCtr, ctr.x + link1->v.x, ctr.y + link1->v.y);
									}
									else 
									{	
										link1a = link_split(&link1->ctr, link1, &p1);
										newCtr = link1a->ctr;
										// link1a is the new link to connect to.
										link0a = link0;
										link0 = LINK_PREV(link0);
										// link0a is the one to free later
										link_connect_gap(link0, link1a);
										link1->p[1].p.l = NULL ;
										link_free(link0a);
										link1 = link1a;
									}
								}
							}// if t == 0 
							else
							{
								if ( link1 == link0 )
								{
									link1->a = 0;
									link_set_aux(link1);
									link1a = LINK_NEXT(link1);
									VEC2D_SET(newCtr, link1->ctr.x + link1->v.x, link1->ctr.y + link1->v.y);
								}
								else
								{
									link1a = link_split(&link1->ctr, link1, &p1);
									// link1a is the new link to connect to.
									link0a = link_split(&link0->ctr, link0, &p0);
									newCtr = link1a->ctr;
									// link0a is the one to free later
									link_connect_gap(link0, link1a);
									link1->p[1].p.l = NULL ;
									link_free(link0a);
									link1 = link1a;
								}
								
							}
							got_in_again = 1;
							next = link1a;
							break; // from the for (t1 = t0 ... )
						}
						if ( t1 == link1->len ) 
						{
							// means we set it once to equal link1->len, don't loop infinitely
							t1 += 0.1;
						}
						else 
						{
							t1 += 0.1;
							// check that t won't extent out of link1.
							if ( t1 > link1->len )
							{
								t1 = link1->len;// so it checks the end point!!!
							}
						}
					}// while// for t1
					t2 = 0;
				} //for link1
				if ( !got_in_again )// means the link ends out of the box.
				{
					link0a = link_split(&link0->ctr, link0, &p0);
					link1a = link_connect_gap(link0,link);// connect to first link.
					link1a->p[1].p.l = NULL;// disconnect the new link from the first one
					link->p[0].p.l = NULL;// disconnect the first one from the new link
					link_free(link0a);
					t0 = link0->len;
					next = NULL;
				}

			}// if 
		}// for t0
	}// for link0
return link;
}



/*!
 * The method creates a new link to connect l0 to l1. The new link is next of l0, and 
 * previous of l1. If that link is shorter than 1, it is discarded, and l0 is connected
 * directly to l1.
 *
 * @param *link : a link in a polylink
 *
 * @return link_type  : the last link
 */
link_type *link_connect_gap(link_type *l0, link_type *l1)
{
	vec2d l1ctr, l0end;
	link_type  *newLink;
	l1ctr = l1->ctr;
	link_end_point(l0,&l0end );

	if ( vec2d_distance(&l0end, &l1ctr) < 1 ) 
	{
		l0->v.x = l1->ctr.x - l0->ctr.x;
		l0->v.y = l1->ctr.y - l0->ctr.y;
		link_set_aux(l0);
		link_set_prev_next(l0,l1);
		return l0;
	}


	newLink = link_1_alloc();

	newLink->ctr.x = l0->ctr.x + l0->v.x;
	newLink->ctr.y = l0->ctr.y + l0->v.y;	
	newLink->v.x = l1->ctr.x - newLink->ctr.x;
	newLink->v.y = l1->ctr.y - newLink->ctr.y;

	newLink->a = 0;
	newLink->type = l0->type;
	newLink->br = l0->br;
	newLink->Fbr = l0->Fbr;	

	// set sequence : l0 --> newLink --> l1
	link_set_prev_next(l0,newLink);
	link_set_prev_next(newLink, l1);
	link_set_aux( newLink ) ;
	return newLink;
}



link_type *newLink(int type1, float x0, float y0, float x1, float y1)
{
	link_type  *newLink ;
	
	newLink = link_1_alloc();
	newLink->v.x = x1 - x0;
	newLink->v.y = y1 - y0;	
	newLink->ctr.x = x0;
	newLink->ctr.y = y0;
	newLink->a = 0;
	newLink->type = type1;
	link_set_aux(newLink);
	return newLink;
}



//api functions
int lessThan(float a, float b){return (a < b);}		
int greaterThan(float a, float b){return ( a > b );}
float boxValueX0(box2d *b){ return b->x0; }
float boxValueX1(box2d *b){ return b->x1; }
float boxValueY0(box2d *b){ return b->y0; }
float boxValueY1(box2d *b){ return b->y1; }
float vecValueX(vec2d *v){return v->x; }
float vecValueY(vec2d *v){return v->y; }

