/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/*************************** 
 ***    LinkFromLine.c   ***
 ***************************/
#include	<stdio.h>
#include	<math.h>

#include	"Uigp/igp.h"

#include	"GpType/GpType.h"

#include	"AgpEdit/AgpEditTool.h"




float
link_distance( vec2d *ctr, link_type *l, vec2d *v, float *u, float *t )
{
vec2d	dv;
float	dt,	cu;


	dv.x = v->x - ctr->x;
	dv.y = v->y - ctr->y;

	*t = -dv.x *l->u.y + dv.y *l->u.x;

	*u = l->u.x * dv.x + l->u.y * dv.y;

	if( *t < 0 ){
		cu = hypot( *u, -1.2*(*t) );

		if( *u < 0 )	*u = -cu;
		else	*u = cu;

		return( cu );
	}

	if( (dt = *t - l->len) > 0 ){
		cu = hypot( *u, 1.2*dt );

		if( *u < 0 )	*u = -cu;
		else	*u = cu;

		return( cu );
	}

	*u -= l->c_prb*(*t) *((*t) - l->len );

	return( *u );
}


float
link_distance2( vec2d *ctr, link_type *l, vec2d *v, float *u, float *t )
{
vec2d	dv;
float	dt,	cu;


	dv.x = v->x - ctr->x;
	dv.y = v->y - ctr->y;

	*t = -dv.x *l->u.y + dv.y *l->u.x;

	*u= l->u.x * dv.x + l->u.y * dv.y;

	if( *t < 0 ){
		if( *u > 0 ) *u -= *t;
		else	*u += *t;

		cu = (*u)*(*u) + 1.44*(*t)*(*t);

		return( cu );
	}

	if( (dt = *t - l->len) > 0 ){
		if( u > 0 ) *u -= dt;
		else	*u += dt;

		cu = (*u)*(*u) + 1.44*dt*dt;

		return( cu );
	}

	*u -=  - l->c_prb*(*t) *((*t) - l->len );
	cu = (*u)*(*u);
	return( cu ); 
}



void
polylink_filter_short_link( vec2d *ctr, link_type *link, float d )
{
link_type *l,	*l1,	*l2;
vec2d	p,	p1,	p2,	pm;


	if ( link && link->len <= d && (LINK_NEXT(link)))
	{
		l1 = LINK_NEXT(link);
		*link = *l1;
		link->p[0].p.l = NULL;
		l2 = LINK_NEXT(link);
		if (l2 != NULL ) 
			l2->p[0].p.l = link;
		link_1_free(l1);
	}

	for( l = link, p = *ctr ; LINK_NEXT(l) != NULL ; l = LINK_NEXT(l) ){

		l1 = LINK_NEXT(l);
		
		if( l1->len > d ){
			p.x += l->v.x;
			p.y += l->v.y;
			continue;
		}

		p1.x = p.x + l->v.x;
		p1.y = p.y + l->v.y;

		l2 = LINK_NEXT(l1);

		if( l2 == NULL ){
			break;
		}

		p2.x = p1.x + l1->v.x;
		p2.y = p1.y + l1->v.y;

		pm.x = 0.5*( p1.x + p2.x);
		pm.y = 0.5*( p1.y + p2.y);



		l2->v.x = p2.x + l2->v.x - pm.x;
		l2->v.y = p2.y + l2->v.y - pm.y;

		l2->ctr = pm;

		link_set_aux( l2 );



		l->v.x = pm.x - p.x;
		l->v.y = pm.y - p.y;

		link_set_aux( l );

		l->p[1].p.l = l2;
		l2->p[0].p.l = l;

		link_1_free( l1 );

		p.x += l->v.x;
		p.y += l->v.y;
	}


	if( l->len < d ){
		l1 = LINK_PREV( l );
		if( l1 != NULL ){
			l1->v.x += l->v.x;
			l1->v.y += l->v.y;
			link_set_aux( l1 );
			l1->p[1] = l->p[1];

			link_1_free( l );
		}
	}

	if( link->len < d ){
		l1 = LINK_NEXT( l );
		if( l1 != NULL ){
			link->v.x += l1->v.x;
			link->v.y += l1->v.y;
			link_set_aux( link );
			link->p[1] = l1->p[1];

			l2 = LINK_NEXT(l1 );
			if( l2 != NULL )
				l2->p[0].p.l = link;

			link_1_free( l1 );
		}
	}

}





float
polylink_distance( vec2d *ctr, link_type  *link, link_type *elink, vec2d  *v,
						link_type **sl, float *u, float *t, float *gt )
{
link_type	*l;
vec2d	cv;
float	cu,	ct,	dt,	len;
float	dis,	tmp;


	*sl = NULL;
	*u = 100;
	*t = 100;


	dis = 100000;

	cv.x = v->x - ctr->x;
	cv.y = v->y - ctr->y;

	len = 0;

	for( l = link ; l != elink && l != NULL ; l = LINK_NEXT(l) ){

		cu = l->u.x * cv.x + l->u.y * cv.y;

		ct = -l->u.y * cv.x + l->u.x * cv.y;

		cv.x -= l->v.x;
		cv.y -= l->v.y;



		if( ct < 0 ){
			tmp = cu*cu + 1.44*ct*ct;

			if( cu > 0 )	cu -= ct;
			else	cu += ct;

//			ct = 0;
		}
		else	if( (dt = ct - l->len) > 0 ){

				tmp = cu*cu + 1.44*dt*dt;

				if( cu > 0 )	cu += dt;
				else	cu -= dt;
//				ct = l->len;
			}
			else	{
					cu -= l->c_prb*ct *(ct - l->len );
					tmp = cu*cu;
				}


			if( tmp >= dis ){
				len += l->len;
				continue;
			}

		dis = tmp;


		*u = cu;
		*t = ct;
		*sl = l;
		*gt = len;

		len += l->len;
	}


	if( *sl == NULL )	return( len );

	if( *t < 0 ){
		if( *sl == link )	*gt = *t;
	}
	else	if( *t > (*sl)->len )
				*gt += ( LINK_NEXT((*sl)) == elink )? *t : (*sl)->len;
			else	*gt += *t;


	return( len );
}



