/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/* ******************************************************************** *\
                          LinkFilter.c				

link_type *
link_filter( vec2d *ctr, link_type *link, float min_width );

if min_width < 0 then min_width = MIN_WIDTH
\* ******************************************************************** */

#include	<math.h>

#include	"GpType/GpType.h"
#include	"LinkTool.h"
#include	"AgpEdit/AgpEditTool.h"
#include	"RlLink/RlType.h"


#define MIN_WIDTH 2.0F

typedef link_type* plink_type;

static int
link_filter_find_segment( gapp_type *gapp, plink_type *links, 
			int Fclose, float min_width );

static void
link_filter_test_segment( gapp_type *gapp, plink_type *links, 
			int Fclose );

static void
link_filter_union_segment( gapp_type *gapp, plink_type *links, 
			int Fclose );

static void
link_filter_set_dis( gapp_type *gapp );

static int 
Next( int ind , int n_ar );

static void 
link_filter_split( gapp_type *gapp, plink_type *links, int ig );

static float	
dis_from_link( vec2d *ctr, link_type *link, vec2d *v );

static link_type *
link_filter_replace( gapp_type *gapp, plink_type *links, 
		vec2d *gctr, link_type *link, int Fclose );

static link_type *
link_from_2points( link_type *link_prev, 
		vec2d *ctr1, vec2d *ctr2, float dis);

static int 
size_segment( int i, gapp_type *gapp );


link_type *
link_filter( vec2d *ctr, link_type *link, float min_width )
{
gapp_type   *gapp;
rl_aux		laux[600];
int			lno;
link_type	*slink, *l1, *l2;
int			i, j;
int			n_pair;
int			Fclose;
plink_type	*links;

	if ( min_width < 0 )
		min_width = MIN_WIDTH;

	gapp = gapp_alloc( 4096 );
	rl_link_gapp( ctr, link, NULL, gapp, laux, &lno );

	for( i = 0 ; i < gapp->no ; i++ )
		gapp->flag[i] = -1;

	links = malloc( sizeof(plink_type)*MAX_POINT_NUMBER);
	for( j = 0; j < lno; j++ )
	{
		laux[j].link->rctr = laux[j].ctr;
		for( i = laux[j].i0; i != laux[j].i1; i++ )
			links[i] = laux[j].link;
	}

	Fclose = 1;
	for ( l1 = link; l1; l1 = LINK_NEXT(l1) )
		l2 = l1;
	if ( link->p[F_BEGIN].p.jnt != l2->p[F_END].p.jnt )
		Fclose = 0;

	n_pair = link_filter_find_segment( gapp, links, Fclose, min_width );
	if ( !n_pair )
	{
		free( links );
		gapp_destroy( gapp );
		return link;
	}
	link_filter_test_segment( gapp, links, Fclose );
	link_filter_union_segment( gapp, links, Fclose );
	link_filter_set_dis( gapp );

	for( i = 0; i < gapp->no; i++ )
	if ( gapp->flag[i] >= 0 )
	{
		j = gapp->flag[i];
		link_filter_split( gapp, links, i );
		link_filter_split( gapp, links, j );
	}

	slink = link_filter_replace( gapp, links, ctr, link, Fclose );
	free( links );

	gapp_destroy( gapp );

	return slink;
}

int
link_filter_find_segment( gapp_type *gapp, plink_type *links, 
			int Fclose, float min_width )
{
vec2d		vec, u;
int			i, j, k;
int			n_pair, no_2;
float		min_width2, vec_len2, p2;

	if ( gapp->no < 10 ) 
		return 0;

	min_width2 = min_width*min_width;
	no_2 = Fclose ? gapp->no/2 : gapp->no;
	n_pair = 0;
	for( i = 0 ; i < gapp->no; i++ )
	for( j = Next( i, gapp->no ), k = 0; 
	     k < no_2; j = Next( j, gapp->no ), k++ )
	{
		if ( !Fclose && j < i )
			break;
		if ( links[i] == links[j] )
			continue;

		u.x = links[i]->u.x + links[j]->u.x;
		u.y = links[i]->u.y + links[j]->u.y;
		p2 = u.x*u.x + u.y*u.y;
		//0.12F - 20 grad, 0.27F - 30 grad
		//(1-cos(alfa))*2 - from theorem of cos
		if ( p2 > 0.6F ) // alfa 45 grad
		{
			if ( LINK_NEXT( links[i] ) == links[j] ||
				 LINK_PREV( links[i] ) == links[j] )
				continue;
			if ( Fclose && LINK_NEXT( links[i] ) == NULL && 
			  			   LINK_PREV( links[j] ) == NULL )
				continue;
			if ( Fclose && LINK_PREV( links[i] ) == NULL && 
			  			   LINK_NEXT( links[j] ) == NULL )
				continue;
		}

		vec.x = gapp->v[j].x - gapp->v[i].x;
		vec.y = gapp->v[j].y - gapp->v[i].y;
		vec_len2 = vec.x*vec.x + vec.y*vec.y;
		if ( vec_len2 < min_width2 )
		{
			gapp->flag[i] = j;
			n_pair++;
		}
	}

	return n_pair;
}

void 
link_filter_test_segment( gapp_type *gapp, plink_type *links, int Fclose )
{
int		n_pair;
int		i, j, k, n;


	for ( i = 0; i < gapp->no; i++ )
	if ( gapp->flag[i] >= 0 )
	{
		n_pair = 2;
		k = gapp->flag[i];

		for ( j = Next( i, gapp->no ), n = 2; 
				j != k; j = Next( j, gapp->no ), n++ )
		if ( gapp->flag[j] >= 0 )
		{
			n_pair++;
			if ( gapp->flag[j] > k )
				k = gapp->flag[j];
			if ( Fclose && j < i )
				 n_pair += size_segment( j, gapp );
		}

		if ( n_pair >= n*0.4F ) 
		{
			for( j = Next( i, gapp->no ); j != k; j = Next( j, gapp->no ) )
				gapp->flag[j] = -1;
			gapp->flag[k] = -1;
			gapp->flag[i] = k;
		}
		else 
			gapp->flag[i] = -1;
	}
}

void 
link_filter_union_segment( gapp_type *gapp, plink_type *links, int Fclose )
{
int		i, i_st, i_prev_segm_start, n_between;

	i_prev_segm_start = -1;
	n_between = -1;
	i_st = -1;

	for ( i = 0; i < gapp->no; i++ )
		if ( gapp->flag[i] >= 0 )
		{
			i_prev_segm_start = i_st = i;
			break;
		}
	if ( i_prev_segm_start < 0 )
		return;

	do  
	{
		if ( gapp->flag[i] >= 0 )
		{
			if ( gapp->flag[i_prev_segm_start] == i || //end of segment	
				 ( n_between >= 0 && n_between <= 4 ) )
			{
				gapp->flag[i_prev_segm_start] = gapp->flag[i];
				gapp->flag[i] = -1;
			}
			else
				i_prev_segm_start = i;
			n_between = -1;
		}
		else if ( gapp->flag[i_prev_segm_start] == i )
			n_between = 0; //end of segment
		else if ( n_between >= 0 )
			n_between++; //between segment

		i = Next( i, gapp->no );
	}
	while ( i != i_st );
}

int 
Next( int ind , int n_ar )
{
	return (ind == n_ar-1) ? 0 : ind+1;
}

void 
link_filter_split( gapp_type *gapp, plink_type *links, int ig )
{
link_type	*link, *l;
vec2d		ctr, v, p;
float		dis, t;	
int			i, bno;
	
	p = gapp->v[ig];
	link = links[ig];
	ctr = link->rctr;

	dis = dis_from_link( &ctr, link, &p );

	if( link == NULL || ABS(dis) > 0.2 )
		return;
	t = -link->u.y * (p.x - ctr.x) + link->u.x * (p.y - ctr.y);
	if( t < 0.0 || t > link->len )	
		return;

	if ( t < 0.1F )
	{
		gapp->v[ig] = ctr;
		if ( link->p[F_BEGIN].bno == -1 )
			links[ig] = LINK_PREV(link);
		else
		{
			bno = link->p[F_BEGIN].bno; 
			links[ig] = link->p[F_BEGIN].p.jnt->brn_no > 1 ?
				link->p[F_BEGIN].p.jnt->brn[1-bno]->link : link;
		}
		return;
	}
	else if ( t > link->len - 0.1F )
	{
		gapp->v[ig].x = ctr.x + link->v.x;
		gapp->v[ig].y = ctr.y + link->v.y;
		links[ig] = link;
		return;
	}
	link_t2xy( &ctr, link, t, &v );

	
	l = link_split( &ctr, link, &v );
	if( LINK_NEXT(l) == NULL )
		l->p[1].p.jnt->brn[l->p[1].bno]->link = l;
	else	
		l->p[1].p.l->p[0].p.l = l;
	if( LINK_NEXT(l) == NULL )
		jnt_set_aux( l->p[1].p.jnt );
	if( LINK_PREV(link) == NULL )
		jnt_set_aux( link->p[0].p.jnt );

	gapp->v[ig] = v;
	links[ig] = link;
	if ( l != NULL )
	{
		l->rctr.x = link->rctr.x + link->v.x;
		l->rctr.y = link->rctr.y + link->v.y;
	}
	for ( i = Next( ig, gapp->no ); links[i] == link && i != ig;
		  i = Next( i, gapp->no ) )
		links[i] = l;
}

void
link_filter_set_dis( gapp_type *gapp )
{
int i, j, k, s_pl, s_mn;
vec2d ctr1, ctr2;
float a, a_pl, a_mn; 
float h, A, B, C, Ap, Bp;
float xu, yu, au;

	for( i = 0; i < gapp->no; i++ )
	if ( gapp->flag[i] >= 0 )
	{
		a_pl = a_mn = gapp->a[i] = 0.0F;
		s_pl = s_mn = 0;
		j  = gapp->flag[i];
		ctr1 = gapp->v[i];
		ctr2 = gapp->v[j];
		A = ctr2.y - ctr1.y;
		B = ctr1.x - ctr2.x;
		h = (float)hypot( B, A );
		if ( ABS( h ) <= 0.001 )
			continue;
		C = -ctr1.x*A - ctr1.y*B;
		Ap = B/h;
		Bp = -A/h;
		xu = ctr1.x + Ap;
		yu = ctr1.y + Bp;
		au = (xu*A + yu*B + C)/h;
		for( k = Next( i, gapp->no ); k != j; k = Next( k, gapp->no ) )
		{
			a = (gapp->v[k].x*A + gapp->v[k].y*B + C)/h;
			if ( a*au > 0.0F )
			{
				s_pl++;
				a_pl += a;
			}
			else if ( a*au < 0.0F )
			{
				s_mn++;
				a_mn += a;
			}
		}
		if ( s_pl && s_mn < s_pl/3 )
			gapp->a[i] = a_pl/s_pl;
		else if ( s_mn && s_pl < s_mn/3 )
			gapp->a[i] = a_mn/s_mn;

		if ( j > i ) 
			i = j;
	}
}

link_type *
link_filter_replace( gapp_type *gapp, plink_type *links, 
	vec2d *gctr, link_type *link, int Fclose )
{
vec2d			ctr1, ctr2, ctr, ctrn;
link_type		*l1, *l1prev, *l2, *l2next;
link_type		*l, *lnext, *l_new, *slink;
jnt_type		*jnt;
int				bno;
int				i, j, Floop;

	Floop = 0;
	slink = NULL;
	bno = link->p[F_BEGIN].bno;
	jnt = link->p[F_BEGIN].p.jnt;
	for( i = 0; i < gapp->no; i++ )
	if ( gapp->flag[i] >= 0 )
	{
		j  = gapp->flag[i];
		if ( j < i && Floop )
			break;
		if ( j < i )
			Floop  = 1;
		l1prev = links[i];
		ctr1 = l1prev->rctr;
		ctr1.x += l1prev->v.x;
		ctr1.y += l1prev->v.y;
		l = l1 = LINK_NEXT(l1prev) ? LINK_NEXT(l1prev) : link;
		ctr = ctr1;

		l2 = links[j];
		ctr2 = l2->rctr;
		ctr2.x += l2->v.x;
		ctr2.y += l2->v.y;
		l2next = LINK_NEXT(l2) ? LINK_NEXT(l2) : link;

		l_new = link_from_2points( l1, &ctr1, &ctr2, gapp->a[i]);
		l_new->rctr	 = ctr1;
		l_new->pr[0] = l1->pr[0];
		l1->pr[0].p	 = NULL;
		if ( LINK_NEXT(l2) == NULL )
		{
			l_new->pr[1] = l2->pr[1];
			l2->pr[1].p	 = NULL;
		}
		link_replace( l1, l2, l_new, l_new );

		do {
			ctrn.x = ctr.x + l->v.x;
			ctrn.y = ctr.y + l->v.y;
			lnext = LINK_NEXT(l);
			if ( !lnext && Fclose )
			{
				lnext = link;
			    if ( l_new->p[F_END].p.jnt != 
					 link->p[F_BEGIN].p.jnt)
				{
					slink = l_new;
					slink->cln = l1prev;
				}
			}
			link_1_free( l );
			l = lnext;
			ctr = ctrn;
		} while ( l != l2next && l );
		if ( l_new && ( LINK_PREV(l_new)==NULL ) && Fclose )
			slink = l_new;
	}

	if ( slink && Fclose )
	{
		*gctr = jnt->gctr = slink->rctr;
		jnt_connet_link( jnt, bno, slink, F_BEGIN);
		l = slink->cln;
		if ( l->p[F_END].bno == -1 )
			l->pr[F_END] = profile_make_copy( l->pr[F_END], l->type );
		jnt_connet_link( jnt, 1-bno, slink->cln, F_END);
		return slink;
	}

	if ( slink && LINK_PREV( slink ) == NULL )
		return slink;

	return link;
}

link_type *
link_from_2points( link_type *link_prev, 
		vec2d *ctr1, vec2d *ctr2, float dis )
{
link_type	*link = NULL;
float	len;

	link = link_1_alloc();
	link->type = link_prev->type;

	link->v.x = ctr2->x - ctr1->x;
	link->v.y = ctr2->y - ctr1->y;
	len = (float)hypot( link->v.x, link->v.y );
	link->a = 0.0F;
	if ( len > 1.0F )
	{
		if ( ABS( dis ) > len/4.0F )
			dis = len/4.0F * ( dis < 0.0F ? -1.0F : 1.0F );
		link->a = dis;
	}

	link_set_aux( link );

	link->br = link_prev->br;
	link->br1 = link_prev->br1;
	link->Fbr = link_prev->Fbr;

	return link;
}

static float
dis_from_link( vec2d *ctr, link_type *l, vec2d *v )
{
vec2d	dv;
float	u,	t;

	dv.x = v->x - ctr->x;
	dv.y = v->y - ctr->y;

	t = -dv.x *l->u.y + dv.y *l->u.x;

	if( t < 0 )
	{
		u = (float)hypot( dv.x, dv.y );
		return( u );
	}

	if( t > l->len )
	{
		u = (float)hypot( dv.x - l->v.x, dv.y - l->v.y );
		return( u );
	}

	u = l->u.x * dv.x + l->u.y * dv.y;
	u -= l->c_prb*t *(t - l->len );

	return( u );
}

int size_segment( int i, gapp_type *gapp )
{
	if ( gapp->flag[i] > i )
		return ( gapp->flag[i] - i + 1 );
	else if ( gapp->flag[i] < i )
		return ( gapp->no + gapp->flag[i] - i + 1 );
	return -1;
}

