/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/******************
 *** LinkIo.c   ***
 ******************/

#include "LinkTool.h"


int
ploylink_write_to_file( vec2d *ctr, link_type *link, char *file )
{
FILE	*fp;

	if( (fp = fopen( file, "wb")) == NULL )
		return( - 1);

	polylink_write( ctr, link, fp );

	fclose( fp );

	return( 1 );
}


int
ploylink_write_to_buffer( vec2d *ctr, link_type *link, char **data, int *bytes )
{
char	file[256];

	gp_filename_get_temporary( "vc", 0, ".ctr", file );
	ploylink_write_to_file( ctr, link, file );

	*bytes = gp_read_buffer_from_file( file, data );

	unlink( file );

	return( 1 );
}


int
polylink_write( vec2d *ctr, link_type *link, FILE *fp )
{
link_type	*l;
int	no;

	for( l = link, no = 0 ; l != NULL ; l = LINK_NEXT(l), no++ );


	fprintf( fp, "%d\n", no );

	fprintf(fp, "%.6f  %.6f\n", ctr->x, ctr->y );


	for( l = link ; l != NULL ; l = LINK_NEXT(l) ){
		if( l->a < 0 ){
			if( l->a > -0.0001 )	l->a = 0;
		}
		else {
			if( l->a < 0.0001 )	l->a = 0;
		}

		l->a = PUSH_TO_RANGE( l->a , -100, 100 );

	
		fprintf( fp, "%.6f  %.6f  %.6f\n", l->v.x, l->v.y, l->a );
	}

	return( 1 );
}



int
ploylink_read_from_file( char *file, vec2d *ctr, link_type **link )
{
FILE	*fp;

	if( (fp = fopen( file, "rb")) == NULL )
		return( -1 );

	ploylink_read( fp, ctr, link );

	fclose( fp );

	return( 1 );
}

int
ploylink_read_from_buffer( char *data, int bytes, vec2d *ctr, link_type **link )
{
char	file[256];

	gp_filename_get_temporary( "vc", 0, ".ctr", file );

	gp_write_buffer_to_file( file, data, bytes );

	ploylink_read_from_file( file, ctr, link );

	unlink( file );

	return( 1 );
}


int
ploylink_read( FILE *fp, vec2d *ctr, link_type **link )
{
link_type	*l,	*pl;
int	i,	no;


	fscanf( fp, "%d", &no );

	fscanf(fp, "%f  %f", &ctr->x, &ctr->y );

	*link = pl = NULL;
	for( i = 0 ; i < no ; i++ ){
		l = link_1_alloc();
		fscanf( fp, "%f  %f  %f", &l->v.x, &l->v.y, &l->a );

		l->type = NF_EDGE;

		link_set_aux( l );

		if( pl != NULL ){
			pl->p[1].p.l = l;
			l->p[0].p.l = pl;
			pl->p[1].bno = l->p[0].bno = -1;
		}
		else	*link = l;

		pl = l;
	}

	pl->p[1].p.l = NULL;
	pl->p[1].bno = -2;

	return( 1 );
}