/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
#include "LinkTool.h"
#include "GpType/LinkType.h"
#include "Ubox/Box2d.h"
#include "Uvec/Vec2d.h"


#include "LinkLayer.h"

#include	<math.h>




void link_get_point_inside(vec2d *gCtr, link_type *link, vec2d *inside)
{

	link_get_point_at_direction(gCtr,link,link_get_inside_direction(gCtr, link),inside);	
}



/******  link_get_point_at_direction  ******
/*! 
 * @desc: 
 * @param vec2d *ctr : 
 * @param link_type *link : 
 * @param direction_type direction : 
 * @param vec2d *point : 
 * @return : void 
*********************************/
void link_get_point_at_direction(vec2d *gCtr,  link_type *link, direction_type direction, vec2d *point)
{
	vec2d gCtr1, ctr1;
	vec2d mid,u;

	link_type *l, *maxL;

	maxL = link;
	//l = link;
	gCtr1 = *gCtr;
	ctr1 = gCtr1;
	for ( l = link; l != NULL ; l = LINK_NEXT(l))
	{
		if (l->len > maxL->len )
		{
			maxL = l;
			gCtr1 = ctr1;
		}
		ctr1.x += l->v.x;
		ctr1.y += l->v.y;
	}

	


	link_t2xy(&gCtr1, maxL, maxL->len / 2, &mid);

	if ( direction == left ) 
	{
		u.x = (-1) * maxL->v.y ;
		u.y = maxL->v.x ;
	}
	else
	{
		u.x = maxL->v.y ;
		u.y = (-1) * maxL->v.x ;
			
	}
	//normalize the vector
	if ( ABS(u.x) > ABS(u.y) )
	{
		u.y = u.y / ABS(u.x);
		u.x = u.x / ABS(u.x);
	}
	else
	{
		u.x = u.x / ABS(u.y);
		u.y = u.y / ABS(u.y);
	}
	
	point->x = mid.x + u.x;
	point->y = mid.y + u.y;
}



/******  *link_get_inside_direction  ******
/*! 
 * @desc: return the direction of the inside area of a contour
 * @param vec2d *gCtr : global ctr for that polylink
 * @param link_type *contour : the contour
 * @return : direction_type 
*********************************/
direction_type link_get_inside_direction(vec2d *ctr, link_type *contour)
{
//	direction_type direction;


	//float A,B,C,p,miu,d,alpha,y;


	float angle;
	vec2d normal,next;
	
	vec2d end;
	vec2d ctr1, minGCtr ;
	
	link_type *l;
	link_type *minLink, *prevMinLink;	


	minLink = contour;	
	ctr1 = *ctr;
	minGCtr = *ctr;

	for (l = contour; l != NULL ; l = LINK_NEXT(l) )
	{	

		if ( ctr1.x < minGCtr .x )
		{
			minLink = l;
			minGCtr = ctr1;
		}
		ctr1.x += l->v.x;
		ctr1.y += l->v.y;
	}

	prevMinLink = LINK_PREV(minLink);
	if (  prevMinLink == NULL ) 
		prevMinLink = polylink_get_last_link(minLink);

	end.x = minGCtr.x + minLink->v.x;
	end.y = minGCtr.y + minLink->v.y;


	
	normal.x = minLink->v.y;
	normal.y = -1 * minLink->v.x;

	next.x = prevMinLink->ctr.x - end.x;
	next.y = prevMinLink->ctr.y - end.y;


	angle = VEC2D_DOT(normal, next);

	if ( angle > 0 ) 
		return right;
	else return left;

}



