/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/*************************** 
 ***    LinkFromLine.c   ***
 ***************************/
#include	<stdio.h>
#include	<math.h>

#include	"Uigp/igp.h"

#include	"GpType/GpType.h"

#include	"AgpEdit/AgpEditTool.h"

#include	"LinkTool.h"


static void	polylink_simplify_curve( vec2d *ctr, link_type *link );
/*
static void	polylink_simplify1( vec2d *ctr, link_type *link );
*/


static void	polylinkC_simplify( vec2d *ctr, link_type **link );

static void	polylinkC_simplify_bisector( vec2d *ctr, link_type **link );



void
polylink_simplify( vec2d *ctr, link_type **link )
{
link_type	*l;
int	type = (*link)->type;

	polylink_simplify_curve( ctr, *link );

	polylink_filter_short_link( ctr, *link, 0.5 );


//	l = polylink_reapproximate( ctr, *link, NULL, NULL );
//	link_free( *link );
//	*link = l;

//	polylink_simplify1( ctr, link );

	polylinkC_simplify_bisector( ctr, link );

	polylinkC_simplify( ctr, link );



	for( l = *link ; l != NULL ; l = LINK_NEXT(l) )	l->type = type;
}


static void
polylink_simplify_curve( vec2d *ctr, link_type *link )
{
link_type	*l;
float	max_a;

	for( l = link ; l != NULL ; l = LINK_NEXT(l) ){
		if( l->len < 1.0 ){
			l->a = l->c_prb = 0;
			continue;
		}

		max_a = 0.25 * l->len;
		if( l->a > 0 ){
			if( l->a > max_a ){
				l->a = max_a;
				l->c_prb = -4.0 *l->a/( l->len*l->len); 
			}
		}
		else	
			if( l->a < -max_a ){
				l->a = -max_a;
				l->c_prb = -4.0 *l->a/( l->len*l->len); 
			}
	}
}




static void
polylinkC_simplify_distance( link_type *link,float df, 
							  link_type **sl, float *t, float *gt,
							  link_type **sl1, float *t1 );

static void
polylinkC_simplify( vec2d *ctr, link_type **link )
{
link_type	*el,	*l,	*sl,	*sl1,	*l0,	*l1,	*l2,	*l3;
vec2d	vt0,	vt1;
float	len,	sinT,	d;
float	t,	gt,	t1;
vec2d	p;

	len = polylink_length( *link, NULL );

	link_set_gCtr( *link, ctr );

	for( el = *link ; LINK_NEXT(el) != NULL ; el = LINK_NEXT(el) );
	el->p[1].p.l = *link;
	(*link)->p[0].p.l = el;
	(*link)->p[0].bno = el->p[1].bno = -1;

	sinT = sin( 45.0*M_PI / 180.0 );

	for( l = *link ; l != el ; l = LINK_NEXT( l ) ){

		if( LINK_NEXT(l) == *link )	break;

		l1 = LINK_PREV(l);

		link_tanget( l1, l1->len, &vt0 );
		link_tanget( l, 0.0, &vt1 );
		d = vt0.x * vt1.x + vt0.y * vt1.y;

		if( d > -sinT )	continue;


		polylinkC_simplify_distance( l, 1.0, &sl, &t, &gt, &sl1, &t1 );

		if( gt < 1.5 )	continue;

		l0 = link_ssplit( sl, t, 0.1 );
		if( l0 != sl ){
			l0->ctr.x = sl->ctr.x + sl->v.x;
			l0->ctr.y = sl->ctr.y + sl->v.y;
		}


		l1 = link_ssplit( sl1, t1, 0.1 );
		if( l1 != sl1 ){
			l1->ctr.x = sl1->ctr.x + sl1->v.x;
			l1->ctr.y = sl1->ctr.y + sl1->v.y;
		}

		l2 = LINK_PREV(l1);

		l3 = LINK_PREV(l0);
		l3->p[1].p.l = NULL;
		l0->p[0].p.l = NULL;
		link_free( l1 );


		p.x = 0.5*( l0->ctr.x + l2->ctr.x + l2->v.x);
		p.y = 0.5*( l0->ctr.y + l2->ctr.y + l2->v.y);

		l0->v.x += l0->ctr.x - p.x;
		l0->v.y += l0->ctr.y - p.y;
		l0->ctr = p;
		link_set_aux( l0 );


		l2->v.x = p.x - l2->ctr.x;
		l2->v.y = p.y - l2->ctr.y;
		link_set_aux( l2 );

		l2->p[1].p.l = l0;
		l0->p[0].p.l = l2;

		*link = l0;
		el = l2;

		l = l0;
	}

	el->p[1].p.l = NULL;
	(*link)->p[0].p.l = NULL;

	*ctr = (*link)->ctr;
}
		




static void
polylinkC_simplify_distance( link_type *link,float df, 
							  link_type **sl, float *t, float *gt,
							  link_type **sl1, float *t1 )
{
float u0,	t0,	gt0,	tt,	len;
link_type	*l,	*l0,	*l1,	*sl0;
vec2d	p;

	l0 = LINK_NEXT(link);

	*sl = NULL;

	*gt = *t = 0;
	for( l = link ; l != NULL ; l = LINK_NEXT(l) ){

		l0 = LINK_NEXT(l);
		if( l0 == link )	break;

		l1 = LINK_NEXT(l0);

		if( l1 == link )	break;


		for( tt = 0.025 ; tt < l->len+0.05 ; tt += 0.1 ){
			if( tt > l->len-0.025 )	tt = l->len-0.025;

			link_t2xy( &l->ctr, l, tt, &p );

			len = polylink_distance( &l1->ctr, l1, link, &p, &sl0, &u0, &t0, &gt0 );

			if( sl0 == NULL || gt0 < 0 || gt0 > len || ABS(u0) > df ){
				*gt += *t;
				return;
			}
			*sl = l;
			*t = tt;

			*sl1 = sl0;
			*t1 = t0;
		}

		*gt += l->len;
	}
}



static int polylinkC_simplify_bisector_one( link_type *link,
							  link_type **sl, float *t,
							  link_type **sl1, float *t1 );



static void
polylinkC_simplify_bisector( vec2d *ctr, link_type **link )
{
link_type	*el,	*sl,	*sl1,	*l0,	*l1,	*l2,	*l3;
float	len0,	len1;
float	t,	t1;
vec2d	p;


	link_set_gCtr( *link, ctr );

	for( el = *link ; LINK_NEXT(el) != NULL ; el = LINK_NEXT(el) );
	el->p[1].p.l = *link;
	(*link)->p[0].p.l = el;
	(*link)->p[0].bno = el->p[1].bno = -1;



	while( polylinkC_simplify_bisector_one( *link, &sl, &t, &sl1, &t1 ) > 0 ){


		l0 = link_ssplit( sl, t, 0.01 );
		if( l0 != sl ){
			l0->ctr.x = sl->ctr.x + sl->v.x;
			l0->ctr.y = sl->ctr.y + sl->v.y;
		}


		l1 = link_ssplit( sl1, t1, 0.01 );
		if( l1 != sl1 ){
			l1->ctr.x = sl1->ctr.x + sl1->v.x;
			l1->ctr.y = sl1->ctr.y + sl1->v.y;
		}


		len1 = polylink_length( l1, l0 );
		len0 = polylink_length( l0, l1 );

		if( len1 > len0 ){
			l2 = l0;
			l0 = l1;
			l1 = l2;
		}


		l2 = LINK_PREV(l1 );
		l3 = LINK_PREV(l0 );
		l3->p[1].p.l = NULL;

		link_free( l1 );



		p.x = 0.5*( l0->ctr.x + l2->ctr.x + l2->v.x);
		p.y = 0.5*( l0->ctr.y + l2->ctr.y + l2->v.y);

		l0->v.x += l0->ctr.x - p.x;
		l0->v.y += l0->ctr.y - p.y;
		l0->ctr = p;
		link_set_aux( l0 );


		l2->v.x = p.x - l2->ctr.x;
		l2->v.y = p.y - l2->ctr.y;
		link_set_aux( l2 );

		l2->p[1].p.l = l0;
		l0->p[0].p.l = l2;

		*link = l0;
		el = l2;
	}


	el->p[1].p.l = NULL;
	(*link)->p[0].p.l = NULL;

	*ctr = (*link)->ctr;
}


static int
polylinkC_simplify_bisector_one( link_type *link,
							  link_type **sl, float *t,
							  link_type **sl1, float *t1 )
{
float u0,	t0,	gt0,	tt,	len,	u;
link_type	*l,	*l0,	*l1,	*sl0;
vec2d	p;
int	Fu;


	*sl = *sl1 = NULL;



	u = 100;
	Fu = 0;

	for( l = link ; l != NULL ; l = LINK_NEXT(l) ){

		l0 = LINK_PREV(l);

		l1 = LINK_NEXT(l);

		if( l1 == link )	break;


		for( tt = 0.025 ; tt < l->len+0.05 ; tt += 0.1 ){
			if( tt > l->len-0.025 )	tt = l->len-0.025;

			link_t2xy( &l->ctr, l, tt, &p );

			len = polylink_distance( &l1->ctr, l1, l0, &p, &sl0, &u0, &t0, &gt0 );

			if( sl0 == NULL || gt0 < 0 || gt0 > len || ABS(u0) > 1.0 ){
				Fu = 0;
				continue;
			}



			if( Fu == 0 ){
				Fu = 1;
				u = u0;
				*sl = l;
				*t = tt;
				*sl1 = sl0;
				*t1 = t0;
				continue;
			}

			if( (u < 0 ) == (u0<0) ){
				u = u0;
				*sl = l;
				*t = tt;
				*sl1 = sl0;
				*t1 = t0;
				continue;
			}

			if( ABS(u) > ABS(u0) ){
				u = u0;
				*sl = l;
				*t = tt;
				*sl1 = sl0;
				*t1 = t0;
			}
			return(1);
		}
	}

	return( -1 );
}