/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/************************ 
 ***    LinkSplit.c   ***
 ***********************/
#include	<stdio.h>
#include	<math.h>

#include	"Uigp/igp.h"

#include	"GpType/NfType.h"
#include	"GpType/LinkType.h"



	



/**** *link_split ****/
/*!
 * Splits the link at point p
 *
 * @param *ctr : global center of this linke
 * @param *link : the link
 * @param *p : point of split on the link.
 *
 * @return link_type  : the link that now starts at point p.
 */

link_type *
link_split( vec2d *ctr, link_type *link, vec2d *p )
{
link_type	*l,*l1;
vec2d	dv,	pv,	pv1;
vec2d	v,	u;
float	t, a;



	l = link_1_alloc();

	*l = *link;

	/* profile */
	dv.x = p->x - ctr->x;
	dv.y = p->y - ctr->y;
	t = -link->u.y * dv.x + link->u.x * dv.y;
	t /= link->len;

	if( link->pr[0].p != NULL ){
		link->pr[1] = l->pr[0] = profile_alloc( link->type );
		profile_afine( link->type, link->pr[0], l->pr[1], t, l->pr[0] );
	}

	if(  l->width[0] != NULL ){
		if( link->type == NF_N3 ){
			link->width[1] = l->width[0] = width_alloc();
			width_afine( link->width[0], l->width[1],
							t, l->width[0] );
		}
	}




	l->ctr = *p;


	l->v.x = ctr->x + link->v.x - p->x;
	l->v.y = ctr->y + link->v.y - p->y;


	link->v.x = p->x - ctr->x;
	link->v.y = p->y - ctr->y;




	/* link->a */
	if( link->c_prb != 0 ){

		a = link->a;

		pv.x = -link->u.y * dv.x + link->u.x*dv.y;
		pv.y =  link->u.x * dv.x + link->u.y*dv.y;


		t = hypot( pv.x , pv.y );
		u.x = -pv.y/t;
		u.y = pv.x/t;
		t = (-u.x  + u.y* link->c_prb* link->len )/
					( 2*link->c_prb*u.y );
		v.x = t;
		v.y = link->c_prb*t*( t - link->len );
		link->a = v.x *u.x + v.y * u.y;

		if( ABS(link->a) > ABS(a) )	link->a = 0;



		pv1.x = link->len - pv.x;
		pv1.y = 0  - pv.y;
		t = hypot( pv1.x , pv1.y );
		u.x = -pv1.y/t;
		u.y = pv1.x/t;
		t = (-u.x  + u.y* link->c_prb* link->len )/
					( 2*link->c_prb*u.y );
		v.x = t - pv.x;
		v.y = link->c_prb*t*( t - link->len ) - pv.y;
		l->a = v.x *u.x + v.y * u.y;

		if( ABS(l->a) > ABS(a) )	l->a = 0;
	}


	link_set_aux( link );
	link_set_aux( l );



	/* POINTER */
	link->p[1].bno = l->p[0].bno = -1;
	link->p[1].p.l = l;
	l->p[0].p.l = link;
	l1 = LINK_NEXT(l);
	if ( l1 != NULL )
		l1->p[0].p.l = l;


	return( l );
}



link_type *
link_split_t( link_type *link, float t0 )
{
link_type	*l,*l1;
vec2d	dv,	pv,	pv1;
vec2d	v,	u;
float	t, a;
	float	h;



	l = link_1_alloc();




	h = link->c_prb * t0 * (t0 - link->len);

	dv.x = t0 * -link->u.y  +  h * link->u.x;
	dv.y = t0 *  link->u.x  +  h * link->u.y;

	*l = *link;


	/* profile */
	t = t0 / link->len;

	if( link->pr[0].p != NULL ){
		link->pr[1] = l->pr[0] = profile_alloc( link->type );
		profile_afine( link->type, link->pr[0], l->pr[1], t, l->pr[0] );
	}

	if(  l->width[0] != NULL ){
		if( link->type == NF_N3 ){
			link->width[1] = l->width[0] = width_alloc();
			width_afine( link->width[0], l->width[1],
							t, l->width[0] );
		}
	}





	l->v.x = link->v.x - dv.x;
	l->v.y = link->v.y - dv.y;


	link->v.x = dv.x;
	link->v.y = dv.y;


	l->ctr.x = link->ctr.x + link->v.x;
	l->ctr.y = link->ctr.y + link->v.y;



	/* link->a */
	if( link->c_prb != 0 ){

		a = link->a;

		pv.x = -link->u.y * dv.x + link->u.x*dv.y;
		pv.y =  link->u.x * dv.x + link->u.y*dv.y;


		t = hypot( pv.x , pv.y );
		u.x = -pv.y/t;
		u.y = pv.x/t;
		t = (-u.x  + u.y* link->c_prb* link->len )/
					( 2*link->c_prb*u.y );
		v.x = t;
		v.y = link->c_prb*t*( t - link->len );
		link->a = v.x *u.x + v.y * u.y;

		if( ABS(link->a) > ABS(a) )	link->a = 0;



		pv1.x = link->len - pv.x;
		pv1.y = 0  - pv.y;
		t = hypot( pv1.x , pv1.y );
		u.x = -pv1.y/t;
		u.y = pv1.x/t;
		t = (-u.x  + u.y* link->c_prb* link->len )/
					( 2*link->c_prb*u.y );
		v.x = t - pv.x;
		v.y = link->c_prb*t*( t - link->len ) - pv.y;
		l->a = v.x *u.x + v.y * u.y;

		if( ABS(l->a) > ABS(a) )	l->a = 0;
	}


	link_set_aux( link );
	link_set_aux( l );



	/* POINTER */
	link->p[1].bno = l->p[0].bno = -1;
	link->p[1].p.l = l;
	l->p[0].p.l = link;
	l1 = LINK_NEXT(l);
	if ( l1 != NULL )
		l1->p[0].p.l = l;


	return( l );
}


link_type *
link_ssplit( link_type *link, float t, float dt )
{
link_type	*l;


	if( t > link->len - dt )
		return( LINK_NEXT(link) );


	if( t < dt ){

		return( link );
	}


	l = link_split_t( link, t );
	return( l );
}
