/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/***********************
 ***	AbsValue.c   ***
 ***********************/
#include        <stdio.h>   
#include	<math.h>
#include	"Uigp/igp.h"

#include        "GpType/GpType.h"
#include        "RelValue.h"

#include        "RelValueIn.h"


#include        "EpType/EpType.h"
#include        "Ep/Ep.h"
#include        "EpValue/EpValue.h"


static void	abs_value_hunch( epValue_type *epValue, ep_type *ep, vec2d *ctr, hunch_type *h );

static void	abs_value_ridge( epValue_type *epValue, ep_type *ep, vec2d *ctr, ridge_type *r );

static void	ridge_aux_link( ridge_type *r );


void
abs_value_agp( gp_array *agp, ep_array *aep )
{
gp_type		*gp;
hunch_type	*h;
jnt_type	*jnt;
brn_type	*b;
link_type	*link;
vec2d	v;
ep_type	*ep;
ridge_type	*r;
  
int     i1,	j1;
int     i,      j,	bno;

        for( i = JSIZE, i1 = aep->Ni0-1 ; i < aep->Ni-JSIZE ; i++, i1++ ){

            for( j = JSIZE, j1 = aep->Nj0-1 ; j < aep->Nj-JSIZE ; j++, j1++ ){
			gp = &agp->gp[i1][j1];

			ep = &aep->ep[i][j];



			for( h = gp->hunch ; h != NULL ; h = h->nh ){

				abs_value_hunch( &aep->epValue, ep, &h->ctr, h );
			}



			for( link = gp->link ; link != NULL ; link = link->nl){

                                if( link->type != NF_RIDGE )    continue; 

				r = link->pr[F_BEGIN].ridge;

				abs_value_ridge( &aep->epValue, ep, &link->ctr, r );

				set_width_ridge_profile( r );
				ridge_aux_link( r );
			}


			for( jnt = gp->jnt ; jnt != NULL ; jnt = jnt->nj ){

                                if( jnt->type != JNT_RIDGE )    continue; 
	
				for( bno = 0 ; bno < jnt->brn_no ; bno++ ){
					b = jnt->brn[bno];
					if( b->direction != F_END )
						continue;

					v.x = jnt->ctr.x + b->ctr.x;
					v.y = jnt->ctr.y + b->ctr.y;

					r = b->link->pr[F_END].ridge;
					abs_value_ridge( &aep->epValue, ep, &v, r );

					set_width_ridge_profile( r );
					ridge_aux_link( r );
				}
			}
	     }
	}

  
}


static void
abs_value_hunch( epValue_type *epValue, ep_type *ep, vec2d *ctr, hunch_type *h )
{
float	val[SP_NO];

	ep_value3( epValue, ep, ctr, val );

	h->gval[SP_Y] =  h->rval[SP_Y] + F2QCOLOR(val[SP_Y]);
	h->gval[SP_Q] =  h->rval[SP_Q] + F2QCOLOR(val[SP_Q]);
	h->gval[SP_A] =  h->rval[SP_A] + F2QCOLOR(val[SP_A]);
}


static void
abs_value_ridge( epValue_type *epValue, ep_type *ep, vec2d *ctr, ridge_type *r )
{
float	val[SP_NO];

	ep_value3( epValue, ep, ctr, val );

	r->mval[SP_Y] =  r->rval[SP_Y] + F2QCOLOR(val[SP_Y]);
	r->mval[SP_Q] =  r->rval[SP_Q] + F2QCOLOR(val[SP_Q]);
	r->mval[SP_A] =  r->rval[SP_A] + F2QCOLOR(val[SP_A]);
}



static void
ridge_aux_link( ridge_type *r )
{ 
        r->lm[SP_Q] = ( r->rval[SP_Q] == 0 )? 0.0 : -r->rval[SP_Q]/ r->width;
 
        r->lm[SP_A] = ( r->rval[SP_A] == 0 )? 0.0 : -r->rval[SP_A]/ r->width;
}
