/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
#include <math.h>

#include	"GpType/GpType.h"
#include	"AgpEdit/AgpEditTool.h"
#include    "RlType.h"
#include    "RlType2.h"


static int rllink_find_arcs_link( link_type *link, link_type **link2);

static int rllink_find_arcs_link_test( link_type *link, link_type *link2);

static int rllink_find_corners_link( link_type *link );

static int compare_tangs( vec2d *tn1, vec2d *tn2, double min_delta );

const float M_2_PI = (float)(M_PI*2.0F);

float MAX_LEN_GROUPE					= 6.0F;
float MAX_LEN_LINK_IN_GROUPE			= 3.0F;
float MIN_DELTA_ANGLE_GROUPE			= (float)(M_PI/4.0F);
int	  MAX_NUMBER_LINKS					= 3;
float MAX_DELTA_ANGLE_BOUND_GROUPE      = (float)(M_PI/4.0F);
float MIN_DELTA_ANGLE_BOUND_GROUPE	    = (float)(M_PI/18.0F);
float LEN_SMALL							= 2.0F;
float LEN_BIG							= 6.0F;
float MIN_DELTA_ANGLE_CORNER_FORLENSMALL= (float)(M_PI/4.0F);
float MIN_DELTA_ANGLE_CORNER_FORLENBIG	= (float)(M_PI/12.0F);


void
rl_link_corner_arc( rl_aux laux[], int lno, gapp_type *gapp )
{
link_type	*l;
int	i, j;

int n_arcs_groupe = 0;
link_type **links_arcs_groupe = NULL;	
int n_corners = 0;
link_type **links_corners = NULL;		
 
	n_arcs_groupe		= 0; 
	links_arcs_groupe	= (link_type **)malloc( sizeof(link_type *)*1200 );
	n_corners			= 0;
	links_corners		= (link_type **)malloc( sizeof(link_type *)*600 );

	rllink_find_arcs( laux[0].link, &laux[0].ctr, 
							&n_arcs_groupe, links_arcs_groupe );
	rllink_find_arcs_union( &n_arcs_groupe, links_arcs_groupe );
	rllink_find_corners( laux[0].link, &laux[0].ctr,
							&n_corners, links_corners );

	laux[0].type = RL_CORNER;

	for( i = 1 ; i < lno ; i++ )
	{
		laux[i].type = RL_NULL;

		l = laux[i].link;

		if ( laux[i].type == RL_CORNER )	continue;

		for ( j = 0; j < n_corners; j++ )
			if ( l == links_corners[j] ){
				laux[i].type = RL_CORNER;
				break;
			}


		if ( laux[i].type == RL_CORNER )	continue;
	
		for ( j = 0; j < n_arcs_groupe; j += 2 )
		{
			if ( ( l == links_arcs_groupe[j] ||
				l == LINK_NEXT(links_arcs_groupe[j+1] ) ) ){
				laux[i].type = RL_CORNER;
				break;
			}
		}
	}

	if( laux[0].link->len < Clen_d )	
		laux[1].type = RL_NULL;
	if( laux[lno-1].link->len < Clen_d )	
		laux[lno-1].type = RL_NULL;

	laux[lno].type = RL_CORNER;

	free( links_arcs_groupe );
	free( links_corners );
}

void rllink_find_arcs( link_type *link, vec2d *gctr,
			int *n_arcs_groupe, link_type **links_arcs_groupe)
{
link_type	*link1, *link2;
vec2d		gctr1;

	if ( (link1 = link) == NULL )
		return;
	gctr1 = *gctr;

	while ( link1 )
	{
		if ( rllink_find_arcs_link( link1, &link2 ) &&
			 rllink_find_arcs_link_test( link1, link2 ) )
		{
			link1->rctr = gctr1;
			links_arcs_groupe[(*n_arcs_groupe)++] = link1;
			links_arcs_groupe[(*n_arcs_groupe)++] = link2;
		}
		gctr1.x += link1->v.x;
		gctr1.y += link1->v.y;
		link1 = LINK_NEXT( link1 );
	}

}

int rllink_find_arcs_link( link_type *link, link_type **link2)
{
link_type	*l, *l_fin;
vec2d		tn1, tn2;
float		len_groupe = 0;
int			n_links = 0;
int			rl_rez = FALSE;

	if ( (l = LINK_PREV( link )) != NULL )
		link_Ntanget_end( l, &tn1 );
	else
		link_Ntanget_beg( link, &tn1 );

	for ( l_fin = link, n_links = 1; l_fin; l_fin = LINK_NEXT(l_fin) )
	{
		if ( n_links++ > MAX_NUMBER_LINKS )
			return rl_rez;
		if ( l_fin->len > MAX_LEN_LINK_IN_GROUPE )
			return rl_rez;
		len_groupe += l_fin->len;
		if ( len_groupe > MAX_LEN_GROUPE )
			return rl_rez;

		if ( (l = LINK_NEXT(l_fin)) != NULL )
			link_Ntanget_beg( l, &tn2 );
		else
			link_Ntanget_end( l_fin, &tn2 );
		if ( compare_tangs( &tn1, &tn2, MIN_DELTA_ANGLE_GROUPE ) != 0 )
		{
			*link2 = l_fin;
			rl_rez = TRUE;
		}
	}

	return rl_rez;
}

int rllink_find_arcs_link_test( link_type *link, link_type *link2)
{//if part link-link2 is OK - return TRUE
link_type	*l1_prev, *l2_next;
vec2d		tn1_e, tn1_b, tn2_e, tn2_b;

	if ( (l1_prev = LINK_PREV( link )) == NULL )
		return TRUE;
	if ( (l2_next = LINK_NEXT( link2 )) == NULL )
		return TRUE;

	link_Ntanget_beg( link, &tn1_b );
	link_Ntanget_end( link2, &tn2_e );
	if ( compare_tangs( &tn2_e, &tn1_b, MIN_DELTA_ANGLE_GROUPE*2.0F/3.0F) != 0 )
		return TRUE;

	link_Ntanget_end( l1_prev, &tn1_e );
	link_Ntanget_end( l2_next, &tn2_b );

	if ( compare_tangs( &tn1_e, &tn1_b, MAX_DELTA_ANGLE_BOUND_GROUPE) != 0 &&
		 compare_tangs( &tn2_e, &tn2_b, MIN_DELTA_ANGLE_BOUND_GROUPE) == 0 )
		return FALSE;
	if ( compare_tangs( &tn1_e, &tn1_b, MIN_DELTA_ANGLE_BOUND_GROUPE) == 0 &&
		 compare_tangs( &tn2_e, &tn2_b, MAX_DELTA_ANGLE_BOUND_GROUPE) != 0 )
		return FALSE;

	return TRUE;
}

void rllink_find_arcs_union( int *n_arcs_groupe, 
							 link_type **links_arcs_groupe)
{
link_type	*link, *link_cmp;
int			i, j, bShift;

	if ( *n_arcs_groupe <= 2 )
		return;

	for ( i = (*n_arcs_groupe)-2; i > 1; )
	{
		link_cmp = links_arcs_groupe[i];
		link = links_arcs_groupe[i-2];
		bShift = 0;
		do {
			if ( link == link_cmp )//|| LINK_NEXT( link ) == link_cmp )
			{
				links_arcs_groupe[i-1] = links_arcs_groupe[i+1];
				if ( i != (*n_arcs_groupe)-2 )
				{//shift array to left on 2
					for ( j = i; j < (*n_arcs_groupe)-2; j++ )
						links_arcs_groupe[j] = links_arcs_groupe[j+2];
					bShift = 1;
				}
				(*n_arcs_groupe) -= 2;
				break; //do
			}
		} while ( link != links_arcs_groupe[i-1] &&
			      (link = LINK_NEXT( link )) != NULL );
		i = !bShift ? i-2 : i;
	}
}

void rllink_find_corners( link_type *link, vec2d *gctr,
			int *n_corners, link_type **links_corners)
{
link_type	*link1;
vec2d		gctr1;

	if ( (link1 = link) == NULL )
		return;
	gctr1 = *gctr;

	while ( link1 )
	{
		if ( link1 != link && //no first link in polylink
			 rllink_find_corners_link( link1 ) )
		{
			link1->rctr = gctr1;
			links_corners[(*n_corners)++] = link1;
		}
		gctr1.x += link1->v.x;
		gctr1.y += link1->v.y;
		link1 = LINK_NEXT( link1 );
	}

}

int rllink_find_corners_link( link_type	*link )
{
link_type	*link_prev;
vec2d		tn1, tn2;
float		min_delta_angle = MIN_DELTA_ANGLE_CORNER_FORLENBIG;
float		min_len;
	
	link_prev = LINK_PREV( link );
	if ( !link_prev )
		return 0;

	link_Ntanget_end( link_prev, &tn1 );
	link_Ntanget_beg( link, &tn2 );

	min_len = MIN( link->len, link_prev->len );
	if ( min_len >= LEN_BIG )
		min_delta_angle = MIN_DELTA_ANGLE_CORNER_FORLENBIG;
	else if ( min_len < LEN_SMALL )
		min_delta_angle = MIN_DELTA_ANGLE_CORNER_FORLENSMALL; 
	else
		min_delta_angle = MIN_DELTA_ANGLE_CORNER_FORLENBIG + (LEN_BIG-min_len)*
		  (MIN_DELTA_ANGLE_CORNER_FORLENSMALL-MIN_DELTA_ANGLE_CORNER_FORLENBIG)/(LEN_BIG-LEN_SMALL);

	if ( compare_tangs( &tn1, &tn2, min_delta_angle ) != 0 )
		return TRUE;

	return FALSE;
}

int compare_tangs( vec2d *tn1, vec2d *tn2, double min_delta )
{
double		al1, al2, delta_al;

	al1 = atan2( tn1->y, tn1->x );
	al2 = atan2( tn2->y, tn2->x );
	delta_al = ABS(al1-al2);
	if ( delta_al > M_PI )
		delta_al = M_2_PI - delta_al;

	if ( delta_al < min_delta )
		return 0;
	if ( al1 < al2 )
		return -1;
	return 1;
}

