/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/*********************
 ***	RlLink.c   ***
 *********************/
#include        <stdio.h>   
#include	<math.h>

#include        "Uigp/igp.h"
#include        "Uprm/prm.h"

#include        "GpType/GpType.h"
#include        "AgpEdit/AgpEditTool.h"
#include        "LinkTool/LinkTool.h"


#include	"UGeoApp/GeoApp.h"

#include        "RlType.h"

rl_type	Rl;


static GP_prm_group Prm_group[] = {
	{ "Rlink",	rl_link_get_prm },
	{ "Corner",	rl_link_corner_get_prm },
};

int	Frl = 0;
gapp_prm	Qgprm;

static GP_prm params[]  = {
	{"RL-F", (char *)&Frl, PRM_BOOLEAN, "%d", "Yes" },
	{"RL-D-weight" , (char *)&Qgprm.dwight, PRM_FLOAT,"%.2f","0.10"},
	{"RL-Ls-error" , (char *)&Qgprm.err, PRM_FLOAT,"%.2f","0.01"},

	{"RL-len-min" , (char *)&Qgprm.min_len,PRM_FLOAT,"%.2f","3.0"},
	{"RL-min-no" , (char *)&Qgprm.Min_pnumber,PRM_INTEGER,"%d","10"},
	{"RL-len-max" , (char *)&Qgprm.max_len,PRM_FLOAT,"%.2f","24.0"},

	{"RL-dev-max" , (char *)&Qgprm.Max_dev,PRM_FLOAT,"%.2f","1.0"},
	{"RL-dev-min" , (char *)&Qgprm.Min_dev,PRM_FLOAT,"%.2f","0.1"},
};


void
rl_link_get_prm(GP_prm **prm, int *prm_no, void (**check)() )
{
	*prm    = params;
	*prm_no  = PRM_NUMBER(params);
	*check = NULL ;
}


void
rl_link_get_group( GP_prm_group **prm_group, int *prm_group_no )
{
        *prm_group_no = PRM_GROUP_SIZE(Prm_group);
        *prm_group = Prm_group;
}



void
rl_link_init( char *file )
{
	gp_prm_init( file, rl_link_get_prm );

	gp_prm_init( file, rl_link_corner_get_prm );
}


void
rl_link_set_lprm()
{
}


void
rl_link_agp( gp_array *agp )
{
gp_type	*gp;
jnt_type	*jnt;
brn_type	*b;
int	i,	j,	k;

	if( Frl == 0 )	return;


/*
	agp_link_in_aux_clear( agp );
*/

	for( i = 0 ; i < agp->Ni ; i++ )
	    for( j = 0 ; j < agp->Nj ; j++ ){
		gp = &agp->gp[i][j];

		for( jnt = gp->jnt ; jnt != NULL ; jnt = jnt->nj ){

			for( k = 0 ; k < jnt->brn_no ; k++ ){
				b = jnt->brn[k];
				if( b->direction != F_BEGIN )	continue;
				if( b->type != BRN_EDGE && b->type != BRN_RIDGE)
					continue;

				rl_link_brn( jnt, k, agp );
			}
		}
	    }


	for( i = 0 ; i < agp->Ni ; i++ )
	    for( j = 0 ; j < agp->Nj ; j++ ){
		gp = &agp->gp[i][j];

		for( jnt = gp->jnt ; jnt != NULL ; jnt = jnt->nj ){

			for( k = 0 ; k < jnt->brn_no ; k++ ){
				b = jnt->brn[k];
				if( b->direction != F_BEGIN )	continue;
				if( b->type != BRN_N3 && b->type != BRN_N3R )
					continue;

				rl_link_brn( jnt, k, agp );
			}
		}
	    }



	for( i = 0 ; i < agp->Ni ; i++ )
	    for( j = 0 ; j < agp->Nj ; j++ ){
		gp = &agp->gp[i][j];

		for( jnt = gp->jnt ; jnt != NULL ; jnt = jnt->nj ){
			if( jnt->type != JNT_N3 )	continue;

			agp_jnt_n3_reposition( agp, jnt );
		}
	    }
}


void
rl_link_countor_agp( gp_array *agp )
{
gp_type	*gp;
jnt_type	*jnt;
brn_type	*b;
int	i,	j,	k;


	for( i = 0 ; i < agp->Ni ; i++ )
	    for( j = 0 ; j < agp->Nj ; j++ ){
			gp = &agp->gp[i][j];

			for( jnt = gp->jnt ; jnt != NULL ; jnt = jnt->nj ){

				for( k = 0 ; k < jnt->brn_no ; k++ ){
					b = jnt->brn[k];
					if( b->direction != F_BEGIN )	continue;
					if( b->type != BRN_EDGE )	continue;
					
					if( b->link->Fbr == 0 )	continue;

					rl_link_brn( jnt, k, agp );
				}
			}
	    }
}


void
rl_link_1( gp_array *agp, link_type *link )
{
jnt_type	*jnt,	*jnt1;
brn_type	*b;
link_type	*l;
int	bno,	br,	Fbr;


	for( l = link ; LINK_PREV(l) != NULL ; l = LINK_PREV(l) );
	jnt = l->p[F_BEGIN].p.jnt;
	bno = l->p[F_BEGIN].bno;

	for( l = link ; LINK_NEXT(l) != NULL ; l = LINK_NEXT(l) );
	jnt1 = l->p[F_END].p.jnt;

	b = jnt->brn[bno];


/*
	if( b->type != BRN_N3 && b->type != BRN_N3R )
		agp_link_in_aux_clear( agp );
*/


	br = b->link->br; 
	Fbr = b->link->Fbr;

	rl_link_brn( jnt, bno, agp );

	for( l = b->link ; l != NULL ; l =LINK_NEXT(l) ){
		l->br = br;
		l->Fbr = Fbr;
	}



	if( jnt->type == JNT_N3 )
		agp_jnt_n3_reposition( agp, jnt );

	if( jnt1->type == JNT_N3 )
		agp_jnt_n3_reposition( agp, jnt1 );


/*
	if( b->type != BRN_N3 && b->type != BRN_N3R )
		agp_link_in_aux( agp );
*/
}





void
rl_link_brn( jnt_type *jnt, int bno, gp_array *agp )
{
brn_type	*b;
gapp_type       *gapp;
//rl_aux	laux[600];
rl_aux		*laux;
int	lno,	nLink;
float	link_len;
rl_type	*rl = &Rl;
link_type	*l0,	*l1;
int	i,	j,	k;
vec2d	ctr;
int	no;

	b = jnt->brn[bno];

	ctr.x = jnt->gctr.x + b->ctr.x;
	ctr.y = jnt->gctr.y + b->ctr.y;

	agp_lnf_clear_link( &ctr, b->link, agp );

	link_len = polylink_length( b->link, NULL );
	no = link_len * 4 + 100;
	gapp = gapp_alloc( no );

	nLink = polylink_link_no( b->link, NULL );
	laux = (rl_aux *)malloc( sizeof(rl_aux) * (nLink+1) );

	rl_link_gapp( &ctr, b->link, NULL, gapp, laux, &lno );
	rl_link_corner( laux, lno, gapp );

	for( i = 0; i < lno ;  ){
		for( j = i+1 ; laux[j].type != RL_CORNER ; j++ );

		if( j == i+1 && laux[i].link->len <= Qgprm.max_len ){
			i = j;
			continue;
		}
 
		gapp->ino = 0;

		rl_approximate( gapp, laux[i].i0, laux[j].i0-1,
					&laux[i].ctr, &laux[j].ctr, &Qgprm, rl );

		rl_to_link( laux[i].link->type, rl, laux, gapp->ino, &l0, &l1 );

		link_replace( laux[i].link, laux[j-1].link, l0, l1 );

		for( k = i ; k < j ; k++ )
			link_1_free( laux[k].link );

 
		i = j;
	}

	agp_lnf_link( &ctr, b->link, agp );

	jnt_load_nf_end_link( b->link );

	if( b->type == BRN_N3 )
		link_width_bes( b->link );

	gapp_destroy( gapp );
	free( laux );
}



void
rl_link_app( gapp_type *gapp, rl_aux laux[],
		vec2d *ctr0, vec2d *ctr1, int i0, int i1,
		link_type **l0, link_type **l1 )
{
rl_type	*rl = &Rl;

	gapp->ino = 0;

	rl_approximate( gapp, i0, i1, ctr0, ctr1, &Qgprm, rl );

		
	rl_to_link( laux[0].link->type, rl, laux, gapp->ino, l0, l1 );
}
