/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
#include        <math.h>

#include        "Uigp/igp.h"
#include        "UGeoApp/GeoApp.h"

#include        "RlType.h"


static void	rl_to_link_one( rl_type *rl, rl_aux qa[], link_type **link );

void
rl_approximate( gapp_type *gapp, int i0, int i1,
                        vec2d *p0, vec2d *p1, gapp_prm *gprm, rl_type *rl )
{
float   ans,    a,	len,	da;
vec2d   p2;
int     i2;
 
 
        ans = geo_approximate_l2( p0, p1,
                        &gapp->v[i0], &gapp->tang[i0], i1 - i0 + 1, gprm,
                        &a, &len, &p2, &i2, &da );

	rl->a = a;
	rl->ctr = *p0;
	rl->v.x = p1->x - p0->x;
	rl->v.y = p1->y - p0->y;

	rl->da = da;

	rl->len = len;
	rl->i0 = i0;
	rl->i1 = i1;

	
 
        if( ans < gprm->err ){
                gapp->ino ++;
		rl->Fsplit = 0;
                return;
        }


	rl->Fsplit = 1;


	if( rl->p[F_BEGIN] == NULL ){
		rl->p[F_BEGIN] = RL_ALLOC();
		rl->p[F_BEGIN]->p[0] = rl->p[F_BEGIN]->p[1] = NULL;
	}

	if( rl->p[F_END] == NULL ){
		rl->p[F_END] = RL_ALLOC();
		rl->p[F_END]->p[0] = rl->p[F_END]->p[1] = NULL;
	}
 
 
 
        i2 += i0;
        rl_approximate( gapp, i0, i2-1, p0, &p2, gprm, rl->p[F_BEGIN] );
        rl_approximate( gapp, i2, i1, &p2, p1, gprm, rl->p[F_END] );
}


void
rl_to_link( int type, rl_type *rl, rl_aux qa[],
		int ino, link_type **l0, link_type **l1 )
{
link_type	*link;

	*l0 = link = link_alloc( type, ino );
	if( link->type == NF_N3 )
		link_alloc_width( link );

	*l1 = link;
	rl_to_link_one( rl, qa, l1 );

	if( (*l0)->type == NF_N3 )
		link_width_bes( *l0 );
}
 
 


static void
rl_to_link_one( rl_type *rl, rl_aux qa[], link_type **link )
{
rl_aux	*qaa;
link_type	*l;
vec2d	v;
float	t;
int	i;

        if( rl->Fsplit == 1 ){
                rl_to_link_one( rl->p[0], qa, link );
                rl_to_link_one( rl->p[1], qa, link );
                return;
        }

	l = *link;


        l->v.x = rl->v.x;
        l->v.y = rl->v.y;
        
        
        l->len = rl->len;
 
        l->u.x = l->v.y/l->len;
        l->u.y = -l->v.x/l->len;
 
        l->c_prb = rl->a;
        l->a = - l->c_prb * l->len * l->len * 0.25;


	for( i = 0 ; rl->i0 > qa[i].i0 ; i++ );
	if( rl->i0 < qa[i].i0 )	i--;
	qaa = &qa[i];

	if( rl->i0 > qa[i].i0 ){
		v.x = rl->ctr.x - qaa->ctr.x;
		v.y = rl->ctr.y - qaa->ctr.y;
		t = -l->u.y * v.x + l->u.x * v.y;
		t = t / qaa->link->len;
	}
	else	t = 0;

	profile_afine( l->type, qaa->link->pr[0],
				qaa->link->pr[1], 1-t, l->pr[0] );
	if( l->type == NF_N3 )
		width_afine( qaa->link->width[0],
				qaa->link->width[1], 1-t, l->width[0] );

	l->br = qaa->link->br;
	l->br1 = qaa->link->br1;
	l->Fbr = qaa->link->Fbr;

	if( LINK_NEXT(l) != NULL ){
		(*link) = (*link)->p[F_END].p.l;
		return;
	}

	for(  ; rl->i1 > qa[i].i0 ; i++ );
	if( rl->i1 <= qa[i].i0 )	i--;
	qaa = &qa[i];

	if( rl->i1 < qa[i].i1-1 ){
		v.x = rl->ctr.x + rl->v.x - qaa->ctr.x;
		v.y = rl->ctr.y + rl->v.y - qaa->ctr.y;
		t = -l->u.y * v.x + l->u.x * v.y;
		t = t / qaa->link->len;
	}
	else	t = 1;

	profile_afine( l->type, qaa->link->pr[0],
				qaa->link->pr[1], 1-t, l->pr[1] );
	if( l->type == NF_N3 )
		width_afine( qaa->link->width[0],
				qaa->link->width[1], 1-t, l->width[1] );
}
