/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
#include        <math.h>

#include        "Uigp/igp.h"
#include        "UGeoApp/GeoApp.h"

#include        "RlType.h"
#include        "RlType2.h"

   
static void	rl_to_link_one( rl_type *rl, rl_aux qa[], link_type **link );

//Grisha 31.05.01
static void set_rl_link( rl_type *rl, vec2d *p0, vec2d *p1, 
				 float a, float len, float da, int i0, int i1 );

//Grisha 31.05.01 ...
void
rl_approximate_2( gapp_type *gapp, int i0, int i1,
                        vec2d *p0, vec2d *p1, gapp_prm *gprm, rl_type *rl )
{
float   ans,    a,	len,	da;
vec2d   p2;
int     i2;
int		il, ir, i1N, i2N;
vec2d   p1N, p2N;
int		find_approximate;
float   a2,	len2, da2;

    ans = geo_approximate_l2( p0, p1,
                        &gapp->v[i0], &gapp->tang[i0], i1 - i0 + 1, gprm,
                        &a, &len, &p2, &i2, &da );

	set_rl_link( rl, p0, p1, a, len, da, i0, i1 );
 
    if( ans < gprm->err )
	{
		gapp->ino ++;
        return;
    }

	rl->Fsplit = 1;

	if( rl->p[F_BEGIN] == NULL ){
		rl->p[F_BEGIN] = RL_ALLOC();
		rl->p[F_BEGIN]->p[0] = rl->p[F_BEGIN]->p[1] = NULL;
	}

	if( rl->p[F_END] == NULL ){
		rl->p[F_END] = RL_ALLOC();
		rl->p[F_END]->p[0] = rl->p[F_END]->p[1] = NULL;
	}

	find_approximate = 0;
	il = i0;
	ir = i1;
	while (il+1 < ir)
	{
		i1N = (il + ir) >> 1;
		p1N = gapp->v[i1N];
		if ( geo_approximate_l2( p0, &p1N,
							&gapp->v[i0], &gapp->tang[i0], i1N - i0 + 1, gprm,
							&a, &len, &p2, &i2, &da ) <  gprm->err )
		{
			il = i1N;
			p2 = gapp->v[il];
			if ( geo_approximate_l2( &p1N, p1,
							&gapp->v[i1N], &gapp->tang[i1N], i1 - i1N + 1, gprm,
							&a2, &len2, &p2N, &i2N, &da2 ) <  gprm->err )
			{
				find_approximate = 1;
				break;
			}
		}
		else
			ir = i1N;
	}
	i2 = il;
	p2 = gapp->v[i2];

	set_rl_link( rl->p[F_BEGIN], p0, &p2, a, len, da, i0, i2-1 );
	gapp->ino++;

	if ( find_approximate > 0 )
	{
		set_rl_link( rl->p[F_END], &p2, p1, a2, len2, da2, i2, i1 );
		gapp->ino++;
	}
	else
		rl_approximate_2( gapp, i2, i1, &p2, p1, gprm, rl->p[F_END] );
}
//... Grisha 31.05.01


static void
rl_to_link_one( rl_type *rl, rl_aux qa[], link_type **link )
{
rl_aux	*qaa;
link_type	*l;
vec2d	v;
float	t;
int	i;

        if( rl->Fsplit == 1 ){
                rl_to_link_one( rl->p[0], qa, link );
                rl_to_link_one( rl->p[1], qa, link );
                return;
        }

	l = *link;


        l->v.x = rl->v.x;
        l->v.y = rl->v.y;
        
        
        l->len = rl->len;
 
        l->u.x = l->v.y/l->len;
        l->u.y = -l->v.x/l->len;
 
        l->c_prb = rl->a;
        l->a = - l->c_prb * l->len * l->len * 0.25F;


	for( i = 0 ; rl->i0 > qa[i].i0 ; i++ );
	if( rl->i0 < qa[i].i0 )	i--;
	qaa = &qa[i];

	if( rl->i0 > qa[i].i0 ){
		v.x = rl->ctr.x - qaa->ctr.x;
		v.y = rl->ctr.y - qaa->ctr.y;
		t = -l->u.y * v.x + l->u.x * v.y;
		t = t / qaa->link->len;
	}
	else	t = 0;

	profile_afine( l->type, qaa->link->pr[0],
				qaa->link->pr[1], 1-t, l->pr[0] );
	if( l->type == NF_N3 )
		width_afine( qaa->link->width[0],
				qaa->link->width[1], 1-t, l->width[0] );

	l->br = qaa->link->br;
	l->br1 = qaa->link->br1;
	l->Fbr = qaa->link->Fbr;

	if( LINK_NEXT(l) != NULL ){
		(*link) = (*link)->p[F_END].p.l;
		return;
	}

	for(  ; rl->i1 > qa[i].i0 ; i++ );
	if( rl->i1 <= qa[i].i0 )	i--;
	qaa = &qa[i];

	if( rl->i1 < qa[i].i1-1 ){
		v.x = rl->ctr.x + rl->v.x - qaa->ctr.x;
		v.y = rl->ctr.y + rl->v.y - qaa->ctr.y;
		t = -l->u.y * v.x + l->u.x * v.y;
		t = t / qaa->link->len;
	}
	else	t = 1;

	profile_afine( l->type, qaa->link->pr[0],
				qaa->link->pr[1], 1-t, l->pr[1] );
	if( l->type == NF_N3 )
		width_afine( qaa->link->width[0],
				qaa->link->width[1], 1-t, l->width[1] );
}


//Grisha 31.05.01 ...
void set_rl_link( rl_type *rl, vec2d *p0, vec2d *p1, 
				 float a, float len, float da, int i0, int i1 )
{
	rl->a		= a;
	rl->ctr		= *p0;
	rl->v.x		= p1->x - p0->x;
	rl->v.y		= p1->y - p0->y;
	rl->da		= da;
	rl->len		= len;
	rl->i0		= i0;
	rl->i1		= i1;
	rl->Fsplit	= 0;
}
//... Grisha 31.05.01

