/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**************************
 ***	CodingAdepth.c	***
 **************************/
#include	<stdio.h>

#include	"Uigp/igp.h"

#include	"GpType/Configuration.h"

#include	"GpType/AdepthType.h"

#include	"Uprm/prm.h"
#include	"Ucode/code.h"
#include	"Ubst/BstType.h"

#include	"CodingAdepth.h"


#ifndef PLAYER

static code_in	Adepth_cp_in =   { -128.0, 128.0, 0.0625, 0.0, 0.0 };
static code_in	Adepth_u_in = { -1.0, 1.0, 0.0078125, 0.0, 0.0 };
static code_in	Adepth_dt_in = { -4.0, 4.0, 0.0078125, 0.0, 0.0 };
static code_in	Adepth_pu_in = { -1.0, 1.0, 0.0078125, 0.0, 0.0 };
static code_in	Adepth_p_in = { -1024, 1024, 0.0625, 0.0, 0.0 };
static code_in	Adepth_d_in = { -10, 100, 0.0078125, 0.0, 0.0 };



static GP_prm  Param[]    =    {
	{"Adepth-cp-d",(char *)&Adepth_cp_in.d, PRM_FLOAT, "%.3f", "0.0625"},
	{"Adepth-u-d",(char *)&Adepth_u_in.d, PRM_FLOAT, "%.3f", "0.0078125"},
	{"Adepth-dt-d",(char *)&Adepth_dt_in.d, PRM_FLOAT, "%.3f", "0.0078125"},

	{"Adepth-p-d",(char *)&Adepth_p_in.d, PRM_FLOAT, "%.3f", "0.0625"},

	{"Adepth-pu-d",(char *)&Adepth_pu_in.d, PRM_FLOAT, "%.3f","0.0078125"},
	{"Adepth-d-d",(char *)&Adepth_d_in.d, PRM_FLOAT, "%.3f", "0.0078125"},
};





void
coding_adepth_get_prm( GP_prm  **prm, int *prm_no, void (**check)())
{
	*prm    = Param;
	*prm_no  = PRM_NUMBER(Param);
	*check = NULL;
}




codingAdepth_type *
coding_adepth_set_lprm( char *file )
{
	codingAdepth_type* ac;

  	gp_prm_init( file, coding_adepth_get_prm );

	ac = ( (codingAdepth_type *)malloc( sizeof( codingAdepth_type ) ) ) ;
	Adepth_cp_in.min = -128.0 - 0.5*Adepth_cp_in.d;
	Adepth_cp_in.max =  128.0 - 0.5*Adepth_cp_in.d;

	coding_set_lprm( &Adepth_cp_in, &ac->Adepth_cp, 1.0 );

	coding_set_lprm( &Adepth_u_in, &ac->Adepth_u, 1.0 );
	coding_set_lprm( &Adepth_dt_in, &ac->Adepth_dt, 1.0 );
	coding_set_lprm( &Adepth_pu_in, &ac->Adepth_pu, 1.0 );

	coding_set_lprm( &Adepth_p_in, &ac->Adepth_p, 1.0 );

	Adepth_d_in.min =  -10.0 - 0.5*Adepth_d_in.d;
	Adepth_d_in.max =  100.0 - 0.5*Adepth_d_in.d;
	coding_set_lprm( &Adepth_d_in, &ac->Adepth_d, 1.0 );

	return ac;
}

#endif




codingAdepth_type *
coding_adepth_set()
{
codingAdepth_type* ac;
float	d;

	ac = ( (codingAdepth_type *)malloc( sizeof( codingAdepth_type ) ) ) ;


	d = 0.0625;
	code_set( &ac->Adepth_cp, -128.0 - 0.5*d, 128.0 - 0.5*d, d, 0, 1.0F );


	code_set( &ac->Adepth_u, -1.0F, 1.0F, 0.0078125, 0, 1.0F );


	code_set( &ac->Adepth_dt, -4.0F, 4.0F, 0.0078125, 0, 1.0F );


	code_set( &ac->Adepth_pu, -1.0F, 1.0F, 0.0078125, 0, 1.0F );


	code_set( &ac->Adepth_p, -1024.0F, 1024.0F, 0.0625, 0, 1.0F );


	d = 0.0078125;
	code_set( &ac->Adepth_d, -10.0 - 0.5*d, 100.0 - 0.5*d, d, 0, 1.0F );

	return ac;
}
