/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
#include        <stdio.h>

#include        "Uigp/igp.h"

#include        "CodingBuffer.h"

static const char *param_sp[] = {
				"0","1","2","3","g",
			  };


static char     Dir_name[512];


void
coding_buffer_initilaze_dir( char *dir, char *name )
{
        sprintf( Dir_name, "%s/%s.temp", dir, name );

	if( gp_is_dir_exist( Dir_name ) == 1 )	return;
	gp_mkdir( Dir_name );

}



void
param_get_file_name ( char *out_name, char* name, int sp )
{
	if( sp == -1 )
        	sprintf (out_name, "%s/%s", Dir_name, name );
	else	
        	sprintf (out_name, "%s/%s_%s", Dir_name, name, param_sp[sp] );
}


void
coding_buffer_remove_dir()
{
	gp_rmdir( Dir_name );
}





/* WRITE */

FILE *
coding_buffer_open_to_write( char *label, int sp )
{
char	name[512];
FILE	*fd;
        sprintf (name,"%s/%s_%s.msg", Dir_name, label,param_sp[sp]);
        if ( (fd = fopen (name, "wb" )) == NULL )
                error ("coding_buffer_open_to_write","can't open to  msg file ");
        return (fd);
}


void
coding_buffer_write( FILE *fd, void *buffer, int length)
{
	if( length == 0 )	return;
        if( fwrite( buffer, 1, length, fd ) == -1 )
                error ("coding_buffer_write","can't write");
}


void
coding_buffer_write_next( FILE *fd, char buffer )
{
        fwrite( &buffer, 1, 1, fd );
}



 
 
/* READ */


FILE *
coding_buffer_open_to_read( char *label, int sp )
{
char name[512];
FILE *fd;
        sprintf (name,"%s/%s_%s.msg", Dir_name, label, param_sp[sp]);
        if ( (fd = fopen (name, "rb" )) == NULL )
                error ("coding_buffer_open_to_read","can't open msg file ");

        return (fd);
}


void
coding_buffer_read( FILE *fd, void *buffer, int length )
{
	if( length == 0 )	return;
        if( fread( buffer, 1, length, fd ) == -1 )
                error ("coding_buffer_read","can't read");
}


int
coding_buffer_read_next( FILE *fd )
{
char	buffer;

	buffer = getc( fd );
	
	return( buffer );
}





/* QUANTIAZE */

void
param_zquantization( float *parray, char *buffer, code *pcod, int length )
{
int i;
int     val;
        for (i = 0; i < length; i++){
                val = ZCODE (parray[i],*pcod);

                if( val < -128 )        buffer[i] = -128;
                else    if( val > 127 ) buffer[i] = 127;
                        else    buffer[i] = val;
        }
}

void
param_quantization( float *parray, u_char *buffer, code *pcod, int length )
{
int i,  val;
        for (i = 0; i < length; i++){
                val = CODE (parray[i],*pcod);

                if( val < 0 )   buffer[i] = 0;
                else    if( val > pcod->imax )  buffer[i] = pcod->imax;
                        else    buffer[i] = val;
        }
}


void
param_zreconstruction( float *parray, char *buffer, code *pcode, int plength )
{
int i;
char temp;

        for ( i = 0;i < plength; i++) {
                temp = (char)buffer[i];
                parray[i] = UNZCODE ((int)temp,*pcode) ;
        }
}

void
param_reconstruction( float *parray, u_char *buffer, code *pcode, int plength)
{
int i;
u_char temp;
        for ( i = 0;i < plength; i++) {
                temp = (char)buffer[i];
                parray[i] = UNCODE ((int)temp,*pcode) ;
        }
}
