/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/****************************
 ***	CodingProfile.c   ***
 ****************************/
#include        "Uprm/prm.h"
#include 	"Ucode/code.h"

#include	"GpType/Configuration.h"

#include 	"CodingProfile.h"




#ifndef PLAYER

static code_in Pval_in =   { -127.0, 127.0, 1.0 };

static code_in Pnull_in =   { -127.0, 127.0, 1.0 };


static code_in Ewidth_in = { 0.5, 4.0, 0.25, 1.5, 0.0 };


static float	Pdmax;


static GP_prm  Param[] = {
	{"P-t", (char*)&Pdmax, PRM_FLOAT, "%.2f", "2.0"},

	{"P-Val-d", (char*)&Pval_in.d, PRM_FLOAT, "%.2f", "1.0"},
	{"P-Null-d", (char*)&Pnull_in.d, PRM_FLOAT, "%.2f", "1.0"},

	{"E-width-d", (char*)&Ewidth_in.d, PRM_FLOAT, "%.2f", "0.25"},
	{"E-width-a", (char*)&Ewidth_in.a, PRM_FLOAT, "%.2f", "1.5"},
};


void
coding_profile_get_prm( GP_prm  **prm, int *prm_no, void  (**check)())
{
        *prm    = Param;
        *prm_no  = PRM_NUMBER(Param);
        *check = NULL;
}



 
codingProfile_type *
coding_profile_set_lprm( char *file )
{
codingProfile_type *pc;

	gp_prm_init( file, coding_profile_get_prm );

	pc  = ( (codingProfile_type *)malloc( sizeof( codingProfile_type ) ) ) ;

	code_set( &pc->Pval, -127,127, Pval_in.d, 0, QCOLOR_UNIT );

	code_set( &pc->Pnull, -127,127, Pnull_in.d, 0, QCOLOR_UNIT );

	code_set( &pc->Ewidth, 0.5, 4, Ewidth_in.d, Ewidth_in.a, 1.0 );

	pc->Pdmax = F2QCOLOR(Pdmax);

	return( pc );
}

#endif






extern	float	P_t[];
extern	float	P_Val_d[];
extern	float	P_Null_d[];
extern	float	E_width_d[];
extern	float	E_width_a[];


codingProfile_type *
coding_profile_set( int iCoding )
{
codingProfile_type *pc;

	pc  = ( (codingProfile_type *)malloc( sizeof( codingProfile_type ) ) ) ;

	code_set( &pc->Pval, -127,127, P_Val_d[iCoding], 0, QCOLOR_UNIT );

	code_set( &pc->Pnull, -127,127, P_Null_d[iCoding], 0, QCOLOR_UNIT );

	code_set( &pc->Ewidth, 0.5, 4, E_width_d[iCoding], E_width_a[iCoding], 1.0 );

	pc->Pdmax = F2QCOLOR(P_t[iCoding]);

	return( pc );
}




void
coding_profile_group_edge( gp_array *agp,
		vec2d *gctr, link_type *link,
		profile_type pr[], int *pr_no,
		int gi[], int *gno, int *Bflag )
{
jnt_type	*jnt;
link_type	*l,	*el;
int	i0,	j0,	i1,	j1;
vec2d	ctr;

	i0 = j0 = -100;

	*Bflag = 0;
	jnt = link->p[F_BEGIN].p.jnt;
	if( jnt->type == JNT_EDGE && jnt->brn_no == 1 )	*Bflag |= 0x01;

	el = link;

	if( jnt->type == JNT_N3 ){
		gctr->x += link->v.x;
		gctr->y += link->v.y;
		link = LINK_NEXT(link);
	}


	*gno = *pr_no = 0;
	for( l = link, ctr = *gctr ; l != NULL ; el = l, l = LINK_NEXT(l ) ){


		i1 = (ctr.x+agp->bgp) * agp->inv_bgp;
		j1 = (ctr.y+agp->bgp) * agp->inv_bgp;

		if( i0 != i1 || j0 != j1 ){
			gi[(*gno)++] = (*pr_no);
			i0 = i1;
			j0 = j1;
		}

		pr[(*pr_no)++] = l->pr[0];

		ctr.x += l->v.x;
		ctr.y += l->v.y;
	}


	jnt = el->p[F_END].p.jnt;
	if( jnt->type != JNT_N3 ){

		i1 = (ctr.x+agp->bgp) * agp->inv_bgp;
		j1 = (ctr.y+agp->bgp) * agp->inv_bgp;

		if( i0 != i1 || j0 != j1 )
                        gi[(*gno)++] = (*pr_no);
 
                pr[(*pr_no)++] = el->pr[1];
	}

	gi[(*gno)] = (*pr_no);

	if( jnt->type == JNT_EDGE && jnt->brn_no == 1 )	*Bflag |= 0x02;
}
