/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/*************************
 ***   QuasiLength.c   ***
 *************************/
#include        "Uigp/igp.h"
#include		"QuasiLength.h"


#define		F2QUASILEN( x )		(( x < 0 )? -x*2 + 0.5 : x*2 + 0.5 ) 


static int	quasi_length( int a, int b );


int
polylink_quasi_length( link_type *link, int qlen[] )
{
link_type	*l;
int	len;
int	i;


	for( l = link, i = 0, len = 0 ; l != NULL ; l = LINK_NEXT(l), i++ ){

		qlen[i] = link_quasi_length( l );

		len += qlen[i];
	}

	return( len );
}



int
link_quasi_length( link_type *link )
{
int	a,	b;
	a = F2QUASILEN( l->v.x );
	b = F2QUASILEN( l->v.y );

	if ( a > 3*b )
		return a;

	if ( b > 3*a )
		return b;

	return ( (a+b)*3/4 );
}


int
vec2d_quasi_length( vec2d *v )
{
int	a,	b;
	a = F2QUASILEN( v->x );
	b = F2QUASILEN( v->y );

	if ( a > 3*b )
		return a;

	if ( b > 3*a )
		return b;

	return ( (a+b)*3/4 );
}




int
quasi_length( int a, int b )
{
	if( a < 0 )	a = -a;
	if( b < 0 )	b = -b;

	if ( a > 3*b )
		return a;
	else if ( b > 3*a )
		return b;
	else
		return ( (a+b)*3/4 );
}