/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/************************
 ***   DecBgLayer.c   ***
 ************************/
#include        "Ucode/code.h"

#include		"Uigp/igp.h"

#include        "Encoding/EncIn.h"

#include        "EpType/EpType.h"
#include        "Ep2/Ep2Type.h"

#include		"Ep2/Ep2.h"

#include        "CodingBgIn.h"



#include        "Coding/CodingDepth.h"


static void	decoding_bg_layer_get( ep_array *aep, int *unset );

static void	decoding_bg_layer_set_all0( ep_array *aep );


static void	decoding_bg_layer_set_all( ep_array *aep, int iLayer );



void
decoding_bg_layer( ep_array *aep, int nL, int iL0, codingDepth_type *dc, bst_type *bst )
{
int	unset,	transparent,	layer_no,	layers[MAX_LAYER];
ep_type	*ep;
int	i,	j,	k,	nLbit,	lbit;


	coding_bg_layer_set_id( aep );

	decoding_bg_layer_get( aep, &unset );

	decoding_bg_layer_set_all0( aep );

	if( unset == 0 ){
//		decoding_bg_layer_set_all0( aep );
		return;
	}


	transparent = bst_get( bst );

	nLbit = coding_ilog2(nL);
	if( transparent + nL > 1 )
		layer_no = bst_get_int( bst, nLbit );


	// use id
	if ( layer_no > 0 && layer_no < nL ){
		for( i = 0 ; i < layer_no ; i++ ){
			layers[i] = bst_get_int( bst, nLbit );
		}
	}
	else	{
		for( i = 0 ; i < nL ; i++ )
			layers[i] = i;
	}


	if( transparent == 1 && layer_no == 0 ){
		decoding_bg_layer_set_all( aep, 255 );
		return;
	}

	if( transparent == 1 && layer_no == 0 ){
		decoding_bg_layer_set_all( aep, layers[0] + iL0 );
		return;
	}



	lbit = coding_ilog2( layer_no ); 

	for( i = LSIZE ; i < aep->Ni-LSIZE ; i++ ){
		ep = &aep->ep[i][LSIZE];
		for( j = LSIZE ; j < aep->Nj-LSIZE ; j++, ep++ ){

			if( ep->iLayer != 0 )	continue;

			for( k = 0 ; k < 4 ; k++ )
				if( (ep->iLayer = ep->ng[k]->iLayer) > 0 )
					break;
			
			if( ep->iLayer > 0 )	continue;

			if( bst_get( bst ) == 0 ){
				ep->iLayer = ep->bound[0]->br.no = 255;
				continue;
			}


			k = 0;
			if( layer_no > 1 )
				k = bst_get_int( bst, lbit );

			ep->iLayer = ep->bound[0]->br.no = layers[k] + iL0;		
		}
	}
}



static void
decoding_bg_layer_get( ep_array *aep, int *unset )
{
ep_type		*ep;
int	i,	j;

	*unset = 0;

	for( i = LSIZE ; i < aep->Ni-LSIZE ; i++ ){
		ep = &aep->ep[i][LSIZE];
		for( j = LSIZE ; j < aep->Nj-LSIZE ; j++, ep++ ){

			if( ep->iLayer != 0 )	continue;

			*unset ++;
		}
	}
}

				
static void
decoding_bg_layer_set_all( ep_array *aep, int iLayer )
{
ep_type		*ep;
int	i,	j;

	for( i = LSIZE ; i < aep->Ni-LSIZE ; i++ ){
		ep = &aep->ep[i][LSIZE];
		for( j = LSIZE ; j < aep->Nj-LSIZE ; j++, ep++ ){

			if( ep->iLayer != 0 )	continue;

			ep->iLayer = ep->bound[0]->br.no = iLayer;
		}
	}
}



static void
decoding_bg_layer_set_all0( ep_array *aep )
{
ep_type		*ep;
int	i,	j;

	for( i = LSIZE ; i < aep->Ni-LSIZE ; i++ ){
		ep = &aep->ep[i][LSIZE];
		for( j = LSIZE ; j < aep->Nj-LSIZE ; j++, ep++ ){

			if( ep->iLayer <= 0 )	continue;

			if( ep->bound[0]->br.no == 255 )
				ep->bound[0]->br.no = ep->iLayer;
		}
	}
}
