/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/************************
 ***   EncLayerBg.c   ***
 ************************/
#include        "Ucode/code.h"

#include		"Uigp/igp.h"

#include        "Encoding/EncIn.h"

#include        "EpType/EpType.h"
#include        "Ep2/Ep2Type.h"

#include		"Ep2/Ep2.h"

#include        "CodingBgIn.h"



#include        "Coding/CodingDepth.h"



static void	encoding_bg_layer_get( ep_array *aep,
			int *unset, int *transparent, int layers[], int *layer_no );


extern int	Lbits,	Lbr_no;




void
encoding_bg_layer( ep_array *aep, int nL, int iL0, codingDepth_type *dc, bst_type *bst )
{
int	unset,	transparent,	layer_no,	layers[MAX_LAYER],	layersInv[MAX_LAYER];
ep_type	*ep;
int	i,	j,	k,	nLbit,	lbit;


	coding_bg_layer_set_id( aep );

	encoding_bg_layer_get( aep, &unset, &transparent, layers, &layer_no );

	if( unset == 0 )	return;


	bst_put( bst, transparent == 0 );

	nLbit = coding_ilog2(nL);
	if( transparent + nL > 1 )
		bst_put_int( bst, layer_no, nLbit );


	// use id
	if ( layer_no > 0 && layer_no < nL )
		for( i = 0 ; i < layer_no ; i++ ){
			bst_put_int( bst, layers[i], nLbit );
			layersInv[ layers[i]] = i;
		}


	if( transparent + layer_no == 1 )	return;


	lbit = coding_ilog2( layer_no ); 

	for( i = LSIZE ; i < aep->Ni-LSIZE ; i++ ){
		ep = &aep->ep[i][LSIZE];
		for( j = LSIZE ; j < aep->Nj-LSIZE ; j++, ep++ ){

			if( ep->iLayer != 0 )	continue;

			for( k = 0 ; k < 4 ; k++ )
				if( (ep->iLayer = ep->ng[k]->iLayer) > 0 )
					break;
			
			if( ep->iLayer > 0 )	continue;

			ep->iLayer = ep->bound[0]->br.no;
			if( ep->iLayer == 255 ){
				bst_put( bst, 0 );
				continue;
			}

			bst_put( bst, 1 );

			if( layer_no > 1 )
				bst_put_int( bst, layersInv[ep->iLayer - iL0 ], lbit );
		
		}
	}
}



static void
encoding_bg_layer_get( ep_array *aep, int *unset, int *transparent, int layers[], int *layer_no )
{
ep_type		*ep;
int	i,	j;
int	mbr,	iLayer;

	*transparent = *layer_no = *unset = 0;
	mbr = 0;

	for( i = LSIZE ; i < aep->Ni-LSIZE ; i++ ){
		ep = &aep->ep[i][LSIZE];
		for( j = LSIZE ; j < aep->Nj-LSIZE ; j++, ep++ ){

			if( ep->iLayer != 0 )	continue;

			*unset ++;
			iLayer = ep->bound[0]->br.no;

			if( iLayer != 255 ){
				if( (mbr & ( 1<< iLayer ) ) == 0 ){
					mbr |= ( 1<< iLayer );
					layers[ (*layer_no)++ ] = iLayer;
				}
			}
			else	*transparent = 1;

			continue;
		}
	}
}

				


