/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/*****************
 *** EncEp.h   ***
 *****************/

#include        "Uigp/igp.h"


#include        "GpType/GpType.h"
#include        "EpType/EpType.h"
#include        "Ep2/Ep2.h"


#include        "CodingBgIn.h"

#include        "Encoding/EncIn.h"



#define		BBUF_SIZE	(4*MAX_GP*MAX_GP)



#define	BVAL_FNAME	"bg"
#define	BDVAL2_FNAME	"bg_d2"
#define	BDVAL_FNAME	"bg_d"



static int	Bval_no[SP_NO],	Bdval_no[SP_NO], Bdval2_no[SP_NO], Bbits;



static void	encoding_ep2_get_value(ep2_array *aep2, code *bval, code *bdval,
		      u_char val[], int *n, char dval[], int *n2, int sp);


static int	encoding_ep_bound_get_dvalue( ep2_array *aep2,
					codingBg_type *bc, int i0, int j0,
					int k, float val[], int sp );

static int	encoding_ep_get_dvalue( ep2_array *aep2,
				codingBg_type *bc, int fdval[], int *dno,
				float val[], int *no, int sp);



void
encoding_ep_open( codingBg_type *bc)
{
int	sp;

	for (sp = 0; sp < SP_NO; sp++) {
		bc->Bval[sp].fp = coding_buffer_open_to_write(BVAL_FNAME, sp);
		bc->Bdval[sp].fp = coding_buffer_open_to_write(BDVAL_FNAME, sp);
		bc->Bdval2[sp].fp = coding_buffer_open_to_write(BDVAL2_FNAME, sp);

		Bval_no[sp] = Bdval_no[sp] = Bdval2_no[sp] = 0;
	}

	Bbits = 0;
}




void
encoding_ep(ep2_array * aep2,codingBg_type *bc, bst_type * bst)
{
float           a[BBUF_SIZE];
u_char          qa[BBUF_SIZE];
char		qa2[BBUF_SIZE];
int             fdval[BBUF_SIZE];
int             sp, n, n2, no, dno, i;

	Bbits -= BST_BITS( bst );

	/* Ep - Ep2 */
	for( sp = 0 ; sp < SP_NO ; sp++ ){


		encoding_ep_get_dvalue(aep2,bc, fdval, &dno, a, &no, sp);
		if (dno == 0) {
			bst_put_int(bst, 0, 1);
			continue;
		}

		bst_put_int(bst, 1, 1);


		for (i = 0; i < dno; i++)
			bst_put_int(bst, fdval[i], 1);

		param_zquantization(a, qa2, &bc->Bdval[sp], no);
		coding_buffer_write(bc->Bdval[sp].fp, qa2, no);

		Bdval_no[sp] += no;
	}




	/* Ep2 */
	for( sp = 0; sp < SP_NO; sp++ ){
		encoding_ep2_get_value(aep2, &bc->Bval[sp], &bc->Bdval2[sp],
				      qa, &n, qa2, &n2, sp);

		coding_buffer_write(bc->Bval[sp].fp, qa, n);

		coding_buffer_write(bc->Bdval2[sp].fp, qa2, n2);

		Bval_no[sp] += n;
		Bdval2_no[sp] += n2;
	}

	Bbits += BST_BITS( bst );
}



void
encoding_ep_close(codingBg_type *bc)
{
	int             sp;
	for (sp = 0; sp < SP_NO; sp++) {
		fclose(bc->Bval[sp].fp);
		fclose(bc->Bdval[sp].fp);
		fclose(bc->Bdval2[sp].fp);
	}
}


static int Bval_bits[SP_NO];
static int Bdval2_bits[SP_NO];
static int Bdval_bits[SP_NO];


void
encoding_ep_huffman( codingBg_type *bc,FILE *cfd )
{
int             sp, msg, cotype;

	for (sp = 0; sp < SP_NO; sp++) {
		encoding_agp_file_append_huffman( cfd, BVAL_FNAME, sp,
					&msg, &Bval_bits[sp], &cotype);

		encoding_agp_file_append_huffman( cfd, BDVAL2_FNAME, sp,
					&msg, &Bdval2_bits[sp], &cotype);

		encoding_agp_file_append_huffman( cfd, BDVAL_FNAME, sp,
					&msg, &Bdval_bits[sp], &cotype);
	}
}



void
encoding_ep_punch_line(FILE * fp)
{
	int             sp, total_bits, bg_no;

	bg_no = total_bits = 0;

	total_bits = Bbits;
	for (sp = 0; sp < SP_NO; sp++) {
		total_bits += Bval_bits[sp] + Bdval_bits[sp] + Bdval2_bits[sp];
		bg_no += Bval_no[sp];
	}

	encoding_print_row(fp, "Bg-", bg_no, 0, total_bits);
}


void
encoding_ep_end_line(FILE * fp)
{
	int             geo, bg, val[SP_NO];
	int             bg_no;

	geo = 0;
	bg = 0;

	val[SP_Y] = Bval_bits[SP_Y] + Bdval_bits[SP_Y] + Bdval2_bits[SP_Y];
	val[SP_Q] = Bval_bits[SP_Q] + Bdval_bits[SP_Q] + Bdval2_bits[SP_Q];
	val[SP_A] = Bval_bits[SP_A] + Bdval_bits[SP_A] + Bdval2_bits[SP_A];
	bg_no = Bval_no[SP_Y] + Bval_no[SP_Q] + Bval_no[SP_A];

	geo = Bbits;

	encoding_print_end_line(fp, "Bg-", bg_no, geo, bg, val);
}
         

	


int
encoding_ep_print_inf(FILE * fp)
{
int	tbits,	bits,	msg;
int	sp;

	msg = tbits = 0;

	fprintf(fp, "\n\n\n");
	fprintf(fp, "Bg -   Hbit: %d\n", Bbits );
	tbits += Bbits;

	for (sp = 0; sp < SP_NO; sp++) {

		bits = Bval_bits[sp] + Bdval_bits[sp] + Bdval2_bits[sp];
		msg += Bval_no[sp] + Bdval_no[sp] + Bdval2_no[sp];

		fprintf(fp, "bg-%d", sp );
		encoing_print_inf( fp, "   V2", Bval_bits[sp], Bval_no[sp] );
		encoing_print_inf( fp, "   D2",Bdval2_bits[sp],Bdval2_no[sp] );
		encoing_print_inf( fp, "   D", Bdval_bits[sp], Bdval_no[sp] );
		encoing_print_inf( fp, "    T", bits, Bval_no[sp] );
		fprintf(fp, "\n" );

		tbits += bits;
	}


	fprintf(fp, "\nTotal -   %d{%.2f}\n", tbits, tbits / (float) msg);
	return (tbits);
}


static void
encoding_ep2_get_value(ep2_array * aep2, code *bval, code *bdval,
		      u_char val[], int *n, char dval[], int *n2, int sp)
{
ep2_type	*ep2;
int	i,	j,	k;
bound2_type	*b2;
float	vdiff,	tmp,	v_prev;
int	cno;


	*n = *n2 = 0;

	for( i = 0 ; i < aep2->Ni ; i++)
	    for( j = 0 ; j < aep2->Nj ; j++){
		ep2 = &aep2->ep[i][j];


		for (k = 0; k < ep2->b_no; k++){

			b2 = ep2->bound[k];

			if( b2->br < 0 )	continue;

			if(B2_INDEPENDENT(*b2)){

				tmp = CODE(b2->val[sp], *bval);
				val[*n] = PUSH_TO_RANGE(tmp, 0,	bval->imax);
				b2->val[sp] = UNCODE((float)val[*n], *bval) ;
				(*n)++;

				continue;
			}
				
			v_prev = ep2_prev_value(aep2, i, j, k, sp);

			vdiff = b2->val[sp] - v_prev;
			cno = ZCODE (vdiff, *bdval);
			dval[*n2] = PUSH_TO_RANGE(cno, -128, 127);

			b2->val[sp] = UNZCODE( cno,  *bdval) + v_prev;
			(*n2)++;
		}
	    }
}


static int DN = 0;

static int
encoding_ep_get_dvalue( ep2_array *aep2,codingBg_type *bc, int fdval[], int *dno,
		       float val[], int *no, int sp)
{
ep2_type       *ep2;
bound2_type    *b2;
int             i, j, k, n, tdno;


	*no = *dno = tdno = 0;

	for (i = 0; i < aep2->Ni; i++)
		for (j = 0; j < aep2->Nj; j++) {
			ep2 = &aep2->ep[i][j];


			for (k = 0; k < ep2->b_no; k++) {

				b2 = ep2->bound[k];

				if( b2->br < 0 )	continue;

				if (b2->no <= 1)
					continue;

				n=encoding_ep_bound_get_dvalue( aep2,bc, i, j, k,
							     &val[*no], sp);

				if (n > 0) {
					tdno++;
					fdval[(*dno)++] = 1;
					(*no) += n - 1;
				} else
					fdval[(*dno)++] = 0;

				DN++;

			}

		}

	if (tdno == 0){
		DN -= *dno;
		*dno = 0;
	}

	return (*dno);
}


static int
encoding_ep_bound_get_dvalue( ep2_array *aep2,codingBg_type *bc, int i0, int j0,
					int k, float val[], int sp )
{
ep2_type	*ep2;
bound2_type	*b2;
ep_type	*ep;
ebgp_type	*bgp;
int	i,	bno,	no;
float	max, dval, app_val;
vec2d	v;

	ep2 = &aep2->ep[i0][j0];
	b2 = ep2->bound[k];

	max = 0;
	no = 0;
	for (i = 0; i < 4; i++){
		if( (ep = ep2->ep[i]) == NULL )	continue;

		for( bno = 0; bno < ep->b_no; bno++ ){

			if (!(b2->Fbound[i] & (1L << bno)))
				continue;

			if( (bgp = ep->bound[bno]->Mbgp) == NULL )
				continue;

			v.x = ep->ctr.x + bgp->ctr.x;
			v.y = ep->ctr.y + bgp->ctr.y;
			app_val = ep2_value( &v, aep2, i0, j0, k, sp);

			dval = bgp->val[sp] - app_val;
			val[no++] = dval;

			if( dval < 0 )		dval = -dval;
			if( dval > max )	max = dval;
		}
	}


	if( max < bc->Td[sp] )	return( 0 );

	return( no );
}
