/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**************************
 ***	EncAdepth.c	***
 **************************/
#include        <stdio.h>

#include        "Uigp/igp.h"

#include        "GpType/GpType.h"

#include        "Ubst/BstType.h"

#include        "Coding/Coding.h"
#include        "Coding/CodingAdepth.h"

#include        "EncIn.h"
#include        "EncAdepth.h"




static void	encoding_view( view_type *vw,
				codingAdepth_type *ad, bst_type *bst );

static void	encoding_plane( view_type *vw,
				vec3d *P, vec3d *N,codingAdepth_type *ad, bst_type *bst );

static void	encoding_layer( coding_type *cprm, gp_array *agp, bst_type *bst );

static void	encoding_adepth_float( float a, code *cd, bst_type *bst );


static int	Ad_bit;


void
encoding_adepth( coding_type *cprm, FILE *fp, gp_array *agp )
{
adepth_type	*ad;
abr_type	*br;
bst_type        *bst;
int	mbr;
int     i,      no,	byte;

        bst = bst_alloc( 100000 );

	ad = agp->ad;

	encoding_view( ad->vw, cprm->ad, bst );

	bst_put_int( bst, (int)ad->Mbr, 16 );



	mbr = ad->Mbr;
	for( no = 0 ;  mbr != 0 ; no++ , mbr >>= 1 ){
		if( !( mbr & 0x01 ) )	continue;

		for( i = 0 ; i < ad->br_no ; i++ ){
			br = &ad->abr[i];
			if( br->no == no )	break;
		}

/*
		bst_put( bst, br->Fplane );

		if( br->Fplane == 0 )	continue;
*/

		encoding_plane( ad->vw, &br->P, &br->N, cprm->ad, bst );
	}


	encoding_layer( cprm, agp, bst );


	byte = ( bst->MaskIndex > 0 )? bst->byte_i+1 : bst->byte_i;
	gp_fwrite_short( fp, byte );
	fwrite( bst->data, 1, byte, fp );
	bst_free( bst );

	Ad_bit = byte*8;
}



static void
encoding_view( view_type *vw, codingAdepth_type  *adc, bst_type *bst )
{
vec2d	vp,	v;
float	tmp;

	bst_put( bst, vw->type == VIEW_PERSPECTIVE );


	if( vw->cp.x != 0 || vw->cp.y != 0 || vw->cp.z != 1 ){
		bst_put( bst, 1 );
		encoding_adepth_float( vw->cp.x, &adc->Adepth_cp, bst );
		encoding_adepth_float( vw->cp.y, &adc->Adepth_cp, bst );
		encoding_adepth_float( vw->cp.z, &adc->Adepth_cp, bst );
	}
	else	bst_put( bst, 0 );



	if( vw->N.x != 0 || vw->N.y != 0 || vw->N.z != -1 ){
		bst_put( bst, 1 );
		encoding_adepth_float( vw->N.x, &adc->Adepth_u, bst );
		encoding_adepth_float( vw->N.y, &adc->Adepth_u, bst );
		bst_put( bst, vw->N.z >= 0 );
	}
	else	bst_put( bst, 0 );


	if( vw->V.x != 0 || vw->V.y != 1 || vw->V.z != 0 ){
		bst_put( bst, 1 );
		encoding_adepth_float( vw->V.x, &adc->Adepth_u, bst );
		encoding_adepth_float( vw->V.y, &adc->Adepth_u, bst );
		bst_put( bst, vw->V.z >= 0 );
	}
	else	bst_put( bst, 0 );



	tmp = 1.0 - ( vw->uv.x1 - vw->uv.x0 )/vw->Upixel * 512;
	if( tmp != 1.0 ){
		bst_put( bst, 1 );
		encoding_adepth_float( tmp, &adc->Adepth_dt, bst );
	}
	else	bst_put( bst, 0 );


	if( vw->uv.x1 == -vw->uv.x0 && vw->uv.y1 == -vw->uv.y0 ){ 
		bst_put( bst, 0 );
		return;
	}


	bst_put( bst, 1 );
	vp.x = vp.y = 0;
	view_vp2pixel( vw, &vp, &v );
	encoding_adepth_float( v.x, &adc->Adepth_p, bst );
	encoding_adepth_float( v.y, &adc->Adepth_p, bst );
}



static void
encoding_plane( view_type *vw, vec3d *P, vec3d *N,
			codingAdepth_type *adc, bst_type *bst )
{
vec3d	dp;
float	d;

	if( N->x != vw->N.x || N->y != vw->N.y || N->z != vw->N.z ){
		bst_put( bst, 1 );

		dp.x = vw->U.x * N->x + vw->U.y * N->y + vw->U.z * N->z;
		dp.y = vw->V.x * N->x + vw->V.y * N->y + vw->V.z * N->z;
		dp.z = vw->N.x * N->x + vw->N.y * N->y + vw->N.z * N->z;

		encoding_adepth_float( dp.x, &adc->Adepth_pu, bst );
		encoding_adepth_float( dp.y, &adc->Adepth_pu, bst );

		encoding_adepth_float( dp.z, &adc->Adepth_pu, bst );
	}
	else	bst_put( bst, 0 );


	dp.x = P->x - vw->cp.x;
	dp.y = P->y - vw->cp.y;
	dp.z = P->z - vw->cp.z;
	d = dp.x * N->x + dp.y * N->y + dp.z * N->z;

	encoding_adepth_float( d, &adc->Adepth_d, bst );
}



static void
encoding_layer( coding_type *cprm, gp_array *agp, bst_type *bst )
{
layer_type	*layer;
int	i,	mbit,	ibit,	jbit;
/*
	if( agp->type == AGP_TRANSPARENT && agp->layer[0]->name[0] != 0 ){
		agp->type = AGP_TRANSPARENT_U;
	}
*/
//	if( agp->type != AGP_TRANSPARENT_U && agp->type != AGP_TRANSPARENT )    return;

	if(  agp->type == AGP_NORMAL )	return;
 
 
	mbit = coding_ilog2( (int)agp->ad->Mbr );
	ibit = coding_ilog2( agp->Ni );
	jbit = coding_ilog2( agp->Nj );


	bst_put_int( bst, agp->layer_no, mbit );	/* new 5-2001 */

	for( i = 0 ; i < agp->layer_no ; i++ ){
		layer = agp->layer[i];
		bst_put_int( bst, layer->Mbr, mbit );
 
		bst_put_int( bst, layer->i0, ibit );
		bst_put_int( bst, layer->i1, ibit );
		bst_put_int( bst, layer->j0, jbit );
		bst_put_int( bst, layer->j1, jbit );

		bst_put( bst, layer->Fdepth_delta );	/* new 5-2001 */

		if( cprm->head & CODING_LAYER_NAME )
			bst_put_string( bst, layer->name, MAX_LAYER_NAME );
	}
}





static void
encoding_adepth_float( float a, code *cd, bst_type *bst )
{
int     cno;
        cno = CODE( a, *cd );
	if( cno < 0 )	cno = 0;
	else	if( cno > cd->imax )	cno = cd->imax;
        bst_put_int( bst, ABS(cno), cd->rbit );
}



void
encoding_adepth_punch_line( FILE *fp )
{
int	tbits;

	tbits = Ad_bit;
 
	encoding_print_row( fp, "Adepth-", 1, 1, tbits );
}
 
 
void
encoding_adepth_end_line( FILE *fp )
{
int	geo,	val[SP_NO];
 
 
	geo = Ad_bit;
	val[SP_Y] = val[SP_Q] = val[SP_A] = 0;
	encoding_print_end_line( fp, "Adepth-", 1, geo, 0, val );
}

