/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**************************
 ***	EncAgp.c	***
 **************************/
#include	<stdio.h>   
#include	<math.h>
#include	<memory.h>


#include	"Uigp/igp.h"

#include	"AgpTool/AgpTool.h"
#include	"AgpDepth/AgpDepth.h"
#include	"AgpColor/AgpColor.h"
#include	"AgpLayer/AgpLayer.h"

#include	"EpType/EpType.h"
#include	"Ep/Ep.h"
#include	"EpBgp/EpBgp.h"

#include	"EpValue/EpValue.h"

#include	"Ep2/Ep2Type.h"
#include	"Ep2/Ep2.h"



#include	"Quantize/Quantize.h"

#include	"N3Prime/N3Prime.h"


#include	"Ucode/code.h"
#include	"Encoding.h"
#include	"EncIn.h"

#include	"EncCenter.h"
#include	"EncJoint.h"
#include	"EncProfile.h"
#include	"EncLink.h"

#include	"EncN3.h"
#include	"EncRidge.h"
#include	"EncHunch.h"
#include	"EncLayer.h"
#include	"EncDepth.h"
#include	"EncAdepth.h"

#include	"CodingBg/EncBg.h"

#include	"Coding/Coding.h"
#include	"Coding/VimVersion.h"
#include	"Coding/CodingDepth.h"
#include	"Coding/CodingBg.h"





#define		CD_N		24



static void	encoding_agp( gp_array *agp, coding_type *cprm, bst_type *bst);

static void	encoding_agp_ridge_hunch( gp_array *agp,
								coding_type *cprm, bst_type *bst );


static void	encoding_agp_open( gp_array *agp, coding_type *cprm );
static void	encoding_agp_close( gp_array *agp, coding_type *cprm );
static void	encoding_agp_huffman( gp_array *agp,
					coding_type *cprm, FILE *cfd );

static int	encoding_agp_build_head( gp_array *agp );




int
encoding_agp_to_file( gp_array *agp, int Lcoding, char *dir, char *name )
{
char	file[256];
FILE    *fp;
int	byte;

	coding_buffer_initilaze_dir( dir, name );
 


	sprintf( file, "%s/%s", dir, name );

	if ( (fp = fopen( file, "wb" )) == NULL )
		return( 0 );


	putc( 'V', fp);
	gp_fwrite_num( fp, VIM_VERSION, 2 ); 


	putc( 'A', fp);
	gp_fwrite_num( fp, 0, 3 );


	encoding_agp_to_Ofile( agp, Lcoding, fp );


	putc( 0, fp);


	byte = ftell( fp );

	fseek( fp, 4, 0L );
	gp_fwrite_num( fp, byte-8, 3 );

	fclose( fp );


	coding_buffer_remove_dir();

	return( 1 );
}





void
encoding_agp_to_Ofile( gp_array *agp, int Lcoding, FILE *fp )
{
coding_type	*cprm;
bst_type	*bst;
int	bst_size;


	cprm = coding_init( Lcoding, agp->bgp );



	agp_connect_jnt( agp );

	agp_link_len( agp, 24 /*agp->bgp*4*/ );

	agp_link_no( agp, 255 );

	quantize_agp( agp, cprm);




	/* write header */
	encoding_agp_file_write_header( fp,
			agp->type, agp->row, agp->column,
			agp->bgp, Lcoding );

	
	cprm->head = encoding_agp_build_head( agp );
	gp_fwrite_num( fp, cprm->head, 1 );

	cprm->N = CD_N;
	gp_fwrite_num( fp, cprm->N, 1 );



	if( agp->type & AGP_DEPTH )
		encoding_adepth( cprm, fp, agp );





	bst_size = agp->column* agp->row/12;
	bst = bst_alloc( MAX( bst_size, 10000) );
 
 
 
	encoding_agp_open( agp, cprm );
 

	encoding_agp( agp, cprm, bst );


	encoding_agp_close( agp, cprm );




	encoding_agp_file_append_bst( fp, -1, bst );

	bst_free( bst );



	encoding_agp_huffman( agp, cprm, fp );

	coding_free( cprm );
}




static void
encoding_agp_open( gp_array *agp, coding_type *cprm )
{
	encoding_center_open(cprm->cc);

	encoding_link_open( cprm->lc);
	encoding_joint_open( cprm->jc );

	encoding_profile_open( cprm->pc );

	encoding_n3_open(cprm->nc);

	encoding_bg_open( cprm );




	if( cprm->head & CODING_RIDGE )
		encoding_ridge_open(cprm->rc);


	if( cprm->head & CODING_HUNCH )
		encoding_hunch_open( cprm->hc );


	if( cprm->head & CODING_DEPTH )
		encoding_layer_open( cprm->depth );

	if( cprm->head & CODING_DEPTH_DELTA )
		encoding_depth_open( cprm->depth );
}





static void
encoding_agp( gp_array *agp, coding_type *cprm, bst_type *bst)
{
int	color_type;


	if( (color_type = agp->color_type ) == AGP_RGB  )
		agp_Rgb_YCrCb( agp );


	encoding_center( agp ,cprm->cc);


	agp_link_in_reorder( agp );
	agp_jnt_brn4( agp );
	encoding_joint( agp, cprm, bst );



	if( agp->type & AGP_DEPTH )
		encoding_layer( agp, cprm->depth, bst );



	encoding_bg( agp, cprm, bst );



	encoding_profile( agp,cprm->pc, bst );



	encoding_agp_ridge_hunch( agp, cprm, bst );



	if( agp->type & AGP_DEPTH && cprm->head & CODING_DEPTH_DELTA ){

		agp_depth_base_add( agp, 0 );

		encoding_depth( agp, cprm->depth, bst );

		agp_depth_base_add( agp, 1 );
	}



	if( color_type == AGP_RGB  )
		agp_YCrCb_Rgb( agp );
}



static void
encoding_agp_ridge_hunch( gp_array *agp, coding_type *cprm, bst_type *bst)
{
hunch_type	*ah[10000];
link_type	*link[10000];
int	nf_no;
int	i,	j;


	agp->Lni = agp->Lnj = cprm->N;
	for( i = 1 ; i < agp->Ni-1 ; i += cprm->N )
		for( j = 1 ; j < agp->Nj-1 ; j += cprm->N ){
			agp_set_box_ij( agp, i, j );


			if( cprm->head & CODING_RIDGE ){
				nf_no = agp_get_ridge_array( agp, link );
				encoding_ridge( link, nf_no,cprm->rc, bst );
			}


			if( cprm->head & CODING_HUNCH ){
				nf_no = agp_get_hunchs_array( agp, ah );
				encoding_hunch( ah, nf_no, cprm->hc, bst );
			}
		}
}


static void
encoding_agp_close( gp_array *agp, coding_type *cprm )
{
	encoding_center_close( cprm->cc );

	encoding_link_close( cprm->lc );
	encoding_joint_close( cprm->jc );

	encoding_profile_close( cprm->pc );


	encoding_n3_close(cprm->nc);

	encoding_bg_close( cprm );




	if( cprm->head & CODING_RIDGE )
		encoding_ridge_close(cprm->rc);
	if( cprm->head & CODING_HUNCH )
		encoding_hunch_close( cprm->hc );


	if( cprm->head & CODING_DEPTH )
		encoding_layer_close( cprm->depth );

	if( cprm->head & CODING_DEPTH_DELTA )
		encoding_depth_close( cprm->depth );
}


static void
encoding_agp_huffman( gp_array *agp, coding_type *cprm, FILE *cfd )
{
	
	encoding_center_huffman( cprm->cc, cfd);

	encoding_joint_huffman( cprm->jc, cfd);

	encoding_link_huffman( cprm->lc, cfd);

	encoding_profile_huffman( cprm->pc, cfd);

	encoding_n3_huffman( cprm->nc, cfd );

	encoding_bg_huffman( cprm, cfd );





	if( cprm->head & CODING_RIDGE )
		encoding_ridge_huffman( cprm->rc, cfd);

	if( cprm->head & CODING_HUNCH )
		encoding_hunch_huffman( cprm->hc, cfd );



	if( cprm->head & CODING_DEPTH )
		encoding_layer_huffman( cprm->depth, cfd );

	if( cprm->head & CODING_DEPTH_DELTA )
		encoding_depth_huffman( cprm->depth, cfd );
}





static int
encoding_agp_build_head( gp_array *agp )
{
int	nf[NF_TYPES];
int	head;
int	i;

	head = 0;

	agp_nf( agp, NULL, nf );

	if( nf[NF_RH] > 0 )	head |= CODING_RH;
	if( nf[NF_HUNCH] + nf[NF_RH] > 0 )	head |= CODING_HUNCH;

	if( nf[NF_RIDGE] + nf[NF_N3R] > 0 )	head |= CODING_RIDGE;



	if( agp->type & AGP_DEPTH )
		head |= CODING_DEPTH;


	for( i = 0 ; i < agp->layer_no ; i++ )
		if( agp->layer[i]->Fdepth_delta == 1 ){
			head |= CODING_DEPTH_DELTA;
			break;
		}

	for( i = 0 ; i < agp->layer_no ; i++ )
		if( agp->layer[i]->name[0] != 0 ){
			head |= CODING_LAYER_NAME;
			break;
		}


	return( head );
}
