/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
#include        <stdio.h>

#include        "Uigp/igp.h"
#include        "Ucode/code.h"

#include        "Coding/CodingJoint.h"

#include        "EncJoint.h"




static void	encoding_joint_get_link_gp( gp_array *agp, link_type *link,
					int i0, int j0, int *i1, int *j1 );


static void	encoding_joint_get_link_no( gp_type *gp,
			int type, link_type *link, int *no, int *cno  );



void
encoding_branch_in( gp_array *agp, jnt_type *jnt, int bno,
				codingJoint_type *jc, bst_type *bst )
{
brn_type	*b;
vec2d	ctr;
int	i0,	j0,	i,	j;
int	di,	dj,	dij;
int	lno,	no;

	b = jnt->brn[bno];
	ctr.x = jnt->gctr.x + b->ctr.x;
	ctr.y = jnt->gctr.y + b->ctr.y;

	i0 = (ctr.x +agp->bgp) * agp->inv_bgp;
	j0 = (ctr.y +agp->bgp) * agp->inv_bgp;

	encoding_joint_get_link_gp( agp, b->link, i0, j0, &i, &j );

	di = i - i0 + 5;
	dj = j - j0 + 5;
	dij = di*11 + dj;
//	coding_buffer_write_next( jc->Jend_f.fp, dij );
	CODING_BUFFER_WRITE_NEXT( jc->Jend_f.fp, dij );


	encoding_joint_get_link_no( &agp->gp[i][j],
		(1L<<NF_N3) | (1L<<NF_N3R)
		, b->link, &lno, &no );
	if( lno > 1 )
                bst_put_int( bst, no, coding_ilog2( lno-1 ) );

	bst_put_int( bst, b->direction, 1 );
}





static void
encoding_joint_get_link_gp( gp_array *agp, link_type *link, int i0, int j0, int *i1, int *j1 )
{
gp_type		*gp;
link_type	*l;
int	i,	j;

	for( i = i0-5 ; i <= i0+5 ; i++ ){
		if( i < 0 || i >= agp->Ni )	continue;

		for( j = j0-5 ; j <= j0+5 ; j++ ){
			if( j < 0 || j >= agp->Nj )	continue;

			gp = &agp->gp[i][j];

			for( l = gp->link ; l != NULL ; l = l->nl ){
				if( l != link )	continue;
				*i1 = i;
				*j1 = j;
				return;
			}
		}
	}
}



static void
encoding_joint_get_link_no( gp_type *gp, int mtype, link_type *link,
				int *no, int *cno  )
{
int	i;
link_type	*ln[MAX_LINK],	*l;

	if( gp->link->nl == NULL ){
		*no = 1;
		*cno = 0;
		return;
	}

	*no = 0;
	for( l = gp->link ; l != NULL ; l = l->nl ){

		if( !( (1L<<l->type) & mtype) )	continue;

		ln[ (*no)++ ] = l;
	}
			
	if( *no == 1 ){
		*cno = 0;
		return;
	}

	for( i = 0; i < *no ; i++ )
		if( ln[i] == link ){
			*cno = i;
			return;
		}
}
