/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
#include	<stdio.h>
#include        <memory.h>

#include        "Uigp/igp.h"
#include        "GpType/GpType.h"

#include        "Ufbst/FbstType.h"

#include        "Ucode/code.h"
#include        "EncIn.h"

#include        "EncCenter.h"

static int	Nfno,	B2,	B1,	B0,	B00;


#include	"EncIn.h"

#include	"Coding/CodingCenter.h"


#define	HCTR_FNAME	"Hctr"
#define	PCTR_FNAME	"Pctr"
#define	YES_FNAME	"Yes"
#define	TYPE_FNAME	"Type"



typedef	struct center_type {
	int	type;
	vec2d	ctr;

}	center_type;

typedef	struct center_box {
	int	no;
	center_type	c[20];

}	center_box;




static void 	encoding_center_block4( gp_array *agp,codingCenter_type *cc,
					int i0, int j0, fbst_type *fbst );

static void	encoding_center_block2( gp_array *agp,codingCenter_type *cc,
			int i0, int j0,
			int gflag, int cno, int cflag,
			int cno1[], int cflag1[],
			center_box cb[], fbst_type *fbst );

static void	encoding_center_block1( gp_type *gp,codingCenter_type *cc,
			int gflag, int cno, center_box cb[4], fbst_type *fbst);
static void	encoding_center_box( codingCenter_type *cc,int gflag,
				center_box *cb, fbst_type *fbst );

static void	encoding_center_get_block2( gp_array *agp, int i0, int j0,
				int *cno, int *cflag,
				int cno1[], int cflag1[], center_box cb[] );

static void	encoding_center_get_block1( gp_array *agp, int i0, int j0, 
                        int *cno, int *cflag, center_box cb[4] );

static void	encoding_center_get_jnt( gp_type *gp, center_box cb[4] );
static void	encoding_center_get_hunch( gp_type *gp, center_box cb[4] );



void
encoding_center_open(codingCenter_type *cc)
{
	cc->Yes_f.fp = coding_buffer_open_to_write( YES_FNAME, 4 );
	cc->Type_f.fp = coding_buffer_open_to_write( TYPE_FNAME, 4 );
	B2 = B1 = B0 = B00 = 0;
	Nfno  = 0;
}
 
 

void
encoding_center( gp_array *agp,codingCenter_type *cc )
{
char	file[256];
static fbst_type	*fbst;
int	i,	j;

	param_get_file_name( file, PCTR_FNAME, -1 );
	fbst = fbst_open_to_write( file );

	for( i = 0 ; i < agp->Ni ; i += 4 )
		for( j = 0 ; j < agp->Nj ; j += 4 )
			encoding_center_block4( agp,cc, i, j, fbst );

	fbst_close( fbst, 1 );
}


static void
encoding_center_block4( gp_array *agp,codingCenter_type *cc, int i0, int j0, fbst_type *fbst )
{
int	cno,	cflag;
int	cno2[4],	cflag2[4];
int	cno1[16],	cflag1[16];
center_box	cb[64];


	encoding_center_get_block2( agp, i0, j0, &cno2[0], &cflag2[0],
					&cno1[0], &cflag1[0], &cb[0] );
	encoding_center_get_block2( agp, i0, j0 +2,
					&cno2[1], &cflag2[1],
					&cno1[4], &cflag1[4], &cb[16] );
	encoding_center_get_block2( agp, i0 +2, j0,
					&cno2[2], &cflag2[2], 
					&cno1[8], &cflag1[8], &cb[32] );
	encoding_center_get_block2( agp, i0 +2, j0 +2,
					&cno2[3], &cflag2[3],
					&cno1[12], &cflag1[12], &cb[48] );

	cno = cno2[0] + cno2[1] + cno2[2] + cno2[3];
	cflag = cflag2[0] & cflag2[1] & cflag2[2] & cflag2[3];


	if( cno == 0 ){
		fbst_put( fbst, 0 );
		return;
	}

	fbst_put( fbst, 1 );
	fbst_put( fbst, cflag );


	encoding_center_block2( agp,cc, i0, j0, cflag, cno2[0], cflag2[0],
				&cno1[0], &cflag1[0], &cb[0], fbst );
	encoding_center_block2( agp,cc, i0, j0 +2, cflag, cno2[1], cflag2[1],
				&cno1[4], &cflag1[4], &cb[16], fbst );
	encoding_center_block2( agp,cc, i0 +2, j0, cflag, cno2[2], cflag2[2],
				&cno1[8], &cflag1[8], &cb[32], fbst );
	encoding_center_block2( agp,cc, i0+2, j0+2, cflag, cno2[3], cflag2[3],
				&cno1[12], &cflag1[12], &cb[48], fbst );
}



static void
encoding_center_block2( gp_array *agp,codingCenter_type *cc, int i0, int j0,
	int gflag, int cno, int cflag,
	int cno1[], int cflag1[], center_box cb[], fbst_type *fbst )
{
	if( i0 >= agp->Ni || j0 >= agp->Nj )    return;

	B2++;

        if( cno == 0 ){  
                fbst_put( fbst, 0 );
                return;
        }
 
        fbst_put( fbst, 1 );

	if( gflag == 0 )
		fbst_put( fbst, cflag );


	if( i0 < agp->Ni && j0 < agp->Nj )
		encoding_center_block1( &agp->gp[i0][j0],cc, cflag,
					cno1[0], &cb[0], fbst );
	if( i0 < agp->Ni && j0+1 < agp->Nj )
		encoding_center_block1( &agp->gp[i0][j0+1],cc, cflag,
					cno1[1], &cb[4], fbst );
	if( i0+1 < agp->Ni && j0 < agp->Nj )
		encoding_center_block1( &agp->gp[i0+1][j0],cc, cflag,
					cno1[2], &cb[8], fbst );
	if( i0+1 < agp->Ni && j0+1 < agp->Nj )
		encoding_center_block1( &agp->gp[i0+1][j0+1],cc, cflag,
					cno1[3], &cb[12], fbst );
}


static void
encoding_center_block1( gp_type *gp,codingCenter_type *cc, int gflag,
			int cno, center_box cb[4], fbst_type *fbst )
{
char	flag;

	B1 ++;
        if( cno == 0 ){  
                fbst_put( fbst, 0 );
                return;
        }
 
        fbst_put( fbst, 1 );

	flag = 0;
	if( cb[0].no > 0 )	flag |= 0x01;
	if( cb[1].no > 0 )	flag |= 0x02;
	if( cb[2].no > 0 )	flag |= 0x04;
	if( cb[3].no > 0 )	flag |= 0x08;
	coding_buffer_write_next( cc->Yes_f.fp, flag );

	encoding_center_box( cc,gflag, &cb[0], fbst );
	encoding_center_box( cc,gflag, &cb[1], fbst );
	encoding_center_box( cc,gflag, &cb[2], fbst );
	encoding_center_box( cc,gflag, &cb[3], fbst );
}

static int      HH,     JJ;

static void
encoding_center_box( codingCenter_type *cc,int gflag, center_box *cb, fbst_type *fbst )
{
center_type	*c;
int	i,	cno;

	B0++;
	if( cb->no == 0 ){
		return;
	}

	B00 ++;

	for( i = 0 ; i < cb->no ; i++ ){
		c = &cb->c[i];

		Nfno++;

		if( c->type > NF_RH ){
			JJ++;
//			coding_buffer_write_next( cc->Type_f.fp, c->type  );
			CODING_BUFFER_WRITE_NEXT( cc->Type_f.fp, c->type  );

			cno = CODE( c->ctr.x, cc->Pctr );
			if( cno >= cc->Pctr.hr )	cno -= cc->Pctr.hr;
			fbst_put_int( fbst, cno, cc->Pctr.rbit-1 );

			cno = CODE( c->ctr.y, cc->Pctr );
			if( cno >= cc->Pctr.hr )	cno -= cc->Pctr.hr;
			fbst_put_int( fbst, cno, cc->Pctr.rbit-1 );

			if( i < cb->no - 1 )	fbst_put( fbst, 1 );
			continue;
		}

		HH++;
//		coding_buffer_write_next( cc->Type_f.fp, c->type  );
		CODING_BUFFER_WRITE_NEXT( cc->Type_f.fp, c->type  );

		cno = CODE( c->ctr.x, cc->Hctr );
		if( cno >= cc->Hctr.hr )	cno -= cc->Hctr.hr;
		fbst_put_int( fbst, cno, cc->Hctr.rbit-1 );

		cno = CODE( c->ctr.y, cc->Hctr );
		if( cno >= cc->Hctr.hr )	cno -= cc->Hctr.hr;
		fbst_put_int( fbst, cno, cc->Hctr.rbit-1 );


		if( i < cb->no - 1 )	fbst_put( fbst, 1 );

		continue;
	}

	if( gflag == 0 )
		fbst_put( fbst, 0 );
}







static void
encoding_center_get_block2( gp_array *agp, int i0, int j0,
				int *cno, int *cflag,
				int cno1[], int cflag1[], center_box cb[] )
{
	if( i0 >= agp->Ni || j0 >= agp->Nj ){
		*cno = *cflag = 0;
		return;
	}


	encoding_center_get_block1( agp, i0, j0,
					&cno1[0], &cflag1[0], &cb[0] );
	encoding_center_get_block1( agp, i0, j0+1,
					&cno1[1], &cflag1[1], &cb[4] );
	encoding_center_get_block1( agp, i0+1, j0,
					&cno1[2], &cflag1[2], &cb[8] );
	encoding_center_get_block1( agp, i0+1, j0+1,
					&cno1[3], &cflag1[3], &cb[12] );

	*cno = cno1[0] + cno1[1] + cno1[2] + cno1[3];
	*cflag = cflag1[0] & cflag1[1] & cflag1[2] & cflag1[3];
}
	




static void
encoding_center_get_block1( gp_array *agp, int i0, int j0,
			int *cno, int *cflag, center_box cb[4] )
{
gp_type *gp;
	if( i0 >= agp->Ni || j0 >= agp->Nj ){
		*cno = *cflag = 0;
		return;
	}

	gp = &agp->gp[i0][j0];

	cb[0].no = cb[1].no = cb[2].no = cb[3].no = 0;


	encoding_center_get_jnt( gp, cb );
	encoding_center_get_hunch( gp, cb );

	*cno = cb[0].no + cb[1].no + cb[2].no + cb[3].no;

	*cflag = ( cb[0].no <= 1 && cb[1].no <= 1 && cb[2].no <= 1 && cb[3].no <=1);
}



static void
encoding_center_get_jnt( gp_type *gp, center_box cb[4] )
{
jnt_type	*h[4][40],	*jnt;
int	hno[4],	no;

	if( gp->jnt == NULL )	return;

	hno[0] = hno[1] = hno[2] = hno[3] = 0;
	for( jnt = gp->jnt ; jnt != NULL ; jnt = jnt->nj ){
		no = ( jnt->ctr.y < 0 ) ? 0 : 1;
		if( jnt->ctr.x > 0 )	no += 2;

		cb[no].c[cb[no].no].ctr = jnt->ctr;
		cb[no].c[cb[no].no].type  = jnt->type + 1;
		cb[no].no++;

		if( hno[no] > 0 )	h[no][ hno[no] - 1]->nj = jnt;
		h[no][ hno[no] ++ ] = jnt;
	}

	jnt = NULL;

	if( hno[3] > 0 ){
		h[3][ hno[3]-1 ]->nj = jnt;
		jnt = h[3][0];
	}

	if( hno[2] > 0 ){
		h[2][ hno[2]-1 ]->nj = jnt;
		jnt = h[2][0];
	}

	if( hno[1] > 0 ){
		h[1][ hno[1]-1 ]->nj = jnt;
		jnt = h[1][0];
	}

	if( hno[0] > 0 ){
		h[0][ hno[0]-1 ]->nj = jnt;
		jnt = h[0][0];
	}

	gp->jnt = jnt;
}






static void
encoding_center_get_hunch( gp_type *gp, center_box cb[4] )
{
hunch_type	*h[4][80],	*hunch;
int	hno[4],	no;

	if( gp->hunch == NULL )	return;

	hno[0] = hno[1] = hno[2] = hno[3] = 0;
	for( hunch = gp->hunch ; hunch != NULL ; hunch = hunch->nh ){
		no = ( hunch->ctr.y < 0 ) ? 0 : 1;
		if( hunch->ctr.x > 0 )	no += 2;

		cb[no].c[cb[no].no].ctr = hunch->ctr;
		cb[no].c[cb[no].no].type  = hunch->type;
		cb[no].no++;


		if( hno[no] > 0 )	h[no][ hno[no] - 1]->nh = hunch;
		h[no][ hno[no] ++ ] = hunch;
	}

	hunch = NULL;

	if( hno[3] > 0 ){
		h[3][ hno[3]-1 ]->nh = hunch;
		hunch = h[3][0];
	}

	if( hno[2] > 0 ){
		h[2][ hno[2]-1 ]->nh = hunch;
		hunch = h[2][0];
	}

	if( hno[1] > 0 ){
		h[1][ hno[1]-1 ]->nh = hunch;
		hunch = h[1][0];
	}

	if( hno[0] > 0 ){
		h[0][ hno[0]-1 ]->nh = hunch;
		hunch = h[0][0];
	}

	gp->hunch = hunch;
}






void
encoding_center_close(codingCenter_type *cc)
{
        fclose( cc->Yes_f.fp );
        fclose( cc->Type_f.fp );
}
 
 

static int Pctr_bits, Yes_f_bits, Hctr_bits,	Type_f_bits;

void
encoding_center_huffman( codingCenter_type *cc,FILE *cfd )
{
char	file[256];
int	cotype,	msg;

	encoding_agp_file_append_huffman( cfd, YES_FNAME, 4,
					&msg, &Yes_f_bits, &cotype );

	encoding_agp_file_append_huffman( cfd, TYPE_FNAME, 4,
					&msg, &Type_f_bits, &cotype );

	param_get_file_name( file, PCTR_FNAME, -1 );
	Pctr_bits = 8*encoding_agp_file_append_file( cfd, -1, file );


	Hctr_bits = cc->Hctr.bits;
}




void
encoding_center_punch_line( FILE *fp, int *num )
{
int     tbits;
	tbits = Yes_f_bits + Pctr_bits + Hctr_bits + Type_f_bits;
	*num = Nfno;

	encoding_print_row( fp, "Center-", Nfno, 0, tbits );
}




void
encoding_center_end_line( FILE *fp )
{
int     geo,    bg,     val[SP_NO];

	geo = Yes_f_bits + Pctr_bits + Hctr_bits + Type_f_bits;
	bg = 0;

	val[SP_Y] = 0;
	val[SP_Q] = 0;
	val[SP_A] = 0;

	encoding_print_end_line( fp, "Center-", Nfno, geo, bg, val );
}



int
encoding_center_print_inf( FILE *fp )
{
int	tbits;

	tbits = 0;

	fprintf (fp,"\n\nCenter - %d   B2: %d   B1: %d   B0: %d   B00: %d\n",
		Nfno, B2, B1, B0, B00 );

	encoing_print_inf( fp, "Location_1", Yes_f_bits, Nfno );
	fprintf( fp, "\n" );
	tbits += Yes_f_bits;

	encoing_print_inf( fp, "Location_1", Type_f_bits, Nfno );
	fprintf( fp, "\n" );
	tbits += Type_f_bits;

	encoing_print_inf( fp, "Coor", Pctr_bits + Hctr_bits, Nfno );
	fprintf( fp, "\n" );
	tbits += Pctr_bits + Hctr_bits;


	encoing_print_inf( fp, "Total", tbits, Nfno );
	fprintf( fp, "\n" );

	return( tbits );
}


void
encoding_center_total_bits( int *fno, int *tbits )
{
	*tbits = Yes_f_bits + Pctr_bits + Hctr_bits + Type_f_bits;
	*fno = Nfno;
}