/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/

#include        "Ucode/code.h"

#include 	"Uigp/igp.h"
#include        "EncDepth.h"
#include        "EncIn.h"



#include	"EncIn.h"
#include        "Coding/CodingDepth.h"



#define	BDEPTH_FNAME	"b_depth"
#define	DEPTH_FNAME	"depth"

#define		BSIZE		2
//#define		MAX_BR		20




static	int	Depth_no,	Bdepth_no,	Dbits;

static void	encoding_depth_br_gp( gp_type *gp,
			float Br_depth[], int Br_depth_no[] );

static void	encoding_depth_gp( gp_type *gp,codingDepth_type *dc, float Br_depth[] );




void
encoding_depth_open(codingDepth_type *dc)
{
	dc->Bdepth.fp = coding_buffer_open_to_write( BDEPTH_FNAME, 4 );
	dc->Depth.fp = coding_buffer_open_to_write( DEPTH_FNAME, 4 );

	Depth_no = 0;
	Bdepth_no = 0;
}







void
encoding_depth( gp_array *agp,codingDepth_type *dc, bst_type *bst )
{
int	i,	j,	n,	k,	no;
float	Br_depth[MAX_BR];
int	Br_depth_no[MAX_BR];
s_char	d[BUF_SIZE];

	Dbits -= BST_BITS( bst );






	for( i = 0 ; i < agp->Ni ; i += BSIZE )
	    for( j = 0 ; j < agp->Nj ; j += BSIZE ){


		for( k = 0 ; k < MAX_BR ; k++ ){
			Br_depth[k] = 0;
			Br_depth_no[k] = 0;
		}



		for( k = i ; k < agp->Ni && k < i+BSIZE ; k++ )
		    for( n = j ; n < agp->Nj && n < j+BSIZE ; n++ )
			encoding_depth_br_gp( &agp->gp[k][n],
					Br_depth, Br_depth_no );


		no = 0;
		for( k = 0 ; k < MAX_BR ; k++ ){
			if( Br_depth_no[k] == 0 )	continue;
			Br_depth[k] /= Br_depth_no[k];
	
                	d[no] = ZCODE( Br_depth[k], dc->Bdepth );

                	Br_depth[k] = UNZCODE( d[no], dc->Bdepth );
			no++;
        	}

        	coding_buffer_write( dc->Bdepth.fp, (u_char *)d, no );

        	Bdepth_no += no;



		for( k = i ; k < agp->Ni && k < i+BSIZE ; k++ )
		    for( n = j ; n < agp->Nj && n < j+BSIZE ; n++ )
			encoding_depth_gp( &agp->gp[k][n],dc, Br_depth );

	    }

			
	Dbits += BST_BITS( bst );
}




void
encoding_depth_close(codingDepth_type *dc)
{
	fclose( dc->Bdepth.fp );
	fclose( dc->Depth.fp );
}

static int Bdepth_bits,Depth_bits;

void
encoding_depth_huffman( codingDepth_type *dc,FILE *cfd )
{
int	msg,	cotype;

	encoding_agp_file_append_huffman( cfd, BDEPTH_FNAME, 4,
						&msg, &Bdepth_bits, &cotype );

	encoding_agp_file_append_huffman( cfd, DEPTH_FNAME, 4,
						&msg, &Depth_bits, &cotype );
}


void
encoding_depth_punch_line( FILE *fp )
{ 
int	total_bits; 

	total_bits = Depth_bits + Bdepth_bits + Dbits;

	encoding_print_row( fp, "Depth-", Depth_no, Depth_no, total_bits );
}



void
encoding_depth_end_line( FILE *fp )
{
int	geo,	bg,	val[SP_NO];

	geo = 0;
	bg = 0;

	val[SP_Y] = Depth_bits + Bdepth_bits + Dbits;
	val[SP_Q] = 0;
	val[SP_A] = 0;

        encoding_print_end_line( fp, "Depth-", Depth_no, geo, bg, val );
}





int
encoding_depth_print_inf( FILE *fp )
{
int	tbits;
 
	tbits = 0;

	fprintf( fp, "\n\n\n" );

	if( Depth_no == 0 ){
		fprintf( fp, "Depth: 0" );
		return( tbits );
	}

	fprintf( fp, "Depth: %d\n", Depth_no );
	encoing_print_inf( fp, "Depth", Depth_bits, Depth_no );
	encoing_print_inf( fp, "Bdepth", Bdepth_bits, Depth_no );
	fprintf( fp, "\n" );

	tbits += Depth_bits + Bdepth_bits + Dbits;


	encoing_print_inf( fp, "\nTotal -   ", tbits, Depth_no );

	return( tbits );
}





static void
encoding_depth_br_gp( gp_type *gp, float Br_depth[], int Br_depth_no[] )
{
bgp_type	*bp;
hunch_type	*h;
link_type	*l;
int	n;



	for( bp = gp->bgp ; bp != NULL ; bp = bp->p ){
		n = bp->br;
		Br_depth[n] += bp->depth;
		Br_depth_no[n] ++;
	}


	for( h = gp->hunch ; h != NULL ; h = h->nh ){
		n = h->br;
		Br_depth[n] += h->depth;
		Br_depth_no[n] ++;
	}


	for( l = gp->link ; l != NULL ; l = l->nl ){

		switch( l->type ){
		case NF_RIDGE:
		case NF_N3R:
			n = l->br;
			Br_depth[n] += l->pr[0].p->depth;
			Br_depth_no[n] ++;
		    break;

		case NF_EDGE:
			n = l->br;
			Br_depth[n] += l->pr[0].p->depth;
			Br_depth_no[n] ++;
		    break;

		case NF_N3:
			n = l->br;
			Br_depth[n] += l->pr[0].p->depth;
			Br_depth_no[n] ++;

			Br_depth[n] += l->width[0]->depth[F_RIGHT];
			Br_depth_no[n] ++;

			Br_depth[n] += l->width[0]->depth[F_LEFT];
			Br_depth_no[n] ++;
		    break;
		}
	}
}




static void
encoding_depth_gp( gp_type *gp,codingDepth_type *dc, float Br_depth[] )
{
bgp_type	*bp;
hunch_type	*h;
link_type	*l;
s_char	d[BUF_SIZE];
int	n,	no;
float	tmp;


	no = 0;

	for( bp = gp->bgp ; bp != NULL ; bp = bp->p ){
		n = bp->br;
		tmp = bp->depth - Br_depth[n];
		d[no++] = ZCODE( tmp, dc->Depth);
	}


	for( h = gp->hunch ; h != NULL ; h = h->nh ){
		n = h->br;
		tmp = h->depth - Br_depth[n];
		d[no++] = ZCODE( tmp, dc->Depth);
	}


	for( l = gp->link ; l != NULL ; l = l->nl ){

		switch( l->type ){
		case NF_RIDGE:
		case NF_N3R:
			n = l->br;

			tmp = l->pr[0].p->depth - Br_depth[n];
			d[no++] = ZCODE( tmp, dc->Depth);
			
			if( LINK_NEXT(l) != NULL )	break;

			tmp = l->pr[1].p->depth - Br_depth[n];
			d[no++] = ZCODE( tmp, dc->Depth);
		    break;

		case NF_EDGE:
			n = l->br;

			tmp = l->pr[0].p->depth - Br_depth[n];
			d[no++] = ZCODE( tmp, dc->Depth);
			
			if( LINK_NEXT(l) != NULL )	break;

			tmp = l->pr[1].p->depth - Br_depth[n];
			d[no++] = ZCODE( tmp, dc->Depth);
		    break;

		case NF_N3:
			n = l->br;

			tmp = l->pr[0].p->depth - Br_depth[n];
			d[no++] = ZCODE( tmp, dc->Depth);

			tmp = l->width[0]->depth[F_RIGHT] - Br_depth[n];
			d[no++] = ZCODE( tmp, dc->Depth);

			tmp = l->width[0]->depth[F_LEFT] - Br_depth[n];
			d[no++] = ZCODE( tmp, dc->Depth);
			

			if( LINK_NEXT(l) != NULL )	break;


			tmp = l->pr[1].p->depth - Br_depth[n];
			d[no++] = ZCODE( tmp, dc->Depth);

			tmp = l->width[1]->depth[F_RIGHT] - Br_depth[n];
			d[no++] = ZCODE( tmp, dc->Depth);

			tmp = l->width[1]->depth[F_LEFT] - Br_depth[n];
			d[no++] = ZCODE( tmp, dc->Depth);
			
		    break;
		}
	}


        coding_buffer_write( dc->Depth.fp, (u_char *)d, no );
	Depth_no += no;
}
