/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**********************
 ***	EncFile.c   ***
 *********************/
#include	<stdio.h>
#include        <malloc.h>
#include        <string.h>

#include        "Uigp/igp.h"

#include        "LossLessCoding/LossLessCoding.h"

#include	"EncIn.h"



static void	encoding_agp_file_put_header( FILE *cfd, int cotype, int size);






void
encoding_agp_file_write_header( FILE *fp,
			int type, int row, int column,
			int bgp, int Lcoding )
{
	gp_fwrite_short( fp, type );
	gp_fwrite_short( fp, row );
	gp_fwrite_short( fp, column );
	gp_fwrite_short( fp, bgp );
	gp_fwrite_short( fp, Lcoding );
}




void
encoding_agp_file_append_huffman( FILE *cfd, char *name, int sp,
					int *msg, int *bits, int *cotype )
{
char	file[128];
bst_type *bst;

        param_get_file_name( file, name, sp );

        bst = lossless_encoding( file, msg, bits, cotype );

        encoding_agp_file_append_bst( cfd, *cotype, bst );

		if( bst != NULL )	bst_free( bst );
}






int
encoding_agp_file_append_bst( FILE *cfd, int cotype, bst_type *bst )
{
int	byte;

	if( cotype == 0x0e ){
		encoding_agp_file_put_header( cfd, cotype, 0 );
		return(0);
	}

	if (bst == NULL)
		return(0);

	byte = ( bst->MaskIndex == 0 )? bst->byte_i : bst->byte_i +1;

	encoding_agp_file_put_header( cfd, cotype, byte );

	if( byte == 0 )	return(0);


	fwrite( bst->data, 1, byte, cfd );

	return( byte );
}


int
encoding_agp_file_append_file( FILE *cfd, int cotype, char *file )
{
FILE	*fp;
int	size,	c;

	size = gp_file_size( file );

	encoding_agp_file_put_header( cfd, cotype, size );


	if( size == 0 )	return(0);



	if ( (fp = fopen( file, "rb" )) == NULL )
		return( 0 );
 
 
	while ((c = getc( fp )) != EOF)
		putc( c, cfd );
 
	fclose( fp );    


	unlink( file );

	return( size );
}






static void
encoding_agp_file_put_header( FILE *cfd, int cotype, int size )
{
u_char	header[4];

	header[0] = ( cotype == -1 )? 0x0f : cotype;

	if( size == 0 ){
		header[0] |= 0x10;
		fwrite( (char *)header, 1, 1, cfd ); 
		return;
	}

	if( size < 256 ){
		header[0] |= 0x20;
		header[1] = size;
		fwrite( (char *)header, 1, 2, cfd ); 
		return;
	}

	if( size < 256*256 ){
		header[0] |= 0x30;
		header[1] = (size>>8)&0xff;
		header[2] = size&0xff;
		fwrite( (char *)header, 1, 3, cfd ); 
		return;
	}

	header[0] |= 0x40;
	header[1] = (size>>16)&0xff; 
	header[2] = (size>>8)&0xff;
        header[3] = size&0xff;
	fwrite( (char *)header, 1, 4, cfd ); 
}
