/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
#include        <stdio.h>
#include        <math.h>
#include        <memory.h>

#include        "Uigp/igp.h"

 
#include        "EncHunch.h"
#include        "EncIn.h"
 
 
#include        "Ucode/code.h"

#include        "Coding/CodingHunch.h"


#define		HVAL_FNAME	"h_val"
#define		HTHETA_FNAME	"h_angle"
#define		HWIDTH_FNAME	"h_width"
#define		HLEN_FNAME	"h_len"

static int	Hunch_no,	Hunch_bits;
static int	Tno;



static int	encoding_hunch_test_valQA( hunch_type *ah[], int nf_no,
						codingHunch_type *hc );

static void	encoding_hunch1( hunch_type *h, int Fqa,
				codingHunch_type *hc, bst_type *bst );

static void	encoding_hunch_treat( hunch_type *h,
				int, codingHunch_type *hc );



 
void
encoding_hunch_open( codingHunch_type *hc )
{

	hc->Hval[SP_Y].fp = coding_buffer_open_to_write( HVAL_FNAME, SP_Y );
	hc->Hval[SP_Q].fp = coding_buffer_open_to_write( HVAL_FNAME, SP_Q );

 
	hc->Hwidth.fp  = coding_buffer_open_to_write( HWIDTH_FNAME, 4 );

	hc->Htheta.fp = coding_buffer_open_to_write ( HTHETA_FNAME, 4 );


	Hunch_no = 0;
	Hunch_bits = 0;

	Tno = 0;

}


void
encoding_hunch( hunch_type *ah[], int hunch_no, codingHunch_type *hc, bst_type *bst )
{
int	i,	Fqa;

	if (hunch_no == 0) return;

	Hunch_no += hunch_no;

	Hunch_bits -= BST_BITS( bst );

	Fqa = encoding_hunch_test_valQA( ah, hunch_no, hc );
	bst_put_int( bst, Fqa, 1 );


	for( i = 0 ; i < hunch_no ; i++ )
		encoding_hunch1( ah[i], Fqa, hc, bst );





	Hunch_bits += BST_BITS( bst );
}



static void
encoding_hunch1( hunch_type *h, int Fqa, codingHunch_type *hc, bst_type *bst )
{
float	tmp;
int	cno,	w_cno,	l_cno;



	encoding_hunch_treat( h, h->type, hc );

	cno = ZCODE( h->rval[SP_Y], hc->Hval[SP_Y] );
	if( cno < -128 )	cno = -128;
	else	if( cno > 127 )	cno = 127;
	CODING_BUFFER_WRITE_NEXT( hc->Hval[SP_Y].fp, cno );

	if( Fqa == 0 ){
		if( ABS(h->rval[SP_Q]) > hc->Hval_t ||
				ABS(h->rval[SP_A]) > hc->Hval_t ){
			bst_put_int( bst, 0, 1 );

			cno = ZCODE( h->rval[SP_Q], hc->Hval[SP_Q] );
			CODING_BUFFER_WRITE_NEXT( hc->Hval[SP_Q].fp, cno );

			cno = ZCODE( h->rval[SP_A], hc->Hval[SP_A] );
			CODING_BUFFER_WRITE_NEXT( hc->Hval[SP_Q].fp, cno );
		}
		else	bst_put_int( bst, 1, 1 );
	}
				

		

	w_cno = CODE( h->g.width, hc->Hwidth );

	tmp = h->g.len - h->g.width;
	l_cno = ZCODE( tmp, hc->Hlen);
	if( l_cno < 0 )	l_cno = 0;

	cno = ( l_cno << hc->Hwidth.rbit ) | w_cno;

	CODING_BUFFER_WRITE_NEXT( hc->Hwidth.fp, cno );


	if( l_cno == 0 )	return;

	cno = CODE( h->g.theta, hc->Htheta );
	if( cno > hc->Htheta.imax ) cno = hc->Htheta.imax;
	CODING_BUFFER_WRITE_NEXT( hc->Htheta.fp, cno );

	Tno++;
}








static int
encoding_hunch_test_valQA( hunch_type *ah[], int nf_no, codingHunch_type *hc )
{
hunch_type *h;
int     i;

        for( i = 0 ; i < nf_no ; i++ ){
		h = ah[i];
		if( ABS(h->rval[SP_Q]) > hc->Hval_t ||
				ABS(h->rval[SP_A]) > hc->Hval_t )
			return( 0 );
	}

	return( 1 );
}






static void
encoding_hunch_treat( hunch_type *h, int type, codingHunch_type *hc )
{
float	width,	len;

	if( type == NF_RH && h->g.len < hc->Hround_min )
		h->g.len = hc->Hround_min;

	if( ABS (h->g.len - h->g.width) < hc->DeltaR ){
		h->g.len = (h->g.len > h->g.width) ? h->g.len :h->g.width;
		h->g.width = h->g.len;					
	}

	if( type != NF_HUNCH )	return;

	len   = h->g.len;
	width = h->g.width;
        if( h->g.width > h->g.len ) {			
		if ( h->g.theta < M_PI/2 )
			h->g.theta +=  M_PI/2;
		else	h->g.theta -=  M_PI/2;

		h->g.len = width;
		h->g.width = len;
	}
}

		

void
encoding_hunch_close( codingHunch_type *hc )
{
	fclose( hc->Hval[SP_Y].fp );
	fclose( hc->Hval[SP_Q].fp );
 
        fclose( hc->Hwidth.fp );

        fclose( hc->Htheta.fp );
}

 

static int	Hwidth_bits,	Htheta_bits;
static int	Hval_bits[SP_NO];

void
encoding_hunch_huffman( codingHunch_type *hc, FILE *cfd )
{
int	cotype,	msg;
 
        encoding_agp_file_append_huffman( cfd, HWIDTH_FNAME, 4,
					&msg, &Hwidth_bits, &cotype);
 
        encoding_agp_file_append_huffman( cfd, HTHETA_FNAME, 4,
					&msg, &Htheta_bits, &cotype);
 

	encoding_agp_file_append_huffman( cfd, HVAL_FNAME, SP_Y,
					&msg, &Hval_bits[SP_Y], &cotype);

	encoding_agp_file_append_huffman( cfd, HVAL_FNAME, SP_Q,
					&msg, &Hval_bits[SP_Q], &cotype);
}


void
encoding_hunch_punch_line( FILE *fp )
{
int	tbits;

        tbits = Htheta_bits + Hwidth_bits;
        tbits += Hunch_bits;

	tbits +=  Hval_bits[SP_Y] + Hval_bits[SP_Q];

 
        encoding_print_row( fp, "Hunch-", Hunch_no, Hunch_no, tbits );
}


void
encoding_hunch_end_line( FILE *fp )
{
int	geo,	bg,	val[SP_NO];

	geo = Hwidth_bits + Htheta_bits + Hunch_bits;
	bg = 0;

	val[SP_Y] = Hval_bits[SP_Y];
	val[SP_Q] = Hval_bits[SP_Q];
	val[SP_A] = 0;

	encoding_print_end_line( fp, "Hunch-", Hunch_no, geo, bg, val );
}
         

int
encoding_hunch_print_inf( FILE *fp )
{
int	tbits;


        tbits = 0;
 
 
        fprintf (fp,"\n\n\nHunch    - %d\n", Hunch_no );

	if( Hunch_no == 0 ){
		fprintf (fp,"\nTotal: %d{%.2f}\n\n", 0, 0.0 );
		return( 0 );
	}


        fprintf(fp,"Size: %d{%.2f:%d}   Angle: %d{%.2f:%d}\n",
                        Hwidth_bits, (float)Hwidth_bits/Hunch_no, Hunch_no,
                        Htheta_bits, (float)Htheta_bits/Tno, Tno );

        tbits += Htheta_bits + Hwidth_bits;



	encoing_print_inf( fp, "Rval-Y", Hval_bits[SP_Y], Hunch_no );
	fprintf (fp,"\n" );
	tbits += Hval_bits[SP_Y];

	encoing_print_inf( fp, "Rval-QA", Hval_bits[SP_Q], Hunch_no );
	fprintf (fp,"\n" );
	tbits += Hval_bits[SP_Q];


	tbits += Hunch_bits;

	fprintf (fp,"\nTotal: %d{%.2f}\n\n",
		tbits,
		( Hunch_no == 0.0 )? 0.0 : tbits/(float)Hunch_no );

	return( tbits );
}



void
encoding_hunch_total_bits( int *fno, int *hbits )
{
	*hbits = Hval_bits[SP_Y] + Hval_bits[SP_Q];
	*fno = Hunch_no;
}