/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**************************
 ***	EncInf.c    ***
 **************************/
#include        <stdio.h>   
#include	<math.h>

#include        "Encoding.h"
#include        "EncIn.h"

#include	"Ucode/code.h"
#include        "EncIn.h"
#include        "EncCenter.h"
#include        "EncJoint.h"
#include        "EncProfile.h"
#include        "EncLink.h"

#include        "EncN3.h"
#include        "EncRidge.h"
#include        "EncHunch.h"
#include        "EncLayer.h"
#include        "EncDepth.h"
#include        "EncAdepth.h"

#include		"../CodingBg/EncBg.h"


static float	Block_no;
static int	Tbits,	File_bits,	Punch_bit;

static void
encoding_agp_punch_line( FILE *fp );

static void
encoding_agp_end_line( FILE *fp );
static void	encoding_print_end_total( FILE *fp );


void
encoding_agp_print_inf( FILE *fp, char *file, int row, int column  )
{
	File_bits = 8*gp_file_size( file );

	Block_no = (float)(row*column) / (48.0*48.0);


	Tbits = 0;
	fprintf( fp, "\n\n\nCoding -\n\n" );



	Tbits += encoding_center_print_inf( fp );

	Tbits += encoding_link_print_inf( fp  );

	Tbits += encoding_joint_print_inf( fp );


	Tbits += encoding_hunch_print_inf( fp );

	Tbits += encoding_ridge_print_inf( fp );


	Tbits += encoding_n3_print_inf( fp );

	Tbits += encoding_profile_print_inf( fp );

	Tbits += encoding_bg_print_inf( fp );

	Tbits += encoding_layer_print_inf( fp );
	Tbits += encoding_depth_print_inf( fp );


    fprintf( fp, "\n\n\nTotal Bits: %d   Blocks: %.2f    Av: %.4f\n\n\n",
		Tbits, Block_no, Tbits/(double)Block_no );


	encoding_agp_punch_line( fp );
	encoding_agp_end_line( fp );
}



static void
encoding_agp_punch_line( FILE *fp )
{
int     nfno,	lno;

	Punch_bit = 0;

	fprintf (fp, "\n\n		PUNCH LINE :\n\n");
	fprintf( fp, "%10s    No      nf/b    bit/b   bits/nf    link   bits/link  T-bits   %% \n",
	" ");

	encoding_hunch_punch_line( fp );

	encoding_ridge_punch_line( fp );

	encoding_n3_punch_line( fp );

	encoding_profile_punch_line( fp );

	encoding_link_punch_line (fp,&lno);

	encoding_joint_punch_line (fp );

	encoding_center_punch_line( fp, &nfno );

	encoding_bg_punch_line( fp );

	encoding_adepth_punch_line( fp );
	encoding_layer_punch_line( fp );
	encoding_depth_punch_line( fp );


	encoding_print_row( fp, "Overhead-", 0, 0, File_bits - Punch_bit );

	encoding_print_row( fp, "Total-", nfno, lno, File_bits );
}



void
encoding_print_row( FILE *fp, char *s, int nf, int link, int bits)
{

        fprintf (fp,"%-10s   %5d   %6.2f   %7.2f   %6.2f  %5d   %5.2f   %7d  %3d\n",
		s,
		nf,
		nf/Block_no,
		bits/Block_no,
		( nf == 0 )? 0: bits/(double)nf,
		link,
		( link == 0 )? 0: bits/(double)link,
		bits,
		(int)(bits*100.0/File_bits +0.5) );

	Punch_bit += bits;
}





static	int Geo_bits,	Bg_bits,	Val_bits[SP_NO],	End_bits;

static void
encoding_agp_end_line( FILE *fp )
{

	End_bits = 0;

	fprintf (fp, "\n\n		END LINE :\n\n");
	fprintf( fp, "%10s      No      geo       bg        Y       Q        A    T-bits    Av\n",
	" ");

	encoding_hunch_end_line( fp );

	encoding_ridge_end_line (fp );

	encoding_n3_end_line( fp );

	encoding_profile_end_line( fp );

	encoding_link_end_line( fp );

	encoding_joint_end_line( fp );

	encoding_center_end_line( fp );

	encoding_bg_end_line( fp );

	encoding_adepth_end_line( fp );
	encoding_layer_end_line( fp );
	encoding_depth_end_line( fp );

	encoding_print_end_total( fp );


}



void
encoding_print_end_line( FILE *fp, char *s, int nf,
					int geo, int bg, int val[] )
{
int	bits;

	bits = geo+ bg + val[SP_Y] + val[SP_Q] + val[SP_A];

        fprintf( fp, "%-10s   %5d   %6d   %6d   %6d  %6d   %6d   %7d   %.2f\n",
		s,
		nf,
		geo,
		bg,
		val[SP_Y],	val[SP_Q],	val[SP_A],
		bits, (nf == 0 )?0.0 :(float)bits/nf );

	Geo_bits += geo;
	Bg_bits += bg;
	Val_bits[SP_Y] += val[SP_Y];
	Val_bits[SP_Q] += val[SP_Q];
	Val_bits[SP_A] += val[SP_A];
	End_bits += bits;
}


static void
encoding_print_end_total( FILE *fp )
{

        fprintf( fp, "%-10s   %5d   %6d   %6d   %6d  %6d   %6d   %7d\n",
		"Overhead",
		0,
		0,
		0,
		0, 0, 0,
		File_bits - End_bits );

        fprintf( fp, "\n" );


        fprintf( fp, "%-10s   %5d   %6d   %6d   %6d  %6d   %6d   %7d\n",
		"Total",
		0,
		Geo_bits,
		Bg_bits,
		Val_bits[SP_Y],	Val_bits[SP_Q],	Val_bits[SP_A],
		File_bits );

        fprintf( fp, "%-10s   %5d   %6.2f   %6.2f   %6.2f  %6.2f   %6.2f   %7.2f\n",
		"Average",
		0,
		(float)Geo_bits/File_bits,
		(float)Bg_bits/File_bits,
		(float)Val_bits[SP_Y]/File_bits,
		(float)Val_bits[SP_Q]/File_bits,
		(float)Val_bits[SP_A]/File_bits,
		(float)File_bits/File_bits );
}








void
encoding_print_compression_ratio( FILE *fp, char *file, int row,int column )
{
int m,n;
double ratio;

	n = gp_file_size(file);

	m = 3 * row*column;
	ratio = (double)m/n ;
	fprintf (fp,"\n\n COMPRESSION RATIO : %.2f\n",ratio );
}	





void
encoing_print_inf( FILE *fp, char *s, int bits, int no )
{
	fprintf( fp,"%s: %d{%.2f:%d}",
		s,
		bits,
		(no == 0 )? no : bits/(float)no,
		no );
}
