/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
#include	<math.h>
#include        "Uprm/prm.h"

#include        "Uigp/igp.h"
#include 	"Ucode/code.h"



#include	"Coding/CodingJoint.h"
#include	"Coding/CodingLink.h"

#include        "EncJoint.h"
#include        "EncLink.h"
#include        "EncN3.h"
#include        "EncIn.h"



#define	JEND_FNAME	"Jend"




static void	encoding_joint_info( jnt_type *jnt,
				coding_type *cprm, bst_type *bst );
static void	encoding_joint_info_jnt_n3( jnt_type *jnt,
				codingN3_type *nc, bst_type *bst );
static void	encoding_joint_info_jnt_edge( jnt_type *jnt, bst_type *bst );
static void	encoding_joint_info_jnt_RN3( jnt_type *jnt,
				codingN3_type *nc, bst_type *bst );




static void	encoding_joint_branch_agp( gp_array *agp,
				coding_type *cprm, bst_type *bst );


static void	encoding_joint_branchs( jnt_type *jnt,
			gp_array *agp,coding_type *cprm, bst_type *bst);




static void	encoding_joint_branch( gp_array *agp,
				jnt_type *jnt,coding_type *cprm, int bno,
				vec2d *base_u, bst_type *bst);


static  void	encoding_joint_last_link( gp_array *agp,
				int type, vec2d *ctr, link_type *link,
				codingJoint_type *jc, bst_type *bst );

static void	encoding_joint_get_jnt_no( gp_type *gp, int type,
                                jnt_type *jnt, int *jnt_no, int *no );

static void	encoding_joint_branch_center( jnt_type *jnt, int bno );






static int	Tjnt_no,	Jheader_bit,	Tend_no,	Tin_no;

void
encoding_joint_open(  codingJoint_type *jc )
{
	jc->Jend_f.fp = coding_buffer_open_to_write( JEND_FNAME, 4 );

	Jheader_bit = 0;
	Tjnt_no = 0;
	Tend_no = 0;
	Tend_no = 0;
}




void
encoding_joint( gp_array *agp,coding_type *cprm, bst_type *bst)
{
gp_type       *gp;
jnt_type	*jnt;
int     i,      j;

	Jheader_bit -= BST_BITS( bst );


	for(i = 0 ; i < agp->Ni ; i++)
		for(j = 0 ; j < agp->Nj ; j++){
                        gp = &agp->gp[i][j];
                        for( jnt = gp->jnt ; jnt != NULL ; jnt = jnt->nj )
				encoding_joint_info( jnt, cprm, bst );
		}



	encoding_joint_branch_agp( agp,cprm, bst );



	Jheader_bit += BST_BITS( bst );
}


static void
encoding_joint_branch_agp( gp_array *agp,coding_type *cprm, bst_type *bst )
{
gp_type	*gp;
jnt_type	*jnt;
brn_type	*b;
int     i,      j,      bno;

	for( i = 0 ; i < agp->Ni ; i++ )
	    for( j = 0 ; j < agp->Nj ; j++ ){
 
		gp = &agp->gp[i][j];

 
 
		for( jnt = gp->jnt ; jnt != NULL ; jnt = jnt->nj ){
			Tjnt_no++;

                    if( jnt->type == JNT_N3 ){
                        b = jnt->brn[0]; 
                        if( b->type != BRN_N3 && b->type != BRN_N3R )
				continue; 
                        if( b->direction != F_BEGIN )        continue;


                        encoding_joint_branch( agp, jnt, cprm,
					0, &cprm->lc->Base_u, bst );
                        continue; 
                    } 

 
                    if( jnt->type == JNT_RN3 )
                        for( bno = 0 ; bno < jnt->brn_no ; bno++ ){ 
                                b = jnt->brn[bno];
                        	if( b->type != BRN_N3 && b->type != BRN_N3R )
					continue; 
                                if( b->direction != F_BEGIN )        continue;

                                encoding_joint_branch( agp, jnt,cprm,
						bno, &cprm->lc->Base_u, bst);
	                       } 
		}
            }




	for( i = 0 ; i < agp->Ni ; i++ )
	    for( j = 0 ; j < agp->Nj ; j++ ){
 
		gp = &agp->gp[i][j];

		for( jnt = gp->jnt ; jnt != NULL ; jnt = jnt->nj )
		    encoding_joint_branchs( jnt, agp,cprm, bst );
	}
}







static void
encoding_joint_info( jnt_type *jnt, coding_type *cprm, bst_type *bst )
{

	switch( jnt->type ){
	case JNT_N3:
		encoding_joint_info_jnt_n3( jnt, cprm->nc, bst );
	    break;
	case JNT_EDGE:
	case JNT_RIDGE:
		encoding_joint_info_jnt_edge( jnt, bst );
	    break;
	case JNT_RN3:
		encoding_joint_info_jnt_RN3( jnt, cprm->nc, bst );
	    break;
	}
}



static void
encoding_joint_info_jnt_n3( jnt_type *jnt, codingN3_type *nc, bst_type *bst )
{
brn_type	*b;
int	i;
char	w[2];

	b = jnt->brn[0];

	if( b->type == BRN_N3 )
		bst_put_int( bst, 0, 1 );
	else	bst_put_int( bst, 1, 1 );


	if( b->direction == F_BEGIN )
		bst_put_int( bst, 1, 1 );
	else	bst_put_int( bst, 0, 1 );


	for( i = 1 ; i < jnt->brn_no ; i++ ){
		b = jnt->brn[i];
		switch( b->type&0x0f ){
		case BRN_NULL:
			bst_put_int( bst, 0, 2 );
		    break;
		case BRN_IN:
			bst_put_int( bst, 1, 2 );
		    break;
		case BRN_EDGE:
			if( b->direction == F_END )
				bst_put_int( bst, 2, 2 );
			else	bst_put_int( bst, 3, 2 );
		    break;
		}
	}

	w[0] = ZCODE (jnt->lwidth - nc->Nwidth.a, nc->Nwidth);
	w[1] = ZCODE (jnt->rwidth - nc->Nwidth.a, nc->Nwidth);

	coding_buffer_write( nc->Nwidth.fp, (unsigned char *)w, 2 );
}




static void
encoding_joint_info_jnt_edge( jnt_type *jnt, bst_type *bst )
{
int	bno,	i;

	for( i = 0, bno = 0 ; i < jnt->brn_no ; i++ ){
                if( jnt->brn[i]->direction == F_END )	continue;
		bno++;
	}
	bst_put_int( bst, bno, 2 );
}


static void
encoding_joint_info_jnt_RN3( jnt_type *jnt, codingN3_type *nc, bst_type *bst )
{
int	bno,	i;
char	w[2];

	for( i = 0, bno = 0 ; i < jnt->brn_no ; i++ ){
                if( jnt->brn[i]->direction == F_END )	continue;
		bno++;
	}
	bst_put_int( bst, bno, 2 );

	for( i = 0 ; i < jnt->brn_no ; i++ ){
                if( jnt->brn[i]->direction == F_END )	continue;

		if( jnt->brn[i]->type == BRN_N3 )
			bst_put_int( bst, 0, 1 );
		else	bst_put_int( bst, 1, 1 );
	}

	if( jnt->brn_no == 2 && jnt->brn[0]->direction == F_END &&
					jnt->brn[1]->direction == F_BEGIN ){
		w[1] = ZCODE (jnt->lwidth - nc->Nwidth.a, nc->Nwidth);
		w[0] = ZCODE (jnt->rwidth - nc->Nwidth.a, nc->Nwidth);
	}
	else	{
		w[0] = ZCODE (jnt->lwidth - nc->Nwidth.a, nc->Nwidth);
		w[1] = ZCODE (jnt->rwidth - nc->Nwidth.a, nc->Nwidth);
	}

	coding_buffer_write( nc->Nwidth.fp, (unsigned char *)w, 2 );
}




static void
encoding_joint_branchs( jnt_type *jnt, gp_array *agp,coding_type *cprm, bst_type *bst)
{
brn_type	*b;
vec2d	base_u;
int	i;


	if( jnt->type == JNT_RN3 )	return;



	if( jnt->type == JNT_N3 ){

/*
		jnt_n3_uvector( jnt, &base_u );
*/
		base_u = jnt->u;

		for( i = 1 ; i < jnt->brn_no ; i++ ){
			b = jnt->brn[i];
			switch( b->type&0x0f ){
			case BRN_NULL:
		    	break;
			case BRN_IN:
				encoding_branch_in( agp, jnt, i, cprm->jc, bst );
				Tin_no++;
			    break;
			case BRN_EDGE:
				if( b->direction == F_END )	break;

				encoding_joint_branch(agp, jnt,cprm, i, &base_u, bst );
			    break;
			}
		}
		return;
	}


	for( i = 0 ; i < jnt->brn_no ; i++ ){
		b = jnt->brn[i];
               	if( b->direction == F_END )   continue;

		encoding_joint_branch( agp, jnt,cprm, i, &cprm->lc->Base_u, bst);
	}
}


static void
encoding_joint_branch( gp_array *agp, jnt_type *jnt,
		coding_type *cprm, int bno, vec2d *base_u, bst_type *bst)
{
brn_type	*b;
vec2d	ctr;

	b = jnt->brn[bno];

	encoding_joint_branch_center( jnt, bno );



	ctr.x = jnt->gctr.x + b->ctr.x;
	ctr.y = jnt->gctr.y + b->ctr.y;
	encoding_link( &ctr, b->link, base_u, cprm->lc,0);

	encoding_joint_last_link( agp, b->type, &ctr, b->link, cprm->jc, bst);

	if( b->type == BRN_N3 )
		encoding_n3_width( b->link, cprm->nc, bst );
}


static	void
encoding_joint_last_link( gp_array *agp, int type,
			vec2d *ctr, link_type *link,
			 codingJoint_type *jc, bst_type *bst )
{
link_type	*l;
vec2d	v;
jnt_type	*jnt;
int	bno,	jnt_no,	no;
int	i0,	j0,	i,	j,	di,	dj,	dij;
	Tend_no ++;

	for( l = link, v = *ctr ; LINK_NEXT(l) != NULL ; l = LINK_NEXT(l) ){
		v.x += l->v.x;
		v.y += l->v.y;
	}

	jnt = l->p[F_END].p.jnt;
	bno = l->p[F_END].bno;
	encoding_joint_branch_center( jnt, bno );

	i0 = (v.x +agp->bgp) * agp->inv_bgp;
	j0 = (v.y +agp->bgp) * agp->inv_bgp;

	i = (jnt->gctr.x+agp->bgp) * agp->inv_bgp;
	j = (jnt->gctr.y+agp->bgp) * agp->inv_bgp;

	di = i - i0 + 5;
	dj = j - j0 + 5;
	dij = di*11 + dj;
//	coding_buffer_write_next( jc->Jend_f.fp, dij );
	CODING_BUFFER_WRITE_NEXT( jc->Jend_f.fp, dij  );

	encoding_joint_get_jnt_no( &agp->gp[i][j], type, jnt, &jnt_no, &no );
	if( jnt_no > 1 )
		bst_put_int( bst, no, coding_ilog2( jnt_no-1 ) );

	if( jnt->type == JNT_N3 && type == BRN_EDGE )
		bst_put_int( bst, bno-1, 1 );
}


static void
encoding_joint_get_jnt_no( gp_type *gp, int type,
				jnt_type *jnt, int *jnt_no, int *no )
{
static int	Mtype[6] = { 0, 0, 8, 6, 18, 18 };
static int	Jtype[5] = { 0, 2, 4, 8, 16 };

jnt_type	*cjnt;
int	mtype = Mtype[type];

	*jnt_no = 0;
	for( cjnt = gp->jnt ; cjnt != NULL ; cjnt = cjnt->nj ){

		if( !(mtype & Jtype[cjnt->type]) )	continue;

		if( jnt == cjnt )	*no = *jnt_no;

		(*jnt_no) ++;
	}
}



static void
encoding_joint_branch_center( jnt_type *jnt, int bno )
{
brn_type	*b;
	if( jnt->type == JNT_N3 )	return;

	b = jnt->brn[bno];

}

	

void
encoding_joint_close(  codingJoint_type *jc )
{
	fclose( jc->Jend_f.fp );
}


static int	Jend_f_bits;

void
encoding_joint_huffman(  codingJoint_type *jc, FILE *cfd)
{
int     msg,	cotype;
 
        encoding_agp_file_append_huffman( cfd, JEND_FNAME, 4,
						&msg, &Jend_f_bits, &cotype);
}


void
encoding_joint_punch_line(FILE *fp )
{ 
int	tbits;
	tbits = Jend_f_bits + Jheader_bit;
	encoding_print_row( fp, "Joint-", Tjnt_no, 0, tbits );
}


void
encoding_joint_end_line( FILE *fp )
{
int	geo,	bg,	val[SP_NO];

	geo = Jend_f_bits + Jheader_bit;
	bg = 0;

	val[SP_Y] = 0;
	val[SP_Q] = 0;
	val[SP_A] = 0;

        encoding_print_end_line( fp, "Joint-", Tjnt_no, geo, bg, val );
}


int
encoding_joint_print_inf( FILE *fp )
{
int	tbits;
	fprintf( fp, "\n\nJoint- %d  H: %d   End: %d\n",
		Tjnt_no, Jheader_bit, Jend_f_bits );

        encoing_print_inf( fp, "End", Jend_f_bits, Tend_no+Tin_no );
        fprintf( fp, "\n" );

	tbits = Jend_f_bits + Jheader_bit;

	fprintf (fp,"\nTotal: %d{%.2f}\n\n", 
                tbits, 
                ( Tjnt_no == 0.0 )? 0.0 : tbits/(float)Tjnt_no );
	return( tbits );
}

