/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/

#include        "Ucode/code.h"

#include 	"Uigp/igp.h"
#include        "EncLayer.h"
#include        "EncIn.h"

#include	"AgpTool/AgpTool.h"



#include        "Coding/CodingDepth.h"


#define	BR_FNAME	"branch"
#define FBR_FNAME	"Fbranch"

#define		BSIZE		2




int	Lbits,	Lbr_no;

static void	encoding_layer_gp( gp_type *gp,
			u_char br[], int *no, u_char fbr[], int *fno );



void
encoding_layer_open(codingDepth_type *dc)
{
	dc->Br_f.fp = coding_buffer_open_to_write( BR_FNAME, 4 );
	dc->Fbr_f.fp = coding_buffer_open_to_write( FBR_FNAME, 4 );

	Lbr_no = 0;
	Lbits = 0;
}




int
encoding_layer( gp_array *agp,codingDepth_type *dc, bst_type *bst )
{
int	i,	j,	n,	k;
u_char	br[BUF_SIZE],	fbr[BUF_SIZE];
int	no,	fno;
int	Mnf,	Mbg,	Fbr;
int	Abr[32],	Lno;
int	mnf;


	Lbits -= BST_BITS( bst );


	agp_br_mask( agp, &Mnf, &Mbg, &Fbr );


	for( i = 0, Lno = 0, mnf = Mnf ; mnf != 0 ; i++, mnf >>= 1 )
		if( mnf & 0x01 ){
			Abr[Lno++] = i;
		}

	if( Lno == 1 ){
		bst_put_int( bst, 0, 1);
		bst_put_int( bst, Abr[0], 8);
	}
	else	bst_put_int( bst, 1, 1);


		

	if( Fbr == 0 ){
		bst_put_int( bst, 0, 1);
	}
	else	bst_put_int( bst, 1, 1);


	if( Lno == 1 && Fbr == 0 ){
		Lbits += BST_BITS( bst );
		return( Mnf | Mbg );
	}




	for( i = 0 ; i < agp->Ni ; i += BSIZE )
	    for( j = 0 ; j < agp->Nj ; j += BSIZE ){


		no = fno = 0;

		for( k = i ; k < agp->Ni && k < i+BSIZE ; k++ )
		    for( n = j ; n < agp->Nj && n < j+BSIZE ; n++ )
			encoding_layer_gp( &agp->gp[k][n],
					br, &no, fbr, &fno );

		if( no == 0 )	continue;


		if( Lno > 1 ){
			for( n = 1 ; n < no ; n ++ )
				if( br[n] != br[0] )	break;
	
			if( n < no ){
				bst_put_int( bst, 1, 1);
       			coding_buffer_write( dc->Br_f.fp, (u_char *)br, no);
			}
			else	{
				bst_put_int( bst, 0, 1);
       			coding_buffer_write( dc->Br_f.fp, (u_char *)br, 1);
			}
		}



		for( n = 0 ; n < fno ; n++ )
			if( fbr[n] != 0 )	break;
	
		if( n < fno ){
			bst_put_int( bst, 1, 1);
			coding_buffer_write( dc->Fbr_f.fp, (u_char *)fbr, fno );
		}
		else	bst_put_int( bst, 0, 1);


		Lbr_no += no;

	    }


	Lbits += BST_BITS( bst );

	return( Mnf | Mbg );
}

static void
encoding_layer_gp( gp_type *gp, u_char br[], int *no, u_char fbr[], int *fno )
{
hunch_type	*h;
jnt_type	*jnt;
brn_type	*b;
int	i;


	for( h = gp->hunch ; h != NULL ; h = h->nh ){
		br[(*no)++] = h->br;
	}


	for( jnt = gp->jnt ; jnt != NULL ; jnt = jnt->nj ){
	    for( i = 0 ; i < jnt->brn_no ; i++ ){
		b = jnt->brn[i];
		if( b->direction != F_BEGIN )	continue;

		switch( b->type ){
		case NF_RIDGE:
		case NF_N3R:
			br[(*no)++] = b->link->br;
		    break;

		case NF_EDGE:
			br[(*no)++] = b->link->br;
			if( b->link->Fbr == 0x04 )
				br[(*no)++] = b->link->br1;
			fbr[(*fno)++] = b->link->Fbr;
		    break;

		case NF_N3:
			br[(*no)++] = b->link->br;
			fbr[(*fno)++] = b->link->Fbr;
		    break;
		}
	    }
	}


}





void
encoding_layer_close(codingDepth_type *dc)
{
	fclose( dc->Br_f.fp );
	fclose( dc->Fbr_f.fp );
}

static int Fbr_f_bits,Br_f_bits;

void
encoding_layer_huffman( codingDepth_type *dc,FILE *cfd )
{
int	msg,	cotype;

	encoding_agp_file_append_huffman( cfd, BR_FNAME, 4,
						&msg, &Br_f_bits, &cotype );
	encoding_agp_file_append_huffman( cfd, FBR_FNAME, 4,
						&msg, &Fbr_f_bits, &cotype );

}


void
encoding_layer_punch_line( FILE *fp )
{ 
int	total_bits; 

	total_bits = Br_f_bits + Fbr_f_bits + Lbits;

	encoding_print_row( fp, "Layer-", Lbr_no, Lbr_no, total_bits );
}



void
encoding_layer_end_line( FILE *fp )
{
int	geo,	bg,	val[SP_NO];

	geo = 0;
	bg = 0;

	val[SP_Y] = Br_f_bits + Fbr_f_bits + Lbits;
	val[SP_Q] = 0;
	val[SP_A] = 0;

        encoding_print_end_line( fp, "Layer-", Lbr_no, geo, bg, val );
}





int
encoding_layer_print_inf( FILE *fp )
{
int	tbits;
 
	tbits = 0;

	fprintf( fp, "\n\n\n" );

	if( Lbr_no == 0 ){
		fprintf( fp, "Layer: 0" );
		return( tbits );
	}

	fprintf( fp, "Layer: %d\n", Lbr_no );
	encoing_print_inf( fp, "Branch", Br_f_bits, Lbr_no );
	encoing_print_inf( fp, "Fbranch", Fbr_f_bits, Lbr_no );
	fprintf( fp, "\n" );

	tbits += Br_f_bits + Fbr_f_bits + Lbits;


	encoing_print_inf( fp, "\nTotal -   ", tbits, Lbr_no );

	return( tbits );
}
