/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
#include        <stdio.h>
#include        <math.h>
 
#include        "Uigp/igp.h"

#include	"EncIn.h"

 
#include	"Coding/CodingLink.h"

#include	"EncLink.h"


#define	LA_FNAME	"La"
#define	LLEN_FNAME	"Llen"
#define	LWIDTH_FNAME	"Lwidth"
#define	LINO_FNAME	"Lino"



/* declare in   Coding/CodingLink.h
 * code_type	Llen,	Lwidth,	La;
 * vec2d	Base_u;
*/

int	Nflinks=0,	Lno = 0;



void
encoding_link_open(codingLink_type *lc)
{
	lc->Lino_f.fp  = coding_buffer_open_to_write( LINO_FNAME, 4 );

	lc->Llen.fp  = coding_buffer_open_to_write( LLEN_FNAME, 4 );
	lc->Lwidth.fp  = coding_buffer_open_to_write( LWIDTH_FNAME, 4 );


	lc->La.fp = coding_buffer_open_to_write( LA_FNAME, 4 );


	Nflinks = Lno = 0;
}



void
encoding_link( vec2d *ctr, link_type *link, vec2d *base_u, codingLink_type *lc,int flag )
{
link_type	*l;
vec2d	*u,	v,	lv,	p;
signed char	w[BUF_SIZE],	len[BUF_SIZE],	a[BUF_SIZE];
u_char	tmp;
int	ino;

code_type       ucode;

	u = base_u;

	/* encoding first link */
	p.x = QCODE( ctr->x, lc->Llen );
	p.y = QCODE( ctr->y, lc->Llen );

	lv.x = ctr->x + link->v.x - p.x;
	lv.y = ctr->y + link->v.y - p.y;

       	v.x = -u->y * lv.x + u->x * lv.y;
       	v.y = u->x * lv.x + u->y * lv.y;

	ucode.d = lc->Llen.d * MAX( ABS(u->x), ABS(u->y) ) - 0.0001;
	ucode.inv_d = 1.0/ucode.d;
	len[0] = ZCODE( v.x, ucode );
	w[0] = ZCODE( v.y, ucode );

	a[0] = ZCODE( link->a, lc->La );

	u = &link->u;



	for( l = LINK_NEXT(link), ino = 1 ; l != NULL ;
						l = LINK_NEXT(l), ino++ ){
        	v.x = -u->y * l->v.x + u->x * l->v.y;
        	v.y = u->x * l->v.x + u->y * l->v.y;

		ucode.d = lc->Llen.d * MAX( ABS(u->x), ABS(u->y) ) - 0.0001;
		ucode.inv_d = 1.0/ucode.d;
		len[ino] = ZCODE( v.x, ucode );
		w[ino] = ZCODE( v.y, ucode );

		a[ino] = ZCODE( l->a, lc->La );

		u = &l->u;
	}


	coding_buffer_write( lc->Llen.fp, (u_char *)len, (flag==1)?ino: ino-1 );
	coding_buffer_write( lc->Lwidth.fp, (u_char *)w, (flag==1)?ino: ino-1 );
	coding_buffer_write( lc->La.fp, (u_char *)a, ino );


	tmp = ino-1;
	coding_buffer_write( lc->Lino_f.fp, &tmp, 1 );


	Nflinks++;
	Lno += ino;
}




void
encoding_link_close(codingLink_type *hc )
{
	fclose( hc->Lino_f.fp );
	fclose( hc->Llen.fp );
	fclose( hc->Lwidth.fp );
	fclose( hc->La.fp );
}






static int Lino_bits;

static int Llen_bits,Lwidth_bits,La_bits;



void
encoding_link_huffman(codingLink_type * lc,FILE *cfd)
{
int	msg,	cotype;

	encoding_agp_file_append_huffman( cfd, LINO_FNAME, 4,
						&msg, &Lino_bits, &cotype );
 
	encoding_agp_file_append_huffman( cfd, LLEN_FNAME, 4,
						&msg, &Llen_bits, &cotype );

	encoding_agp_file_append_huffman( cfd, LWIDTH_FNAME, 4,
						&msg, &Lwidth_bits, &cotype );
	
	encoding_agp_file_append_huffman( cfd, LA_FNAME, 4,
						&msg, &La_bits, &cotype );
}





void
encoding_link_punch_line( FILE *fp,int *num )
{
int	tbits;

	tbits = (Lino_bits + Llen_bits + Lwidth_bits + La_bits);
	*num = Nflinks;
 
	encoding_print_row( fp, "Link-", Nflinks, Lno, tbits );
}


void
encoding_link_end_line( FILE *fp )
{
int	geo,	bg,	val[SP_NO];

	geo = Lino_bits + Llen_bits + Lwidth_bits + La_bits;
	bg = 0;

	val[SP_Y] = 0;
	val[SP_Q] = 0;
	val[SP_A] = 0;

        encoding_print_end_line( fp, "Link-", Lno, geo, bg, val );
}


int
encoding_link_print_inf( FILE *fp )
{
int	bits,	tbits;

	tbits = 0;

	fprintf( fp, "\n\n" );

	if( Lno == 0 ){
		fprintf( fp, "Link : %d     Polyline: %d\n", Lno, Nflinks );
		return(0);
	}


	fprintf( fp, "Link : %d     Polyline: %d     Ino: %d{%.4f} \n",
				Lno, Nflinks,
				Lino_bits, (float)Lino_bits/Nflinks ); 
	tbits  += Lino_bits;

 

	bits = Llen_bits + Lwidth_bits + La_bits;
	fprintf (fp, "Link:  L:%d{%.4f}   W:%d{%.4f}   A:%d{%.4f}   T:%d{%.4f- %d}\n",
			Llen_bits, (float)Llen_bits/Lno,
			Lwidth_bits, (float)Lwidth_bits/Lno,
			La_bits, (float)La_bits/Lno,
			bits, (float)bits/Lno, Lno  );
			
	tbits  += bits;

    


	fprintf (fp,"\nTotal: %d{%.2f}\n\n",
                tbits,
                ( Lno == 0.0 )? 0.0 : tbits/(float)Lno );

	return( tbits );
}
