/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/

#include        "Uprm/prm.h"

#include 	"Ucode/code.h"

#include 	"Uigp/igp.h"

#include	"Coding/CodingN3.h"

#include        "EncN3.h"
#include        "EncIn.h"


#define		NWIDTH_FNAME	"n3_width"



static	int	Nno,	Nlinks,	Nbits;









void
encoding_n3_open(codingN3_type *nc)
{
	nc->Nwidth.fp = coding_buffer_open_to_write( NWIDTH_FNAME, 4 );

	Nno =  Nlinks = 0;
}


void
encoding_n3_width( link_type *link,codingN3_type *nc, bst_type *bst )
{
signed char	w[256];
link_type	*l;
width_type	*width;
int	n;
float	tmp;

	Nbits -= BST_BITS( bst );

	Nno += 1;

	n = 0;

	for( l = LINK_NEXT(link) ; l != NULL ; l = LINK_NEXT(l) ){

		width = l->width[F_BEGIN];

		tmp = width->w[F_RIGHT]- nc->Nwidth.a;
		w[n++] = ZCODE( tmp, nc->Nwidth );


		tmp = -width->w[F_LEFT]- nc->Nwidth.a;
		w[n++] = ZCODE( tmp, nc->Nwidth );
	}


	if( n > 0 )
        	coding_buffer_write( nc->Nwidth.fp, (u_char *)w, n );

	Nlinks += n;

	Nbits += BST_BITS( bst );
}




void
encoding_n3_close(codingN3_type *nc)
{
	fclose( nc->Nwidth.fp );
}


static int Nwidth_bits;

void
encoding_n3_huffman( codingN3_type *nc,FILE *cfd )
{
int	msg,	tmp;

	encoding_agp_file_append_huffman( cfd, NWIDTH_FNAME, 4,
						&msg, &Nwidth_bits, &tmp );
}


void
encoding_n3_punch_line( FILE *fp )
{ 
int	total_bits; 

	total_bits = Nwidth_bits + Nbits;

	encoding_print_row( fp, "N3-", Nno, Nlinks, total_bits );
}



void
encoding_n3_end_line( FILE *fp )
{
int	geo,	bg,	val[SP_NO];

	geo = 0;
	bg = 0;

	val[SP_Y] = Nwidth_bits + Nbits;
	val[SP_Q] = 0;
	val[SP_A] = 0;

        encoding_print_end_line( fp, "N3-", Nlinks, geo, bg, val );
}





int
encoding_n3_print_inf( FILE *fp )
{
int	tbits;
 
	tbits = 0;

	fprintf( fp, "\n\n\n" );

	if( Nno == 0 ){
		fprintf( fp, "N3: %d   Link: %d\n", Nno, Nlinks );
		return( tbits );
	}

	fprintf( fp, "N3: %d\n", Nno );
	encoing_print_inf( fp, "Width", Nwidth_bits, Nlinks );
	fprintf( fp, "\n" );

	tbits += Nwidth_bits;


	encoing_print_inf( fp, "\nTotal -   ", tbits, Nlinks );

	return( tbits );
}
