/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/

#include        "EncIn.h"

#include        "GpType/GpType.h"
#include        "EncProfile.h"

#include	"Coding/CodingProfile.h"

#define		BUF_SIZE	(16*MAX_GP*MAX_GP)



#define PVAL_FNAME	"profile"
#define PNULL_FNAME	"profile_null"
#define EWIDTH_FNAME	"edge_w"


static int	Eno,	Epr,	Epr_null,	Ebits;
static int	N3no,	N3pr;



static void	encoding_profile_n3( gp_array *agp,codingProfile_type *pc,
			vec2d *lctr, link_type *link, bst_type *bst );

static void	encoding_profile_n3_side( gp_array *agp,codingProfile_type *pc,
			vec2d *lctr, link_type *link, int side,
			s_char dval[], int *vno, int *zno );

static void	encoding_profile_edge( gp_array *agp,codingProfile_type *pc,
		vec2d *gctr, link_type *link, bst_type *bst );



void
encoding_profile_open(codingProfile_type *pc)
{
	pc->Ewidth.fp = coding_buffer_open_to_write( EWIDTH_FNAME, 4 );
	pc->Pval.fp = coding_buffer_open_to_write( PVAL_FNAME, 4 );
	pc->Pnull.fp = coding_buffer_open_to_write( PNULL_FNAME, 4 );

	Eno = Epr = Epr_null = Ebits = 0;
	N3no = N3pr = 0;
}


void
encoding_profile( gp_array *agp,codingProfile_type *pc, bst_type *bst )
{
int	i,	j,	bno;
gp_type	*gp;
jnt_type	*jnt;
brn_type	*b;
vec2d	ctr;

	Ebits -= BST_BITS( bst );

	for( i = 0 ; i < agp->Ni ; i++ )
	    for( j = 0 ; j < agp->Nj ; j++ ){

		gp = &agp->gp[i][j];

		for( jnt = gp->jnt ; jnt != NULL ; jnt = jnt->nj ){

			for( bno = 0 ; bno < jnt->brn_no ; bno++ ){
				b = jnt->brn[bno];	

				if( b->direction != F_BEGIN )	continue;
				ctr.x = jnt->gctr.x + b->ctr.x;
				ctr.y = jnt->gctr.y + b->ctr.y;

				if( b->type == BRN_EDGE ){
					encoding_profile_edge( agp,pc,
							&ctr, b->link, bst );
					continue;
				}

				if( b->type == BRN_N3 ){
					encoding_profile_n3( agp,pc,
							&ctr, b->link, bst );
					continue;
				}
			}
		}
	    }

	Ebits += BST_BITS( bst );
}


static void
encoding_profile_edge( gp_array *agp,codingProfile_type *pc,
		vec2d *gctr, link_type *link, bst_type *bst )
{
profile_type	pr[400];
int	gi[300],	gno,	pr_no,	Bflag;

edge_type	*e;
float	width,	tmp;
s_char	w[BUF_SIZE],	dval[BUF_SIZE];

qcolor_type	Rval[300],	Lval[300];
qcolor_type	Rbval[300],	Lbval[300];

int	vno,	no;
int	i,	j;

	Eno ++;

	coding_profile_group_edge( agp, gctr, link,
                		pr, &pr_no, gi, &gno, &Bflag );

	if( gno == 0 )	return;

	for( i = 0 ; i < gno ; i++ ){

		width = 0;
		Rval[i] = Rbval[i] = 0;
		Lval[i] = Lbval[i] = 0;
		no = 0;

		for( j = gi[i] ; j < gi[i+1] ; j++ ){
			e = pr[j].edge;

			width += e->width;
			Rval[i] += e->side[F_RIGHT].val[SP_Y];
			Rbval[i] += e->side[F_RIGHT].bval[SP_Y];

			Lval[i] += e->side[F_LEFT].val[SP_Y];
			Lbval[i] += e->side[F_LEFT].bval[SP_Y];

			no++;
		}

		width /= no;
		tmp = width - pc->Ewidth.a;
		w[i] = ZCODE( tmp, pc->Ewidth );

		Rval[i] /= no;
		Rbval[i] /= no;

		Lval[i] /= no;
		Lbval[i] /= no;
	}



	coding_buffer_write( pc->Ewidth.fp, (u_char *)w, gno );
	Epr += gno;


	if( Bflag != 0 && gno == 1 ){
		tmp = -(Rval[0] - Rbval[0]);
		dval[0] = ZCODE( tmp, pc->Pnull );
		tmp = Lval[0] - Lbval[0];
		dval[1] = ZCODE( tmp, pc->Pnull );
		coding_buffer_write( pc->Pnull.fp, (u_char *)dval, 2 );
		Epr_null += 2;
		return;
	}

	if( Bflag == 0x03 && gno == 2 ){
		tmp = -(Rval[0] - Rbval[0]);
		dval[0] = ZCODE( tmp, pc->Pnull );

		tmp = Lval[0] - Lbval[0];
		dval[1] = ZCODE( tmp, pc->Pnull );

		tmp = -(Rval[1] - Rbval[1]);
		dval[2] = ZCODE( tmp, pc->Pnull );

		tmp = Lval[1] - Lbval[1];
		dval[3] = ZCODE( tmp, pc->Pnull );
		coding_buffer_write( pc->Pnull.fp, (u_char *)dval, 4 );

		Epr_null += 4;
		return;
	}



	if( Bflag & 0x01 ){
		Rbval[0] = Rbval[1];
		Lbval[0] = Lbval[1];
	}


	if( Bflag & 0x02 ){
		Rbval[gno-1] = Rbval[gno-2];
		Lbval[gno-1] = Lbval[gno-2];
	}



	for( i = 0, no = 0, vno=0 ; i < gno ; i++ ){
		tmp = -(Rval[i] - Rbval[i]);
		dval[no++] = ZCODE( tmp, pc->Pval );
		if( ABS(tmp) > pc->Pdmax )	vno++;

		tmp = Lval[i] - Lbval[i];
		dval[no++] = ZCODE( tmp, pc->Pval );
		if( ABS(tmp) > pc->Pdmax )	vno++;
	}


        if( vno == 0 ){
                bst_put_int( bst, 0, 1 );
                return;
        }


        bst_put_int( bst, 1, 1 );
	coding_buffer_write( pc->Pval.fp, (u_char *)dval, no );
}




static void
encoding_profile_n3( gp_array *agp,codingProfile_type *pc,
			vec2d *lctr, link_type *link, bst_type *bst )
{
s_char	dval[BUF_SIZE];
int	vno,	zno;

	N3no++;

	vno = zno = 0;


	encoding_profile_n3_side( agp,pc, lctr, link, F_RIGHT, dval, &vno, &zno );

	encoding_profile_n3_side( agp,pc, lctr, link, F_LEFT, dval, &vno, &zno );


	if( vno == 0 )	return;

	N3pr += 0.5*vno;

        if( zno == 0 ){
                bst_put_int( bst, 0, 1 );
                return;
        }

        bst_put_int( bst, 1, 1 );



	coding_buffer_write( pc->Pval.fp, (u_char *)dval, vno );
}






static void	encoding_profile_n3_side( gp_array *agp,codingProfile_type *pc,
			vec2d *lctr, link_type *link, int side,
			s_char dval[], int *vno, int *zno )
{
link_type	*l,	*pl;
in_type	*in;
n3_type	*n3;
qcolor_type	val;
float	tmp;
int	no;
int	i0,	j0,	i1,	j1;
vec2d	ctr;
jnt_type	*jnt;
int	flag;

	i0 = j0 = -100;

	no = 0;
	val = 0;

	flag = F_BEGIN;
	jnt = link->p[0].p.jnt;
	if( jnt->type == JNT_N3 && jnt->brn[2-side]->type == BRN_IN && 
				jnt->brn[2-side]->link->type == BRN_N3 )
		flag = F_END;

	for( l = link, ctr = *lctr ; l != NULL ; l = LINK_NEXT(l ) ){


		if( flag == F_BEGIN ){
			i1 = (ctr.x+agp->bgp) * agp->inv_bgp;
			j1 = (ctr.y+agp->bgp) * agp->inv_bgp;

			if( i0 != i1 || j0 != j1 ){
				if( no > 0 ){
					tmp = val / no;
					dval[(*vno)++] = ZCODE( tmp, pc->Pval );
					if( ABS(tmp) > pc->Pdmax )
						(*zno)++;

					val = 0;
					no = 0;
				}
				i0 = i1;
				j0 = j1;
			}

			n3 = l->pr[0].n3;
			val += n3->side[side].val[SP_Y] -
					n3->side[side].bval[SP_Y]
				;

			no++;
		}

		if( l->in[side] != NULL ){
			for( in = l->in[side] ; in->p != NULL ; in = in->p );

			if( in->jnt->brn[in->bno]->link->type == BRN_N3 ){
				flag = in->direct;

				i0 = j0 = -100;
			}
		}


		ctr.x += l->v.x;
		ctr.y += l->v.y;



		pl = l;
	}



	if( flag == F_BEGIN ){
		i1 = (ctr.x+agp->bgp) * agp->inv_bgp;
		j1 = (ctr.y+agp->bgp) * agp->inv_bgp;

		if( i0 != i1 || j0 != j1 ){
			if( no > 0 ){
				tmp = val / no;
				dval[(*vno)++] = ZCODE( tmp, pc->Pval );
				if( ABS(tmp) > pc->Pdmax )	(*zno)++;

				val = 0;
				no = 0;
			}
			i0 = i1;
			j0 = j1;
		}

		n3 = pl->pr[1].n3;
		val += n3->side[side].val[SP_Y] - n3->side[side].bval[SP_Y];

		no++;
	}


	if( no > 0 ){
		tmp = val / no;
		dval[(*vno)++] = ZCODE( tmp, pc->Pval );
		if( ABS(tmp) > pc->Pdmax )	(*zno)++;
	}
}








void
encoding_profile_close(codingProfile_type *pc)
{
	fclose( pc->Ewidth.fp );
	fclose( pc->Pval.fp );
	fclose( pc->Pnull.fp );
}

static int Ewidth_bits,Pval_bits,Pnull_bits;

void
encoding_profile_huffman(codingProfile_type *pc, FILE *cfd )
{
int	msg,	cotype;

	encoding_agp_file_append_huffman( cfd, EWIDTH_FNAME, 4,
						&msg, &Ewidth_bits, &cotype );
	encoding_agp_file_append_huffman( cfd, PVAL_FNAME, 4,
						&msg, &Pval_bits, &cotype );

	encoding_agp_file_append_huffman( cfd, PNULL_FNAME, 4,
						&msg, &Pnull_bits, &cotype );
}


void
encoding_profile_punch_line( FILE *fp )
{
int	total_bits;

	total_bits = Ewidth_bits + Pval_bits + Pnull_bits + Ebits;

	encoding_print_row( fp, "Profile-",
		Eno+N3no+Epr_null, Epr+N3pr, total_bits );
}


void
encoding_profile_end_line( FILE *fp )
{
int	geo,	bg,	val[SP_NO];

	geo = 0;
	bg = 0;

	val[SP_Y] = Ewidth_bits + Pval_bits + Pnull_bits + Ebits;
	val[SP_Q] = 0;
	val[SP_A] = 0;

        encoding_print_end_line( fp, "Profile-",
				Epr+N3pr, geo, bg, val );
}
         

	



int
encoding_profile_print_inf( FILE *fp )
{
int	tbits;
 
	tbits = 0;

	fprintf( fp, "\n\n\n" );
	fprintf( fp, "Profile:   Edge- %d  N3- %d\n", Eno, N3no );

	encoing_print_inf( fp, "Width", Ewidth_bits, Epr );
	fprintf( fp, "\n" );

	encoing_print_inf( fp, "Rval", Pval_bits, Epr+N3pr );
	fprintf( fp, "\n" );

	encoing_print_inf( fp, "Null val", Pnull_bits, Epr_null );
	fprintf( fp, "\n" );

	tbits = Ewidth_bits + Pnull_bits + Pval_bits + Ebits;
	encoing_print_inf( fp, "Total", tbits, Epr+N3pr );
	fprintf( fp, "\n" );

	return( tbits );
}


void
encoding_profile_total_bits( int *fno, int *hbits )
{
	*hbits = Pval_bits + Pnull_bits + Ebits;
	*fno = Epr+N3pr;
}