/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/

#include        "Uprm/prm.h"

#include        "Uigp/igp.h"
#include 	"Ucode/code.h"

#include        "EncRidge.h"
#include        "EncIn.h"


#include        "Coding/CodingRidge.h"


#define		BUF_SIZE	(16*MAX_GP*MAX_GP)


#define	RVAL_FNAME	"ridge_val"
#define	RLM_FNAME	"ridge_lm"

static int Ridge_no,	Rpr_no,	Ridge_bits;


static int	encoding_ridge_test_valQA( link_type *link[],
				codingRidge_type *rc ,int ridge_no );



static int	encoding_ridge_link( link_type *link, int Fqa,
				codingRidge_type *rc, bst_type *bst );

static int	encoding_ridge_sgn( link_type *l, int sp );

static void	encoding_ridge_val(  ridge_type *r, int Fqa, int sign,
				codingRidge_type *rc, bst_type *bst );



void
encoding_ridge_open(codingRidge_type *rc)
{

	rc->Rlm.fp = coding_buffer_open_to_write( RLM_FNAME, 4 );

	rc->Rval[SP_Y].fp = coding_buffer_open_to_write( RVAL_FNAME, SP_Y );
	rc->Rval[SP_Q].fp = coding_buffer_open_to_write( RVAL_FNAME, SP_Q );

	Ridge_bits = 0;
	Ridge_no = Rpr_no = 0;
}


void
encoding_ridge( link_type *link[], int ridge_no,
				codingRidge_type *rc, bst_type *bst)
{
int	Fqa;
int	i;

	if( ridge_no == 0 )	return;


	Ridge_bits -= BST_BITS( bst );

	Ridge_no +=  ridge_no;


	Fqa = encoding_ridge_test_valQA( link,rc, ridge_no );
	bst_put_int( bst, Fqa, 1 );

	for( i = 0 ; i < ridge_no ; i++ )
		encoding_ridge_link( link[i], Fqa, rc, bst );



	Ridge_bits += BST_BITS( bst );
}


void
encoding_ridge_close(codingRidge_type *rc)
{
	fclose( rc->Rlm.fp );

	fclose( rc->Rval[SP_Y].fp);
	fclose( rc->Rval[SP_Q].fp);
}




static int Rlm_bits,RvalSP_Y_bits,RvalSP_Q_bits;




void
encoding_ridge_huffman( codingRidge_type *rc,FILE *cfd )
{
int	msg,	cotype;

	encoding_agp_file_append_huffman( cfd, RLM_FNAME, 4,
					&msg, &Rlm_bits, &cotype );


	encoding_agp_file_append_huffman( cfd, RVAL_FNAME, SP_Y,
					&msg, &RvalSP_Y_bits ,&cotype);

	encoding_agp_file_append_huffman( cfd, RVAL_FNAME, SP_Q,
					&msg, &RvalSP_Q_bits ,&cotype);
}






static int
encoding_ridge_link( link_type *link, int Fqa,
				codingRidge_type *rc, bst_type *bst )
{
link_type	*l,	*nl;
ridge_type	*r;
int	sign;

	sign = encoding_ridge_sgn( link, SP_Y );
	bst_put_int( bst, sign, 1 );


	for( l = link ; l != NULL ; l = LINK_NEXT(l) ){
		r = l->pr[0].ridge;

		encoding_ridge_val( r, Fqa, sign ,rc, bst );


		nl = l;
	}


	r = nl->pr[1].ridge;
	encoding_ridge_val( r, Fqa, sign ,rc, bst );


	return( 1 );
}


static void
encoding_ridge_val( ridge_type *r, int Fqa, int sign,
				codingRidge_type *rc, bst_type *bst )
{
float	tmp;
int	cno;


	Rpr_no++;


	tmp = ( sign == 1 )? -r->lm[SP_Y] : r->lm[SP_Y];
	cno = CODE( tmp, rc->Rlm );
	if( cno > rc->Rlm.imax )	cno = rc->Rlm.imax;
	CODING_BUFFER_WRITE_NEXT( rc->Rlm.fp, cno );


	tmp = ( sign == 1 )? r->rval[SP_Y] : -r->rval[SP_Y];
	cno = ZCODE( tmp, rc->Rval[SP_Y] );
	if( cno < -128 )	cno = -128;
	else	if( cno > 127 )	cno = 127;
	CODING_BUFFER_WRITE_NEXT( rc->Rval[SP_Y].fp, cno );

	if( Fqa == 0 ){
		if( ABS(r->rval[SP_Q]) > rc->Rval_t ||
				ABS(r->rval[SP_A]) > rc->Rval_t ){

			bst_put_int( bst, 0, 1 );

			cno = ZCODE( r->rval[SP_Q], rc->Rval[SP_Q] );
			CODING_BUFFER_WRITE_NEXT( rc->Rval[SP_Q].fp, cno );

			cno = ZCODE( r->rval[SP_A], rc->Rval[SP_A] );
			CODING_BUFFER_WRITE_NEXT( rc->Rval[SP_Q].fp, cno );
		}
		else    bst_put_int( bst, 1, 1 );
	}
}






static int
encoding_ridge_test_valQA( link_type *link[],codingRidge_type *rc ,int ridge_no )
{
link_type	*l,	*nl;
ridge_type	*r;
int	i;


	for( i = 0 ; i < ridge_no ; i++ ){

		for( l = link[i] ; l != NULL ; l = LINK_NEXT(l) ){
			r = l->pr[0].ridge;

			if( ABS(r->rval[SP_Q]) > rc->Rval_t ||
				ABS(r->rval[SP_A]) > rc->Rval_t )
					return( 0 );
			
			nl = l;
		}

		r = nl->pr[1].ridge;

		if( ABS(r->rval[SP_Q]) > rc->Rval_t ||
			ABS(r->rval[SP_A]) > rc->Rval_t )
				return( 0 );
	}

	return( 1 );
}





static int
encoding_ridge_sgn( link_type *link, int sp )
{
int     i,	m,	p,	pr_no;
link_type	*l;
ridge_type *r;


	if (sp == SP_Y) {
		r = link->pr[0].ridge;
		if( r->lm[sp] < 0)
			return (1);
		return (0);
	}

	m = p = 0;
	for( l = link ; l != NULL ; l = LINK_NEXT(l) ){
		pr_no = ( LINK_NEXT(l) != NULL )? 1 : 2;
		for( i = 0 ; i < pr_no ; i++ ){
			r = l->pr[i].ridge;
			if( r->rval[sp] < 0 )
				m++;
			else
				p++;
		}
	}

	if (m < p)
		return (1);
	return (0);
}



void
encoding_ridge_punch_line( FILE *fp )
{ 
int	tbits;
 
	tbits = Rlm_bits;
	tbits  += RvalSP_Y_bits + RvalSP_Q_bits;

	tbits += Ridge_bits;

	encoding_print_row( fp, "Ridge-", Ridge_no, Rpr_no, tbits );
}


void
encoding_ridge_end_line( FILE *fp )
{
int	geo,	bg,	val[SP_NO];
int	bit,	bit3;

	geo = 0;
	bg = 0;

	bit = Ridge_bits;
	bit3 = bit/3;

	val[SP_Y] = Rlm_bits + RvalSP_Y_bits + bit - 2*bit3;
	val[SP_Q] = (RvalSP_Q_bits + bit3)/2;
	val[SP_A] = (RvalSP_Q_bits + bit3)/2;
	

        encoding_print_end_line( fp, "Ridge-", Rpr_no, geo, bg, val );
}






int
encoding_ridge_print_inf( FILE *fp )
{
int	tbits;

	fprintf( fp, "\n\n\nRidge   - %d    Profile: %d\n", Ridge_no, Rpr_no );

	if( Ridge_no == 0 )	return( 0 );


	tbits = 0;


	encoing_print_inf( fp, "Lm", Rlm_bits, Rpr_no );
	fprintf( fp, "\n");
	tbits += Rlm_bits;


	encoing_print_inf( fp, "Rval-Y", RvalSP_Y_bits, Rpr_no );
	fprintf( fp, "\n" );
	tbits += RvalSP_Y_bits;

	encoing_print_inf( fp, "Rval-QA", RvalSP_Q_bits, Rpr_no );
	fprintf( fp, "\n" );
	tbits += RvalSP_Q_bits;



	tbits += Ridge_bits;
	encoing_print_inf( fp, "\nTotal", tbits, Rpr_no );

	return( tbits );
}

