/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
#include	<stdio.h>
#include	<stdlib.h>

#include	"Uigp/igp.h"

#include	"Ubst/BstType.h"
#include	"Ucst/CstType.h"

#include	"Bits.h"

#define	ByteLength	8
static int get_bit_mask(FILE * input, unsigned char *mask);
static int get_coded_char( u_char original, u_char *cod, u_char bmask, u_char flag);
static void bit_encoding(FILE * input, unsigned char mask, bst_type * bst);
static void bit_encoding_cst(cst_type * input, unsigned char mask, bst_type * bst);
static int get_bit_mask_cst(cst_type *input, unsigned char *mask);


int
bitstream_encoding(FILE * input, char *output_file)
{
FILE	*fp;
bst_type *bst;
int     i,	n,	j;
u_char mask;


	if( (fp = fopen( output_file, "wb" )) == NULL )
		return -1;

	n = get_bit_mask(input, &mask);

	putc( mask, fp );

	if (mask == 0) {
		gp_fwrite_num( fp, n, 4 );
		

		fclose( fp );
		return 0;
	}

	for (i = 0, j = 0; i < ByteLength; i++)
		if ((mask & (1 << i)) != 0)
			j++;


	bst = bst_alloc( j * n +1);
	fseek( input, 0, SEEK_SET );
	bit_encoding(input, mask, bst);

	bst_write( fp, bst );
	bst_free( bst );

	fclose( fp );
	return 0;
}


bst_type*
bitstream_encoding_cst(cst_type * input)
{
bst_type *bst;
int i, n, j;
u_char mask;

	n = get_bit_mask_cst(input, &mask);


	if (mask == 0) {
		bst = bst_alloc(6); /*1 byte for mask + 4 bytes form n*/
		bst_put_int(bst, 0, 8);
		bst_put_align_num( bst, n, 4 );

		return bst;
	}

	for (i = 0, j = 0; i < ByteLength; i++)
		if ((mask & (1 << i)) != 0)
			j++;

	
	bst = bst_alloc( j * n +2);
	bst_put_int(bst, mask, 8);
	bit_encoding_cst(input, mask, bst);


	return bst;
}


unsigned long bitstream_estimate(FILE * input, int *size)
{
int     c, i, n = 0;
u_char bit_mask, ones_count;
u_char mask = 0;
unsigned long new_size;
char    temp;
unsigned char ch;


	while ((c = getc(input)) != EOF) {		
		ch = (unsigned char) c;
		temp = (char) ch;
		if (temp < 0) {
			mask |= (1 << (ByteLength - 1));
			ch = (unsigned char) (-temp);
		} else
			ch = (unsigned char) temp;
		mask |= ch;
		n++;
	}
	/* calculate number of 1's in mask*/
	bit_mask = 1;
	ones_count = 0;
	for (i = 0; i < 8; i++) {
		if (bit_mask & mask) ones_count++;
		bit_mask <<= 1;
	}

	new_size = ones_count*n;

	fseek( input, 0, SEEK_SET );
	return (new_size & 0x07) ? ((new_size >> 3) + 3) : ((new_size >> 3) + 2);
}


unsigned long bitstream_estimate_cst(cst_type * input)
{
int     c, i;
u_char bit_mask, ones_count;
u_char mask = 0;
unsigned long new_size;
char    temp;
unsigned char ch;


	for (i = 0; i < input->byte_no; i++) {
		
		c = CST_GET_SCHAR(input);
		ch = (unsigned char) c;
		temp = (char) ch;
		if (temp < 0) {
			mask |= (1 << (ByteLength - 1));
			ch = (unsigned char) (-temp);
		} else
			ch = (unsigned char) temp;
		mask |= ch;

	}
	cst_rewind(input);
	/* calculate number of 1's in mask*/
	bit_mask = 1;
	ones_count = 0;
	for (i = 0; i < 8; i++) {
		if (bit_mask & mask) ones_count++;
		bit_mask <<= 1;
	}

	new_size = ones_count*input->byte_no;

	return (new_size & 0x07) ? ((new_size >> 3) + 3) : ((new_size >> 3) + 2);
}

static void 
bit_encoding_cst(cst_type * input, unsigned char mask, bst_type * bst)
{
unsigned char flag = 0;
int     i, n = 0, c;
unsigned char ch, cod;

	flag = (mask & (1 << (ByteLength - 1)));
	for (i = 0; i < ByteLength - 1; i++)
		if ((mask & (1 << i)) != 0)
			n++;
	if (flag != 0)
		n++;
	for (i = 0; i < input->byte_no; i++) {
		c = CST_GET_SCHAR(input);
		ch = (unsigned char) c;
		get_coded_char(ch, &cod, mask, flag);
		bst_put_int(bst, (int) cod, n);
	}

	cst_rewind(input);
}


static int 
get_bit_mask(FILE * input, unsigned char *mask)
{
int     c, n = 0;
char    temp;
unsigned char ch;

	*mask = 0;
	while ((c = getc(input)) != EOF) {
		ch = (unsigned char) c;
		temp = (char) ch;
		if (temp < 0) {
			*mask |= (1 << (ByteLength - 1));
			ch = (unsigned char) (-temp);
		} else
			ch = (unsigned char) temp;
		*mask |= ch;
		n++;
	}
	return (n);
}


static int 
get_coded_char( u_char original, u_char *cod, u_char bmask, u_char flag)
{
int     i, j = 0, temp;
unsigned char c;

	if (bmask == 255) {
		*cod = original;
		return (ByteLength);
	}
	if (flag != 0) {
		temp = (int) (char) original;
		c = ((temp < 0) ? (unsigned char) (-temp) : (u_char) temp);
	}
	else	c = original;
	*cod = 0;
	for (i = 0; i < ByteLength - 1; i++) {
		if ((bmask & (1 << i)) == 0)
			continue;
		if ((c & (1 << i)) != 0)
			*cod |= (1 << j);
		j++;
	}
	if (flag != 0) {
		if (temp < 0)
			*cod |= (1 << j);
		j++;
	}
	return (j);
}


static void 
bit_encoding(FILE * input, unsigned char mask, bst_type * bst)
{
unsigned char flag = 0;
int     i, n = 0, c;
unsigned char ch, cod;

	flag = (mask & (1 << (ByteLength - 1)));
	for (i = 0; i < ByteLength - 1; i++)
		if ((mask & (1 << i)) != 0)
			n++;
	if (flag != 0)
		n++;
	while ((c = getc(input)) != EOF) {
		ch = (unsigned char) c;
		get_coded_char(ch, &cod, mask, flag);
		bst_put_int(bst, (int) cod, n);
	}
}


static int 
get_bit_mask_cst(cst_type *input, unsigned char *mask)
{
int     c, n = 0, i;
char    temp;
unsigned char ch;
 
	*mask = 0;
	for (i = 0; i < input->byte_no; i++) {
		c = CST_GET_SCHAR(input);
		ch = (unsigned char) c;
		temp = (char) ch;
		if (temp < 0) {
			*mask |= (1 << (ByteLength - 1));
			ch = (unsigned char) (-temp);
		} else
			ch = (unsigned char) temp;
		*mask |= ch;
		n++;
	}
	cst_rewind(input);
	return (n);
}


