/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
#include "HuffmanLocal.h"

void huff_count_bytes( FILE *input, unsigned long *long_counts );
void huff_convert_tree_to_code( NODE *nodes,
                           CODE *codes,
                           unsigned int code_so_far,
                           int bits,
                           int node );
void huff_output_counts( fbst_type *obst, NODE *nodes );
void huff_scale_counts( unsigned long *long_counts, NODE *nodes );
void huff_compress_data( FILE *input, fbst_type *obst, CODE *codes );
void huff_count_bytes_cst( cst_type *input, unsigned long *counts , unsigned long *saved_counts);
void huff_output_counts_cst( bst_type *obst, NODE *nodes );
bst_type* huff_compress_data_cst( cst_type *input, CODE *codes, NODE* nodes, 
							unsigned long *saved_counts );
							
							
void huffman_encoding( FILE *input, fbst_type *output )
{
unsigned long *counts;
NODE	*nodes;
CODE	*codes;
int	root_node;

	counts = (unsigned long *) calloc( 256, sizeof( unsigned long ) );
	nodes = (NODE *) calloc( 514, sizeof( NODE ) );
	codes = (CODE *) calloc( 257, sizeof( CODE ) );


	huff_count_bytes( input, counts );

	huff_scale_counts( counts, nodes );

	huff_output_counts( output, nodes );

	root_node = huff_build_tree( nodes );

	huff_convert_tree_to_code( nodes, codes, 0, 0, root_node );


	huff_compress_data( input, output, codes );


	free( (char *) counts );
	free( (char *) nodes );
	free( (char *) codes );
}

							
bst_type* huffman_encoding_cst( cst_type *input)
{
unsigned long *counts, *saved_counts;
NODE	*nodes;
CODE	*codes;
int	root_node;
bst_type *output;

	counts = (unsigned long *) calloc( 256, sizeof( unsigned long ) );
	saved_counts = (unsigned long *) calloc( 256, sizeof( unsigned long ) );
	nodes = (NODE *) calloc( 514, sizeof( NODE ) );
	codes = (CODE *) calloc( 257, sizeof( CODE ) );

	huff_count_bytes_cst( input, counts , saved_counts);

	huff_scale_counts( counts, nodes );

	root_node = huff_build_tree( nodes );

	huff_convert_tree_to_code( nodes, codes, 0, 0, root_node );

	output = huff_compress_data_cst( input, codes, nodes, saved_counts );	

	free( (char *) counts );
	free( (char *) saved_counts );
	free( (char *) nodes );
	free( (char *) codes );

	return output;
}


unsigned long huffman_estimate(FILE *input)
{
unsigned long *counts;
NODE	*nodes;
CODE	*codes;
int	root_node, i;
unsigned long bits = 0, new_size;

	counts = (unsigned long *) calloc( 256, sizeof( unsigned long ) );
	nodes = (NODE *) calloc( 514, sizeof( NODE ) );
	codes = (CODE *) calloc( 257, sizeof( CODE ) );

	huff_count_bytes( input, counts );

	huff_scale_counts( counts, nodes );

	root_node = huff_build_tree( nodes );

	huff_convert_tree_to_code( nodes, codes, 0, 0, root_node );

	for (i = 0; i < 256; i++) {
		bits += codes[i].code_bits*counts[i];
	}
	new_size = (bits & 0x07) ? ((bits >> 3) + 1) : (bits >> 3);

	free( (char *) counts );
	free( (char *) nodes );
	free( (char *) codes );

	return (new_size + 259);
}


unsigned long huffman_estimate_cst( cst_type *input)
{
unsigned long *counts, *saved_counts;
NODE	*nodes;
CODE	*codes;
int	root_node, i;
unsigned long bits = 0, new_size;

	counts = (unsigned long *) calloc( 256, sizeof( unsigned long ) );
	saved_counts = (unsigned long *) calloc( 256, sizeof( unsigned long ) );
	nodes = (NODE *) calloc( 514, sizeof( NODE ) );
	codes = (CODE *) calloc( 257, sizeof( CODE ) );


	huff_count_bytes_cst( input, counts , saved_counts);

	huff_scale_counts( counts, nodes );

	root_node = huff_build_tree( nodes );

	huff_convert_tree_to_code( nodes, codes, 0, 0, root_node );

	for (i = 0; i < 256; i++) {
		bits += codes[i].code_bits*counts[i];
	}
	new_size = (bits & 0x07) ? ((bits >> 3) + 1) : (bits >> 3);

	free( (char *) counts );
	free( (char *) saved_counts );
	free( (char *) nodes );
	free( (char *) codes );

	return(new_size + 259);
}


void huff_convert_tree_to_code( 
				NODE *nodes,
				CODE *codes,
				unsigned int code_so_far,
				int bits,
				int node)
{
	if ( node <= END_OF_STREAM ) {
		codes[ node ].code = code_so_far;
		codes[ node ].code_bits = bits;
		return;
	}



	huff_convert_tree_to_code( nodes, codes, code_so_far, bits+1,
                          nodes[ node ].child_0 );

	huff_convert_tree_to_code( nodes, codes, code_so_far | (1L<<bits),
                          bits+1, nodes[ node ].child_1 );
}


void huff_scale_counts( unsigned long *counts, NODE *nodes )
{
    unsigned long max_count;
    int i;

    max_count = 0;
    for ( i = 0 ; i < 256 ; i++ )
       if ( counts[ i ] > max_count )
	   max_count = counts[ i ];
    if ( max_count == 0 ) {
	counts[ 0 ] = 1;
	max_count = 1;
    }
    max_count = max_count / 255;
    max_count = max_count + 1;
    for ( i = 0 ; i < 256 ; i++ ) {
	nodes[ i ].count = (unsigned int) ( counts[ i ] / max_count );
	if ( nodes[ i ].count == 0 && counts[ i ] != 0 )
	    nodes[ i ].count = 1;
    }
    nodes[ END_OF_STREAM ].count = 1;
}


void
huff_output_counts( fbst_type *obst, NODE *nodes )
{
int	first;
int	last;
int	next;
int	i;

	first = 0;
	while ( first < 255 && nodes[ first ].count == 0 )
		first++;
/*
 * Each time I hit the start of the loop, I assume that first is the
 * number for a run of non-zero values.  The rest of the loop is
 * concerned with finding the value for last, which is the end of the
 * run, and the value of next, which is the start of the next run.
 * At the end of the loop, I assign next to first, so it starts in on
 * the next run.
 */
	for ( ; first < 256 ; first = next ) {
		last = first + 1;
		for ( ; ; ) {
			for ( ; last < 256 ; last++ )
				if ( nodes[ last ].count == 0 )	break;

			last--;
			for ( next = last + 1; next < 256 ; next++ )
				if ( nodes[ next ].count != 0 )	break;

			if ( next > 255 )	break;
			if ( ( next - last ) > 3 )	break;
			last = next;
		};
/*
 * Here is where I output first, last, and all the counts in between.
 */
		fbst_put_int( obst, first, 8 );
		fbst_put_int( obst, last, 8 );

		for ( i = first ; i <= last ; i++ ) 
			fbst_put_int( obst, nodes[ i ].count, 8 );
	}

	fbst_put_int( obst, 0, 8 );
}


void huff_output_counts_cst( bst_type *obst, NODE *nodes )
{
int	first;
int	last;
int	next;
int	i;

	first = 0;
	while ( first < 255 && nodes[ first ].saved_count == 0 )
		first++;
/*
 * Each time I hit the start of the loop, I assume that first is the
 * number for a run of non-zero values.  The rest of the loop is
 * concerned with finding the value for last, which is the end of the
 * run, and the value of next, which is the start of the next run.
 * At the end of the loop, I assign next to first, so it starts in on
 * the next run.
 */
	for ( ; first < 256 ; first = next ) {
		last = first + 1;
		for ( ; ; ) {
			for ( ; last < 256 ; last++ )
				if ( nodes[ last ].saved_count == 0 )	break;

			last--;
			for ( next = last + 1; next < 256 ; next++ )
				if ( nodes[ next ].saved_count != 0 )	break;

			if ( next > 255 )	break;
			if ( ( next - last ) > 3 )	break;
			last = next;
		};
/*
 * Here is where I output first, last, and all the counts in between.
 */
		bst_put_int( obst, first, 8 );
		bst_put_int( obst, last, 8 );

		for ( i = first ; i <= last ; i++ ) 
			bst_put_int( obst, nodes[ i ].saved_count, 8 );
	}

	bst_put_int( obst, 0, 8 );
}


#ifndef SEEK_SET
#define SEEK_SET 0
#endif

void huff_count_bytes( FILE *input, unsigned long *counts )
{
    long input_marker;
    int c;

    input_marker = ftell( input );
    while ( ( c = getc( input )) != EOF )
	counts[ c ]++;
    fseek( input, input_marker, SEEK_SET );
}


void huff_compress_data( FILE *input, fbst_type *obst, CODE *codes )
{
    int c;

    while ( ( c = getc( input ) ) != EOF )
        fbst_put_int( obst, (unsigned long) codes[ c ].code,
                    codes[ c ].code_bits );
    fbst_put_int( obst, (unsigned long) codes[ END_OF_STREAM ].code,
		codes[ END_OF_STREAM ].code_bits );
}



void huff_count_bytes_cst( cst_type *input, unsigned long *counts , unsigned long *saved_counts)
{
    int c, i;

	for (i = 0; i < input->byte_no; i++) {
		c = CST_GET_UCHAR( input );
		counts[ c ]++;
	}
    cst_rewind(input);
	/* save the cpunts for calculated needed space*/
	for (i = 0; i < 256; i++)
		saved_counts[i] = counts[i]; 
}


bst_type* huff_compress_data_cst( cst_type *input, CODE *codes, NODE* nodes, unsigned long *saved_counts )
{
	bst_type *output;
    int c, i;
	unsigned long bits = 0;

	/*calculate needed size for bst_type */
	for (i = 0; i < 256; i++) {
		bits += codes[i].code_bits*saved_counts[i];
	}
	bits = (bits & 0x07) ? ((bits >> 3) + 1) : (bits >> 3);

	/*allocate the space  number of needed bytes + maximum 259 symbols for counts*/
	output = bst_alloc(bits + 259);	
	/*output = bst_alloc(huffman_estimate_cst(input));	*/

	/* write counts*/
	huff_output_counts_cst( output, nodes );

	/*do the rest*/
	for (i = 0; i < input->byte_no; i++) {
		c = CST_GET_UCHAR( input );
        bst_put_int( output, (unsigned long) codes[ c ].code,
                    codes[ c ].code_bits );
	}

    bst_put_int( output, (unsigned long) codes[ END_OF_STREAM ].code,
		codes[ END_OF_STREAM ].code_bits );

	cst_rewind(input);
	/*bst_rewind(output);*/
	return output;
}
