/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
#include "HuffmanLocal.h"


void huff_count_bytes_extended( FILE *input, unsigned long *counts, char* ext_bits );

void huff_compress_data_extended( FILE *input, fbst_type *obst, CODE *codes, char* ext_bits );


void huff_count_bytes_extended_cst( cst_type *input, unsigned long *counts , 
							  unsigned long *saved_counts, unsigned long *needed_bytes,
							  char* ext_bits);
bst_type* huff_compress_data_extended_cst( cst_type *input, CODE *codes, NODE* nodes, 
							unsigned long *saved_counts , unsigned long needed_bytes,
							char* ext_bits);


void huffman_encoding_extended( FILE *input, fbst_type *output )
{
unsigned long *counts;
NODE	*nodes;
CODE	*codes;
int	root_node;
char *ptr, *ext_bits, bits;
unsigned char mask, i;

	counts = (unsigned long *) calloc( 256, sizeof( unsigned long ) );
	nodes = (NODE *) calloc( 514, sizeof( NODE ) );
	codes = (CODE *) calloc( 257, sizeof( CODE ) );
	/* buffer with number of needed bits per each char. saved only for pisitive
		char. same value for negative, exept -128 for which the value is 8 and 
		saved in 128 offset */
	ext_bits = (char *) malloc(129*sizeof(char));

	/*fill the buffer */
	ptr = ext_bits;
	mask = 1;
	bits = 0;
	for (i = 0 ; i < 129; i++, ptr++) {
		if (mask & i) {
			/* next level ; incr bits*/
			bits ++;
			mask <<= 1;
		}

		*ptr = bits;
	}


	huff_count_bytes_extended( input, counts, ext_bits );

	huff_scale_counts( counts, nodes );

	huff_output_counts( output, nodes );

	root_node = huff_build_tree( nodes );

	huff_convert_tree_to_code( nodes, codes, 0, 0, root_node );

	huff_compress_data_extended( input, output, codes , ext_bits);


	free( (char *) counts );
	free( (char *) nodes );
	free( (char *) codes );
	free( ext_bits );
}


bst_type* huffman_encoding_extended_cst( cst_type *input)
{
unsigned long *counts, *saved_counts;
NODE	*nodes;
CODE	*codes;
int	root_node;
bst_type *output;
unsigned long needed_bytes;
char *ptr, *ext_bits, bits;
unsigned char mask, i;

	counts = (unsigned long *) calloc( 256, sizeof( unsigned long ) );
	saved_counts = (unsigned long *) calloc( 256, sizeof( unsigned long ) );
	nodes = (NODE *) calloc( 514, sizeof( NODE ) );
	codes = (CODE *) calloc( 257, sizeof( CODE ) );
	/* buffer with number of needed bits per each char. saved only for pisitive
		char. same value for negative, exept -128 for which the value is 8 and 
		saved in 128 offset */
	ext_bits = (char *) malloc(129*sizeof(char));

	cst_rewind(input);
	ptr = ext_bits;

	/*fill the buffer */
	mask = 1;
	bits = 0;
	for (i = 0 ; i < 129; i++, ptr++) {
		if (mask & i) {
			/* next level ; incr bits*/
			bits ++;
			mask <<= 1;
		}

		*ptr = bits;
	}

	huff_count_bytes_extended_cst( input, counts , saved_counts, &needed_bytes, ext_bits);

	huff_scale_counts( counts, nodes );

	root_node = huff_build_tree( nodes );

	huff_convert_tree_to_code( nodes, codes, 0, 0, root_node );

	output = huff_compress_data_extended_cst( input, codes, nodes, saved_counts, needed_bytes, ext_bits );	

	free( (char *) counts );
	free( (char *) saved_counts );
	free( (char *) nodes );
	free( (char *) codes );
	free( ext_bits );

	return output;
}


void huff_count_bytes_extended( FILE *input, unsigned long *counts, char* ext_bits )
{
    long input_marker;
	unsigned char uc;
	int in_c;

	
    input_marker = ftell( input );
    while ( ( in_c = getc( input )) != EOF ) {
		uc = (unsigned char) in_c;
		counts[ ext_bits[uc] ]++;
	}
    fseek( input, input_marker, SEEK_SET );
}


void huff_compress_data_extended( FILE *input, fbst_type *obst, CODE *codes, char* ext_bits )
{
	unsigned char uc;
	int j;
	int in_c;
	char mask, c;

    while ( ( in_c = getc( input ) ) != EOF ) {
		c = (char) in_c;
		if (c == -128) {
			fbst_put_int( obst, (unsigned long) codes[ 8 ].code, codes[ 8 ].code_bits );
			fbst_put_int( obst, 0x80, 8 );					
			continue;
		}

		if (c < 0) uc = -c;
		else uc = c;
		j = ext_bits[uc] ;
		fbst_put_int( obst, (unsigned long) codes[ j].code, codes[ j ].code_bits );
		if (j > 0) {
			mask = (1 << (j - 1));
			fbst_put_int( obst, (c >= 0) ? ((unsigned long) (uc & (~mask) )) : ((unsigned long) uc), j );
		}
	}

    fbst_put_int( obst, (unsigned long) codes[ END_OF_STREAM ].code,
		codes[ END_OF_STREAM ].code_bits );
	
}



void huff_count_bytes_extended_cst( cst_type *input, unsigned long *counts , 
							  unsigned long *saved_counts, unsigned long *needed_bytes,
							  char* ext_bits)
{
	unsigned char uc;
	int i;
	unsigned long needed_bits = 0;
	char c;

	for (i = 0; i < input->byte_no; i++) {
		c = CST_GET_SCHAR( input );
		uc = ( c < 0) ? -c : c;
		counts[ ext_bits[uc] ]++;
		needed_bits += ext_bits[uc];
	}
    cst_rewind(input);

	/* save the cpunts for calculated needed space*/
	for (i = 0; i < 256; i++)
		saved_counts[i] = counts[i]; 

	*needed_bytes = (needed_bits & 0x03) ? needed_bits >> 3 :  ((needed_bits >> 3 ) + 1);
}


bst_type* huff_compress_data_extended_cst( cst_type *input, CODE *codes, NODE* nodes, 
									 unsigned long *saved_counts , unsigned long needed_bytes,
									 char* ext_bits)
{

	bst_type *bst;
	unsigned char uc;
	char c;
	int i,j;
	unsigned long bits = 0;
	char mask;

	/*calculate needed size for bst_type */
	for (i = 0; i < 256; i++) {
		bits += codes[i].code_bits*saved_counts[i];
	}
	bits = (bits & 0x07) ? ((bits >> 3) + 1) : (bits >> 3);

	/*allocate the space  number of needed bytes + maximum 259 symbols for counts*/
	bst = bst_alloc(bits + needed_bytes + 259);	
	/*output = bst_alloc(huffman_estimate_cst(input));	*/

	/* write counts*/
	huff_output_counts_cst( bst, nodes );

	/*do the rest*/
	for (i = 0; i < input->byte_no; i++) {
		c = CST_GET_SCHAR( input );

		if (c < 0) uc = -c;
		else uc = c;
		j = ext_bits[uc] ;
		bst_put_int( bst, (unsigned long) codes[ j].code, codes[ j ].code_bits );
		if (j > 0) {
			mask = (1 << (j - 1));
			bst_put_int( bst, (c >= 0) ? ((unsigned long) (uc & (~mask) )) : ((unsigned long) uc), j );
		}
	}

    bst_put_int( bst, (unsigned long) codes[ END_OF_STREAM ].code,
		codes[ END_OF_STREAM ].code_bits );

	cst_rewind(input);

	return bst;
}
