/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/

/************************
 ***	Quantize.c    ***
 ************************/
#include        <stdio.h>   
#include	<math.h>

#include        "Uprm/prm.h"
#include        "Ucode/code.h"
#include        "Uigp/igp.h"


#include        "QuantizeIn.h"


#include        "Coding/CodingCenter.h"
#include        "Coding/CodingJoint.h"
#include        "Coding/CodingLink.h"
#include        "Coding/CodingN3.h"




static void	quantize_jnt_clear_in( jnt_type *jnt );

static void	quantize_jnt_center( gp_type *gp, jnt_type *jnt,coding_type *cprm );
static jnt_type *  //Grisha 19.12.00
quantize_brn( gp_array *agp, jnt_type *jnt, int bno,coding_type *cprm );

static void
quantize_JNT_N3_brn_aux( jnt_type *jnt,codingN3_type *nc );
static  void
quantize_JNT_RN3_brn_aux( jnt_type *jnt,codingN3_type *nc );


static void
quantize_links( vec2d *ctr, link_type *link, vec2d *dv ,codingLink_type *lc);

static void
quantize_link( vec2d *ctr, link_type *l, vec2d *dv,codingLink_type *lc );


static void	quantize_branch_center( jnt_type *jnt, int bno );


static int	quantize_brn_link_filter( gp_array *agp,
						jnt_type *jnt, int bno,
						jnt_type *njnt, int nbno );
static void	quantize_n3_width( link_type *link ,codingN3_type *nc);





void
quantize_agp( gp_array *agp,coding_type *cprm )
{
gp_type	*gp;
jnt_type	*jnt, *jntNext; //Grisha 19.12.00
brn_type	*b;
int	i,	j,	bno;


/*
	agp_link_in_aux_clear( agp );
*/

	for( i = 0 ; i < agp->Ni ; i++ )
		for( j = 0 ; j < agp->Nj ; j++ ){
			gp = &agp->gp[i][j];

			for( jnt = gp->jnt ; jnt != NULL ; jnt = jnt->nj ){

				quantize_jnt_center( gp, jnt,cprm );
				quantize_jnt_clear_in( jnt );
			}

			gp->link = NULL;
		}



	for( i = 0 ; i < agp->Ni ; i++ )
	    for( j = 0 ; j < agp->Nj ; j++ ){
		gp = &agp->gp[i][j];

		for( jnt = gp->jnt ; jnt != NULL ;	jnt = jntNext )	//Grisha 19.12.00
		{

			jntNext = jnt->nj;	//Grisha 19.12.00

			if( jnt->type == JNT_N3 ){
				b = jnt->brn[0];
				if( b->direction != F_BEGIN )	continue;

				jntNext = quantize_brn( agp, jnt, 0 ,cprm); //Grisha 19.12.00
				continue;
			}


			if( jnt->type != JNT_RN3 )	continue;


			for( bno = 0 ; bno < jnt->brn_no ; bno++ ){
				b = jnt->brn[bno];
				if( b->direction != F_BEGIN )	continue;

				jntNext = quantize_brn( agp, jnt, bno,cprm ); //Grisha 19.12.00
			}
		}
	    }


	for( i = 0 ; i < agp->Ni ; i++ )
	    for( j = 0 ; j < agp->Nj ; j++ ){
		gp = &agp->gp[i][j];

		for( jnt = gp->jnt ; jnt != NULL ; jnt = jntNext )	//Grisha 19.12.00
		{

			jntNext = jnt->nj;	//Grisha 19.12.00

			if( jnt->type == JNT_RN3 )	continue;


			bno = ( jnt->type == JNT_N3 )? 1 : 0;
			for( ; bno < jnt->brn_no ; bno++ ){
				b = jnt->brn[bno];
				if( b->direction != F_BEGIN )	continue;
				if( b->type == BRN_NULL )	continue;
				if( b->type == BRN_IN )	continue;

				jntNext = quantize_brn( agp, jnt, bno,cprm ); //Grisha 19.12.00
			}
		}
	    }








	for( i = 0 ; i < agp->Ni ; i++ )
		for( j = 0 ; j < agp->Nj ; j++ ){
			gp = &agp->gp[i][j];

			for( jnt = gp->jnt ; jnt != NULL ; jnt = jnt->nj )
				free( jnt->cjnt );
		}


	agp_geomtric_aux( agp );
	agp_link_in_reorder( agp );
/*
	agp_link_in_aux( agp );
*/
}




static void
quantize_jnt_clear_in( jnt_type *jnt )
{
brn_type	*b;

	if( jnt->type != JNT_N3 )	return;

	b = jnt->brn[1];
	if( b->type == BRN_IN ){
		free( b->in );
		b->in = NULL;
		b->link->in[F_RIGHT] = b->link->in[F_LEFT] = NULL;
	}

	b = jnt->brn[2];
	if( b->type == BRN_IN ){
		free( b->in );
		b->in = NULL;
		b->link->in[F_RIGHT] = b->link->in[F_LEFT] = NULL;
	}
}



static void
quantize_jnt_center( gp_type *gp, jnt_type *jnt,coding_type *cprm )
{
int	i,	cno;
qjnt_type	*qj;

	jnt->cjnt = (jnt_type *)QJNT_ALLOC();
	qj = (qjnt_type *)jnt->cjnt;

	qj->ctr = jnt->ctr;

	for( i = 0 ; i < jnt->brn_no ; i++ )
		qj->b[i].ctr = jnt->brn[i]->ctr;


	cno = CODE( jnt->ctr.x, cprm->cc->Pctr );
	jnt->ctr.x = UNCODE( cno, cprm->cc->Pctr );

	cno = CODE( jnt->ctr.y, cprm->cc->Pctr );
	jnt->ctr.y = UNCODE( cno, cprm->cc->Pctr );


	qj->ctr.x -= jnt->ctr.x;
	qj->ctr.y -= jnt->ctr.y;

	
	jnt->gctr.x = gp->ctr.x + jnt->ctr.x;
	jnt->gctr.y = gp->ctr.y + jnt->ctr.y;



	if( jnt->type == JNT_RN3 )
		quantize_JNT_RN3_brn_aux( jnt,cprm->nc );
}



static void
quantize_JNT_N3_brn_aux( jnt_type *jnt,codingN3_type *nc )
{
brn_type	*b;
int	cno;
qjnt_type	*qj;

	qj = (qjnt_type *)jnt->cjnt;


	
	qj->b[1].ctr = jnt->brn[1]->ctr;
	qj->b[2].ctr = jnt->brn[2]->ctr;


	cno = ZCODE (jnt->lwidth - nc->Nwidth.a, nc->Nwidth);
	jnt->lwidth = UNZCODE (cno,nc->Nwidth) + nc->Nwidth.a;

	cno = ZCODE (jnt->rwidth - nc->Nwidth.a, nc->Nwidth);
	jnt->rwidth = UNZCODE (cno,nc->Nwidth) + nc->Nwidth.a;



	jnt_n3_load_u( jnt );

	b = jnt->brn[1];
	qj->b[1].ctr.x = qj->b[1].ctr.x - b->ctr.x + qj->ctr.x;
	qj->b[1].ctr.y = qj->b[1].ctr.y - b->ctr.y + qj->ctr.y;

	b = jnt->brn[2];
	qj->b[2].ctr.x = qj->b[2].ctr.x - b->ctr.x + qj->ctr.x;
	qj->b[2].ctr.y = qj->b[2].ctr.y - b->ctr.y + qj->ctr.y;
}



static  void
quantize_JNT_RN3_brn_aux( jnt_type *jnt,codingN3_type *nc )
{
int	cno;
	cno = ZCODE (jnt->lwidth - nc->Nwidth.a, nc->Nwidth);
	jnt->lwidth = UNZCODE (cno,nc->Nwidth) + nc->Nwidth.a;

	cno = ZCODE (jnt->rwidth - nc->Nwidth.a, nc->Nwidth);
	jnt->rwidth = UNZCODE (cno,nc->Nwidth) + nc->Nwidth.a;
}





static jnt_type *  //Grisha 19.12.00
quantize_brn( gp_array *agp, jnt_type *jnt, int bno,coding_type *cprm )
{
qjnt_type	*qj;
jnt_type	*njnt, *jntNext, *jntNextNext;
brn_type	*b;
link_type	*l;
vec2d	ctr;
int	nbno;

	jntNext = jnt->nj;
	jntNextNext = jntNext == NULL ? NULL : jntNext->nj;

	b = jnt->brn[bno];

	for( l = b->link ; LINK_NEXT(l) != NULL ; l = LINK_NEXT(l) );

	njnt = l->p[F_END].p.jnt;
	nbno = l->p[F_END].bno;


	if( jnt->type != JNT_N3 )
		quantize_branch_center( jnt, bno );

	if( njnt->type != JNT_N3 )
		quantize_branch_center( njnt, nbno );



	qj = (qjnt_type *)jnt->cjnt;

	ctr.x = jnt->gctr.x + b->ctr.x;
	ctr.y = jnt->gctr.y + b->ctr.y;
	quantize_links( &ctr, b->link, &qj->b[bno].ctr,cprm->lc );


	jnt_load_nf_end_link( b->link );


	if( quantize_brn_link_filter( agp, jnt, bno, njnt, nbno ) == 0 )
	{
		return ( jntNext == njnt ) ? jntNextNext : jntNext; //Grisha 19.12.00
	}


	agp_lnf_link( &ctr, b->link, agp );


	if( b->type == NF_N3 )
		quantize_n3_width( b->link,cprm->nc );




	if( jnt->type == JNT_N3 && bno == 0 )
		quantize_JNT_N3_brn_aux( jnt ,cprm->nc);

	if( njnt->type == JNT_N3 && nbno == 0 )
		quantize_JNT_N3_brn_aux( njnt ,cprm->nc);


	link_set_geo_aux( b->link );

	return jntNext; //Grisha 19.12.00
}




static void
quantize_branch_center( jnt_type *jnt, int bno )
{
qjnt_type	*qj;
brn_type	*b;

	qj = (qjnt_type *)jnt->cjnt;

	b = jnt->brn[bno];

	qj->b[bno].ctr.x = qj->ctr.x + b->ctr.x;
	qj->b[bno].ctr.y = qj->ctr.y + b->ctr.y;

  
 	b->ctr.x = b->ctr.y = 0.0;

 
	qj->b[bno].ctr.x -= b->ctr.x;
	qj->b[bno].ctr.y -= b->ctr.y;
}










static void
quantize_links( vec2d *ctr, link_type *link, vec2d *dv ,codingLink_type *lc)
{
link_type	*l;
vec2d	v,	dv1;

	dv1 = *dv;
	
	for( l = link, v = *ctr ; l != NULL ; l = LINK_NEXT(l) ){

		quantize_link( &v, l, &dv1 ,lc);

		v.x += l->v.x;
		v.y += l->v.y;
	}
}


static void
quantize_link( vec2d *ctr, link_type *l, vec2d *dv,codingLink_type *lc )
{
int	cno;
vec2d	v,	w;

	v.x = ctr->x + dv->x + l->v.x;
	v.y = ctr->y + dv->y + l->v.y;

	w.x = QCODE( v.x, lc->Llen );
	w.y = QCODE( v.y, lc->Llen );

	l->v.x = w.x - ctr->x;
	l->v.y = w.y - ctr->y;

	dv->x = v.x - w.x;
	dv->y = v.y - w.y;


	cno = ZCODE( l->a, lc->La );
	l->a = UNZCODE( cno, lc->La );

	link_set_aux( l );
}


void
quantize_link_center( vec2d *ctr,codingLink_type *lc )
{
	ctr->x = QCODE( ctr->x, lc->Llen );
	ctr->y = QCODE( ctr->y, lc->Llen );
}


void
quantize_link_a( float *a,codingLink_type *lc )
{
int	cno;
	cno = ZCODE( (*a), lc->La );
	*a = UNZCODE( cno, lc->La );
}




static int
quantize_brn_link_filter( gp_array *agp,
			jnt_type *jnt, int bno, jnt_type *njnt, int nbno )
{
brn_type	*b,	*nb;
link_type	*l,	*pl,	*tl;
int	ino;

	b = jnt->brn[bno];
	nb = njnt->brn[nbno];

	pl = NULL;
	for( l = b->link, ino = 0 ; l != NULL ; ){

		if( l->v.x != 0 || l->v.y != 0 ){
			ino++;
			pl = l;
			l = LINK_NEXT(l);
			continue;
		}

		tl = l;
		l = LINK_NEXT(l);

		if( l == NULL ){
			if( pl == NULL ){
				link_free( b->link );
				jnt_free_brn( jnt, bno );
				if( jnt->brn_no == 0 )
					agp_clear_jnt( agp, jnt );

				jnt_free_brn( njnt, nbno );
				if( njnt->brn_no == 0 )
					agp_clear_jnt( agp, njnt );

				return( 0 );
			}
			nb->link = pl;
			pl->pr[1] = tl->pr[1];
		}
		else	l->p[F_BEGIN] = tl->p[F_BEGIN];


		if( pl != NULL ){
			pl->p[F_END] = tl->p[F_END];
			if( l != NULL )
				pl->pr[1] = l->pr[0];
		}
		else	b->link = l;


		free( tl->pr[0].p );
		free( tl );
	}

	return( ino );
}




static void
quantize_n3_width( link_type *link ,codingN3_type *nc)
{
int	cno;
link_type	*l;
width_type	*width;
float	temp;

	for( l = link ; l != NULL ; l = LINK_NEXT(l) ){
		width = l->width[F_BEGIN];

		temp = -width->w[F_LEFT] - nc->Nwidth.a;
		cno = ZCODE( temp, nc->Nwidth );
		width->w[F_LEFT] = -(UNZCODE( cno, nc->Nwidth) + nc->Nwidth.a);

		temp = width->w[F_RIGHT] - nc->Nwidth.a;
		cno = ZCODE( temp, nc->Nwidth );
		width->w[F_RIGHT] = UNZCODE( cno, nc->Nwidth) + nc->Nwidth.a;

		width = l->width[F_END];
	}

	temp = -width->w[F_LEFT] - nc->Nwidth.a;
	cno = ZCODE( temp, nc->Nwidth );
	width->w[F_LEFT] = -(UNZCODE( cno, nc->Nwidth) + nc->Nwidth.a);

	temp = width->w[F_RIGHT] - nc->Nwidth.a;
	cno = ZCODE( temp, nc->Nwidth );
	width->w[F_RIGHT] = UNZCODE( cno, nc->Nwidth) + nc->Nwidth.a;
}
