/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/************************
 ***   Quantize16.c   ***
 ************************/
#include        <stdio.h>   
#include	<math.h>

#include        "Ucode/code.h"
#include        "Uigp/igp.h"


#include        "QuantizeIn.h"



#define         QCODE16( a )    ( (((int)(a*16)) + 0.5)* 0.0625 )

#define         CODE16( a )     ((int)(a*16))

#define         UNCODE16( a )    ( (  ((int)(a*16)) + 0.5)* 0.0625 )



static void	quantize_jnt_clear_in( jnt_type *jnt );

static void	quantize_jnt_center( gp_type *gp, jnt_type *jnt );
static jnt_type * quantize_brn( gp_array *agp, jnt_type *jnt, int bno );

static void	quantize_JNT_N3_brn_aux( jnt_type *jnt );
static void	quantize_JNT_RN3_brn_aux( jnt_type *jnt );


static void	quantize_links( vec2d *ctr, link_type *link, vec2d *dv );

static void	quantize_link( vec2d *ctr, link_type *l, vec2d *dv );


static void	quantize_branch_center( jnt_type *jnt, int bno );


static void	quantize_n3_width( link_type *link );




static float	qcode16( float a );

static float	qzcode16( float a );




void
quantize16_agp( gp_array *agp )
{
gp_type	*gp;
jnt_type	*jnt, *jntNext;
brn_type	*b;
int	i,	j,	bno;



	for( i = 0 ; i < agp->Ni ; i++ )
		for( j = 0 ; j < agp->Nj ; j++ ){
			gp = &agp->gp[i][j];

			for( jnt = gp->jnt ; jnt != NULL ; jnt = jnt->nj ){

				quantize_jnt_center( gp, jnt );
				quantize_jnt_clear_in( jnt );
			}

			gp->link = NULL;
		}



	for( i = 0 ; i < agp->Ni ; i++ )
	    for( j = 0 ; j < agp->Nj ; j++ ){
		gp = &agp->gp[i][j];

		for( jnt = gp->jnt ; jnt != NULL ; jnt = jntNext ){

			jntNext = jnt->nj;

			if( jnt->type == JNT_N3 ){
				b = jnt->brn[0];
				if( b->direction != F_BEGIN )	continue;

				jntNext = quantize_brn( agp, jnt, 0 );
				continue;
			}


			if( jnt->type != JNT_RN3 )	continue;


			for( bno = 0 ; bno < jnt->brn_no ; bno++ ){
				b = jnt->brn[bno];
				if( b->direction != F_BEGIN )	continue;

				jntNext = quantize_brn( agp, jnt, bno );
			}
		}
	    }


	for( i = 0 ; i < agp->Ni ; i++ )
	    for( j = 0 ; j < agp->Nj ; j++ ){
		gp = &agp->gp[i][j];

		for( jnt = gp->jnt ; jnt != NULL ; jnt = jntNext ) {

			jntNext = jnt->nj;

			if( jnt->type == JNT_RN3 )	continue;


			bno = ( jnt->type == JNT_N3 )? 1 : 0;
			for( ; bno < jnt->brn_no ; bno++ ){
				b = jnt->brn[bno];
				if( b->direction != F_BEGIN )	continue;
				if( b->type == BRN_NULL )	continue;
				if( b->type == BRN_IN )	continue;

				jntNext = quantize_brn( agp, jnt, bno );
			}
		}
	    }








	for( i = 0 ; i < agp->Ni ; i++ )
		for( j = 0 ; j < agp->Nj ; j++ ){
			gp = &agp->gp[i][j];

			for( jnt = gp->jnt ; jnt != NULL ; jnt = jnt->nj )
				free( jnt->cjnt );
		}


	agp_geomtric_aux( agp );
	agp_link_in_reorder( agp );
}




static void
quantize_jnt_clear_in( jnt_type *jnt )
{
brn_type	*b;

	if( jnt->type != JNT_N3 )	return;

	b = jnt->brn[1];
	if( b->type == BRN_IN ){
		free( b->in );
		b->in = NULL;
		b->link->in[F_RIGHT] = b->link->in[F_LEFT] = NULL;
	}

	b = jnt->brn[2];
	if( b->type == BRN_IN ){
		free( b->in );
		b->in = NULL;
		b->link->in[F_RIGHT] = b->link->in[F_LEFT] = NULL;
	}
}



static void
quantize_jnt_center( gp_type *gp, jnt_type *jnt )
{
int	i;
qjnt_type	*qj;

	jnt->cjnt = (jnt_type *)QJNT_ALLOC();
	qj = (qjnt_type *)jnt->cjnt;

	qj->ctr = jnt->ctr;

	for( i = 0 ; i < jnt->brn_no ; i++ )
		qj->b[i].ctr = jnt->brn[i]->ctr;


	jnt->ctr.x = qcode16( jnt->ctr.x );
	jnt->ctr.y = qcode16( jnt->ctr.y );


	qj->ctr.x -= jnt->ctr.x;
	qj->ctr.y -= jnt->ctr.y;

	
	jnt->gctr.x = gp->ctr.x + jnt->ctr.x;
	jnt->gctr.y = gp->ctr.y + jnt->ctr.y;



	if( jnt->type == JNT_RN3 )
		quantize_JNT_RN3_brn_aux( jnt );
}



static void
quantize_JNT_N3_brn_aux( jnt_type *jnt )
{
brn_type	*b;
qjnt_type	*qj;

	qj = (qjnt_type *)jnt->cjnt;


	
	qj->b[1].ctr = jnt->brn[1]->ctr;
	qj->b[2].ctr = jnt->brn[2]->ctr;



	jnt->lwidth = qzcode16( jnt->lwidth );
	jnt->rwidth = qzcode16( jnt->rwidth );




	jnt_n3_load_u( jnt );

	b = jnt->brn[1];
	qj->b[1].ctr.x = qj->b[1].ctr.x - b->ctr.x + qj->ctr.x;
	qj->b[1].ctr.y = qj->b[1].ctr.y - b->ctr.y + qj->ctr.y;

	b = jnt->brn[2];
	qj->b[2].ctr.x = qj->b[2].ctr.x - b->ctr.x + qj->ctr.x;
	qj->b[2].ctr.y = qj->b[2].ctr.y - b->ctr.y + qj->ctr.y;
}



static  void
quantize_JNT_RN3_brn_aux( jnt_type *jnt )
{
	jnt->lwidth = qzcode16( jnt->lwidth );
	jnt->rwidth = qzcode16( jnt->rwidth );
}





static jnt_type * 
quantize_brn( gp_array *agp, jnt_type *jnt, int bno )
{
qjnt_type	*qj;
jnt_type	*njnt, *jntNext, *jntNextNext;
brn_type	*b;
link_type	*l;
vec2d	ctr;
int	nbno;

	jntNext = jnt->nj;
	jntNextNext = jntNext == NULL ? NULL : jntNext->nj;

	b = jnt->brn[bno];

	for( l = b->link ; LINK_NEXT(l) != NULL ; l = LINK_NEXT(l) );

	njnt = l->p[F_END].p.jnt;
	nbno = l->p[F_END].bno;


	if( jnt->type != JNT_N3 )
		quantize_branch_center( jnt, bno );

	if( njnt->type != JNT_N3 )
		quantize_branch_center( njnt, nbno );



	qj = (qjnt_type *)jnt->cjnt;

	ctr.x = jnt->gctr.x + b->ctr.x;
	ctr.y = jnt->gctr.y + b->ctr.y;
	quantize_links( &ctr, b->link, &qj->b[bno].ctr );


	jnt_load_nf_end_link( b->link );


	agp_lnf_link( &ctr, b->link, agp );


	if( b->type == NF_N3 )
		quantize_n3_width( b->link );




	if( jnt->type == JNT_N3 && bno == 0 )
		quantize_JNT_N3_brn_aux( jnt );

	if( njnt->type == JNT_N3 && bno == 0 )
		quantize_JNT_N3_brn_aux( njnt );


	link_set_geo_aux( b->link );

	return jntNext;
}




static void
quantize_branch_center( jnt_type *jnt, int bno )
{
qjnt_type	*qj;
brn_type	*b;

	qj = (qjnt_type *)jnt->cjnt;

	b = jnt->brn[bno];

	qj->b[bno].ctr.x = qj->ctr.x + b->ctr.x;
	qj->b[bno].ctr.y = qj->ctr.y + b->ctr.y;

  
 	b->ctr.x = b->ctr.y = 0.0;

 
	qj->b[bno].ctr.x -= b->ctr.x;
	qj->b[bno].ctr.y -= b->ctr.y;
}










static void
quantize_links( vec2d *ctr, link_type *link, vec2d *dv )
{
link_type	*l;
vec2d	v,	dv1;

	dv1 = *dv;
	
	for( l = link, v = *ctr ; l != NULL ; l = LINK_NEXT(l) ){

		quantize_link( &v, l, &dv1 );

		v.x += l->v.x;
		v.y += l->v.y;
	}
}


static void
quantize_link( vec2d *ctr, link_type *l, vec2d *dv )
{
vec2d	v,	w;

	v.x = ctr->x + dv->x + l->v.x;
	v.y = ctr->y + dv->y + l->v.y;

	w.x = qcode16( v.x );
	w.y = qcode16( v.y );

	l->v.x = w.x - ctr->x;
	l->v.y = w.y - ctr->y;

	dv->x = v.x - w.x;
	dv->y = v.y - w.y;


	l->a = qzcode16( l->a );

	link_set_aux( l );
}



static void
quantize_n3_width( link_type *link )
{
link_type	*l;
width_type	*width;

	for( l = link ; l != NULL ; l = LINK_NEXT(l) ){
		width = l->width[F_BEGIN];

		width->w[F_LEFT] = qzcode16( width->w[F_LEFT] );
		width->w[F_RIGHT] = qzcode16( width->w[F_RIGHT] );

		width = l->width[F_END];
	}

	width->w[F_LEFT] = qzcode16( width->w[F_LEFT] );
	width->w[F_RIGHT] = qzcode16( width->w[F_RIGHT] );
}



static float
qcode16( float a )
{
float	b;
int	c;

	b = a *16;
	c = (int)b;

	if( b == c && (c & 0x0f) != 0 )	return( a );

	a = ( c + 0.5 )*0.0625;

	return( a );
}


static float
qzcode16( float a )
{
float	b;
int	c;

	b = a *16;
	c = (int)b;

	if( b == c )	return( a );

	if( a < 0 )	c = b - 0.5;
	else	c = b + 0.5;

	a = c * 0.0625;

	return( a );
}
