/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/

/**************************
 ***   EpBgpPut1.c     	***
 **************************/

#include <stdio.h>
#include <math.h>

#include	"Uigp/igp.h"
#include        "Uprm/prm.h"


#include        "GpType/GpType.h"

#include        "Ep/Ep.h"
#include        "EpBgp.h"



static void	ep_bgp_put_middle_1( gp_type *gp, ep_type *ep );

static void	ep_bgp_put_al_1( ep_type *ep, al_type *al,
					int side, int Fprofile );



void
ep_bgp_put_1( gp_array *agp, ep_array *aep, int Fprofile )
{
gp_type		*gp;
ep_type		*ep;

al_type	*al;

int	i,	j,	i1,	j1;


	for( i = LSIZE, i1 = aep->Ni0; i < aep->Ni-LSIZE ; i++, i1++ )
	    for( j = LSIZE, j1 = aep->Nj0; j < aep->Nj-LSIZE ; j++, j1++ ){

		gp = &agp->gp[i1][j1];

		ep = &aep->ep[i][j];

		ep_bgp_put_middle_1( gp, ep );


		for( al = ep->al ; al != NULL ; al = al->nal ){
			if( !(al->flag & 0x30 ) )       continue;

			ep_bgp_put_al_1( ep, al, F_RIGHT, Fprofile );
			ep_bgp_put_al_1( ep, al, F_LEFT, Fprofile );
		}

	    }

}




static void
ep_bgp_put_al_1( ep_type *ep, al_type *al, int side, int Fprofile )
{
el_type	*l;
bound_type	*b;
ebgp_type	*bp;
int     bno;
edge_type	*e;


	bno = al->bno[side];

	b = ep->bound[bno];

	if( (bp = b->bgp) == NULL )	return;


	l = al->link;


	if( al->flag & 0x10 ){
		e = l->pr[0].edge;

		e->side[side].bval[SP_Y] = F2QCOLOR(bp->val[SP_Y]);
		e->side[side].bval[SP_Q] = F2QCOLOR(bp->val[SP_Q]);
		e->side[side].bval[SP_A] = F2QCOLOR(bp->val[SP_A]);


		if( Fprofile ){
			e->side[side].val[SP_Y] = F2QCOLOR(bp->val[SP_Y]);
			e->side[side].val[SP_Q] = F2QCOLOR(bp->val[SP_Q]);
			e->side[side].val[SP_A] = F2QCOLOR(bp->val[SP_A]);
		}
	}


	if( al->flag & 0x20 && LINK_NEXT(l) == NULL ){
		e = l->pr[1].edge;

		e->side[side].bval[SP_Y] = F2QCOLOR(bp->val[SP_Y]);
		e->side[side].bval[SP_Q] = F2QCOLOR(bp->val[SP_Q]);
		e->side[side].bval[SP_A] = F2QCOLOR(bp->val[SP_A]);


		if( Fprofile ){
			e->side[side].val[SP_Y] = F2QCOLOR(bp->val[SP_Y]);
			e->side[side].val[SP_Q] = F2QCOLOR(bp->val[SP_Q]);
			e->side[side].val[SP_A] = F2QCOLOR(bp->val[SP_A]);
		}
	}
}




static void
ep_bgp_put_middle_1( gp_type *gp, ep_type *ep )
{
bgp_type	*bgp;
ebgp_type	*bp;
int	bno;
float	dis;

	for( bgp = gp->bgp ; bgp != NULL ; bgp = bgp->p ){
		bno = ep_bound_point( ep, &bgp->ctr, &dis );
		if( bno == -1 )	continue;

		bp = ep->bound[bno]->bgp;
		bgp->val[SP_Y] = F2QCOLOR(bp->val[SP_Y]);
		bgp->val[SP_Q] = F2QCOLOR(bp->val[SP_Q]);
		bgp->val[SP_A] = F2QCOLOR(bp->val[SP_A]);
	}
}
