/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/

/**************************
 ***   EpBoundBr.c	***
 **************************/

#include <stdio.h>
#include <math.h>

#include	"Uigp/igp.h"
#include        "Uprm/prm.h"


#include        "GpType/GpType.h"

#include        "Ep/Ep.h"
#include        "EpBgp.h"


static void	ep_br_set( ep_type *ep );

static void	ep_br_fill( ep_type *ep );


void
aep_br_set( ep_array *aep )
{
ep_type		*ep;

int	i,	j;


	for( i = JSIZE ; i < aep->Ni-JSIZE ; i++ )
		for( j = JSIZE ; j < aep->Nj-JSIZE ; j++ ){

			ep = &aep->ep[i][j];

			ep_br_set( ep );
		}
}


static void
ep_br_set( ep_type *ep )
{
bound_type	*b;
al_type	*al;
int	bno,	i;
int	mbound;


/*
	if( ep->al == NULL ){
		for( i = 0 ; i < ep->b_no ; i++ ){
			b = ep->bound[i];
			b->br.no = 255;
			b->br.flag = 0;
		}

		return;
	}
*/

	for( i = 0 ; i < ep->b_no ; i++ ){
		b = ep->bound[i];
		b->br.no = 255;
		b->br.flag = 0;
	}



	mbound = 0;
	for( al = ep->al ; al != NULL ; al = al->nal ){

		if( al->link->Fbr != 0 &&
				al->bno[F_RIGHT] == al->bno[F_LEFT] ){
			bno = al->bno[F_RIGHT];
			ep->bound[bno]->br.no = 255;
			mbound |= ( 1L<<bno);
			continue;
		}

		if( al->link->Fbr & 0x04 ){
			bno = al->bno[F_RIGHT];
			ep->bound[bno]->br.no = al->link->br;
			ep->bound[bno]->br.flag = 1;

			bno = al->bno[F_LEFT];
			ep->bound[bno]->br.no = al->link->br1;
			ep->bound[bno]->br.flag = 1;

			continue;
		}

		bno = al->bno[F_RIGHT];
		if( !(al->link->Fbr & (1L<< F_RIGHT)) && !(mbound &(1L<<bno)) ){
			ep->bound[bno]->br.no = al->link->br;
			ep->bound[bno]->br.flag = 1;
		}

		bno = al->bno[F_LEFT];
		if( !(al->link->Fbr & (1L<< F_LEFT)) && !(mbound &(1L<<bno)) ){
			ep->bound[bno]->br.no = al->link->br;
			ep->bound[bno]->br.flag = 1;
		}
	}
}




void
aep_br_fill( ep_array *aep )
{
ep_type		*ep;

int	i,	j;


	for( i = LSIZE ; i < aep->Ni-LSIZE ; i++ )
		for( j = LSIZE ; j < aep->Nj-LSIZE ; j++ ){

			ep = &aep->ep[i][j];

			ep_br_fill( ep );
		}
}


static void
ep_br_fill( ep_type *ep )
{
ep_type	*nep;
bound_type	*b;
int	i,	n,	k;


	for( i = 0 ; i < ep->b_no ; i++ ){
		b = ep->bound[i];
		if( b->br.no != 255 )	continue;

		for( n = 0 ; n < 9 ; n++ ){

			if( b->Fcont[n] == 0 )  continue;

			nep = ep->ng[n];

			for( k = 0 ; k < nep->b_no ; k++ ){

				if( !(b->Fcont[n] & (1<<k ) ) )
					continue;
				if( nep->bound[k]->br.no == 255 )
					continue;

				b->br.no = nep->bound[k]->br.no;
				break;
			}

			if( b->br.no != 255 )	break;
		}
	}
}
