/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
#include	<stdio.h>
#include	"ImageType/ImageType.h"


image_type *
image_create( int row, int column, int depth, int pad, u_char *data )
{
image_type	*im;

	im = ( image_type *)malloc( sizeof( image_type ) );

	im->row = row;
	im->column = column;
	im->depth = depth;

	im->column_byte = (( im->column*im->depth + pad -1 )/pad ) *pad;
	im->pad = im->column_byte - im->column*im->depth;

	if( data == NULL )
		im->data = (u_char *)malloc( im->row* im->column_byte );
	else	im->data = data;


	im->format = ( depth == 1 )? IMAGE_FORMAT_Y : IMAGE_FORMAT_RGB;


	im->file = NULL;

	im->palette = NULL;

	return( im );
}



image_type *
image_createYUV420( int height, int width, u_char *data )
{
image_type	*im;

	// width and height must be even
	if( (height & 0x01) == 0 || (width&0x01) == 0 )	return( NULL );

	im = ( image_type *)malloc( sizeof( image_type ) );

	im->row = height;
	im->column = width;
	im->depth = 2;

	im->column_byte = im->column;
	im->pad = 0;

	if( data == NULL )
		im->data = (u_char *)malloc( (int)(im->row* im->column*1.5) );
	else	im->data = data;


	im->format = IMAGE_FORMAT_YUV420;


	im->file = NULL;

	im->palette = NULL;

	return( im );
}



void
image_recreate( image_type *im, int row, int column, int depth, int pad )
{

	if( im->row == row && im->column == column && im->depth == depth )
		return;

	free( im->data );

	im->row = row;
	im->column = column;
	im->depth = depth;

	im->column_byte = (( im->column*im->depth + pad -1 )/pad ) *pad;
	im->pad = im->column_byte - im->column*im->depth;

	im->data = (u_char *)malloc( im->row* im->column_byte );


	if( im->palette != NULL ){
		palette_destroy( im->palette );
		im->palette = NULL;
	}
}




void
image_destroy( image_type *im, int Fdata )
{

	if( Fdata == 1 )
		free( im->data );

	if( im->file != NULL )
		free( im->file );

	if( im->palette != NULL )
		palette_destroy( im->palette );

	free( im );
}




image_type * 
image_make_copy( image_type *sim, image_type *im )
{

	if( im != NULL && ( im->row != sim->row || im->column != sim->column || im->depth != sim->depth ) ){
		image_destroy( im, 1 );
		im = NULL;
	}



	if( sim->format == IMAGE_FORMAT_YUV420 ){
		if( im == NULL )
			im = image_createYUV420( sim->row, sim->column, NULL );

		memcpy( im->data, sim->data, (int)(im->row* im->column*1.5) );
		return( im );
	}



	if( im == NULL )
		im = image_create( sim->row, sim->column, sim->depth, 1, NULL );

	memcpy( im->data, sim->data, im->column_byte * im->row );

	return ( im );
}