/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**************************
 ***	Image4Tool.c	***
 *************************/

#include "ImageType/ImageType.h"



image_type *
image4_sample( image_type *sim, int srow, int scol )
{
u_int	*sp,	*p;
image_type	*im;
int	i,	j,	i1,	j1;
int	r,	g,	b,	n,	k,	algin;
float	dup2 = 1.0F / (srow*scol);

	im = image_create( sim->row/srow, sim->column/scol, 4, 1, NULL );

	algin = sim->column - scol;
	
	p = (u_int *)im->data;
	for( i = 0, i1 = 0 ; i < im->row ; i++, i1 += srow )
	    for( j = 0, j1 = 0 ; j < im->column ; j++, j1 += scol ){

		sp = IMAGE4_PIXEL( sim, i1, j1 );
		r = g = b = 0;
		for( k = 0 ; k < srow ; k++, sp += algin )
			for( n = 0 ; n < scol ; n++, sp++ ){
/*
				r += ((*sp)&0xff0000);
*/
				r += ((*sp) >> 16);
				g += ((*sp)&0x00ff00);
				b += ((*sp)&0x0000ff);
			}

		r *= dup2;
		g *= dup2;
		b *= dup2;

/*
		*p++ = (r&0xff0000) | (g&0x00ff00) | (b&0x0000ff);
*/
		*p++ = (r<<16) | (g&0x00ff00) | (b&0x0000ff);
	    }


	return( im );
}



image_type *
image4_dup( image_type *sim, int drow, int dcol )
{
u_int	*sp,	*p,	*sp0;
image_type	*im;
int	i,	j,	i1,	j1;

	im = image_create( sim->row*drow, sim->column*dcol, 4, 1, NULL );

	
	p = (u_int *)im->data;
	sp0 = (u_int *)sim->data;
	for( i = 0 ; i < sim->row ; i++, sp0 += sim->column ){


		for( i1 = 0 ; i1 < drow ; i1++ ){
			sp = sp0;
			for( j = 0 ; j < sim->column ; j++, sp++ )
				for( j1 = 0 ; j1 < dcol ; j1++ )
					*p++ = *sp;
		}
	}

	return( im );
}

/*
image_type *
image4_subtract( image_type *sim1, image_type *sim0 )
{
u_int	*sp0,	*sp1;
u_int	*p;
image_type	*im;
int	row,	col,	i,	j;
int	r,	g,	b,	tmp;

	row = MIN( IMAGE_ROW(sim0), IMAGE_ROW(sim1) ); 
	col = MIN( IMAGE_COLUMN(sim0), IMAGE_COLUMN(sim1) ); 

	im = image_create( row, col, 4, 1, NULL );

	sp0 = (u_int *)sim0->data;
	sp1 = (u_int *)sim1->data;
	p = (u_int *)im->data;
	for( i = 0 ; i < im->row ; i++ ){
		for( j = 0 ; j < im->column ; j++ ){
			tmp = IMAGE4_RED(*sp1) - IMAGE4_RED(*sp0) + 128;
			r = PUSH_TO_RANGE( tmp, 0, 255 );

			tmp = IMAGE4_GREEN(*sp1) - IMAGE4_GREEN(*sp0) + 128;
			g = PUSH_TO_RANGE( tmp, 0, 255 );

			tmp = IMAGE4_BLUE(*sp1) - IMAGE4_BLUE(*sp0) + 128;
			b = PUSH_TO_RANGE( tmp, 0, 255 );

			sp0++;
			sp1++;

			*p++ = IMAGE4_RGB( r, g, b );
		}
	}

	return( im );
}
*/






void
image4_mark( image_type *im, int row0, int col0, int row, int col, int color )
{
int	i,	j;

	for( i = 0 ; i < row ; i++ ){
		*IMAGE4_PIXEL( im, row0 + i, col0 ) = color;
		*IMAGE4_PIXEL( im, row0 + i, col0+col ) = color;
	}


	for( j = 0 ; j < col ; j++ ){
		*IMAGE4_PIXEL( im, row0, col0 + j ) = color;
		*IMAGE4_PIXEL( im, row0 + row, col0 + j ) = color;
	}
}



void
image4_const( image_type *im, int color )
{
u_int	*p;
int	i,	j;

	p = (u_int *)im->data;
	for( i = 0 ; i < im->row ; i++ )
		for( j = 0 ; j < im->column ; j++ )
			*p++ = color;
}



void
image4_Vflip( image_type *im )
{
u_int	*p0,	*p1,	tmp;
int	i,	j,	hrow;

	hrow = im->row / 2;
	for( i = 0 ; i < hrow  ; i++ ){

		p0 = IMAGE4_PIXEL( im, i, 0 );
		p1 = IMAGE4_PIXEL( im, im->row-1-i, 0 );
		for( j = 0 ; j < im->column ; j++, p0++, p1++ ){
			tmp = *p0;
			*p0 = *p1;
			*p1 = tmp;
		}
	}
}


image_type *
image4_Vflip2( image_type *sim, image_type *tim)
{
u_int	*p0,	*p1,	tmp;
int	i,	j,	hrow;

	if ( tim == NULL )
		tim = image_create( sim->row, sim->column, 4, 1, NULL );
	
	p0 = (u_int *)sim->data;
	p1 = (u_int *)tim->data;

	for ( i = 0 ; i < sim->row ; i++ )
		for ( j = 0 ; j < sim->column ; j++)
			*p1++ = *p0++;

	hrow = tim->row / 2;
	for( i = 0 ; i < hrow  ; i++ ){

		p0 = IMAGE4_PIXEL( tim, i, 0 );
		p1 = IMAGE4_PIXEL( tim, tim->row-1-i, 0 );
		for( j = 0 ; j < tim->column ; j++, p0++, p1++ ){
			tmp = *p0;
			*p0 = *p1;
			*p1 = tmp;
		}
	}
	return ( tim );
}


image_type *
image4_Hflip2( image_type *sim, image_type *tim)
{
u_int	*p0,	*p1,	tmp;
int	i,	j,	hcol;

	if ( tim == NULL )
		tim = image_create( sim->row, sim->column, 4, 1, NULL );
	
	p0 = (u_int *)sim->data;
	p1 = (u_int *)tim->data;
	
	for ( i = 0 ; i < sim->row ; i++ )
		for ( j = 0 ; j < sim->column ; j++)
			*p1++ = *p0++;

	hcol = tim->column / 2;
	for( i = 0 ; i < tim->row  ; i++ ){

		p0 = IMAGE4_PIXEL( tim, i, 0 );
		p1 = IMAGE4_PIXEL( tim, i, tim->column-1 );
		for( j = 0 ; j < hcol ; j++, p0++, p1-- ){
			tmp = *p0;
			*p0 = *p1;
			*p1 = tmp;
		}
	}
	return ( tim );
}


image_type *
image4_rotate90( image_type *sim, image_type *tim )
{
u_int	*sp,	*tp;
int	i,	j;

	if ( tim == NULL )
		tim = image_create( sim->column, sim->row, 4, 1, NULL );
	
	sp = (u_int *)sim->data;
	for ( i = 0 ; i < sim->row ; i++ ){
		tp = IMAGE4_PIXEL( tim, tim->row-1, i );

		for ( j = 0 ; j < sim->column ; j++){
			*tp = *sp++;
			tp -= IMAGE_COLUMN( tim );
		}
	}

	return( tim );

}


image_type *
image4_rotate180( image_type *sim, image_type *tim)
{
u_int	*p0,	*p1,	tmp;
int	i,	j;

	if ( tim == NULL )
		tim = image_create( sim->row, sim->column, 4, 1, NULL );
	
	p0 = (u_int *)sim->data;
	p1 = (u_int *)tim->data;

	for( i = 0 ; i < tim->row; i++ ){

		p1 = IMAGE4_PIXEL( tim, tim->row-i-1, tim->column-1 );
		for( j = 0 ; j < tim->column ; j++, p0++, p1-- ){
			tmp = *p0;
			*p0 = *p1;
			*p1 = tmp;
		}
	}
	return ( tim );
}


image_type *
image4_rotate270( image_type *sim, image_type *tim )
{
u_int	*sp,	*tp;
int	i,	j;

	if ( tim == NULL )
		tim = image_create( sim->column, sim->row, 4, 1, NULL );
	
	sp = (u_int *)sim->data;
	for ( i = 0 ; i < sim->row ; i++ ){
		//tp = IMAGE4_PIXEL( tim, 0, i );
		tp = IMAGE4_PIXEL( tim, 0, sim->row-i-1 );

		for ( j = 0 ; j < sim->column ; j++){
			*tp = *sp++;
			tp += IMAGE_COLUMN( tim );
		}
	}

	return( tim );

}


void
image4_copy_in( image_type *sim, image_type *im, int row0, int col0 )
{
u_int	*p,	*sp;
int	i,	j,	algin;

	
	algin = im->column - sim->column;
	sp = (u_int *)sim->data;
	p = IMAGE4_PIXEL( im, row0, col0 );

	for( i = 0 ; i < sim->row ; i++, p += algin ){
		for( j = 0 ; j < sim->column ; j++ )
			*p++ = *sp++;
	}
}

image_type * 
image4_copy( image_type *sim )
{
image_type	*im;
u_int	*p,	*sp;
int	i,	j;

	im = image_create( sim->row, sim->column, 4, 1, NULL );
	sp = (u_int *)sim->data;
	p = (u_int *)im->data;

	for( i = 0 ; i < sim->row ; i++){
		for( j = 0 ; j < sim->column ; j++ )
			*p++ = *sp++;
	}

	return ( im );
}



image_type *
image4_to_red( image_type *sim )
{
u_int	*sp;
u_char	*p;
image_type	*im;
int	row,	col,	i,	j;

	row = IMAGE_ROW(sim);
	col = IMAGE_COLUMN(sim);

	im = image_create( row, col, 1, 1, NULL );

	sp = (u_int *)sim->data;
	p = (u_char *)im->data;
	for( i = 0 ; i < im->row ; i++ ){
		for( j = 0 ; j < im->column ; j++ ){
			*p++ = IMAGE4_RED(*sp);
			sp++;
		}
	}

	return( im );
}

image_type *
image4_to_green( image_type *sim )
{
u_int	*sp;
u_char	*p;
image_type	*im;
int	row,	col,	i,	j;

	row = IMAGE_ROW(sim);
	col = IMAGE_COLUMN(sim);

	im = image_create( row, col, 1, 1, NULL );

	sp = (u_int *)sim->data;
	p = (u_char *)im->data;
	for( i = 0 ; i < im->row ; i++ ){
		for( j = 0 ; j < im->column ; j++ ){
			*p++ = IMAGE4_GREEN(*sp);
			sp++;
		}
	}

	return( im );
}

image_type *
image4_to_blue( image_type *sim )
{
u_int	*sp;
u_char	*p;
image_type	*im;
int	row,	col,	i,	j;

	row = IMAGE_ROW(sim);
	col = IMAGE_COLUMN(sim);

	im = image_create( row, col, 1, 1, NULL );

	sp = (u_int *)sim->data;
	p = (u_char *)im->data;
	for( i = 0 ; i < im->row ; i++ ){
		for( j = 0 ; j < im->column ; j++ ){
			*p++ = IMAGE4_BLUE(*sp);
			sp++;
		}
	}

	return( im );
}


image_type *
image4_to_y( image_type *sim )
{
u_int	*sp;
u_char	*p;
image_type	*im;
int	row,	col,	i,	j;
int	R,	G,	B,	Y;

	row = IMAGE_ROW(sim);
	col = IMAGE_COLUMN(sim);

	im = image_create( row, col, 1, 1, NULL );

	sp = (u_int *)sim->data;
	p = (u_char *)im->data;
	for( i = 0 ; i < im->row ; i++ ){
		for( j = 0 ; j < im->column ; j++ ){
			R = IMAGE4_RED(*sp);
			G = IMAGE4_GREEN(*sp);
			B = IMAGE4_BLUE(*sp);
			sp++;

			Y  =  0.29900 * R + 0.58700 * G + 0.11400 * B + 0.5;

			*p++ = PUSH_TO_RANGE( Y, 0, 255 );

		}
	}

	return( im );
}


image_type *
image4_from_y( image_type *sim )
{
u_char	*sp;
u_int	*p;
image_type	*im;
int	row,	col,	i,	j;
int	y;

	row = IMAGE_ROW(sim);
	col = IMAGE_COLUMN(sim);

	im = image_create( row, col, 4, 1, NULL );

	sp = (u_char *)sim->data;
	p = (u_int *)im->data;
	for( i = 0 ; i < im->row ; i++ ){
		for( j = 0 ; j < im->column ; j++ ){

			y = *sp++;
			*p++ = IMAGE4_RGB( y, y, y );
		}
	}

	return( im );
}
