/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**************************
 ***	ImageCut.c	***
 *************************/

#include "ImageType/ImageType.h"


static image_type *	image1_cut( image_type *sim,
				int row0, int col0, int row, int col );


image_type *
image_cut( image_type *sim, int row0, int col0, int row, int col )
{
image_type	*im;
 
        switch( sim->depth ){
        case 1:
                im = image1_cut( sim, row0, col0, row, col );
            break;
 
        case 4:
                im = image4_cut( sim, row0, col0, row, col );
            break;
        }

	return( im );
} 




image_type *
image4_cut( image_type *sim, int row0, int col0, int row, int col )
{
u_int	*sp,	*p;
image_type	*im;
int	i,	j;
int	row1,	col1;
int	r,	c,	r0,	c0,	rr0,	cc0;

	im = image_create( row, col, 4, 1, NULL );

	if( row0 >= IMAGE_ROW(sim ) || col0 >= IMAGE_COLUMN(sim) ){
		p = (u_int *)im->data;
		for( i = 0 ; i < row ; i++ )
			for( j = 0 ; j < col ; j++ )
				*p++ = 0;

		return( im );
	}


	r0 = ( row0 < 0 )? 0 : row0;
	rr0 = r0 - row0;
	
	if( (row1 = row0 + row) > IMAGE_ROW(sim) )
		row1 = IMAGE_ROW(sim);
	r = row1 - r0;

	c0 = ( col0 < 0 )? 0 : col0;
	cc0 = c0 - col0;
	if( ( col1 = col0 + col) > IMAGE_COLUMN(sim) )
		col1 = IMAGE_COLUMN(sim);
	c = col1 - c0;




	for( i = 0 ; i < r ; i++ ){
		sp = IMAGE4_PIXEL( sim, r0+i, c0 );
		p = IMAGE4_PIXEL( im, rr0+i, cc0 );

		for( j = 0 ; j < c ; j++ )
			*p++ = *sp++;
	}

	if( row0 < 0 ){
		p = (u_int *)im->data;
		for( i = 0 ; i < rr0 ; i++ )
			for( j = 0 ; j < col ; j++ )
				*p++ = 0;
	}

	if( r < row ){
		p = IMAGE4_PIXEL( im, rr0+r, 0 );
		for( i = rr0 +r ; i < row ; i++ )
			for( j = 0 ; j < col ; j++ )
				*p++ = 0;
	}

	if( col0 < 0 )
		for( i = 0 ; i < r ; i++ ){
			p = IMAGE4_PIXEL( im, rr0+i, 0 );

			for( j = 0 ; j < cc0 ; j++ )
				*p++ = 0;
		}

	if( c < col )
		for( i = 0 ; i < r ; i++ ){
			p = IMAGE4_PIXEL( im, rr0+i, cc0+c );

			for( j = cc0 + c ; j < col ; j++ )
				*p++ = 0;
		}


	return( im );
}




static image_type *
image1_cut( image_type *sim, int row0, int col0, int row, int col )
{
u_char	*sp,	*p;
image_type	*im;
int	i,	j;
int	row1,	col1;
int	r,	c,	r0,	c0,	rr0,	cc0;

	im = image_create( row, col, 1, 1, NULL );

	if( row0 >= IMAGE_ROW(sim ) || col0 >= IMAGE_COLUMN(sim) ){
		p = (u_char *)im->data;
		for( i = 0 ; i < row ; i++ )
			for( j = 0 ; j < col ; j++ )
				*p++ = 0;

		return( im );
	}


	r0 = ( row0 < 0 )? 0 : row0;
	rr0 = r0 - row0;
	
	if( (row1 = row0 + row) > IMAGE_ROW(sim) )
		row1 = IMAGE_ROW(sim);
	r = row1 - r0;

	c0 = ( col0 < 0 )? 0 : col0;
	cc0 = c0 - col0;
	if( ( col1 = col0 + col) > IMAGE_COLUMN(sim) )
		col1 = IMAGE_COLUMN(sim);
	c = col1 - c0;




	for( i = 0 ; i < r ; i++ ){
		sp = IMAGE_PIXEL( sim, r0+i, c0 );
		p = IMAGE_PIXEL( im, rr0+i, cc0 );

		for( j = 0 ; j < c ; j++ )
			*p++ = *sp++;
	}

	if( row0 < 0 ){
		p = (u_char *)im->data;
		for( i = 0 ; i < rr0 ; i++ )
			for( j = 0 ; j < col ; j++ )
				*p++ = 0;
	}

	if( r < row ){
		p = IMAGE_PIXEL( im, rr0+r, 0 );
		for( i = rr0 +r ; i < row ; i++ )
			for( j = 0 ; j < col ; j++ )
				*p++ = 0;
	}

	if( col0 < 0 )
		for( i = 0 ; i < r ; i++ ){
			p = IMAGE_PIXEL( im, rr0+i, 0 );

			for( j = 0 ; j < cc0 ; j++ )
				*p++ = 0;
		}

	if( c < col )
		for( i = 0 ; i < r ; i++ ){
			p = IMAGE_PIXEL( im, rr0+i, cc0+c );

			for( j = cc0 + c ; j < col ; j++ )
				*p++ = 0;
		}


	return( im );
}
