/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/*************************
 ***	ImageExtend.c   ***
 *************************/

#include "ImageType/ImageType.h"



/*
Image_extend: add to the image a colored framework
parameters:
row - new heigth of image
column - new width of image
*/

image_type *
image_extend( image_type *im, int row, int column, int color, image_type *rim )
{
u_int	*sp,	*tp;
int	row_diff,	column_diff;
int	i,	j;

	if ( rim != NULL && ( rim->row != row || rim->column != column) ){
		image_destroy( rim, 1 );
		rim = NULL;
	}


	if( rim == NULL )
		rim = image_create(row, column, 4, 1, NULL );



	row_diff = (row - im->row)/2;
	column_diff = (column - im->column)/2;


	sp = (u_int *)im->data;
	tp = (u_int *)rim->data;
	
	for ( i = 0 ; i < rim->row ; i++ )
		for ( j = 0 ; j < rim->column ; j++ )
			*tp++ = color;

	image4_copy_in( im, rim, row_diff, column_diff );

	return ( rim );
}

/*
Image_border_rect: add to the image a colored framework, within its borders
parameters:
cycle: the depth of the framework of the image
r0, c0 - the top left point of the framework
row0, col0 - width and height of framework
*/
image_type *    
image_border_rect(image_type * im, int color, int cycle,
				  int r0, int c0, int row0, int col0)
{
int	col = IMAGE_COLUMN(im);
unsigned int *pixel;
int	i,	j;

	if(im == NULL)
		return NULL;
	

	for( i = 1; i <= cycle; i++ )
	{
		// rows
		pixel = IMAGE4_PIXEL( im, r0-i, c0-i);
		for (j = c0-i; j <= c0+col0+i; j++)
			*pixel++ = color;
		pixel = IMAGE4_PIXEL( im, r0+row0+i, c0-i);
		for (j = c0-i; j <= c0+col0+i; j++)
			*pixel++ = color;

		// columns
		pixel = IMAGE4_PIXEL( im, r0-i+1, c0-i );
		for (j = r0-i+1; j < r0+row0+i; j++){
			*pixel = color;
			pixel += col;
		}
		pixel = IMAGE4_PIXEL( im, r0-i+1, c0+col0+i );
		for (j = r0-i+1; j < r0+row0+i; j++){
			*pixel = color;
			pixel += col;
		}
	}


	return( im );
}

