/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**************************
 ***	ImageSubtruct.c	***
 *************************/

#include	"ImageType/ImageType.h"

static image_type *	image1_subtract( image_type *sim1, image_type *sim0 );


image_type *
image_subtract( image_type *sim1, image_type *sim0 )
{
image_type	*im;
 
        switch( sim1->depth ){
        case 1:
                im = image1_subtract( sim1, sim0 );
            break;
 
        case 4:
                im = image_subtract( sim1, sim0 );
            break;
        }

	return( im );
} 


static image_type *
image1_subtract( image_type *sim1, image_type *sim0 )
{
u_char	*sp0,	*sp1;
u_char	*p;
image_type	*im;
int	row,	col,	i,	j;
int	tmp;

	row = MIN( IMAGE_ROW(sim0), IMAGE_ROW(sim1) ); 
	col = MIN( IMAGE_COLUMN(sim0), IMAGE_COLUMN(sim1) ); 

	im = image_create( row, col, 1, 1, NULL );

	p = (u_char *)im->data;
	for( i = 0 ; i < im->row ; i++ ){
		sp0 = IMAGE_PIXEL( sim0, i, 0 );
		sp1 = IMAGE_PIXEL( sim1, i, 0 );
		for( j = 0 ; j < im->column ; j++ ){
			tmp = (*sp1) - (*sp0) + 128;
			*p++ = PUSH_TO_RANGE( tmp, 0, 255 );


			sp0++;
			sp1++;
		}
	}

	return( im );
}



image_type *
image4_subtract( image_type *sim1, image_type *sim0 )
{
u_int	*sp0,	*sp1;
u_int	*p;
image_type	*im;
int	row,	col,	i,	j;
int	r,	g,	b,	tmp;

	row = MIN( IMAGE_ROW(sim0), IMAGE_ROW(sim1) ); 
	col = MIN( IMAGE_COLUMN(sim0), IMAGE_COLUMN(sim1) ); 

	im = image_create( row, col, 4, 1, NULL );

	p = (u_int *)im->data;
	for( i = 0 ; i < im->row ; i++ ){
		sp0 = IMAGE4_PIXEL( sim0, i, 0 );
		sp1 = IMAGE4_PIXEL( sim1, i, 0 );

		for( j = 0 ; j < im->column ; j++ ){
			tmp = IMAGE4_RED(*sp1) - IMAGE4_RED(*sp0) + 128;
			r = PUSH_TO_RANGE( tmp, 0, 255 );

			tmp = IMAGE4_GREEN(*sp1) - IMAGE4_GREEN(*sp0) + 128;
			g = PUSH_TO_RANGE( tmp, 0, 255 );

			tmp = IMAGE4_BLUE(*sp1) - IMAGE4_BLUE(*sp0) + 128;
			b = PUSH_TO_RANGE( tmp, 0, 255 );

			sp0++;
			sp1++;

			*p++ = IMAGE4_RGB( r, g, b );
		}
	}

	return( im );
}


