/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
#ifndef         _IMAGE_TOOL_
#define         _IMAGE_TOOL_


#ifdef __cplusplus
extern "C" {
#endif

#include	"Uigp/igp.h"
#include	"Umat/Mat2fType.h"
#include	"Uvec/Vec2d.h"
#include	"Ubox/Box2d.h"


#define	image_destory	image_destroy


	/* ImageAux.c */
void	image_box(image_type *im, box2i *b, int *i0, int *j0, int *row, int *column );


	/* Image4to1.c */
image_type *	image_1to4( image_type *sim );
image_type *	image_4to1( image_type *sim );


	/* Image4Tool.c */
image_type *	image4_sample( image_type *sim, int srow, int scol );

image_type *	image4_dup( image_type *sim, int drow, int dcol );


void		image4_mark( image_type *im, int row0, int col0,
					int row, int col, int color );

void		image4_const( image_type *im, int color );

void		image4_Vflip( image_type *im );

image_type *	image4_Vflip2( image_type *sim, image_type *tim );

image_type *    image4_Hflip2( image_type *sim, image_type *tim );

void		image4_copy_in( image_type *sim,
				image_type *im, int row0, int col0 );

image_type * 	image4_copy( image_type *sim );

image_type *	image4_to_red( image_type *sim );

image_type *	image4_to_green( image_type *sim );

image_type *	image4_to_blue( image_type *sim );

image_type *	image4_to_y( image_type *sim );

image_type *	image4_from_y( image_type *sim );

image_type *	image4_rotate90( image_type *sim, image_type *tim );

image_type *	image4_rotate180( image_type *sim, image_type *tim);

image_type *	image4_rotate270( image_type *sim, image_type *tim );


	/* ImageResample.c */
image_type *	image_resample( image_type *sim, int row, int col );


	/* ImageSample.c - float version grisha*/ 
image_type *	image_sample( image_type *sim, float srow, float scol );

image_type *	image_sample_transparent( image_type *sim, float srow, float scol );





	/* ImageSamplingF.c - two pass float */
image_type * image_samplefc( image_type *sim, int drow, int dcol );


	/* ImageSampleDown.c */
image_type * image_sample_down( image_type *sim, int drow, int dcol );

	/* ImageSampleDownTransparent.c */
image_type * image_sample_down_transparent( image_type *sim, int drow, int dcol );
void image_sample_down_fix_transparent( image_type *im );


	/* ImageCut.c */
image_type *	image_cut( image_type *sim,
			int row0, int col0, int row, int col );

image_type *	image4_cut( image_type *sim,
			int row0, int col0, int row, int col );

	/* ImageSubtruct.c */
image_type *	image_subtract( image_type *sim1, image_type *sim0 );
image_type *	image4_subtract( image_type *sim1, image_type *sim0 );


	/* ImageRgbYCrCb.c */
image_type *	image_Rgb_YCrCb( image_type *im, int Falloc );


	/* ImageRgbYiq.c */
image_type *image_Rgb_Yiq( image_type *im, int Falloc );


	/* ImageComparison */
void	image_comparison( image_type *im0, image_type *im1,
                float var[],    /* Variabilty */
                float dev[],
                float snr[],    /* Signal to noise ratio */
                float average[],
                int peak[] );

void	image4_comparisonG( image_type *im0, image_type *im1, float *var, float *dev, float *average );


	/* ImageEqual.c   */
int		image_equal( image_type *im0, image_type *im1 );


	/* ImageCutUarray.c */
void	image4_cut_uarray( image_type *im,
                        int x0, int y0, int row, int column,
                        u_char *aY,
                        u_char *aQ,
                        u_char *aA,
                        int acolumn);



	/* ImageFile.c */
void	image_file_attach_name( image_type *im,
			char *dir, char *name, char *extension );

void	image_file_attach_nameF( image_type *im, char *fname );

int	image_file_size( image_type *im );



        /* ImagePreview.c */
image_type *    image_preview( image_type *im, int size, int color );


        /* ImageTransparent.c */
image_type *    image_transparent(image_type * sim,
                float trans, int backcolor, image_type *im); 

void	image_transparent_set(image_type *bim, box2i *b,
						image_type *im, int transpart_index );

void	image_transparent_copy_in(image_type *bim, box2i *b,
						image_type *im, int transpart_index );

void	imageT_clear( image_type *im, int color );

image_type *    image_border_fade(image_type * sim,
								float trans, int backcolor,
								int r0, int c0, int row, int column, image_type *im); 


	/* ImageHistogram.c */
int	image_histogram( image_type *im, int h[256] );

image_type *	image_histogram_equlization( image_type *sim );

image_type *	image_histogram_curve( image_type *sim, 
		float partL, int limL, float partR, int limR );


	/* ImageConvolution.c */
image_type *	image_convolution( image_type *sim, mat2f_type *m );
float	image_mat2f_inner( image_type *im, int i0, int j0, mat2f_type *m );


	/* ImageEdgePixel.c */
image_type *	image_edge_pixel( image_type *sim, mat2f_type *m, float T );
image_type *	image_edge_pixel_value( image_type *sim, mat2f_type *m, float a, float b );

image_type *	image_edge_gradient( image_type *sim, mat2f_type *mx, mat2f_type *my, float T );
image_type *	image_edge_gradient_value( image_type *sim, mat2f_type *mx, mat2f_type *my, float a, float b );


	/*ImageSline.c */
image_type *	image_sline(  vec2d *n, float d0, float d1, float a,
							int c0, int c1, int c2, int row, int col );
image_type *	image_S_cosinos(  float a, float n, int row, int col );





	/* ImageFourier.c */
image_type *	image_fourier_amplitod( image_type *im ); 

image_type *	image_fourier(image_type *im);

image_type *	image_fourier_inverse(image_type *im);



    /* Image24To8.c */
image_type * image_24to8(image_type * im, palette_type * palette, box2i * pBox );

image_type * image_24to8_UNKNOWN(image_type * im, palette_type * palette, box2i * pBox);

image_type * image_24to8_smart(image_type * im, palette_type * palette, box2i * pBox);

image_type * image_24to8_222(image_type * sim, palette_type * p, box2i * pBox);

image_type * image_24to8_gray(image_type * sim, palette_type * p, box2i * pBox);

image_type * image_24to8_HALFTONE(image_type * sim, palette_type * p, box2i * pBox);

image_type * image_24to8_BW(image_type * im, palette_type * p, box2i * pBox);

image_type * image_24to8_WB(image_type * im, palette_type * p, box2i * pBox);

image_type *	image_24to8_ADAPTIVE4(image_type * im, palette_type * palette, box2i * pBox);

void    image_difference(image_type * im1, image_type * im2, box2i * b);


    /* Image24To8DitherFloyd.c */
image_type *	image_24to8_dithering_floyd(image_type * im, palette_type * palette, box2i * pBox );

image_type *	image_24to8_UNKNOWN_dithering_floyd(image_type * im, palette_type * palette, box2i * pBox );

image_type *	image_24to8_smart_dithering_floyd(image_type * im, palette_type * palette, box2i * pBox );

image_type *	image_24to8_222_dithering_floyd(image_type * im, palette_type * palette, box2i * pBox );

image_type *	image_24to8_gray_dithering_floyd(image_type * sim, palette_type * p, box2i * pBox);

image_type *	image_24to8_HALFTONE_dithering_floyd(image_type * im, palette_type * palette, box2i * pBox );

image_type *	image_color_222_dithering_floyd( image_type *im, image_type *new_im );


    /* ImageDitherFloyd.c */
image_type *	image_dithering_floyd(image_type * im, image_type * new_im);


	/*	Image8To24.c */
image_type *	image_8to24( image_type *im, palette_type *palette );
image_type *	image_8to24_transparent( image_type *sim, int transparent_index, palette_type *p );

void	image_8to24_copy( image_type *sim, image_type *im, 
						 int row0, int col0, palette_type *p );
void	image_8to24_copy_transparent( image_type *sim, image_type *im, 
									 int row0, int col0, int transparent_index, palette_type *p);


	/* ImageSmartPalette.c */
palette_type *	image_createSmartPalette(int n, image_type * im );

palette_type *	image_createSmartPaletteTransparent( int n, int nImages, image_type *images[] );
palette_type *	image_createSmartPaletteTransparent2( int n, int nImages, image_type *images[] );


	/* ImageAdaptivePalette.c */
void	image_adaptive_palette( image_type *images[], int nImage,
					   palette_type *palette, int nColor );


	/* ImageDifference.c */
void	image_difference_rectangle( image_type *im0, image_type *im1,
							int *i0, int *i1, int *j0, int *j1 );

void	image1_difference_rectangle( image_type *im0, image_type *im1,
							int *i0, int *i1, int *j0, int *j1 );


void	image_color_rectangle( image_type *im, int color,
							  int *i0, int *i1, int *j0, int *j1 );



	/* ImageExtend.c */
image_type *	image_extend( image_type *im, int row, int column, int color, image_type *rim );
image_type *	image_border_rect(image_type * im, int backcolor, int cycle, 
				  int r0, int c0, int row0, int col0);


	/* ImageEnhancement.c */
#define ENHANCEMENT_OBJECT		0
#define ENHANCEMENT_TEXT		1
#define ENHANCEMENT_BACKGROUND	2

image_type *	image_enhancement( image_type *sim, int enhan_type );


	/* ImageGrayEnhancement.c */
image_type *	image_gray_enhancement( image_type *sim, int enhan_type );


	/* ImageNegative.c		   */
void image_negative( image_type *im );


	/* ImageResize.c		   */
image_type *	image_resize( image_type *imd, image_type *ims, int row, int col );

	/* ImageHsiTune.c*/
image_type *	image_HSI_tune( image_type *sim, float HUE, float Saturation, float Intensity );
float	image_Saturation( image_type *sim );

	/* ImageLogo.c */
void image_add_logo( image_type *sim, image_type *logo, int x, int y );


#ifdef __cplusplus
}
#endif

#endif
