/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
#ifndef         _IMAGE_
#define         _IMAGE_


#ifdef __cplusplus
extern "C" {
#endif

#include	"Uigp/igp.h"


#ifdef WIN32
#pragma warning(disable :4244)
#endif



//typedef	unsigned char   u_char;


#include	"PaletteType.h"


#define	IMAGE_FORMAT_RGB	1
#define	IMAGE_FORMAT_Y		2
#define	IMAGE_FORMAT_YUV420	3



typedef struct	image_type {

	int	column;	//width
	int	row;	//height

	int	depth;	// byte per pixel
	int	pad;	// num of additional bytes at the end of the line (complete to 4*n bytes, so we read a whole int)

	int	column_byte;	/* line length : depth*column + pad */

	int	format;

	u_char	*data;

	palette_type	*palette;

	char	*file;
} image_type;


#define		IMAGE_PIXEL( im, row, column )\
		( (im)->data + (row)*(im)->column_byte +\
		(column)*(im)->depth )

#define		IMAGE4_PIXEL( im, row, col ) \
		( ((u_int *)(im)->data) + (row)*(im)->column + (col) )



#define		IMAGE_ROW( im )	((im)->row)
#define		IMAGE_COLUMN( im )	((im)->column)
#define		IMAGE_DATA_ADRESS( im )	((im)->data )

#define		IMAGE_DATA( im )	((im)->data )
#define		IMAGE4_DATA( im )	((u_int *)((im)->data) )


#define		IMAGE_DATA_SIZE( im )   ( IMAGE_ROW( im )* IMAGE_COLUMN( im ) )



/* This defination is only for im->depth == 4 */

#define         SP_0    SP_R
#define         SP_1    SP_G
#define         SP_2    SP_B




#ifdef WIN32
#define		IMAGE4_RED( pixel )		( ((pixel) >> 16) &0xff )
#define		IMAGE4_GREEN( pixel )	( ((pixel) >> 8 ) &0xff )
#define		IMAGE4_BLUE( pixel )	(  (pixel)&0xff )

#define		IMAGE4_RGB( r, g, b ) 	(((r)<<16) | ((g)<<8) | (b) )
#endif


#ifdef __linux
#define		IMAGE4_RED( pixel )		( ((pixel) >> 16) &0xff )
#define		IMAGE4_GREEN( pixel )	( ((pixel) >> 8 ) &0xff )
#define		IMAGE4_BLUE( pixel )	(  (pixel)&0xff )

#define		IMAGE4_RGB( r, g, b ) 	(((r)<<16) | ((g)<<8) | (b) )
#endif



#ifdef PALMOS

#define		IMAGE4_BLUE( pixel )	(( (pixel) >> 16 ) &0xff )
#define		IMAGE4_GREEN( pixel )	( ((pixel) >> 8 ) &0xff )
#define		IMAGE4_RED( pixel )		(  (pixel)&0xff )

#define		IMAGE4_RGB( r, g, b ) 	(((b)<<16) | ((g)<<8) | (r) )
#endif


#ifndef IMAGE4_RGB

#define		IMAGE4_BLUE( pixel )	( ((pixel) >> 16 ) &0xff )
#define		IMAGE4_GREEN( pixel )	( ((pixel) >> 8 ) &0xff )
#define		IMAGE4_RED( pixel )		(  (pixel)&0xff )

#define		IMAGE4_RGB( r, g, b ) 	(((b)<<16) | ((g)<<8) | (r) )
#endif




	/* Image.c */
image_type *	image_create( int row, int column,
						int depth, int pad, u_char *data );

image_type *	image_createYUV420( int height, int width, u_char *data );


void	image_recreate( image_type *im, int row, int column,
						int depth, int pad );

void	image_destroy( image_type *im, int Fdata );



image_type *	image_make_copy( image_type *sim, image_type *im );



#include	"ImageTool.h"


#ifdef __cplusplus
}
#endif

#endif
