/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
#ifndef __PALETTE_TYPE_HASH_H__
#define __PALETTE_TYPE_HASH_H__

#include "PaletteType.h"





/*
Minor : (binary)RRRGGGBBB where each RRR,GGG,BBB is 3 MSB of the red/green/blue value.
*/



//*

#define BITS_PER_BASE_COLOR 3
#define RED_MASK (0x1CD)
#define GREEN_MASK (0x038)
#define BLUE_MASK (0x007)
#define RGB_SHIFT_RIGHT (8 - (BITS_PER_BASE_COLOR))
#define RED_SHIFT_LEFT ((BITS_PER_BASE_COLOR) + (BITS_PER_BASE_COLOR))
#define HALF_RANGE (1 << (8 - BITS_PER_BASE_COLOR - 1))
#define FULL_RANGE (1 << (8 - BITS_PER_BASE_COLOR ))
#define GREEN_SHIFT_LEFT (BITS_PER_BASE_COLOR)

#define RGB2MINOR(r,g,b) (	(((r & 0xFF) >> (RGB_SHIFT_RIGHT))	<< (RED_SHIFT_LEFT)) | \
							(((g & 0xFF)>>(RGB_SHIFT_RIGHT) ) << (GREEN_SHIFT_LEFT)) | \
							((b & 0xFF)>>(RGB_SHIFT_RIGHT))  )


#define MINOR2R(minor) (((minor & RED_MASK)>>(RED_SHIFT_LEFT)) << (RGB_SHIFT_RIGHT))
#define MINOR2G(minor) (((minor & GREEN_MASK)>>(GREEN_SHIFT_LEFT)) << (RGB_SHIFT_RIGHT))
#define MINOR2B(minor) ((minor & BLUE_MASK) << (RGB_SHIFT_RIGHT) )


#define MINOR2R_MIDDLE(minor) (MINOR2R(minor) + (HALF_RANGE))
#define MINOR2G_MIDDLE(minor) (MINOR2G(minor) + (HALF_RANGE))
#define MINOR2B_MIDDLE(minor) (MINOR2B(minor) + (HALF_RANGE))




#define RED_UPPEST RED_MASK
#define GREEN_UPPEST GREEN_MASK
#define BLUE_UPPEST BLUE_MASK

#define RED_UP_1_FLOOR(minor) (minor + ((1 << BITS_PER_BASE_COLOR)*(1 << BITS_PER_BASE_COLOR)))
#define RED_DOWN_1_FLOOR(minor) (minor - ((1<<BITS_PER_BASE_COLOR)*(1<<BITS_PER_BASE_COLOR)))
#define GREEN_UP_1_FLOOR(minor) (minor + (1<<BITS_PER_BASE_COLOR))
#define GREEN_DOWN_1_FLOOR(minor) (minor - (1<<BITS_PER_BASE_COLOR))
#define BLUE_UP_1_FLOOR(minor) (minor + 1)
#define BLUE_DOWN_1_FLOOR(minor) (minor - 1)


/*

Major : (binary)RGB where each r,g,b, is 1 if high value, or 0 if low value.
Minor : (binary)RRGGBB where each RR,GG,BB is 0,01,10,11 according to it's value.

	
#define MINOR2MAJOR(minor) ( (((minor)>>1) & 1) + (((minor)>>3) & 1) + (((minor)>>5) & 1))
#define MAJOR2MINOR(major, sub_index) 	\
		(	\
			(((major) & 4)<<3) | (((sub_index) & 4) << 2) | 	\
			(((major) & 2)<<2) | (((sub_index) & 2) << 1) |		\
			(((major) & 1)<<1) | (((sub_index) & 1) )			\
		)




#define RGB2MINOR(r,g,b) (	(((r) >> 6)	<< 4) |  (((g)>>6 ) << 2) | ((b)>>6)  )
#define MINOR2R_MIDDLE(minor) ((((minor)>>4) << 6)+ 0x1F)
#define MINOR2G_MIDDLE(minor) ((((minor)>>2) << 6) + 0x1F)
#define MINOR2B_MIDDLE(minor) (((minor) << 6) + 0x1F)

#define MINOR2MAJOR(minor) ( (((minor)>>1) & 1) + (((minor)>>3) & 1) + (((minor)>>5) & 1))
#define MAJOR2MINOR(major, sub_index) 	\
		(	\
			(((major) & 4)<<3) | (((sub_index) & 4) << 2) | 	\
			(((major) & 2)<<2) | (((sub_index) & 2) << 1) |		\
			(((major) & 1)<<1) | (((sub_index) & 1) )			\
		)
//*/

typedef struct palette_hash_entry
{
	int red, green, blue;	
	long minor; // the minor partitioning
	struct palette_hash_entry *nextEntry;
	int palette_index;	
} palette_hash_entry;


typedef struct palette_type_hash
{
	
	int type;		
	int	NumberOfRanges;
	//int* entries; // array of pointers to the data
	palette_hash_entry** data;// array of minor ranges
} palette_type_hash;


void palette_hash_destroy(palette_type_hash *hash);
palette_type_hash *palette_hash_create_from_palette(palette_type *palette);
palette_type_hash *palette_hash_alloc(int numOfColors);
palette_hash_entry* palette_hash_entry_alloc(int red, int green, int blue, int major_minor, int palette_index);
palette_hash_entry* palette_hash_entry_alloc_copy(palette_hash_entry* source);

image_type *image_24to8_UNKNOWN_HASH(image_type * im, palette_type_hash *hash , box2i * pBox);



#endif //__PALETTE_TYPE_HASH_H__