/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/

#include	"Uigp/igp.h"
#include	"Uvec/Vec2d.h"
#include	"Box2d.h"


box2d *box2d_alloc()
{
	box2d *box= (box2d*)malloc(sizeof(box2d));
	box->x0 = box->y0 = box->x1 = box->y1 = 0;
	return box;
}


int
box2_segment( box2d *b, vec2d *p, vec2d *p1, int Fp1, vec2d *v0, vec2d *v1 )
{
vec2d	n;
float	t0,	t1,	y0,	y1;

	if( Fp1 == 1 ){
		n.x = p1->x - p->x;
		n.y = p1->y - p->y;
	}
	else	n = *p1;

	if( n.x == 0 && n.y == 0 )	return( 0 );

	if( n.y == 0 ){

/*
		if( v0->y < b->y0 || v0->y > b->y1 )	return( 0 );
*/
		if( p->y < b->y0 || p->y > b->y1 )	return( 0 );

		t0 = (b->x0 - p->x )/n.x;
		t1 = (b->x1 - p->x )/n.x;

		v0->x = p->x + t0 *n.x;
		v0->y = p->y;
/*
		if( p->x < b->x0 || p->x > b->x1 )	return( 0 );
*/
		if( v0->x < b->x0 || v0->x > b->x1 )	return( 0 );


		v1->x = p->x + t1 *n.x;
		v1->y = p->y;

		return( 1 );
	}


	if( n.x == 0 ){

/*
		if( v0->x < b->x0 || v0->x > b->x1 )	return( 0 );
*/
		if( p->x < b->x0 || p->x > b->x1 )	return( 0 );
		t0 = (b->y0 - p->y )/n.y;
		t1 = (b->y1 - p->y )/n.y;

		v0->x = p->x;
		v0->y = p->y + t0 *n.y;
		if( v0->y < b->y0 || v0->y > b->y1 )	return( 0 );

		v1->x = p->x;
		v1->y = p->y + t1 *n.y;

		return( 1 );
	}




	if( n.x > 0 ){
		t0 = (b->x0 - p->x )/n.x;
		t1 = (b->x1 - p->x )/n.x;
	}
	else	{
		t0 = (b->x1 - p->x )/n.x;
		t1 = (b->x0 - p->x )/n.x;
	}

	if( n.y > 0 ){
		y0 = (b->y0 - p->y )/n.y;
		y1 = (b->y1 - p->y )/n.y;
	}
	else	{
		y0 = (b->y1 - p->y )/n.y;
		y1 = (b->y0 - p->y )/n.y;
	}

	if( t0 < y0 )	t0 = y0;
	if( t1 > y1 )	t1 = y1;


	v0->x = p->x + t0 *n.x;
	v0->y = p->y + t0 *n.y;
	if( v0->x < b->x0-0.001 || v0->x > b->x1+0.001 )	return( 0 );
	if( v0->y < b->y0-0.001 || v0->y > b->y1+0.001 )	return( 0 );

	v1->x = p->x + t1 *n.x;
	v1->y = p->y + t1 *n.y;

	return( 1 );
}



void
box_shift( box2d *b, float x, float y )
{
	b->x0 += x;
	b->x1 += x;

	b->y0 += y;
	b->y1 += y;
}

void
box_update( box2d *b, box2d *b1 )
{
	if( b->x0 > b1->x0 )	b->x0 = b1->x0;
	if( b->x1 < b1->x1 )	b->x1 = b1->x1;

	if( b->y0 > b1->y0 )	b->y0 = b1->y0;
	if( b->y1 < b1->y1 )	b->y1 = b1->y1;
}


void
box2d_union( box2d ab[], int no, box2d *b )
{
int	i;
	*b = ab[0];

	for( i = 1 ; i < no ; i++ ){
		if( b->x0 > ab[i].x0 )	b->x0 = ab[i].x0;
		if( b->x1 < ab[i].x1 )	b->x1 = ab[i].x1;

		if( b->y0 > ab[i].y0 )	b->y0 = ab[i].y0;
		if( b->y1 < ab[i].y1 )	b->y1 = ab[i].y1;
	}
}


void
box2d_round( box2d *b )
{
	if( b->x0 < 0 )	b->x0 = ((int)b->x0) -1;
	else	b->x0 = ((int)b->x0);

	if( b->y0 < 0 )	b->y0 = ((int)b->y0) -1;
	else	b->y0 = ((int)b->y0);


	if( b->x1 < 0 )	b->x1 = ((int)b->x1);
	else	b->x1 = ((int)b->x1+1);

	if( b->y1 < 0 )	b->y1 = ((int)b->y1);
	else	b->y1 = ((int)b->y1+1);
}