/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/*******************
 ***	GpIo.c	***
 *******************/


#include <stdio.h>



#ifndef CE
#include <fcntl.h>                        /* for O_BINARY */
#endif

#include	"igp.h"




void
gp_fwrite_bits( FILE *fp, int code, int no )
{
	while ( no-- != 0 ) {
 
		if ( code & 1L )
			fprintf( fp, "1 " );
		else
			fprintf( fp, "0 " );
 
		code >>= 1;
	}
 
	fprintf( fp, "\n");
}





int
gp_file_create( char *data, int bytes, char *file )
{
FILE	*fp;


	if ( (fp = fopen( file, "wb" )) == NULL )
		return( 0 );

	fwrite( data, 1, bytes, fp );

	fclose( fp );

	return( 1 );
}


int
gp_file_copy( char *ifile, char *ofile )
{
FILE	*ifp,	*fp;
int	c;

	if ( (ifp = fopen( ifile, "rb" )) == NULL )
		return( 0 );

	if ( (fp = fopen( ofile, "wb" )) == NULL )
		return( 0 );

	 while ((c = getc( ifp )) != EOF) 
		putc( c, fp );


	fclose( ifp );
	fclose( fp );

	return( 1 );
}


int
gp_file_append( char *file, int i0, FILE *fp )
{
FILE	*ifp;
int	c;

	if ( (ifp = fopen( file, "rb" )) == NULL )
		return( 0 );


	if( i0 > 0 )	fseek( ifp, i0, 0L );


	 while ((c = getc( ifp )) != EOF) 
		putc( c, fp );


	fclose( ifp );

	return( 1 );
}


int
gp_file_compare( char *file1, char *file2 )
{
FILE	*fp1, *fp2;
int	c1,	c2;

	if ( (fp1 = fopen( file1, "rb" )) == NULL )
		return( -2 );

	if ( (fp2 = fopen( file2, "rb" )) == NULL ){
		fclose( fp1 );
		return( -2 );
	}


	while ((c1 = getc( fp1 )) != EOF){
		if( (c2 = getc( fp2 )) == EOF || c1 != c2 ){
			fclose( fp1 );
			fclose( fp2 );
			return( -1 );
		}
	}
	

	if( getc( fp2 ) != EOF )	return( -1 );

	fclose( fp1 );
	fclose( fp2 );

	return( 1 );
}


int gp_write_INTtable_to_file(char *file, char *name, int table[], int no )
{
FILE *fp;
int	i;

	if( (fp = fopen( file, "wb")) == NULL )	return( -1 );

	fprintf( fp, "const int %s[] = {", name );

	for( i = 0 ; i < no-1 ; i++ )
		fprintf( fp, "%16d,\n", table[i] );

	fprintf( fp, "%16d };\n", table[i] );

	fclose( fp);

	return( 1 );
}



int gp_read_buffer_from_file(char *szFile, u_char **buffer)
{
	FILE *file;
	int size, asize;
	file = fopen(szFile,"rb");
	if ( file == NULL ) 
	{
		*buffer = NULL;
		return 0;
	}

	size = gp_file_size( szFile );

	*buffer = (u_char *) malloc(size * sizeof(u_char) + 2);
	asize = fread( (*buffer), sizeof(u_char), size, file );
	fclose(file);
	return asize;
	
}


int gp_read_file_to_buffer(char *szFile, u_char **data, int *bytes )
{
FILE *fp;

	*bytes = 0;
	if( (fp = fopen(szFile,"rb")) == NULL )
		return( -1 );


	*bytes = gp_file_size( szFile );

	*data = (u_char *) malloc(*bytes * sizeof(u_char) + 2);
	*bytes = fread( (*data), sizeof(u_char), *bytes, fp );
	fclose(fp);

	return *bytes;
}


void gp_write_buffer_to_file(char *szFile, void *buffer, int dwSize)
{
	FILE *file;
	//
	file = fopen(szFile, "wb");

	//
	if(file == NULL)
		return;

	//
	fwrite(buffer, dwSize, 1, file);

	//
	fclose(file);
}
