/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/

/************************
 ***   GpFunction.c   ***
 ************************/
#include <string.h>
#include <stdlib.h>

#include	"igp.h"


int
gp_ilog2( int a )
{
int	i,	b;

	if( a < 0 )     a = -a;
 
	for( i = 0, b  = 1; a >= b ; i++, b <<= 1 );

	return( i );
}


int
gp_bound2( int a )
{
int	i,	b;

	if( a < 0 )     a = -a;
 
	for( i = 0, b = 1; a >= b ; i++, b <<= 1 );

	return( b );
}



/*********  gp_strsep  **********
* @Description	:	
* 
* @Param  char **sptr :  
* @Param  const char *delim :  
* @Return  char *  :  
*****************************************/
char *gp_strsep(char **sptr, const char *delim)
{
	char *start, *ret;
	start = ret = *sptr;
	if ((ret == NULL) || ret == '\0') {
	   return (NULL);
	}

	while (*ret != '\0' &&
		   strchr(delim, *ret) == NULL) {
		ret++;
	}
	if (*ret == '\0') {
		*sptr = NULL;
	} else {
	    *ret = '\0';
	    ret++;
	    *sptr = ret;
	}
	return (start);
}

/*********  *gp_strcasestr  **********
* @Description	:	finds a string in another, NOT case sensitice
* 
* @Param  const char *haystack :string to look at  
* @Param  const char *needle :  string to look for
* @Return  char*  :  pointer in haystack of needle
*****************************************/
char *gp_strcasestr(const char *haystack, const char *needle)
{
  int nlen, hlen;
  int ix;

  if (needle == NULL || haystack == NULL) return (NULL);
  nlen = strlen(needle);
  hlen = strlen(haystack);
 
  for (ix = 0; ix + nlen <= hlen; ix++) {
    if (gp_strnicmp(needle, haystack + ix, nlen) == 0) {
      return ((char *)haystack + ix);
    }
  }
  return (NULL);
}


char *gp_strlwr(char *szString)
{
	char *szResult = szString;
	if (szString)
	{
		char *szPtr = szString;
		while(*szPtr != '\0')
		{
			if (*szPtr >= 'A' && *szPtr <= 'Z')
			{
				*szPtr = (*szPtr) + ('a' - 'A');
			}
			szPtr++;
		}
	}
	return szResult;
} // gp_strlwr


/*
int gp_strnicmp(const char *szString1, const char *szString2, size_t count)
{
	int nResult = -1;
	if (szString1 && szString2 && count > 0)
	{
		char *szStr1 = malloc(count+1);
		char *szStr2 = malloc(count+1);
		strncpy(szStr1, szString1, count);
		strncpy(szStr2, szString2, count);
		nResult = strncmp(gp_strlwr(szStr1), gp_strlwr(szStr2), count);
		free (szStr1);
		free (szStr2);
	}
	return nResult;
} // gp_strnicmp
*/

int 
gp_strnicmp(const char *s1, const char *s2, int n)
{
char	*p1,	*p2,	a1,	a2;
int	i;

	for( i = 0, p1 = (char *)s1, p2 = (char *)s2 ; i < n ; i++, p1++, p2++ ){
		if( *p1 == *p2 )	continue;

		if( *p1 >= 'A' && *p1 <= 'Z' )
			a1 = (*p1) + ('a' - 'A');
		else	a1 = *p1;

		if( *p2 >= 'A' && *p2 <= 'Z' )
			a2 = (*p2) + ('a' - 'A');
		else	a2 = *p2;

		if( a1 != a2 )	return( 1 );
	}

	return 0;
} // gp_strnicmp


int 
gp_stricmp(const char *s1, const char *s2)
{
char	*p1,	*p2,	a1,	a2;
int	i;

	for( i = 0, p1 = (char *)s1, p2 = (char *)s2 ; *p1 != 0 && &p2 != 0 ; i++, p1++, p2++ ){
		if( *p1 == *p2 )	continue;

		if( *p1 >= 'A' && *p1 <= 'Z' )
			a1 = (*p1) + ('a' - 'A');
		else	a1 = *p1;

		if( *p2 >= 'A' && *p2 <= 'Z' )
			a2 = (*p2) + ('a' - 'A');
		else	a2 = *p2;

		if( a1 != a2 )	return( 1 );
	}

	if( *p1 != *p2 )	return( 1 );

	return 0;
} // gp_strnicmp