/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/************************
 *** AppJet2.cArray.c ***
 ************************/
#include <stdio.h>
#include <malloc.h>	

#include	"Uigp/igp.h"
#include	"Jet23Type.h"

static void	jet23_array_alloc( jet23_array *ajet, int ni, int nj );


void
jet23_array_load( char *jetmap_name, jet23_array *ajet )
{
char	jetmap_file[50],	garbage[50];
FILE	*fp;
int	ni,	nj,	i,	j;
float	dxy;

/*
	sprintf( jetmap_file, "%s.jet",jetmap_name );
*/
	sprintf( jetmap_file, "%s",jetmap_name );

	if( (fp = fopen( jetmap_file, "r" ) ) == NULL )
		error("load_jetmap","can't open jetmap file" );


	fscanf( fp,"%[^G]" ,garbage);
	fscanf( fp,"%s %d %d %f" ,garbage,  &ni, &nj, &dxy );

	jet23_array_alloc( ajet, ni, nj );

	ajet->dxy = dxy;
	ajet->inv_dxy = 1.0 / dxy;
	ajet->dh = 0.5*dxy;


	ajet->Xmin = ajet->Ymin = 0;

	ajet->Xmax = ajet->Ni * ajet->dxy;
	ajet->Ymax = ajet->Nj * ajet->dxy;

	for( i = 0; i < ajet->Ni ; i++ )
		for( j = 0; j < ajet->Nj ; j++ )
			load_jet2( fp, 3, (jet2 * )&ajet->jet[i][j] );

}



void
jet23_array_realloc( jet23_array *ajet, int ni, int nj )
{
	if( ajet->Ni == ni && ajet->Nj == nj )	return;

	jet23_array_free( ajet );

	jet23_array_alloc( ajet, ni, nj );
}


static void
jet23_array_alloc( jet23_array *ajet, int ni, int nj )
{
int          i;
int          byte;

	ajet->Ni  = ni ;
	ajet->Nj  = nj ;

	ajet->jet = (jet23 **)malloc( ajet->Ni * sizeof( jet23 * ) );

	byte = ajet->Nj * sizeof(jet23);

	for( i = 0 ; i < ajet->Ni ; i++ )
		ajet->jet[i] = (jet23 *)malloc( byte );
}



void
jet23_array_free( jet23_array *ajet )
{
int          i;

	for( i = 0 ; i < ajet->Ni ; i++ ){
		free( ajet->jet[i] );
		ajet->jet[i] = NULL;
	}

	free( ajet->jet );
	ajet->Ni = ajet->Nj = 0;
}
