/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/************************
 *** jet2_array.c ***
 ************************/
#include <stdio.h>

#include	"Uigp/igp.h"
#include	"Jet2Type.h"
#include	"Uunity/unity.h"

float
jet2map_value( jet2_array *ajet, float x, float y )
{
jet2	f;

	jet2map_jet2( ajet, x, y, &f );

	return( J00(f) );
}


int
jet2map_jet2( jet2_array *ajet, float x, float y, jet2 *f )
{
int	i,	j;
float	x0,	y0;
jet2	u,	t,	t1;

	if( x < ajet->Xmin + ajet->dh || x > ajet->Xmax - ajet->dh ||
			y < ajet->Ymin + ajet->dh || y > ajet->Ymax - ajet->dh )
		return( 0 );

	unity2_set_paramter( ajet->dh, ajet->dh );

	i = (x - ajet->Xmin ) *ajet->inv_dxy;
	j = (y - ajet->Ymin ) *ajet->inv_dxy;

        x0 = x - ( ajet->Xmin + i*ajet->dxy +ajet->dh );
        y0 = y - ( ajet->Ymin + j*ajet->dxy +ajet->dh);

	unity2_jet2( &u, x0, y0 );
	shift_jet2( &ajet->jet[i][j], x0, y0, &t );
	product_jet2( &u, &t, f );


	if( i+1 < ajet->Ni ){
		unity2_jet2( &u, x0 - ajet->dxy, y0 );
		if( J00(u) != 0 ){
			shift_jet2( &ajet->jet[i+1][j], x0 - ajet->dxy, y0, &t);
			product_jet2( &u, &t, &t1 );
			*f = sum_jet2( f, &t1, 2 );
		}
	}


	if( i+1 < ajet->Ni && j+1 < ajet->Nj  ){
		unity2_jet2( &u, x0 - ajet->dxy, y0 - ajet->dxy );
		if( J00(u) != 0 ){
			shift_jet2( &ajet->jet[i+1][j+1],
					x0 -ajet->dxy, y0 -ajet->dxy, &t );
			product_jet2( &u, &t, &t1 );
			*f = sum_jet2( f, &t1, 2 );
		}
	}


	if( j+1 < ajet->Nj  ){
		unity2_jet2( &u, x0 , y0 - ajet->dxy );
		if( J00(u) != 0 ){
			shift_jet2( &ajet->jet[i][j+1], x0, y0 - ajet->dxy, &t);
			product_jet2( &u, &t, &t1 );
			*f = sum_jet2( f, &t1, 2 );
		}
	}


	if( i > 0 && j+1 < ajet->Nj  ){
		unity2_jet2( &u, x0+ajet->dxy , y0 - ajet->dxy );
		if( J00(u) != 0 ){
			shift_jet2( &ajet->jet[i-1][j+1],
					x0 +ajet->dxy, y0-ajet->dxy, &t ) ;
			product_jet2( &u, &t, &t1 );
			*f = sum_jet2( f, &t1, 2 );
		}
	}


	if( i > 0 ){
		unity2_jet2( &u, x0+ajet->dxy , y0 );
		if( J00(u) != 0 ){
			shift_jet2( &ajet->jet[i-1][j], x0 + ajet->dxy, y0, &t);
			product_jet2( &u, &t, &t1 );
			*f = sum_jet2( f, &t1, 2 );
		}
	}


	if( i > 0 && j > 0 ){
		unity2_jet2( &u, x0+ajet->dxy , y0+ajet->dxy );
		if( J00(u) != 0 ){
			shift_jet2( &ajet->jet[i-1][j-1],
					x0 + ajet->dxy, y0+ajet->dxy, &t );
			product_jet2( &u, &t, &t1 );
			*f = sum_jet2( f, &t1, 2 );
		}
	}


	if( j > 0 ){
		unity2_jet2( &u, x0 , y0+ajet->dxy );
		if( J00(u) != 0 ){
			shift_jet2( &ajet->jet[i][j-1], x0,  y0+ajet->dxy, &t ) ;
			product_jet2( &u, &t, &t1 );
			*f = sum_jet2( f, &t1, 2 );
		}
	}


	if( i+1 < ajet->Ni && j > 0 ){
		unity2_jet2( &u, x0 - ajet->dxy , y0+ajet->dxy );
		if( J00(u) != 0 ){
			shift_jet2( &ajet->jet[i+1][j-1],
					x0 -ajet->dxy,  y0+ajet->dxy, &t );
			product_jet2( &u, &t, &t1 );
			*f = sum_jet2( f, &t1, 2 );
		}
	}
	return( 1 );

}
