/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/

#ifndef _WINDOWS
#include	<sys/times.h>
#else
#include	<time.h>
#endif

#include	<sys/timeb.h>


#include   "Uigp/igp.h"
#include   "GpTime.h"


#include   <stdio.h>

void
gp_time_init( gp_time_type *t )
{
	t->utime = 0;
	t->stime = 0;
	t->cutime = 0;
	t->cstime = 0;

	t->sec = 0;

	t->clock = 0;
}


void
gp_time_start( gp_time_type *t )
{
#if !defined _WINDOWS && !defined WIN32 
struct tms      ptm;
#endif
struct timeb    tp;

#if !defined _WINDOWS && !defined WIN32
	times( &ptm );
	t->utime0 = ptm.tms_utime;
	t->cutime0 = ptm.tms_cutime;
	t->stime0 = ptm.tms_stime;
	t->cstime0 = ptm.tms_cstime;
 #endif

	ftime( &tp );
	t->sec0 = tp.time;
	t->mil_sec0 = tp.millitm;

	t->clock0 = clock();
}


void
gp_time_add( gp_time_type *t )
{
struct timeb    tp;
#ifndef _WINDOWS
struct tms      tms1;
#endif
int	sec,	mil_sec;

#ifndef _WINDOWS
	times( &tms1 );

	t->utime += tms1.tms_utime - t->utime0;
	t->stime += tms1.tms_stime - t->stime0;
	t->cutime += tms1.tms_cutime - t->cutime0;
	t->cstime += tms1.tms_cstime - t->cstime0;
 #endif

	t->clock += clock() - t->clock0;

	ftime( &tp );
	sec = tp.time;
	mil_sec = tp.millitm;

	t->sec += sec - t->sec0 + ((int)mil_sec - t->mil_sec0)/1000.0;
}





void 
gp_time_print( FILE *fp, char *string, gp_time_type *t )
{
	fprintf( fp, "\n%-10s  time:  ", string);
#ifndef _WINDOWS
	fprintf( fp, "CPU:  %.2fs      ", (t->utime + t->cutime) / 60.);
	fprintf( fp, "sys:  %.2fs ", (t->stime + t->cstime) / 60.);
	fprintf( fp, "   %.2fs", t->clock/1000000.0 );
#else
	fprintf( fp, "   %.2fs", (float )t->clock / CLOCKS_PER_SEC );
#endif

	fprintf( fp, "   %.2fs", t->sec );
	fprintf( fp, "\n");
}

void 
gp_time_msg( char *string, gp_time_type *t )
{
char	msg[256];

#ifndef _WINDOWS
	sprintf( msg, "%-10s  time:  %.2fs       %.2fs",
					string,
					(float )t->clock / 1000000.0,
					t->sec );
#else
	sprintf( msg, "%-10s  time:  %.2fs       %.2fs",
					string,
					(float )t->clock / CLOCKS_PER_SEC,
					t->sec );
#endif

	warning( "time", msg );
}




int
gp_base_time()
{
struct	timeb	tp;

	ftime( &tp );
	return( tp.time );
}


int
gp_time( int base_time )
{
struct	timeb	tp;
int	sec;

	ftime( &tp );
	sec = tp.time - base_time;


	return( sec *1000 + tp.millitm );
}


char * 
gp_dateTime_iso681( char *buf, u_int timeStamp )
{
time_t ltime;
struct tm *gmt;//,	*timeInfo;

//	time(&ltime);
	ltime = timeStamp;
	if( ltime == 0 )
		ltime = time(0);
    gmt = localtime(&ltime);
//	gmt = gmtime( &ltime );

	sprintf( buf, "%.4d-%.2d-%.2d %.2d:%.2d:%.2d",
				1900+gmt->tm_year, gmt->tm_mon+1, gmt->tm_mday,
				gmt->tm_hour, gmt->tm_min, gmt->tm_sec );

	return( buf );
}


char * 
gp_time_iso681( char *buf, u_int timeStamp )
{
struct tm *gmt;//,	*timeInfo;

	gmt = localtime((time_t*)&timeStamp);
//	gmt = gmtime( (time_t*)&timeStamp );

	sprintf( buf, "%.2d:%.2d:%.2d",
				gmt->tm_hour, gmt->tm_min, gmt->tm_sec );
	return( buf );
}