/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
#include "hrtime.h"

HRTimer::HRTimer()
{
#ifdef WIN32
	init();
#endif
	reset();
}

void 
HRTimer::reset()
{
	GET_LONG( total_ ) = 0L;
	GET_PLONG( ptotal_ ) = 0L;
}

unsigned long 
HRTimer::sec()
{
	return long(GET_LONG( total_ ) / SCALE_LIMIT);
}

unsigned long 
HRTimer::msec()
{
	return long(GET_LONG( total_ ) % SCALE_LIMIT);
}

unsigned long 
HRTimer::p_sec()
{
	return long(GET_PLONG( ptotal_ ) / SCALE_LIMIT);
}

unsigned long 
HRTimer::p_msec()
{
	return long(GET_PLONG( ptotal_ ) % SCALE_LIMIT);
}

#if defined( WIN32 )

void 
HRTimer::init( void )
{
	hr_ptime_t sf;
	QueryPerformanceFrequency( &sf );

	GET_LONG(  scale_factor_ )  = SCALE_LIMIT / CLOCKS_PER_SEC;
	GET_PLONG( scale_pfactor_ ) = SCALE_LIMIT / GET_PLONG( sf );
}

void 
HRTimer::start()
{
	start_  = clock();
	QueryPerformanceCounter( &pstart_ ); 
}

void 
HRTimer::stop()
{
	LARGE_INTEGER li;
	QueryPerformanceCounter( &li ); 
	GET_LONG(total_)  += (GET_LONG(clock()) - GET_LONG(start_)) * GET_LONG( scale_factor_ ); 
	GET_PLONG(ptotal_) += (GET_PLONG(li) - GET_PLONG(pstart_)) * GET_PLONG( scale_pfactor_ ); 
}


#else  // WIN32
#define NANO 1000000000

void 
HRTimer::start()
{
	start_  = gethrtime();
	pstart_ = gethrvtime();
}

void 
HRTimer::stop()
{
	total_  += (gethrtime() - start_) / (NANO / SCALE_LIMIT);
	ptotal_ += (gethrvtime() - pstart_) / (NANO / SCALE_LIMIT);
}

#endif // WIN32

