/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
#ifndef __HRTIME_H__
#define __HRTIME_H__

#define SCALE_LIMIT    1000  

#if defined( WIN32 )
#	include <Windows.h>
#	include <time.h>
#	define hr_time_t      clock_t
#	define hr_ptime_t     LARGE_INTEGER
#	define GET_LONG(x)    x 
#	define GET_PLONG(x)   x.QuadPart
#else  
#	include <sys/time.h>
#	define hr_time_t  hrtime_t
#	define hr_ptime_t hrtime_t
#	define GET_LONG(x)  x 
#	define GET_PLONG(x) x
#endif // WIN32

class HRTimer 
{
public:
	HRTimer();

	void reset();

	// Start timer
	void start();

	// Stop Timer
	void stop();

	// Pause Timer
	void pause();

	// Resume Timer
	void resume();

	unsigned long sec();
	unsigned long msec();

	unsigned long p_sec();
	unsigned long p_msec();

private:
	void init( void );
	hr_time_t start_;
	hr_time_t total_;

	hr_ptime_t pstart_;
	hr_ptime_t ptotal_;

	hr_time_t scale_factor_;
	hr_ptime_t scale_pfactor_;
};




#endif // __HRTIME_H__