/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
#include "BitVector.h"
#include <stdlib.h>

BitVector * createBitVector(unsigned short k)
{
	BitVector * p = (BitVector *)malloc(sizeof(BitVector));
	if (!p) return p;
	p->length = k;
	p->data = (unsigned long *)calloc((p->length + 31) / 32, sizeof(unsigned long));
	if (!p->data)
	{
		free(p);
		return NULL;
	}
	return p;
}

void deleteBitVector(BitVector * p)
{
	if (p->data)
		free(p->data);
	free(p);
}

void setBitVector(BitVector * pBV, int i)
{
	if ((i < 0)||(i >= pBV->length))
		return;
	pBV->data[i / 32] |= (1L << (i % 32));
}

int firstSetBitOfVector(BitVector * pBV)
{
	unsigned long dw;
	int i, j, k;

	k = -1;
	for (i = 0; i < (pBV->length + 31) / 32; i++)
	{
		if (dw = pBV->data[i])
		{
			for (j = 0; j < 32; j++)
			{
				if (dw & (1L << j))
				{
					k = i * 32 + j;
					break;
				}
			}
			break;
		}
	}
	return k;
}

int lastSetBitOfVector(BitVector * pBV)
{
	unsigned long dw;
	int i, j, k;
	k = -1;

	for (i = (pBV->length + 31) / 32 - 1; i >= 0; i--)
	{
		if (dw = pBV->data[i])
		{
			for (j = 31; j >= 0; j--)
			{
				if (dw & (1L << j))
				{
					k = i * 32 + j;
					break;
				}
			}
			break;
		}
	}
	return k;
}
