/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/

#include	<stdio.h>
#include	<stdlib.h>
#include	<math.h>
#include	<string.h>

extern "C" {
#include	"Uigp/igp.h"
#include	"BMP/ImageBmp.h"
#include	"ImageType/ImageType.h"
#include	"ImageType/ImageTool.h"
#include "Encoding/Encoding.h"
#include "AgpTool/AgpTool.h"
}


#include "VimTransform.h"


static void	read_command_line(int argc, char *argv[],
				  int *filetr, int *coding, 
				  char *inFile, char *outFile );






int
main( int argc, char **argv )
{
char	inFile[256],	outFile[256];
int		filter,	coding;
image_type	*im;
gp_array *agp,	*cagp;
char	dir[256],	name[256],	extension[256],	file[256];


	read_command_line( argc, argv, &filter, &coding, inFile, outFile );



	if( (im = image_read_bmp_file( inFile )) == NULL )
		error( "VimEncoder", "Read image: failed" );



	int ret = vim_transform( im, filter, &cagp );
	if( ret < 0 )
		error( "VimEncoder", "vim transform: failed" );

	agp = agp_make_copy( cagp );
	vim_release( cagp );



	gp_filename_split( outFile, dir, name, extension );
	sprintf( file, "%s%s.vim", name, extension );
	encoding_agp_to_file( agp, coding, dir, file );


	agp_destroy( agp );

	image_destory( im, 1 );
	return( 1 );
}






static char *Usage = 
	"Usage: VimEncoder    [-F#]  [-C#]  imageFile  outFile";



static void	
read_command_line(int argc, char *argv[],
				  int *filetr, int *coding, 
				  char *inFile, char *outFile )
{
int     k;

	k = 1;


	*filetr = 0;
	if (k < argc && gp_strnicmp(argv[k], "-F", 2) == 0  ){
		sscanf( argv[k]+2, "%d", filetr );
		k++;
	}

	*coding = 0;
	if (k < argc && gp_strnicmp(argv[k], "-C", 2) == 0  ){
		sscanf( argv[k]+2, "%d", coding );
		k++;
	}




	if (k < argc) {
		sprintf( inFile, "%s", argv[k]);
		k++;
	}
	else	error("read_command_line", Usage);
	
	if (k < argc) {
		sprintf( outFile, "%s", argv[k]);
		k++;
	}
	else	error("read_command_line", Usage);

}
